/********************************************************************************
** Form generated from reading UI file 'google_fonts_widget.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GOOGLE_FONTS_WIDGET_H
#define UI_GOOGLE_FONTS_WIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QTableView>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "widgets/font/font_size_widget.hpp"

namespace glaxnimate {
namespace gui {
namespace font {

class Ui_GoogleFontsWidget
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_google_error;
    QProgressBar *google_fonts_progress;
    QToolButton *button_google_refresh;
    QTableView *view_google_fonts;
    QLabel *label;
    QLabel *label_3;
    QLabel *label_2;
    QLineEdit *edit_family;
    QVBoxLayout *verticalLayout;
    QListWidget *view_style;
    QGroupBox *groupBox;
    QFormLayout *formLayout;
    QLabel *label_4;
    QComboBox *combo_system;
    QLabel *label_5;
    QComboBox *combo_category;
    glaxnimate::gui::font::FontSizeWidget *size_widget;

    void setupUi(QWidget *glaxnimate__gui__font__GoogleFontsWidget)
    {
        if (glaxnimate__gui__font__GoogleFontsWidget->objectName().isEmpty())
            glaxnimate__gui__font__GoogleFontsWidget->setObjectName("glaxnimate__gui__font__GoogleFontsWidget");
        glaxnimate__gui__font__GoogleFontsWidget->resize(736, 566);
        gridLayout = new QGridLayout(glaxnimate__gui__font__GoogleFontsWidget);
        gridLayout->setObjectName("gridLayout");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_google_error = new QLabel(glaxnimate__gui__font__GoogleFontsWidget);
        label_google_error->setObjectName("label_google_error");
        label_google_error->setText(QString::fromUtf8(""));

        horizontalLayout_2->addWidget(label_google_error);

        google_fonts_progress = new QProgressBar(glaxnimate__gui__font__GoogleFontsWidget);
        google_fonts_progress->setObjectName("google_fonts_progress");

        horizontalLayout_2->addWidget(google_fonts_progress);

        button_google_refresh = new QToolButton(glaxnimate__gui__font__GoogleFontsWidget);
        button_google_refresh->setObjectName("button_google_refresh");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("view-refresh");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_google_refresh->setIcon(icon);

        horizontalLayout_2->addWidget(button_google_refresh);


        gridLayout->addLayout(horizontalLayout_2, 4, 0, 1, 3);

        view_google_fonts = new QTableView(glaxnimate__gui__font__GoogleFontsWidget);
        view_google_fonts->setObjectName("view_google_fonts");
        view_google_fonts->setAlternatingRowColors(true);
        view_google_fonts->setSelectionMode(QAbstractItemView::SingleSelection);
        view_google_fonts->setSelectionBehavior(QAbstractItemView::SelectRows);
        view_google_fonts->setShowGrid(false);
        view_google_fonts->setSortingEnabled(true);

        gridLayout->addWidget(view_google_fonts, 3, 0, 1, 1);

        label = new QLabel(glaxnimate__gui__font__GoogleFontsWidget);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        label_3 = new QLabel(glaxnimate__gui__font__GoogleFontsWidget);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 0, 2, 1, 1);

        label_2 = new QLabel(glaxnimate__gui__font__GoogleFontsWidget);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 0, 1, 1, 1);

        edit_family = new QLineEdit(glaxnimate__gui__font__GoogleFontsWidget);
        edit_family->setObjectName("edit_family");

        gridLayout->addWidget(edit_family, 2, 0, 1, 1);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        view_style = new QListWidget(glaxnimate__gui__font__GoogleFontsWidget);
        view_style->setObjectName("view_style");

        verticalLayout->addWidget(view_style);

        groupBox = new QGroupBox(glaxnimate__gui__font__GoogleFontsWidget);
        groupBox->setObjectName("groupBox");
        formLayout = new QFormLayout(groupBox);
        formLayout->setObjectName("formLayout");
        label_4 = new QLabel(groupBox);
        label_4->setObjectName("label_4");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label_4);

        combo_system = new QComboBox(groupBox);
        combo_system->setObjectName("combo_system");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(combo_system->sizePolicy().hasHeightForWidth());
        combo_system->setSizePolicy(sizePolicy);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, combo_system);

        label_5 = new QLabel(groupBox);
        label_5->setObjectName("label_5");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label_5);

        combo_category = new QComboBox(groupBox);
        combo_category->setObjectName("combo_category");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, combo_category);


        verticalLayout->addWidget(groupBox);


        gridLayout->addLayout(verticalLayout, 2, 1, 2, 1);

        size_widget = new glaxnimate::gui::font::FontSizeWidget(glaxnimate__gui__font__GoogleFontsWidget);
        size_widget->setObjectName("size_widget");

        gridLayout->addWidget(size_widget, 1, 2, 3, 1);

        gridLayout->setColumnStretch(0, 20);
        gridLayout->setColumnStretch(1, 5);
        gridLayout->setColumnStretch(2, 5);

        retranslateUi(glaxnimate__gui__font__GoogleFontsWidget);

        QMetaObject::connectSlotsByName(glaxnimate__gui__font__GoogleFontsWidget);
    } // setupUi

    void retranslateUi(QWidget *glaxnimate__gui__font__GoogleFontsWidget)
    {
        button_google_refresh->setText(tr2i18n("Reload", nullptr));
        label->setText(tr2i18n("Font", nullptr));
        label_3->setText(tr2i18n("Size", nullptr));
        label_2->setText(tr2i18n("Font Style", nullptr));
        groupBox->setTitle(tr2i18n("Filters", nullptr));
        label_4->setText(tr2i18n("Writing System", nullptr));
        label_5->setText(tr2i18n("Category", nullptr));
        (void)glaxnimate__gui__font__GoogleFontsWidget;
    } // retranslateUi

};

} // namespace font
} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace font {
namespace Ui {
    class GoogleFontsWidget: public Ui_GoogleFontsWidget {};
} // namespace Ui
} // namespace font
} // namespace gui
} // namespace glaxnimate

#endif // GOOGLE_FONTS_WIDGET_H

