/********************************************************************************
** Form generated from reading UI file 'emoji_set_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EMOJI_SET_DIALOG_H
#define UI_EMOJI_SET_DIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_EmojiSetDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTableWidget *emoji_set_view;
    QProgressBar *progress_bar;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushButton_2;
    QPushButton *button_view_website;
    QPushButton *button_download;
    QPushButton *button_add_emoji;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushButton_3;

    void setupUi(QDialog *EmojiSetDialog)
    {
        if (EmojiSetDialog->objectName().isEmpty())
            EmojiSetDialog->setObjectName("EmojiSetDialog");
        EmojiSetDialog->resize(800, 600);
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("face-smile");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        EmojiSetDialog->setWindowIcon(icon);
        verticalLayout = new QVBoxLayout(EmojiSetDialog);
        verticalLayout->setObjectName("verticalLayout");
        emoji_set_view = new QTableWidget(EmojiSetDialog);
        if (emoji_set_view->columnCount() < 9)
            emoji_set_view->setColumnCount(9);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        emoji_set_view->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        emoji_set_view->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        emoji_set_view->setHorizontalHeaderItem(2, __qtablewidgetitem2);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        emoji_set_view->setHorizontalHeaderItem(3, __qtablewidgetitem3);
        QTableWidgetItem *__qtablewidgetitem4 = new QTableWidgetItem();
        emoji_set_view->setHorizontalHeaderItem(4, __qtablewidgetitem4);
        QTableWidgetItem *__qtablewidgetitem5 = new QTableWidgetItem();
        emoji_set_view->setHorizontalHeaderItem(5, __qtablewidgetitem5);
        QTableWidgetItem *__qtablewidgetitem6 = new QTableWidgetItem();
        emoji_set_view->setHorizontalHeaderItem(6, __qtablewidgetitem6);
        QTableWidgetItem *__qtablewidgetitem7 = new QTableWidgetItem();
        emoji_set_view->setHorizontalHeaderItem(7, __qtablewidgetitem7);
        QTableWidgetItem *__qtablewidgetitem8 = new QTableWidgetItem();
        emoji_set_view->setHorizontalHeaderItem(8, __qtablewidgetitem8);
        emoji_set_view->setObjectName("emoji_set_view");
        emoji_set_view->setEditTriggers(QAbstractItemView::NoEditTriggers);
        emoji_set_view->setSelectionMode(QAbstractItemView::SingleSelection);
        emoji_set_view->setSelectionBehavior(QAbstractItemView::SelectRows);
        emoji_set_view->setIconSize(QSize(72, 72));
        emoji_set_view->setShowGrid(false);

        verticalLayout->addWidget(emoji_set_view);

        progress_bar = new QProgressBar(EmojiSetDialog);
        progress_bar->setObjectName("progress_bar");
        progress_bar->setValue(24);

        verticalLayout->addWidget(progress_bar);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushButton_2 = new QPushButton(EmojiSetDialog);
        pushButton_2->setObjectName("pushButton_2");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("view-refresh");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        pushButton_2->setIcon(icon1);

        horizontalLayout->addWidget(pushButton_2);

        button_view_website = new QPushButton(EmojiSetDialog);
        button_view_website->setObjectName("button_view_website");
        button_view_website->setEnabled(false);
        QIcon icon2;
        iconThemeName = QString::fromUtf8("internet-web-browser");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_view_website->setIcon(icon2);

        horizontalLayout->addWidget(button_view_website);

        button_download = new QPushButton(EmojiSetDialog);
        button_download->setObjectName("button_download");
        button_download->setEnabled(false);
        QIcon icon3;
        iconThemeName = QString::fromUtf8("download");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_download->setIcon(icon3);

        horizontalLayout->addWidget(button_download);

        button_add_emoji = new QPushButton(EmojiSetDialog);
        button_add_emoji->setObjectName("button_add_emoji");
        button_add_emoji->setEnabled(false);
        QIcon icon4;
        iconThemeName = QString::fromUtf8("smiley-add");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon4 = QIcon::fromTheme(iconThemeName);
        } else {
            icon4.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_add_emoji->setIcon(icon4);

        horizontalLayout->addWidget(button_add_emoji);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushButton_3 = new QPushButton(EmojiSetDialog);
        pushButton_3->setObjectName("pushButton_3");
        QIcon icon5;
        iconThemeName = QString::fromUtf8("dialog-close");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon5 = QIcon::fromTheme(iconThemeName);
        } else {
            icon5.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        pushButton_3->setIcon(icon5);

        horizontalLayout->addWidget(pushButton_3);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(EmojiSetDialog);
        QObject::connect(pushButton_3, &QPushButton::clicked, EmojiSetDialog, qOverload<>(&QDialog::reject));
        QObject::connect(pushButton_2, SIGNAL(clicked()), EmojiSetDialog, SLOT(reload_sets()));
        QObject::connect(button_view_website, SIGNAL(clicked()), EmojiSetDialog, SLOT(view_website()));
        QObject::connect(emoji_set_view, SIGNAL(currentCellChanged(int,int,int,int)), EmojiSetDialog, SLOT(set_selected(int)));
        QObject::connect(button_download, SIGNAL(clicked()), EmojiSetDialog, SLOT(download_selected()));
        QObject::connect(button_add_emoji, SIGNAL(clicked()), EmojiSetDialog, SLOT(add_emoji()));

        QMetaObject::connectSlotsByName(EmojiSetDialog);
    } // setupUi

    void retranslateUi(QDialog *EmojiSetDialog)
    {
        EmojiSetDialog->setWindowTitle(tr2i18n("Emoji Set", nullptr));
        QTableWidgetItem *___qtablewidgetitem = emoji_set_view->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(tr2i18n("Name", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = emoji_set_view->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(tr2i18n("License", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = emoji_set_view->horizontalHeaderItem(8);
        ___qtablewidgetitem2->setText(tr2i18n("Downloaded", nullptr));
        pushButton_2->setText(tr2i18n("Reload", nullptr));
        button_view_website->setText(tr2i18n("View Website", nullptr));
        button_download->setText(tr2i18n("Download", nullptr));
        button_add_emoji->setText(tr2i18n("Add Emoji...", nullptr));
        pushButton_3->setText(tr2i18n("Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class EmojiSetDialog: public Ui_EmojiSetDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // EMOJI_SET_DIALOG_H

