/****************************************************************************
** Meta object code from reading C++ file 'color_2d_slider.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/external/Qt-Color-Widgets/include/QtColorWidgets/color_2d_slider.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'color_2d_slider.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13color_widgets13Color2DSliderE_t {};
} // unnamed namespace

template <> constexpr inline auto color_widgets::Color2DSlider::qt_create_metaobjectdata<qt_meta_tag_ZN13color_widgets13Color2DSliderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "color_widgets::Color2DSlider",
        "colorChanged",
        "",
        "QColor",
        "colorSelected",
        "componentXChanged",
        "Component",
        "componentX",
        "componentYChanged",
        "componentY",
        "setColor",
        "c",
        "setHue",
        "h",
        "setSaturation",
        "s",
        "setValue",
        "v",
        "setComponentX",
        "setComponentY",
        "color",
        "hue",
        "saturation",
        "value",
        "Hue",
        "Saturation",
        "Value"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(QColor)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'colorSelected'
        QtMocHelpers::SignalData<void(QColor)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'componentXChanged'
        QtMocHelpers::SignalData<void(enum Component)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'componentYChanged'
        QtMocHelpers::SignalData<void(enum Component)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 9 },
        }}),
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(const QColor &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 11 },
        }}),
        // Slot 'setHue'
        QtMocHelpers::SlotData<void(qreal)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 13 },
        }}),
        // Slot 'setSaturation'
        QtMocHelpers::SlotData<void(qreal)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 15 },
        }}),
        // Slot 'setValue'
        QtMocHelpers::SlotData<void(qreal)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 17 },
        }}),
        // Slot 'setComponentX'
        QtMocHelpers::SlotData<void(enum Component)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'setComponentY'
        QtMocHelpers::SlotData<void(enum Component)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(20, 0x80000000 | 3, QMC::Readable | QMC::Writable | QMC::Designable | QMC::Scriptable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'hue'
        QtMocHelpers::PropertyData<qreal>(21, QMetaType::QReal, QMC::Readable | QMC::Writable | QMC::Scriptable | QMC::Stored | QMC::StdCppSet),
        // property 'saturation'
        QtMocHelpers::PropertyData<qreal>(22, QMetaType::QReal, QMC::Readable | QMC::Writable | QMC::Scriptable | QMC::Stored | QMC::StdCppSet),
        // property 'value'
        QtMocHelpers::PropertyData<qreal>(23, QMetaType::QReal, QMC::Readable | QMC::Writable | QMC::Scriptable | QMC::Stored | QMC::StdCppSet),
        // property 'componentX'
        QtMocHelpers::PropertyData<enum Component>(7, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'componentY'
        QtMocHelpers::PropertyData<enum Component>(9, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Component'
        QtMocHelpers::EnumData<enum Component>(6, 6, QMC::EnumFlags{}).add({
            {   24, Component::Hue },
            {   25, Component::Saturation },
            {   26, Component::Value },
        }),
    };
    return QtMocHelpers::metaObjectData<Color2DSlider, qt_meta_tag_ZN13color_widgets13Color2DSliderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject color_widgets::Color2DSlider::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets13Color2DSliderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets13Color2DSliderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13color_widgets13Color2DSliderE_t>.metaTypes,
    nullptr
} };

void color_widgets::Color2DSlider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Color2DSlider *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 1: _t->colorSelected((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 2: _t->componentXChanged((*reinterpret_cast<std::add_pointer_t<enum Component>>(_a[1]))); break;
        case 3: _t->componentYChanged((*reinterpret_cast<std::add_pointer_t<enum Component>>(_a[1]))); break;
        case 4: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 5: _t->setHue((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 6: _t->setSaturation((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 7: _t->setValue((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 8: _t->setComponentX((*reinterpret_cast<std::add_pointer_t<enum Component>>(_a[1]))); break;
        case 9: _t->setComponentY((*reinterpret_cast<std::add_pointer_t<enum Component>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Color2DSlider::*)(QColor )>(_a, &Color2DSlider::colorChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Color2DSlider::*)(QColor )>(_a, &Color2DSlider::colorSelected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Color2DSlider::*)(Component )>(_a, &Color2DSlider::componentXChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Color2DSlider::*)(Component )>(_a, &Color2DSlider::componentYChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QColor*>(_v) = _t->color(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->hue(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->saturation(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->value(); break;
        case 4: *reinterpret_cast<enum Component*>(_v) = _t->componentX(); break;
        case 5: *reinterpret_cast<enum Component*>(_v) = _t->componentY(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setColor(*reinterpret_cast<QColor*>(_v)); break;
        case 1: _t->setHue(*reinterpret_cast<qreal*>(_v)); break;
        case 2: _t->setSaturation(*reinterpret_cast<qreal*>(_v)); break;
        case 3: _t->setValue(*reinterpret_cast<qreal*>(_v)); break;
        case 4: _t->setComponentX(*reinterpret_cast<enum Component*>(_v)); break;
        case 5: _t->setComponentY(*reinterpret_cast<enum Component*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *color_widgets::Color2DSlider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *color_widgets::Color2DSlider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets13Color2DSliderE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int color_widgets::Color2DSlider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void color_widgets::Color2DSlider::colorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void color_widgets::Color2DSlider::colorSelected(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void color_widgets::Color2DSlider::componentXChanged(Component _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void color_widgets::Color2DSlider::componentYChanged(Component _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
