/********************************************************************************
** Form generated from reading UI file 'snippets.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SNIPPETS_H
#define UI_SNIPPETS_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDockWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "widgets/docks/snippet_list_widget.hpp"

QT_BEGIN_NAMESPACE

class Ui_dock_snippets
{
public:
    QWidget *dockWidgetContents;
    QVBoxLayout *verticalLayout;
    glaxnimate::gui::SnippetListWidget *widget;

    void setupUi(QDockWidget *dock_snippets)
    {
        if (dock_snippets->objectName().isEmpty())
            dock_snippets->setObjectName("dock_snippets");
        dock_snippets->resize(400, 300);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("dialog-scripts")));
        dock_snippets->setWindowIcon(icon);
        dockWidgetContents = new QWidget();
        dockWidgetContents->setObjectName("dockWidgetContents");
        verticalLayout = new QVBoxLayout(dockWidgetContents);
        verticalLayout->setObjectName("verticalLayout");
        widget = new glaxnimate::gui::SnippetListWidget(dockWidgetContents);
        widget->setObjectName("widget");

        verticalLayout->addWidget(widget);

        dock_snippets->setWidget(dockWidgetContents);

        retranslateUi(dock_snippets);

        QMetaObject::connectSlotsByName(dock_snippets);
    } // setupUi

    void retranslateUi(QDockWidget *dock_snippets)
    {
        dock_snippets->setWindowTitle(tr2i18n("Snippets", nullptr));
    } // retranslateUi

};

namespace Ui {
    class dock_snippets: public Ui_dock_snippets {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SNIPPETS_H

