/********************************************************************************
** Form generated from reading UI file 'gradients.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GRADIENTS_H
#define UI_GRADIENTS_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDockWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "widgets/shape_style/gradient_list_widget.hpp"

QT_BEGIN_NAMESPACE

class Ui_dock_gradients
{
public:
    QWidget *dockWidgetContents;
    QVBoxLayout *verticalLayout;
    glaxnimate::gui::GradientListWidget *widget_gradients;

    void setupUi(QDockWidget *dock_gradients)
    {
        if (dock_gradients->objectName().isEmpty())
            dock_gradients->setObjectName("dock_gradients");
        dock_gradients->resize(400, 300);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("color-gradient")));
        dock_gradients->setWindowIcon(icon);
        dockWidgetContents = new QWidget();
        dockWidgetContents->setObjectName("dockWidgetContents");
        verticalLayout = new QVBoxLayout(dockWidgetContents);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        widget_gradients = new glaxnimate::gui::GradientListWidget(dockWidgetContents);
        widget_gradients->setObjectName("widget_gradients");

        verticalLayout->addWidget(widget_gradients);

        dock_gradients->setWidget(dockWidgetContents);

        retranslateUi(dock_gradients);

        QMetaObject::connectSlotsByName(dock_gradients);
    } // setupUi

    void retranslateUi(QDockWidget *dock_gradients)
    {
        dock_gradients->setWindowTitle(tr2i18n("Gradients", nullptr));
    } // retranslateUi

};

namespace Ui {
    class dock_gradients: public Ui_dock_gradients {};
} // namespace Ui

QT_END_NAMESPACE

#endif // GRADIENTS_H

