/********************************************************************************
** Form generated from reading UI file 'stalefiles_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STALEFILES_DIALOG_H
#define UI_STALEFILES_DIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

namespace glaxnimate {
namespace gui {

class Ui_StalefilesDialog
{
public:
    QVBoxLayout *verticalLayout;
    QListWidget *list_widget;
    QHBoxLayout *horizontalLayout;
    QPushButton *button_open;
    QPushButton *button_delete;
    QPushButton *button_delete_all;
    QSpacerItem *horizontalSpacer;
    QPushButton *button_close;

    void setupUi(QDialog *glaxnimate__gui__StalefilesDialog)
    {
        if (glaxnimate__gui__StalefilesDialog->objectName().isEmpty())
            glaxnimate__gui__StalefilesDialog->setObjectName("glaxnimate__gui__StalefilesDialog");
        glaxnimate__gui__StalefilesDialog->resize(493, 437);
        verticalLayout = new QVBoxLayout(glaxnimate__gui__StalefilesDialog);
        verticalLayout->setObjectName("verticalLayout");
        list_widget = new QListWidget(glaxnimate__gui__StalefilesDialog);
        list_widget->setObjectName("list_widget");

        verticalLayout->addWidget(list_widget);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        button_open = new QPushButton(glaxnimate__gui__StalefilesDialog);
        button_open->setObjectName("button_open");
        button_open->setEnabled(false);
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("document-revert");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_open->setIcon(icon);

        horizontalLayout->addWidget(button_open);

        button_delete = new QPushButton(glaxnimate__gui__StalefilesDialog);
        button_delete->setObjectName("button_delete");
        button_delete->setEnabled(false);
        QIcon icon1;
        iconThemeName = QString::fromUtf8("edit-delete");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_delete->setIcon(icon1);

        horizontalLayout->addWidget(button_delete);

        button_delete_all = new QPushButton(glaxnimate__gui__StalefilesDialog);
        button_delete_all->setObjectName("button_delete_all");
        QIcon icon2(QIcon::fromTheme(QString::fromUtf8("edit-clear")));
        button_delete_all->setIcon(icon2);

        horizontalLayout->addWidget(button_delete_all);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        button_close = new QPushButton(glaxnimate__gui__StalefilesDialog);
        button_close->setObjectName("button_close");
        QIcon icon3(QIcon::fromTheme(QString::fromUtf8("dialog-close")));
        button_close->setIcon(icon3);

        horizontalLayout->addWidget(button_close);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(glaxnimate__gui__StalefilesDialog);
        QObject::connect(button_close, &QPushButton::clicked, glaxnimate__gui__StalefilesDialog, qOverload<>(&QDialog::reject));
        QObject::connect(button_open, &QPushButton::clicked, glaxnimate__gui__StalefilesDialog, qOverload<>(&QDialog::accept));
        QObject::connect(button_delete, SIGNAL(clicked()), glaxnimate__gui__StalefilesDialog, SLOT(delete_selected()));
        QObject::connect(button_delete_all, SIGNAL(clicked()), glaxnimate__gui__StalefilesDialog, SLOT(delete_all()));
        QObject::connect(list_widget, SIGNAL(currentRowChanged(int)), glaxnimate__gui__StalefilesDialog, SLOT(current_changed(int)));

        QMetaObject::connectSlotsByName(glaxnimate__gui__StalefilesDialog);
    } // setupUi

    void retranslateUi(QDialog *glaxnimate__gui__StalefilesDialog)
    {
        glaxnimate__gui__StalefilesDialog->setWindowTitle(tr2i18n("Autosave Files", nullptr));
        button_open->setText(tr2i18n("Load", nullptr));
        button_delete->setText(tr2i18n("Delete", nullptr));
        button_delete_all->setText(tr2i18n("Delete All", nullptr));
        button_close->setText(tr2i18n("Close", nullptr));
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class StalefilesDialog: public Ui_StalefilesDialog {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // STALEFILES_DIALOG_H

