/****************************************************************************
** Meta object code from reading C++ file 'timeline_items.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/gui/widgets/timeline/timeline_items.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'timeline_items.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate3gui8timeline8LineItemE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::gui::timeline::LineItem::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate3gui8timeline8LineItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::gui::timeline::LineItem",
        "removed",
        "",
        "quintptr",
        "id",
        "clicked",
        "selected",
        "replace_selection"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'removed'
        QtMocHelpers::SignalData<void(quintptr)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'clicked'
        QtMocHelpers::SignalData<void(quintptr, bool, bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Bool, 6 }, { QMetaType::Bool, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LineItem, qt_meta_tag_ZN10glaxnimate3gui8timeline8LineItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::gui::timeline::LineItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8timeline8LineItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8timeline8LineItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate3gui8timeline8LineItemE_t>.metaTypes,
    nullptr
} };

void glaxnimate::gui::timeline::LineItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LineItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->removed((*reinterpret_cast<std::add_pointer_t<quintptr>>(_a[1])), QPrivateSignal()); break;
        case 1: _t->clicked((*reinterpret_cast<std::add_pointer_t<quintptr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LineItem::*)(quintptr , QPrivateSignal)>(_a, &LineItem::removed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LineItem::*)(quintptr , bool , bool )>(_a, &LineItem::clicked, 1))
            return;
    }
}

const QMetaObject *glaxnimate::gui::timeline::LineItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::gui::timeline::LineItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8timeline8LineItemE_t>.strings))
        return static_cast<void*>(this);
    return QGraphicsObject::qt_metacast(_clname);
}

int glaxnimate::gui::timeline::LineItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::gui::timeline::LineItem::removed(quintptr _t1, QPrivateSignal _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void glaxnimate::gui::timeline::LineItem::clicked(quintptr _t1, bool _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}
namespace {
struct qt_meta_tag_ZN10glaxnimate3gui8timeline14ObjectLineItemE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::gui::timeline::ObjectLineItem::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate3gui8timeline14ObjectLineItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::gui::timeline::ObjectLineItem"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ObjectLineItem, qt_meta_tag_ZN10glaxnimate3gui8timeline14ObjectLineItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::gui::timeline::ObjectLineItem::staticMetaObject = { {
    QMetaObject::SuperData::link<LineItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8timeline14ObjectLineItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8timeline14ObjectLineItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate3gui8timeline14ObjectLineItemE_t>.metaTypes,
    nullptr
} };

void glaxnimate::gui::timeline::ObjectLineItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ObjectLineItem *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *glaxnimate::gui::timeline::ObjectLineItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::gui::timeline::ObjectLineItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8timeline14ObjectLineItemE_t>.strings))
        return static_cast<void*>(this);
    return LineItem::qt_metacast(_clname);
}

int glaxnimate::gui::timeline::ObjectLineItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = LineItem::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN10glaxnimate3gui8timeline17KeyframeSplitItemE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::gui::timeline::KeyframeSplitItem::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate3gui8timeline17KeyframeSplitItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::gui::timeline::KeyframeSplitItem"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KeyframeSplitItem, qt_meta_tag_ZN10glaxnimate3gui8timeline17KeyframeSplitItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::gui::timeline::KeyframeSplitItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8timeline17KeyframeSplitItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8timeline17KeyframeSplitItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate3gui8timeline17KeyframeSplitItemE_t>.metaTypes,
    nullptr
} };

void glaxnimate::gui::timeline::KeyframeSplitItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KeyframeSplitItem *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *glaxnimate::gui::timeline::KeyframeSplitItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::gui::timeline::KeyframeSplitItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8timeline17KeyframeSplitItemE_t>.strings))
        return static_cast<void*>(this);
    return QGraphicsObject::qt_metacast(_clname);
}

int glaxnimate::gui::timeline::KeyframeSplitItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsObject::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN10glaxnimate3gui8timeline14AnimatableItemE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::gui::timeline::AnimatableItem::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate3gui8timeline14AnimatableItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::gui::timeline::AnimatableItem",
        "add_keyframe",
        "",
        "index",
        "remove_keyframe",
        "transition_changed",
        "model::KeyframeTransition::Descriptive",
        "before",
        "after",
        "update_keyframe",
        "model::KeyframeBase*",
        "kf"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'add_keyframe'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'remove_keyframe'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'transition_changed'
        QtMocHelpers::SlotData<void(model::KeyframeTransition::Descriptive, model::KeyframeTransition::Descriptive)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 6, 8 },
        }}),
        // Slot 'update_keyframe'
        QtMocHelpers::SlotData<void(int, model::KeyframeBase *)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 10, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AnimatableItem, qt_meta_tag_ZN10glaxnimate3gui8timeline14AnimatableItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::gui::timeline::AnimatableItem::staticMetaObject = { {
    QMetaObject::SuperData::link<LineItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8timeline14AnimatableItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8timeline14AnimatableItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate3gui8timeline14AnimatableItemE_t>.metaTypes,
    nullptr
} };

void glaxnimate::gui::timeline::AnimatableItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AnimatableItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->add_keyframe((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->remove_keyframe((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->transition_changed((*reinterpret_cast<std::add_pointer_t<model::KeyframeTransition::Descriptive>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<model::KeyframeTransition::Descriptive>>(_a[2]))); break;
        case 3: _t->update_keyframe((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<model::KeyframeBase*>>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObject *glaxnimate::gui::timeline::AnimatableItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::gui::timeline::AnimatableItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8timeline14AnimatableItemE_t>.strings))
        return static_cast<void*>(this);
    return LineItem::qt_metacast(_clname);
}

int glaxnimate::gui::timeline::AnimatableItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = LineItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN10glaxnimate3gui8timeline16PropertyLineItemE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::gui::timeline::PropertyLineItem::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate3gui8timeline16PropertyLineItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::gui::timeline::PropertyLineItem"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PropertyLineItem, qt_meta_tag_ZN10glaxnimate3gui8timeline16PropertyLineItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::gui::timeline::PropertyLineItem::staticMetaObject = { {
    QMetaObject::SuperData::link<LineItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8timeline16PropertyLineItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8timeline16PropertyLineItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate3gui8timeline16PropertyLineItemE_t>.metaTypes,
    nullptr
} };

void glaxnimate::gui::timeline::PropertyLineItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PropertyLineItem *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *glaxnimate::gui::timeline::PropertyLineItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::gui::timeline::PropertyLineItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8timeline16PropertyLineItemE_t>.strings))
        return static_cast<void*>(this);
    return LineItem::qt_metacast(_clname);
}

int glaxnimate::gui::timeline::PropertyLineItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = LineItem::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN10glaxnimate3gui8timeline18ObjectListLineItemE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::gui::timeline::ObjectListLineItem::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate3gui8timeline18ObjectListLineItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::gui::timeline::ObjectListLineItem"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ObjectListLineItem, qt_meta_tag_ZN10glaxnimate3gui8timeline18ObjectListLineItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::gui::timeline::ObjectListLineItem::staticMetaObject = { {
    QMetaObject::SuperData::link<LineItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8timeline18ObjectListLineItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8timeline18ObjectListLineItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate3gui8timeline18ObjectListLineItemE_t>.metaTypes,
    nullptr
} };

void glaxnimate::gui::timeline::ObjectListLineItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ObjectListLineItem *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *glaxnimate::gui::timeline::ObjectListLineItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::gui::timeline::ObjectListLineItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8timeline18ObjectListLineItemE_t>.strings))
        return static_cast<void*>(this);
    return LineItem::qt_metacast(_clname);
}

int glaxnimate::gui::timeline::ObjectListLineItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = LineItem::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
