/****************************************************************************
** Meta object code from reading C++ file 'color_list_widget.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/external/Qt-Color-Widgets/include/QtColorWidgets/color_list_widget.hpp"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'color_list_widget.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13color_widgets15ColorListWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto color_widgets::ColorListWidget::qt_create_metaobjectdata<qt_meta_tag_ZN13color_widgets15ColorListWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "color_widgets::ColorListWidget",
        "colorsChanged",
        "",
        "QList<QColor>",
        "wheelShapeChanged",
        "ColorWheel::ShapeEnum",
        "shape",
        "colorSpaceChanged",
        "ColorWheel::ColorSpaceEnum",
        "space",
        "wheelRotatingChanged",
        "rotating",
        "setWheelShape",
        "setColorSpace",
        "setWheelRotating",
        "emit_changed",
        "handle_removed",
        "color_changed",
        "row",
        "colors",
        "wheelShape",
        "colorSpace",
        "wheelRotating"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'colorsChanged'
        QtMocHelpers::SignalData<void(const QList<QColor> &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'wheelShapeChanged'
        QtMocHelpers::SignalData<void(ColorWheel::ShapeEnum)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'colorSpaceChanged'
        QtMocHelpers::SignalData<void(ColorWheel::ColorSpaceEnum)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'wheelRotatingChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'setWheelShape'
        QtMocHelpers::SlotData<void(ColorWheel::ShapeEnum)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'setColorSpace'
        QtMocHelpers::SlotData<void(ColorWheel::ColorSpaceEnum)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'setWheelRotating'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'emit_changed'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handle_removed'
        QtMocHelpers::SlotData<void(int)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'color_changed'
        QtMocHelpers::SlotData<void(int)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'colors'
        QtMocHelpers::PropertyData<QList<QColor>>(19, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'wheelShape'
        QtMocHelpers::PropertyData<ColorWheel::ShapeEnum>(20, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'colorSpace'
        QtMocHelpers::PropertyData<ColorWheel::ColorSpaceEnum>(21, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'wheelRotating'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ColorListWidget, qt_meta_tag_ZN13color_widgets15ColorListWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN13color_widgets15ColorListWidgetE[] = {
    QMetaObject::SuperData::link<color_widgets::ColorWheel::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject color_widgets::ColorListWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractWidgetList::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets15ColorListWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets15ColorListWidgetE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN13color_widgets15ColorListWidgetE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13color_widgets15ColorListWidgetE_t>.metaTypes,
    nullptr
} };

void color_widgets::ColorListWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ColorListWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->colorsChanged((*reinterpret_cast<std::add_pointer_t<QList<QColor>>>(_a[1]))); break;
        case 1: _t->wheelShapeChanged((*reinterpret_cast<std::add_pointer_t<ColorWheel::ShapeEnum>>(_a[1]))); break;
        case 2: _t->colorSpaceChanged((*reinterpret_cast<std::add_pointer_t<ColorWheel::ColorSpaceEnum>>(_a[1]))); break;
        case 3: _t->wheelRotatingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->setWheelShape((*reinterpret_cast<std::add_pointer_t<ColorWheel::ShapeEnum>>(_a[1]))); break;
        case 5: _t->setColorSpace((*reinterpret_cast<std::add_pointer_t<ColorWheel::ColorSpaceEnum>>(_a[1]))); break;
        case 6: _t->setWheelRotating((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->emit_changed(); break;
        case 8: _t->handle_removed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->color_changed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ColorListWidget::*)(const QList<QColor> & )>(_a, &ColorListWidget::colorsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorListWidget::*)(ColorWheel::ShapeEnum )>(_a, &ColorListWidget::wheelShapeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorListWidget::*)(ColorWheel::ColorSpaceEnum )>(_a, &ColorListWidget::colorSpaceChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorListWidget::*)(bool )>(_a, &ColorListWidget::wheelRotatingChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QList<QColor>*>(_v) = _t->colors(); break;
        case 1: *reinterpret_cast<ColorWheel::ShapeEnum*>(_v) = _t->wheelShape(); break;
        case 2: *reinterpret_cast<ColorWheel::ColorSpaceEnum*>(_v) = _t->colorSpace(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->wheelRotating(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setColors(*reinterpret_cast<QList<QColor>*>(_v)); break;
        case 1: _t->setWheelShape(*reinterpret_cast<ColorWheel::ShapeEnum*>(_v)); break;
        case 2: _t->setColorSpace(*reinterpret_cast<ColorWheel::ColorSpaceEnum*>(_v)); break;
        case 3: _t->setWheelRotating(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *color_widgets::ColorListWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *color_widgets::ColorListWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets15ColorListWidgetE_t>.strings))
        return static_cast<void*>(this);
    return AbstractWidgetList::qt_metacast(_clname);
}

int color_widgets::ColorListWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractWidgetList::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void color_widgets::ColorListWidget::colorsChanged(const QList<QColor> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void color_widgets::ColorListWidget::wheelShapeChanged(ColorWheel::ShapeEnum _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void color_widgets::ColorListWidget::colorSpaceChanged(ColorWheel::ColorSpaceEnum _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void color_widgets::ColorListWidget::wheelRotatingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
