/********************************************************************************
** Form generated from reading UI file 'font_preview_widget.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FONT_PREVIEW_WIDGET_H
#define UI_FONT_PREVIEW_WIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGraphicsView>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace glaxnimate {
namespace gui {
namespace font {

class Ui_FontPreviewWidget
{
public:
    QVBoxLayout *verticalLayout;
    QGraphicsView *view;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QDoubleSpinBox *spin_zoom;

    void setupUi(QWidget *font__FontPreviewWidget)
    {
        if (font__FontPreviewWidget->objectName().isEmpty())
            font__FontPreviewWidget->setObjectName("font__FontPreviewWidget");
        font__FontPreviewWidget->resize(730, 299);
        verticalLayout = new QVBoxLayout(font__FontPreviewWidget);
        verticalLayout->setObjectName("verticalLayout");
        view = new QGraphicsView(font__FontPreviewWidget);
        view->setObjectName("view");

        verticalLayout->addWidget(view);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        spin_zoom = new QDoubleSpinBox(font__FontPreviewWidget);
        spin_zoom->setObjectName("spin_zoom");
        spin_zoom->setMinimum(1.000000000000000);
        spin_zoom->setMaximum(9999.000000000000000);
        spin_zoom->setSingleStep(10.000000000000000);
        spin_zoom->setStepType(QAbstractSpinBox::AdaptiveDecimalStepType);
        spin_zoom->setValue(100.000000000000000);

        horizontalLayout->addWidget(spin_zoom);


        verticalLayout->addLayout(horizontalLayout);

        QWidget::setTabOrder(spin_zoom, view);

        retranslateUi(font__FontPreviewWidget);
        QObject::connect(spin_zoom, SIGNAL(valueChanged(double)), font__FontPreviewWidget, SLOT(zoom_changed(double)));

        QMetaObject::connectSlotsByName(font__FontPreviewWidget);
    } // setupUi

    void retranslateUi(QWidget *font__FontPreviewWidget)
    {
#if QT_CONFIG(tooltip)
        spin_zoom->setToolTip(tr2i18n("Preview Scale", nullptr));
#endif // QT_CONFIG(tooltip)
        spin_zoom->setSuffix(tr2i18n("%", nullptr));
        (void)font__FontPreviewWidget;
    } // retranslateUi

};

} // namespace font
} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace font {
namespace Ui {
    class FontPreviewWidget: public Ui_FontPreviewWidget {};
} // namespace Ui
} // namespace font
} // namespace gui
} // namespace glaxnimate

#endif // FONT_PREVIEW_WIDGET_H

