/********************************************************************************
** Form generated from reading UI file 'fill_tool_widget.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FILL_TOOL_WIDGET_H
#define UI_FILL_TOOL_WIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace glaxnimate {
namespace gui {

class Ui_FillToolWidget
{
public:
    QVBoxLayout *verticalLayout;
    QRadioButton *check_fill;
    QRadioButton *check_stroke;
    QRadioButton *check_both;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *FillToolWidget)
    {
        if (FillToolWidget->objectName().isEmpty())
            FillToolWidget->setObjectName("FillToolWidget");
        FillToolWidget->resize(146, 90);
        verticalLayout = new QVBoxLayout(FillToolWidget);
        verticalLayout->setObjectName("verticalLayout");
        check_fill = new QRadioButton(FillToolWidget);
        check_fill->setObjectName("check_fill");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("format-fill-color");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        check_fill->setIcon(icon);
        check_fill->setChecked(true);

        verticalLayout->addWidget(check_fill);

        check_stroke = new QRadioButton(FillToolWidget);
        check_stroke->setObjectName("check_stroke");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("format-stroke-color");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        check_stroke->setIcon(icon1);

        verticalLayout->addWidget(check_stroke);

        check_both = new QRadioButton(FillToolWidget);
        check_both->setObjectName("check_both");
        QIcon icon2;
        iconThemeName = QString::fromUtf8("edit-paste-style");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        check_both->setIcon(icon2);

        verticalLayout->addWidget(check_both);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        retranslateUi(FillToolWidget);

        QMetaObject::connectSlotsByName(FillToolWidget);
    } // setupUi

    void retranslateUi(QWidget *FillToolWidget)
    {
        check_fill->setText(tr2i18n("Fill", nullptr));
        check_stroke->setText(tr2i18n("Stroke", nullptr));
        check_both->setText(tr2i18n("Fill and Stroke", nullptr));
        (void)FillToolWidget;
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class FillToolWidget: public Ui_FillToolWidget {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // FILL_TOOL_WIDGET_H

