/********************************************************************************
** Form generated from reading UI file 'font_style_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FONT_STYLE_DIALOG_H
#define UI_FONT_STYLE_DIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "widgets/font/external_font_widget.hpp"
#include "widgets/font/font_preview_widget.hpp"
#include "widgets/font/font_style_widget.hpp"
#include "widgets/font/google_fonts_widget.hpp"

namespace glaxnimate {
namespace gui {
namespace font {

class Ui_FontStyleDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tab_widget;
    QWidget *tab_system;
    QVBoxLayout *verticalLayout_2;
    glaxnimate::gui::font::FontStyleWidget *widget_system;
    QWidget *tab_google;
    QVBoxLayout *verticalLayout_3;
    glaxnimate::gui::font::GoogleFontsWidget *google_fonts_widget;
    QWidget *tab_custom;
    QVBoxLayout *verticalLayout_4;
    glaxnimate::gui::font::ExternalFontWidget *custom_font_widget;
    glaxnimate::gui::font::FontPreviewWidget *preview;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *glaxnimate__gui__font__FontStyleDialog)
    {
        if (glaxnimate__gui__font__FontStyleDialog->objectName().isEmpty())
            glaxnimate__gui__font__FontStyleDialog->setObjectName("glaxnimate__gui__font__FontStyleDialog");
        glaxnimate__gui__font__FontStyleDialog->resize(834, 659);
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("dialog-text-and-font");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        glaxnimate__gui__font__FontStyleDialog->setWindowIcon(icon);
        verticalLayout = new QVBoxLayout(glaxnimate__gui__font__FontStyleDialog);
        verticalLayout->setObjectName("verticalLayout");
        tab_widget = new QTabWidget(glaxnimate__gui__font__FontStyleDialog);
        tab_widget->setObjectName("tab_widget");
        tab_system = new QWidget();
        tab_system->setObjectName("tab_system");
        verticalLayout_2 = new QVBoxLayout(tab_system);
        verticalLayout_2->setObjectName("verticalLayout_2");
        widget_system = new glaxnimate::gui::font::FontStyleWidget(tab_system);
        widget_system->setObjectName("widget_system");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(widget_system->sizePolicy().hasHeightForWidth());
        widget_system->setSizePolicy(sizePolicy);

        verticalLayout_2->addWidget(widget_system);

        tab_widget->addTab(tab_system, QString());
        tab_google = new QWidget();
        tab_google->setObjectName("tab_google");
        verticalLayout_3 = new QVBoxLayout(tab_google);
        verticalLayout_3->setSpacing(0);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        google_fonts_widget = new glaxnimate::gui::font::GoogleFontsWidget(tab_google);
        google_fonts_widget->setObjectName("google_fonts_widget");

        verticalLayout_3->addWidget(google_fonts_widget);

        tab_widget->addTab(tab_google, QString());
        tab_custom = new QWidget();
        tab_custom->setObjectName("tab_custom");
        verticalLayout_4 = new QVBoxLayout(tab_custom);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        custom_font_widget = new glaxnimate::gui::font::ExternalFontWidget(tab_custom);
        custom_font_widget->setObjectName("custom_font_widget");

        verticalLayout_4->addWidget(custom_font_widget);

        tab_widget->addTab(tab_custom, QString());

        verticalLayout->addWidget(tab_widget);

        preview = new glaxnimate::gui::font::FontPreviewWidget(glaxnimate__gui__font__FontStyleDialog);
        preview->setObjectName("preview");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(preview->sizePolicy().hasHeightForWidth());
        preview->setSizePolicy(sizePolicy1);

        verticalLayout->addWidget(preview);

        buttonBox = new QDialogButtonBox(glaxnimate__gui__font__FontStyleDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        verticalLayout->setStretch(0, 10);
        verticalLayout->setStretch(1, 4);

        retranslateUi(glaxnimate__gui__font__FontStyleDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, glaxnimate__gui__font__FontStyleDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, glaxnimate__gui__font__FontStyleDialog, qOverload<>(&QDialog::reject));
        QObject::connect(widget_system, SIGNAL(font_changed(QFont)), preview, SLOT(set_font(QFont)));
        QObject::connect(google_fonts_widget, SIGNAL(font_changed(QFont)), preview, SLOT(set_font(QFont)));
        QObject::connect(custom_font_widget, SIGNAL(font_changed(QFont)), preview, SLOT(set_font(QFont)));

        tab_widget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(glaxnimate__gui__font__FontStyleDialog);
    } // setupUi

    void retranslateUi(QDialog *glaxnimate__gui__font__FontStyleDialog)
    {
        glaxnimate__gui__font__FontStyleDialog->setWindowTitle(tr2i18n("Font Style", nullptr));
        tab_widget->setTabText(tab_widget->indexOf(tab_system), tr2i18n("System", nullptr));
        tab_widget->setTabText(tab_widget->indexOf(tab_google), tr2i18n("Google Fonts", nullptr));
        tab_widget->setTabText(tab_widget->indexOf(tab_custom), tr2i18n("Custom", nullptr));
    } // retranslateUi

};

} // namespace font
} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace font {
namespace Ui {
    class FontStyleDialog: public Ui_FontStyleDialog {};
} // namespace Ui
} // namespace font
} // namespace gui
} // namespace glaxnimate

#endif // FONT_STYLE_DIALOG_H

