/********************************************************************************
** Form generated from reading UI file 'stroke.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STROKE_H
#define UI_STROKE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDockWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "widgets/shape_style/stroke_style_widget.hpp"

QT_BEGIN_NAMESPACE

class Ui_dock_stroke
{
public:
    QWidget *dockWidgetContents;
    QVBoxLayout *verticalLayout;
    glaxnimate::gui::StrokeStyleWidget *stroke_style_widget;

    void setupUi(QDockWidget *dock_stroke)
    {
        if (dock_stroke->objectName().isEmpty())
            dock_stroke->setObjectName("dock_stroke");
        dock_stroke->resize(400, 300);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("object-stroke-style")));
        dock_stroke->setWindowIcon(icon);
        dockWidgetContents = new QWidget();
        dockWidgetContents->setObjectName("dockWidgetContents");
        verticalLayout = new QVBoxLayout(dockWidgetContents);
        verticalLayout->setObjectName("verticalLayout");
        stroke_style_widget = new glaxnimate::gui::StrokeStyleWidget(dockWidgetContents);
        stroke_style_widget->setObjectName("stroke_style_widget");

        verticalLayout->addWidget(stroke_style_widget);

        dock_stroke->setWidget(dockWidgetContents);

        retranslateUi(dock_stroke);

        QMetaObject::connectSlotsByName(dock_stroke);
    } // setupUi

    void retranslateUi(QDockWidget *dock_stroke)
    {
        dock_stroke->setWindowTitle(tr2i18n("Stroke", nullptr));
    } // retranslateUi

};

namespace Ui {
    class dock_stroke: public Ui_dock_stroke {};
} // namespace Ui

QT_END_NAMESPACE

#endif // STROKE_H

