/********************************************************************************
** Form generated from reading UI file 'export_image_sequence_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXPORT_IMAGE_SEQUENCE_DIALOG_H
#define UI_EXPORT_IMAGE_SEQUENCE_DIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <klocalizedstring.h>

namespace glaxnimate {
namespace gui {

class Ui_ExportImageSequenceDialog
{
public:
    QGridLayout *gridLayout;
    QLabel *label_4;
    QComboBox *input_format;
    QLabel *label_2;
    QSpinBox *input_frame_to;
    QLineEdit *input_name;
    QSpinBox *input_frame_step;
    QLabel *label_5;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;
    QLineEdit *input_path;
    QSpinBox *input_frame_from;
    QLabel *label_3;
    QLabel *label;
    QPushButton *button_select;
    QProgressBar *progress_bar;

    void setupUi(QDialog *glaxnimate__gui__ExportImageSequenceDialog)
    {
        if (glaxnimate__gui__ExportImageSequenceDialog->objectName().isEmpty())
            glaxnimate__gui__ExportImageSequenceDialog->setObjectName("glaxnimate__gui__ExportImageSequenceDialog");
        glaxnimate__gui__ExportImageSequenceDialog->resize(358, 276);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("folder-images	")));
        glaxnimate__gui__ExportImageSequenceDialog->setWindowIcon(icon);
        gridLayout = new QGridLayout(glaxnimate__gui__ExportImageSequenceDialog);
        gridLayout->setObjectName("gridLayout");
        label_4 = new QLabel(glaxnimate__gui__ExportImageSequenceDialog);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 3, 0, 1, 1);

        input_format = new QComboBox(glaxnimate__gui__ExportImageSequenceDialog);
        input_format->setObjectName("input_format");

        gridLayout->addWidget(input_format, 1, 2, 1, 1);

        label_2 = new QLabel(glaxnimate__gui__ExportImageSequenceDialog);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        input_frame_to = new QSpinBox(glaxnimate__gui__ExportImageSequenceDialog);
        input_frame_to->setObjectName("input_frame_to");

        gridLayout->addWidget(input_frame_to, 3, 1, 1, 2);

        input_name = new QLineEdit(glaxnimate__gui__ExportImageSequenceDialog);
        input_name->setObjectName("input_name");

        gridLayout->addWidget(input_name, 1, 1, 1, 1);

        input_frame_step = new QSpinBox(glaxnimate__gui__ExportImageSequenceDialog);
        input_frame_step->setObjectName("input_frame_step");
        input_frame_step->setMinimum(1);

        gridLayout->addWidget(input_frame_step, 4, 1, 1, 2);

        label_5 = new QLabel(glaxnimate__gui__ExportImageSequenceDialog);
        label_5->setObjectName("label_5");

        gridLayout->addWidget(label_5, 4, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 5, 2, 1, 1);

        buttonBox = new QDialogButtonBox(glaxnimate__gui__ExportImageSequenceDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout->addWidget(buttonBox, 7, 0, 1, 3);

        input_path = new QLineEdit(glaxnimate__gui__ExportImageSequenceDialog);
        input_path->setObjectName("input_path");

        gridLayout->addWidget(input_path, 0, 1, 1, 1);

        input_frame_from = new QSpinBox(glaxnimate__gui__ExportImageSequenceDialog);
        input_frame_from->setObjectName("input_frame_from");

        gridLayout->addWidget(input_frame_from, 2, 1, 1, 2);

        label_3 = new QLabel(glaxnimate__gui__ExportImageSequenceDialog);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 2, 0, 1, 1);

        label = new QLabel(glaxnimate__gui__ExportImageSequenceDialog);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        button_select = new QPushButton(glaxnimate__gui__ExportImageSequenceDialog);
        button_select->setObjectName("button_select");
        QIcon icon1;
        QString iconThemeName = QString::fromUtf8("folder-open");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_select->setIcon(icon1);

        gridLayout->addWidget(button_select, 0, 2, 1, 1);

        progress_bar = new QProgressBar(glaxnimate__gui__ExportImageSequenceDialog);
        progress_bar->setObjectName("progress_bar");
        progress_bar->setValue(24);

        gridLayout->addWidget(progress_bar, 6, 0, 1, 3);

#if QT_CONFIG(shortcut)
        label_4->setBuddy(input_frame_to);
        label_2->setBuddy(input_name);
        label_5->setBuddy(input_frame_step);
        label_3->setBuddy(input_frame_from);
        label->setBuddy(input_path);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(input_path, button_select);
        QWidget::setTabOrder(button_select, input_name);
        QWidget::setTabOrder(input_name, input_format);
        QWidget::setTabOrder(input_format, input_frame_from);
        QWidget::setTabOrder(input_frame_from, input_frame_to);
        QWidget::setTabOrder(input_frame_to, input_frame_step);

        retranslateUi(glaxnimate__gui__ExportImageSequenceDialog);
        QObject::connect(buttonBox, SIGNAL(accepted()), glaxnimate__gui__ExportImageSequenceDialog, SLOT(render()));
        QObject::connect(button_select, SIGNAL(clicked()), glaxnimate__gui__ExportImageSequenceDialog, SLOT(pick_path()));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, glaxnimate__gui__ExportImageSequenceDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(glaxnimate__gui__ExportImageSequenceDialog);
    } // setupUi

    void retranslateUi(QDialog *glaxnimate__gui__ExportImageSequenceDialog)
    {
        glaxnimate__gui__ExportImageSequenceDialog->setWindowTitle(tr2i18n("Export as Image Sequence", nullptr));
        label_4->setText(tr2i18n("To Frame", nullptr));
        label_2->setText(tr2i18n("File Name", nullptr));
#if QT_CONFIG(tooltip)
        input_name->setToolTip(tr2i18n("Use {frame} to represent the frame number", nullptr));
#endif // QT_CONFIG(tooltip)
        input_name->setText(tr2i18n("Frame {frame}", nullptr));
        label_5->setText(tr2i18n("Frame Step", nullptr));
        label_3->setText(tr2i18n("From Frame", nullptr));
        label->setText(tr2i18n("Path", nullptr));
        button_select->setText(tr2i18n("Select...", nullptr));
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class ExportImageSequenceDialog: public Ui_ExportImageSequenceDialog {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // EXPORT_IMAGE_SEQUENCE_DIALOG_H

