/********************************************************************************
** Form generated from reading UI file 'tooloptions.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLOPTIONS_H
#define UI_TOOLOPTIONS_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDockWidget>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_dock_tool_options
{
public:
    QWidget *dockWidgetContents;
    QVBoxLayout *verticalLayout;
    QScrollArea *dock_tool_options_scroll_area;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_9;
    QStackedWidget *tool_settings_widget;
    QWidget *page;

    void setupUi(QDockWidget *dock_tool_options)
    {
        if (dock_tool_options->objectName().isEmpty())
            dock_tool_options->setObjectName("dock_tool_options");
        dock_tool_options->resize(400, 300);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("settings-configure")));
        dock_tool_options->setWindowIcon(icon);
        dockWidgetContents = new QWidget();
        dockWidgetContents->setObjectName("dockWidgetContents");
        verticalLayout = new QVBoxLayout(dockWidgetContents);
        verticalLayout->setObjectName("verticalLayout");
        dock_tool_options_scroll_area = new QScrollArea(dockWidgetContents);
        dock_tool_options_scroll_area->setObjectName("dock_tool_options_scroll_area");
        dock_tool_options_scroll_area->setFrameShape(QFrame::Shape::NoFrame);
        dock_tool_options_scroll_area->setHorizontalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
        dock_tool_options_scroll_area->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 382, 262));
        verticalLayout_9 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_9->setSpacing(0);
        verticalLayout_9->setObjectName("verticalLayout_9");
        verticalLayout_9->setContentsMargins(0, 0, 0, 0);
        tool_settings_widget = new QStackedWidget(scrollAreaWidgetContents);
        tool_settings_widget->setObjectName("tool_settings_widget");
        page = new QWidget();
        page->setObjectName("page");
        tool_settings_widget->addWidget(page);

        verticalLayout_9->addWidget(tool_settings_widget);

        dock_tool_options_scroll_area->setWidget(scrollAreaWidgetContents);

        verticalLayout->addWidget(dock_tool_options_scroll_area);

        dock_tool_options->setWidget(dockWidgetContents);

        retranslateUi(dock_tool_options);

        tool_settings_widget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(dock_tool_options);
    } // setupUi

    void retranslateUi(QDockWidget *dock_tool_options)
    {
        dock_tool_options->setWindowTitle(tr2i18n("&Tool Options", nullptr));
    } // retranslateUi

};

namespace Ui {
    class dock_tool_options: public Ui_dock_tool_options {};
} // namespace Ui

QT_END_NAMESPACE

#endif // TOOLOPTIONS_H

