/****************************************************************************
** Meta object code from reading C++ file 'gradient_editor.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/external/Qt-Color-Widgets/include/QtColorWidgets/gradient_editor.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'gradient_editor.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13color_widgets14GradientEditorE_t {};
} // unnamed namespace

template <> constexpr inline auto color_widgets::GradientEditor::qt_create_metaobjectdata<qt_meta_tag_ZN13color_widgets14GradientEditorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "color_widgets::GradientEditor",
        "backgroundChanged",
        "",
        "QBrush",
        "stopsChanged",
        "QGradientStops",
        "selectedStopChanged",
        "setOrientation",
        "Qt::Orientation",
        "setSelectedStop",
        "stop",
        "setSelectedColor",
        "QColor",
        "color",
        "addStop",
        "removeStop",
        "dialogUpdate",
        "c",
        "background",
        "stops",
        "gradient",
        "QLinearGradient",
        "orientation",
        "selectedStop",
        "selectedColor"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'backgroundChanged'
        QtMocHelpers::SignalData<void(const QBrush &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'stopsChanged'
        QtMocHelpers::SignalData<void(const QGradientStops &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'selectedStopChanged'
        QtMocHelpers::SignalData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'setOrientation'
        QtMocHelpers::SlotData<void(Qt::Orientation)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Slot 'setSelectedStop'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'setSelectedColor'
        QtMocHelpers::SlotData<void(const QColor &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'addStop'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'removeStop'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'dialogUpdate'
        QtMocHelpers::SlotData<void(const QColor &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'background'
        QtMocHelpers::PropertyData<QBrush>(18, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'stops'
        QtMocHelpers::PropertyData<QGradientStops>(19, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'gradient'
        QtMocHelpers::PropertyData<QLinearGradient>(20, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'orientation'
        QtMocHelpers::PropertyData<Qt::Orientation>(22, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'selectedStop'
        QtMocHelpers::PropertyData<int>(23, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'selectedColor'
        QtMocHelpers::PropertyData<QColor>(24, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GradientEditor, qt_meta_tag_ZN13color_widgets14GradientEditorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject color_widgets::GradientEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets14GradientEditorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets14GradientEditorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13color_widgets14GradientEditorE_t>.metaTypes,
    nullptr
} };

void color_widgets::GradientEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GradientEditor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->backgroundChanged((*reinterpret_cast<std::add_pointer_t<QBrush>>(_a[1]))); break;
        case 1: _t->stopsChanged((*reinterpret_cast<std::add_pointer_t<QGradientStops>>(_a[1]))); break;
        case 2: _t->selectedStopChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->setOrientation((*reinterpret_cast<std::add_pointer_t<Qt::Orientation>>(_a[1]))); break;
        case 4: _t->setSelectedStop((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->setSelectedColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 6: _t->addStop(); break;
        case 7: _t->removeStop(); break;
        case 8: _t->dialogUpdate((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (GradientEditor::*)(const QBrush & )>(_a, &GradientEditor::backgroundChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (GradientEditor::*)(const QGradientStops & )>(_a, &GradientEditor::stopsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (GradientEditor::*)(int )>(_a, &GradientEditor::selectedStopChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QBrush*>(_v) = _t->background(); break;
        case 1: *reinterpret_cast<QGradientStops*>(_v) = _t->stops(); break;
        case 2: *reinterpret_cast<QLinearGradient*>(_v) = _t->gradient(); break;
        case 3: *reinterpret_cast<Qt::Orientation*>(_v) = _t->orientation(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->selectedStop(); break;
        case 5: *reinterpret_cast<QColor*>(_v) = _t->selectedColor(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setBackground(*reinterpret_cast<QBrush*>(_v)); break;
        case 1: _t->setStops(*reinterpret_cast<QGradientStops*>(_v)); break;
        case 2: _t->setGradient(*reinterpret_cast<QLinearGradient*>(_v)); break;
        case 3: _t->setOrientation(*reinterpret_cast<Qt::Orientation*>(_v)); break;
        case 4: _t->setSelectedStop(*reinterpret_cast<int*>(_v)); break;
        case 5: _t->setSelectedColor(*reinterpret_cast<QColor*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *color_widgets::GradientEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *color_widgets::GradientEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets14GradientEditorE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int color_widgets::GradientEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void color_widgets::GradientEditor::backgroundChanged(const QBrush & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void color_widgets::GradientEditor::stopsChanged(const QGradientStops & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void color_widgets::GradientEditor::selectedStopChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
