/****************************************************************************
** Meta object code from reading C++ file 'color_selector.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/external/Qt-Color-Widgets/include/QtColorWidgets/color_selector.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'color_selector.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13color_widgets13ColorSelectorE_t {};
} // unnamed namespace

template <> constexpr inline auto color_widgets::ColorSelector::qt_create_metaobjectdata<qt_meta_tag_ZN13color_widgets13ColorSelectorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "color_widgets::ColorSelector",
        "wheelShapeChanged",
        "",
        "ColorWheel::ShapeEnum",
        "shape",
        "colorSpaceChanged",
        "ColorWheel::ColorSpaceEnum",
        "space",
        "wheelRotatingChanged",
        "rotating",
        "updateModeChanged",
        "UpdateMode",
        "dialogModalityChanged",
        "Qt::WindowModality",
        "colorSelected",
        "QColor",
        "c",
        "showDialog",
        "setWheelShape",
        "setColorSpace",
        "setWheelRotating",
        "accept_dialog",
        "reject_dialog",
        "update_old_color",
        "updateMode",
        "dialogModality",
        "wheelShape",
        "colorSpace",
        "wheelRotating",
        "Confirm",
        "Continuous"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'wheelShapeChanged'
        QtMocHelpers::SignalData<void(ColorWheel::ShapeEnum)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'colorSpaceChanged'
        QtMocHelpers::SignalData<void(ColorWheel::ColorSpaceEnum)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'wheelRotatingChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'updateModeChanged'
        QtMocHelpers::SignalData<void(enum UpdateMode)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 2 },
        }}),
        // Signal 'dialogModalityChanged'
        QtMocHelpers::SignalData<void(Qt::WindowModality)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 2 },
        }}),
        // Signal 'colorSelected'
        QtMocHelpers::SignalData<void(const QColor &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'showDialog'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setWheelShape'
        QtMocHelpers::SlotData<void(ColorWheel::ShapeEnum)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setColorSpace'
        QtMocHelpers::SlotData<void(ColorWheel::ColorSpaceEnum)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'setWheelRotating'
        QtMocHelpers::SlotData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'accept_dialog'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'reject_dialog'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'update_old_color'
        QtMocHelpers::SlotData<void(const QColor &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'updateMode'
        QtMocHelpers::PropertyData<enum UpdateMode>(24, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'dialogModality'
        QtMocHelpers::PropertyData<Qt::WindowModality>(25, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'wheelShape'
        QtMocHelpers::PropertyData<ColorWheel::ShapeEnum>(26, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'colorSpace'
        QtMocHelpers::PropertyData<ColorWheel::ColorSpaceEnum>(27, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'wheelRotating'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'UpdateMode'
        QtMocHelpers::EnumData<enum UpdateMode>(11, 11, QMC::EnumFlags{}).add({
            {   29, UpdateMode::Confirm },
            {   30, UpdateMode::Continuous },
        }),
    };
    return QtMocHelpers::metaObjectData<ColorSelector, qt_meta_tag_ZN13color_widgets13ColorSelectorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN13color_widgets13ColorSelectorE[] = {
    QMetaObject::SuperData::link<color_widgets::ColorWheel::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject color_widgets::ColorSelector::staticMetaObject = { {
    QMetaObject::SuperData::link<ColorPreview::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets13ColorSelectorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets13ColorSelectorE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN13color_widgets13ColorSelectorE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13color_widgets13ColorSelectorE_t>.metaTypes,
    nullptr
} };

void color_widgets::ColorSelector::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ColorSelector *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->wheelShapeChanged((*reinterpret_cast<std::add_pointer_t<ColorWheel::ShapeEnum>>(_a[1]))); break;
        case 1: _t->colorSpaceChanged((*reinterpret_cast<std::add_pointer_t<ColorWheel::ColorSpaceEnum>>(_a[1]))); break;
        case 2: _t->wheelRotatingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->updateModeChanged((*reinterpret_cast<std::add_pointer_t<enum UpdateMode>>(_a[1]))); break;
        case 4: _t->dialogModalityChanged((*reinterpret_cast<std::add_pointer_t<Qt::WindowModality>>(_a[1]))); break;
        case 5: _t->colorSelected((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 6: _t->showDialog(); break;
        case 7: _t->setWheelShape((*reinterpret_cast<std::add_pointer_t<ColorWheel::ShapeEnum>>(_a[1]))); break;
        case 8: _t->setColorSpace((*reinterpret_cast<std::add_pointer_t<ColorWheel::ColorSpaceEnum>>(_a[1]))); break;
        case 9: _t->setWheelRotating((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->accept_dialog(); break;
        case 11: _t->reject_dialog(); break;
        case 12: _t->update_old_color((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ColorSelector::*)(ColorWheel::ShapeEnum )>(_a, &ColorSelector::wheelShapeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorSelector::*)(ColorWheel::ColorSpaceEnum )>(_a, &ColorSelector::colorSpaceChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorSelector::*)(bool )>(_a, &ColorSelector::wheelRotatingChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorSelector::*)(UpdateMode )>(_a, &ColorSelector::updateModeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorSelector::*)(Qt::WindowModality )>(_a, &ColorSelector::dialogModalityChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorSelector::*)(const QColor & )>(_a, &ColorSelector::colorSelected, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum UpdateMode*>(_v) = _t->updateMode(); break;
        case 1: *reinterpret_cast<Qt::WindowModality*>(_v) = _t->dialogModality(); break;
        case 2: *reinterpret_cast<ColorWheel::ShapeEnum*>(_v) = _t->wheelShape(); break;
        case 3: *reinterpret_cast<ColorWheel::ColorSpaceEnum*>(_v) = _t->colorSpace(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->wheelRotating(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setUpdateMode(*reinterpret_cast<enum UpdateMode*>(_v)); break;
        case 1: _t->setDialogModality(*reinterpret_cast<Qt::WindowModality*>(_v)); break;
        case 2: _t->setWheelShape(*reinterpret_cast<ColorWheel::ShapeEnum*>(_v)); break;
        case 3: _t->setColorSpace(*reinterpret_cast<ColorWheel::ColorSpaceEnum*>(_v)); break;
        case 4: _t->setWheelRotating(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *color_widgets::ColorSelector::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *color_widgets::ColorSelector::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets13ColorSelectorE_t>.strings))
        return static_cast<void*>(this);
    return ColorPreview::qt_metacast(_clname);
}

int color_widgets::ColorSelector::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ColorPreview::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void color_widgets::ColorSelector::wheelShapeChanged(ColorWheel::ShapeEnum _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void color_widgets::ColorSelector::colorSpaceChanged(ColorWheel::ColorSpaceEnum _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void color_widgets::ColorSelector::wheelRotatingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void color_widgets::ColorSelector::updateModeChanged(UpdateMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void color_widgets::ColorSelector::dialogModalityChanged(Qt::WindowModality _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void color_widgets::ColorSelector::colorSelected(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
