/****************************************************************************
** Meta object code from reading C++ file 'glaxnimate_window.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/gui/widgets/dialogs/glaxnimate_window.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'glaxnimate_window.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate3gui16GlaxnimateWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::gui::GlaxnimateWindow::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate3gui16GlaxnimateWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::gui::GlaxnimateWindow",
        "document_save",
        "",
        "document_save_as",
        "document_export",
        "document_export_as",
        "document_export_sequence",
        "view_fit",
        "copy",
        "paste",
        "cut",
        "duplicate_selection",
        "delete_selected",
        "copyDebugInfo",
        "document_new",
        "document_open_dialog",
        "document_reload",
        "document_treeview_current_changed",
        "model::VisualNode*",
        "node",
        "document_treeview_selection_changed",
        "std::vector<model::VisualNode*>",
        "selected",
        "deselected",
        "scene_selection_changed",
        "layer_new_menu",
        "layer_delete",
        "layer_duplicate",
        "layer_top",
        "layer_raise",
        "layer_lower",
        "layer_bottom",
        "help_about_env",
        "help_manual",
        "refresh_title",
        "document_open_recent",
        "QUrl",
        "url",
        "save_frame_bmp",
        "save_frame_svg",
        "tool_triggered",
        "checked",
        "validate_tgs",
        "switch_composition",
        "model::Composition*",
        "comp",
        "index",
        "ipc_signal_connections",
        "enable",
        "ipc_error",
        "QLocalSocket::LocalSocketError",
        "socketError",
        "ipc_read",
        "ipc_write_time",
        "model::FrameTime",
        "t",
        "ipc_draw_background",
        "QPainter*",
        "painter",
        "warning",
        "message",
        "title",
        "status",
        "choose_option",
        "QVariant",
        "label",
        "QVariantMap",
        "options",
        "get_open_image_file",
        "dir",
        "convert_to_path",
        "glaxnimate::model::ShapeElement*",
        "shape",
        "document",
        "glaxnimate::model::Document*",
        "current_item",
        "glaxnimate::model::VisualNode*",
        "current_shape",
        "current_shape_container",
        "glaxnimate::model::Object*",
        "current_composition",
        "glaxnimate::model::Composition*",
        "fill_color",
        "QColor",
        "stroke_color"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'document_save'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'document_save_as'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'document_export'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'document_export_as'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'document_export_sequence'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'view_fit'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'copy'
        QtMocHelpers::SlotData<void() const>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'paste'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'cut'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'duplicate_selection'
        QtMocHelpers::SlotData<void() const>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'delete_selected'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'copyDebugInfo'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'document_new'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'document_open_dialog'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'document_reload'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'document_treeview_current_changed'
        QtMocHelpers::SlotData<void(model::VisualNode *)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'document_treeview_selection_changed'
        QtMocHelpers::SlotData<void(const std::vector<model::VisualNode*> &, const std::vector<model::VisualNode*> &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { 0x80000000 | 21, 23 },
        }}),
        // Slot 'scene_selection_changed'
        QtMocHelpers::SlotData<void(const std::vector<model::VisualNode*> &, const std::vector<model::VisualNode*> &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { 0x80000000 | 21, 23 },
        }}),
        // Slot 'layer_new_menu'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'layer_delete'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'layer_duplicate'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'layer_top'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'layer_raise'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'layer_lower'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'layer_bottom'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'help_about_env'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'help_manual'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'refresh_title'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'document_open_recent'
        QtMocHelpers::SlotData<void(const QUrl &)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Slot 'save_frame_bmp'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'save_frame_svg'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'tool_triggered'
        QtMocHelpers::SlotData<void(bool)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 41 },
        }}),
        // Slot 'validate_tgs'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'switch_composition'
        QtMocHelpers::SlotData<void(model::Composition *, int)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 44, 45 }, { QMetaType::Int, 46 },
        }}),
        // Slot 'ipc_signal_connections'
        QtMocHelpers::SlotData<void(bool)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 48 },
        }}),
        // Slot 'ipc_error'
        QtMocHelpers::SlotData<void(QLocalSocket::LocalSocketError)>(49, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 50, 51 },
        }}),
        // Slot 'ipc_read'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ipc_write_time'
        QtMocHelpers::SlotData<void(model::FrameTime)>(53, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 54, 55 },
        }}),
        // Slot 'ipc_draw_background'
        QtMocHelpers::SlotData<void(QPainter *)>(56, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 57, 58 },
        }}),
        // Method 'warning'
        QtMocHelpers::MethodData<void(const QString &, const QString &) const>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 60 }, { QMetaType::QString, 61 },
        }}),
        // Method 'warning'
        QtMocHelpers::MethodData<void(const QString &) const>(59, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 60 },
        }}),
        // Method 'status'
        QtMocHelpers::MethodData<void(const QString &) const>(62, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 60 },
        }}),
        // Method 'choose_option'
        QtMocHelpers::MethodData<QVariant(const QString &, const QVariantMap &, const QString &) const>(63, 2, QMC::AccessPublic, 0x80000000 | 64, {{
            { QMetaType::QString, 65 }, { 0x80000000 | 66, 67 }, { QMetaType::QString, 61 },
        }}),
        // Method 'choose_option'
        QtMocHelpers::MethodData<QVariant(const QString &, const QVariantMap &) const>(63, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 64, {{
            { QMetaType::QString, 65 }, { 0x80000000 | 66, 67 },
        }}),
        // Method 'get_open_image_file'
        QtMocHelpers::MethodData<QString(const QString &, const QString &) const>(68, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 61 }, { QMetaType::QString, 69 },
        }}),
        // Method 'get_open_image_file'
        QtMocHelpers::MethodData<QString(const QString &) const>(68, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QString, {{
            { QMetaType::QString, 61 },
        }}),
        // Method 'convert_to_path'
        QtMocHelpers::MethodData<glaxnimate::model::ShapeElement *(glaxnimate::model::ShapeElement *)>(70, 2, QMC::AccessPublic, 0x80000000 | 71, {{
            { 0x80000000 | 71, 72 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'document'
        QtMocHelpers::PropertyData<glaxnimate::model::Document*>(73, 0x80000000 | 74, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'current_item'
        QtMocHelpers::PropertyData<glaxnimate::model::VisualNode*>(75, 0x80000000 | 76, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'current_shape'
        QtMocHelpers::PropertyData<glaxnimate::model::ShapeElement*>(77, 0x80000000 | 71, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'current_shape_container'
        QtMocHelpers::PropertyData<glaxnimate::model::Object*>(78, 0x80000000 | 79, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'current_composition'
        QtMocHelpers::PropertyData<glaxnimate::model::Composition*>(80, 0x80000000 | 81, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'fill_color'
        QtMocHelpers::PropertyData<QColor>(82, 0x80000000 | 83, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'stroke_color'
        QtMocHelpers::PropertyData<QColor>(84, 0x80000000 | 83, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GlaxnimateWindow, qt_meta_tag_ZN10glaxnimate3gui16GlaxnimateWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::gui::GlaxnimateWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<KXmlGuiWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui16GlaxnimateWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui16GlaxnimateWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate3gui16GlaxnimateWindowE_t>.metaTypes,
    nullptr
} };

void glaxnimate::gui::GlaxnimateWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GlaxnimateWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->document_save(); break;
        case 1: _t->document_save_as(); break;
        case 2: _t->document_export(); break;
        case 3: _t->document_export_as(); break;
        case 4: _t->document_export_sequence(); break;
        case 5: _t->view_fit(); break;
        case 6: _t->copy(); break;
        case 7: _t->paste(); break;
        case 8: _t->cut(); break;
        case 9: _t->duplicate_selection(); break;
        case 10: _t->delete_selected(); break;
        case 11: _t->copyDebugInfo(); break;
        case 12: _t->document_new(); break;
        case 13: _t->document_open_dialog(); break;
        case 14: _t->document_reload(); break;
        case 15: _t->document_treeview_current_changed((*reinterpret_cast<std::add_pointer_t<model::VisualNode*>>(_a[1]))); break;
        case 16: _t->document_treeview_selection_changed((*reinterpret_cast<std::add_pointer_t<std::vector<model::VisualNode*>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::vector<model::VisualNode*>>>(_a[2]))); break;
        case 17: _t->scene_selection_changed((*reinterpret_cast<std::add_pointer_t<std::vector<model::VisualNode*>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::vector<model::VisualNode*>>>(_a[2]))); break;
        case 18: _t->layer_new_menu(); break;
        case 19: _t->layer_delete(); break;
        case 20: _t->layer_duplicate(); break;
        case 21: _t->layer_top(); break;
        case 22: _t->layer_raise(); break;
        case 23: _t->layer_lower(); break;
        case 24: _t->layer_bottom(); break;
        case 25: _t->help_about_env(); break;
        case 26: _t->help_manual(); break;
        case 27: _t->refresh_title(); break;
        case 28: _t->document_open_recent((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 29: _t->save_frame_bmp(); break;
        case 30: _t->save_frame_svg(); break;
        case 31: _t->tool_triggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 32: _t->validate_tgs(); break;
        case 33: _t->switch_composition((*reinterpret_cast<std::add_pointer_t<model::Composition*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 34: _t->ipc_signal_connections((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 35: _t->ipc_error((*reinterpret_cast<std::add_pointer_t<QLocalSocket::LocalSocketError>>(_a[1]))); break;
        case 36: _t->ipc_read(); break;
        case 37: _t->ipc_write_time((*reinterpret_cast<std::add_pointer_t<model::FrameTime>>(_a[1]))); break;
        case 38: _t->ipc_draw_background((*reinterpret_cast<std::add_pointer_t<QPainter*>>(_a[1]))); break;
        case 39: _t->warning((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 40: _t->warning((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 41: _t->status((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 42: { QVariant _r = _t->choose_option((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 43: { QVariant _r = _t->choose_option((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 44: { QString _r = _t->get_open_image_file((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 45: { QString _r = _t->get_open_image_file((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 46: { glaxnimate::model::ShapeElement* _r = _t->convert_to_path((*reinterpret_cast<std::add_pointer_t<glaxnimate::model::ShapeElement*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<glaxnimate::model::ShapeElement**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 46:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< glaxnimate::model::ShapeElement* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< glaxnimate::model::Composition* >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< glaxnimate::model::Document* >(); break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< glaxnimate::model::Object* >(); break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< glaxnimate::model::ShapeElement* >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< glaxnimate::model::VisualNode* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<glaxnimate::model::Document**>(_v) = _t->document(); break;
        case 1: *reinterpret_cast<glaxnimate::model::VisualNode**>(_v) = _t->current_document_node(); break;
        case 2: *reinterpret_cast<glaxnimate::model::ShapeElement**>(_v) = _t->current_shape(); break;
        case 3: *reinterpret_cast<glaxnimate::model::Object**>(_v) = _t->current_shape_container_script(); break;
        case 4: *reinterpret_cast<glaxnimate::model::Composition**>(_v) = _t->current_composition(); break;
        case 5: *reinterpret_cast<QColor*>(_v) = _t->current_color(); break;
        case 6: *reinterpret_cast<QColor*>(_v) = _t->secondary_color(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->set_current_document_node(*reinterpret_cast<glaxnimate::model::VisualNode**>(_v)); break;
        case 4: _t->set_current_composition(*reinterpret_cast<glaxnimate::model::Composition**>(_v)); break;
        case 5: _t->set_current_color(*reinterpret_cast<QColor*>(_v)); break;
        case 6: _t->set_secondary_color(*reinterpret_cast<QColor*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::gui::GlaxnimateWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::gui::GlaxnimateWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui16GlaxnimateWindowE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "glaxnimate::gui::SelectionManager"))
        return static_cast< glaxnimate::gui::SelectionManager*>(this);
    return KXmlGuiWindow::qt_metacast(_clname);
}

int glaxnimate::gui::GlaxnimateWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KXmlGuiWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 47)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 47;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 47)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 47;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
