/****************************************************************************
** Meta object code from reading C++ file 'animatable.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/core/model/animation/animatable.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'animatable.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate5model12KeyframeBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::KeyframeBase::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model12KeyframeBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::KeyframeBase",
        "transition_changed",
        "",
        "KeyframeTransition::Descriptive",
        "before",
        "after",
        "value",
        "QVariant",
        "time"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'transition_changed'
        QtMocHelpers::SignalData<void(KeyframeTransition::Descriptive, KeyframeTransition::Descriptive)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'value'
        QtMocHelpers::PropertyData<QVariant>(6, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'time'
        QtMocHelpers::PropertyData<double>(8, QMetaType::Double, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KeyframeBase, qt_meta_tag_ZN10glaxnimate5model12KeyframeBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::KeyframeBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model12KeyframeBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model12KeyframeBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model12KeyframeBaseE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::KeyframeBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KeyframeBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->transition_changed((*reinterpret_cast<std::add_pointer_t<KeyframeTransition::Descriptive>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KeyframeTransition::Descriptive>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KeyframeBase::*)(KeyframeTransition::Descriptive , KeyframeTransition::Descriptive )>(_a, &KeyframeBase::transition_changed, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QVariant*>(_v) = _t->value(); break;
        case 1: *reinterpret_cast<double*>(_v) = _t->time(); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::KeyframeBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::KeyframeBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model12KeyframeBaseE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int glaxnimate::model::KeyframeBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::model::KeyframeBase::transition_changed(KeyframeTransition::Descriptive _t1, KeyframeTransition::Descriptive _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN10glaxnimate5model14AnimatableBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::AnimatableBase::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model14AnimatableBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::AnimatableBase",
        "keyframe_added",
        "",
        "index",
        "KeyframeBase*",
        "keyframe",
        "keyframe_removed",
        "keyframe_updated",
        "value_mismatch",
        "keyframe_index",
        "time",
        "keyframe_count",
        "value",
        "QVariant",
        "animated"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'keyframe_added'
        QtMocHelpers::SignalData<void(int, KeyframeBase *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'keyframe_removed'
        QtMocHelpers::SignalData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'keyframe_updated'
        QtMocHelpers::SignalData<void(int, KeyframeBase *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Method 'value_mismatch'
        QtMocHelpers::MethodData<bool() const>(8, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'keyframe_index'
        QtMocHelpers::MethodData<int(double) const>(9, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::Double, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'keyframe_count'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'value'
        QtMocHelpers::PropertyData<QVariant>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'animated'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AnimatableBase, qt_meta_tag_ZN10glaxnimate5model14AnimatableBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::AnimatableBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model14AnimatableBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model14AnimatableBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model14AnimatableBaseE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::AnimatableBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AnimatableBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->keyframe_added((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KeyframeBase*>>(_a[2]))); break;
        case 1: _t->keyframe_removed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->keyframe_updated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KeyframeBase*>>(_a[2]))); break;
        case 3: { bool _r = _t->value_mismatch();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { int _r = _t->keyframe_index((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KeyframeBase* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KeyframeBase* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AnimatableBase::*)(int , KeyframeBase * )>(_a, &AnimatableBase::keyframe_added, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnimatableBase::*)(int )>(_a, &AnimatableBase::keyframe_removed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnimatableBase::*)(int , KeyframeBase * )>(_a, &AnimatableBase::keyframe_updated, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->keyframe_count(); break;
        case 1: *reinterpret_cast<QVariant*>(_v) = _t->value(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->animated(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->set_undoable(*reinterpret_cast<QVariant*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::AnimatableBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::AnimatableBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model14AnimatableBaseE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "BaseProperty"))
        return static_cast< BaseProperty*>(this);
    return QObject::qt_metacast(_clname);
}

int glaxnimate::model::AnimatableBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::model::AnimatableBase::keyframe_added(int _t1, KeyframeBase * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void glaxnimate::model::AnimatableBase::keyframe_removed(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void glaxnimate::model::AnimatableBase::keyframe_updated(int _t1, KeyframeBase * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN10glaxnimate5model6detail24AnimatedPropertyPositionE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::detail::AnimatedPropertyPosition::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model6detail24AnimatedPropertyPositionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::detail::AnimatedPropertyPosition",
        "bezier_set",
        "",
        "math::bezier::Bezier",
        "bezier",
        "split_segment",
        "index",
        "factor",
        "set_bezier",
        "glaxnimate::math::bezier::Bezier"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'bezier_set'
        QtMocHelpers::SignalData<void(const math::bezier::Bezier &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Method 'split_segment'
        QtMocHelpers::MethodData<void(int, qreal)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 }, { QMetaType::QReal, 7 },
        }}),
        // Method 'set_bezier'
        QtMocHelpers::MethodData<bool(glaxnimate::math::bezier::Bezier)>(8, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 9, 4 },
        }}),
        // Method 'bezier'
        QtMocHelpers::MethodData<glaxnimate::math::bezier::Bezier() const>(4, 2, QMC::AccessPublic, 0x80000000 | 9),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AnimatedPropertyPosition, qt_meta_tag_ZN10glaxnimate5model6detail24AnimatedPropertyPositionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::detail::AnimatedPropertyPosition::staticMetaObject = { {
    QMetaObject::SuperData::link<detail::AnimatedProperty<QPointF>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model6detail24AnimatedPropertyPositionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model6detail24AnimatedPropertyPositionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model6detail24AnimatedPropertyPositionE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::detail::AnimatedPropertyPosition::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AnimatedPropertyPosition *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->bezier_set((*reinterpret_cast<std::add_pointer_t<math::bezier::Bezier>>(_a[1]))); break;
        case 1: _t->split_segment((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 2: { bool _r = _t->set_bezier((*reinterpret_cast<std::add_pointer_t<glaxnimate::math::bezier::Bezier>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { glaxnimate::math::bezier::Bezier _r = _t->bezier();
            if (_a[0]) *reinterpret_cast<glaxnimate::math::bezier::Bezier*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< glaxnimate::math::bezier::Bezier >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AnimatedPropertyPosition::*)(const math::bezier::Bezier & )>(_a, &AnimatedPropertyPosition::bezier_set, 0))
            return;
    }
}

const QMetaObject *glaxnimate::model::detail::AnimatedPropertyPosition::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::detail::AnimatedPropertyPosition::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model6detail24AnimatedPropertyPositionE_t>.strings))
        return static_cast<void*>(this);
    return detail::AnimatedProperty<QPointF>::qt_metacast(_clname);
}

int glaxnimate::model::detail::AnimatedPropertyPosition::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = detail::AnimatedProperty<QPointF>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::model::detail::AnimatedPropertyPosition::bezier_set(const math::bezier::Bezier & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
