/********************************************************************************
** Form generated from reading UI file 'shape_parent_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SHAPE_PARENT_DIALOG_H
#define UI_SHAPE_PARENT_DIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

namespace glaxnimate {
namespace gui {

class Ui_ShapeParentDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTreeView *view_document_node;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushButton;
    QPushButton *pushButton_2;

    void setupUi(QDialog *ShapeParentDialog)
    {
        if (ShapeParentDialog->objectName().isEmpty())
            ShapeParentDialog->setObjectName("ShapeParentDialog");
        ShapeParentDialog->resize(394, 425);
        verticalLayout = new QVBoxLayout(ShapeParentDialog);
        verticalLayout->setObjectName("verticalLayout");
        view_document_node = new QTreeView(ShapeParentDialog);
        view_document_node->setObjectName("view_document_node");
        view_document_node->setSelectionMode(QAbstractItemView::ExtendedSelection);
        view_document_node->header()->setVisible(false);
        view_document_node->header()->setMinimumSectionSize(32);
        view_document_node->header()->setDefaultSectionSize(32);
        view_document_node->header()->setStretchLastSection(false);

        verticalLayout->addWidget(view_document_node);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushButton = new QPushButton(ShapeParentDialog);
        pushButton->setObjectName("pushButton");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("dialog-cancel");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        pushButton->setIcon(icon);

        horizontalLayout->addWidget(pushButton);

        pushButton_2 = new QPushButton(ShapeParentDialog);
        pushButton_2->setObjectName("pushButton_2");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("dialog-ok");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        pushButton_2->setIcon(icon1);

        horizontalLayout->addWidget(pushButton_2);


        verticalLayout->addLayout(horizontalLayout);

        QWidget::setTabOrder(view_document_node, pushButton_2);
        QWidget::setTabOrder(pushButton_2, pushButton);

        retranslateUi(ShapeParentDialog);
        QObject::connect(pushButton, &QPushButton::clicked, ShapeParentDialog, qOverload<>(&QDialog::reject));
        QObject::connect(pushButton_2, &QPushButton::clicked, ShapeParentDialog, qOverload<>(&QDialog::accept));
        QObject::connect(view_document_node, SIGNAL(clicked(QModelIndex)), ShapeParentDialog, SLOT(select(QModelIndex)));
        QObject::connect(view_document_node, SIGNAL(doubleClicked(QModelIndex)), ShapeParentDialog, SLOT(select_and_accept(QModelIndex)));

        pushButton_2->setDefault(true);


        QMetaObject::connectSlotsByName(ShapeParentDialog);
    } // setupUi

    void retranslateUi(QDialog *ShapeParentDialog)
    {
        ShapeParentDialog->setWindowTitle(tr2i18n("Destination", nullptr));
        pushButton->setText(tr2i18n("Cancel", nullptr));
        pushButton_2->setText(tr2i18n("Move", nullptr));
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class ShapeParentDialog: public Ui_ShapeParentDialog {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // SHAPE_PARENT_DIALOG_H

