/****************************************************************************
** Meta object code from reading C++ file 'document.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/core/model/document.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'document.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate5model8DocumentE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::Document::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model8DocumentE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::Document",
        "filename_changed",
        "",
        "n",
        "current_time_changing",
        "FrameTime",
        "t",
        "current_time_changed",
        "record_to_keyframe_changed",
        "r",
        "graphics_invalidated",
        "find_by_uuid",
        "glaxnimate::model::DocumentNode*",
        "QUuid",
        "find_by_name",
        "name",
        "find_by_type_name",
        "QVariantList",
        "type_name",
        "undo",
        "redo",
        "get_best_name",
        "node",
        "suggestion",
        "set_best_name",
        "filename",
        "current_time",
        "record_to_keyframe",
        "assets",
        "Object*",
        "metadata",
        "QVariantMap"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'filename_changed'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'current_time_changing'
        QtMocHelpers::SignalData<void(FrameTime)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'current_time_changed'
        QtMocHelpers::SignalData<void(FrameTime)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'record_to_keyframe_changed'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'graphics_invalidated'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'find_by_uuid'
        QtMocHelpers::MethodData<glaxnimate::model::DocumentNode *(const QUuid &) const>(11, 2, QMC::AccessPublic, 0x80000000 | 12, {{
            { 0x80000000 | 13, 3 },
        }}),
        // Method 'find_by_name'
        QtMocHelpers::MethodData<glaxnimate::model::DocumentNode *(const QString &) const>(14, 2, QMC::AccessPublic, 0x80000000 | 12, {{
            { QMetaType::QString, 15 },
        }}),
        // Method 'find_by_type_name'
        QtMocHelpers::MethodData<QVariantList(const QString &) const>(16, 2, QMC::AccessPublic, 0x80000000 | 17, {{
            { QMetaType::QString, 18 },
        }}),
        // Method 'undo'
        QtMocHelpers::MethodData<bool()>(19, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'redo'
        QtMocHelpers::MethodData<bool()>(20, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'get_best_name'
        QtMocHelpers::MethodData<QString(glaxnimate::model::DocumentNode *, const QString &) const>(21, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 12, 22 }, { QMetaType::QString, 23 },
        }}),
        // Method 'get_best_name'
        QtMocHelpers::MethodData<QString(glaxnimate::model::DocumentNode *) const>(21, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QString, {{
            { 0x80000000 | 12, 22 },
        }}),
        // Method 'set_best_name'
        QtMocHelpers::MethodData<void(glaxnimate::model::DocumentNode *, const QString &) const>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 22 }, { QMetaType::QString, 23 },
        }}),
        // Method 'set_best_name'
        QtMocHelpers::MethodData<void(glaxnimate::model::DocumentNode *) const>(24, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 12, 22 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'filename'
        QtMocHelpers::PropertyData<QString>(25, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'current_time'
        QtMocHelpers::PropertyData<double>(26, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable, 2),
        // property 'record_to_keyframe'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 3),
        // property 'assets'
        QtMocHelpers::PropertyData<Object*>(28, 0x80000000 | 29, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'metadata'
        QtMocHelpers::PropertyData<QVariantMap>(30, 0x80000000 | 31, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Document, qt_meta_tag_ZN10glaxnimate5model8DocumentE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::Document::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model8DocumentE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model8DocumentE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model8DocumentE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::Document::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Document *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->filename_changed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->current_time_changing((*reinterpret_cast<std::add_pointer_t<FrameTime>>(_a[1]))); break;
        case 2: _t->current_time_changed((*reinterpret_cast<std::add_pointer_t<FrameTime>>(_a[1]))); break;
        case 3: _t->record_to_keyframe_changed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->graphics_invalidated(); break;
        case 5: { glaxnimate::model::DocumentNode* _r = _t->find_by_uuid((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])));
            if (_a[0]) *reinterpret_cast<glaxnimate::model::DocumentNode**>(_a[0]) = std::move(_r); }  break;
        case 6: { glaxnimate::model::DocumentNode* _r = _t->find_by_name((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<glaxnimate::model::DocumentNode**>(_a[0]) = std::move(_r); }  break;
        case 7: { QVariantList _r = _t->find_by_type_name((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->undo();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->redo();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { QString _r = _t->get_best_name((*reinterpret_cast<std::add_pointer_t<glaxnimate::model::DocumentNode*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 11: { QString _r = _t->get_best_name((*reinterpret_cast<std::add_pointer_t<glaxnimate::model::DocumentNode*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->set_best_name((*reinterpret_cast<std::add_pointer_t<glaxnimate::model::DocumentNode*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 13: _t->set_best_name((*reinterpret_cast<std::add_pointer_t<glaxnimate::model::DocumentNode*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< glaxnimate::model::DocumentNode* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< glaxnimate::model::DocumentNode* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< glaxnimate::model::DocumentNode* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< glaxnimate::model::DocumentNode* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Document::*)(const QString & )>(_a, &Document::filename_changed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(FrameTime )>(_a, &Document::current_time_changing, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(FrameTime )>(_a, &Document::current_time_changed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(bool )>(_a, &Document::record_to_keyframe_changed, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)()>(_a, &Document::graphics_invalidated, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Object* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->filename(); break;
        case 1: *reinterpret_cast<double*>(_v) = _t->current_time(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->record_to_keyframe(); break;
        case 3: *reinterpret_cast<Object**>(_v) = _t->assets_obj(); break;
        case 4: *reinterpret_cast<QVariantMap*>(_v) = _t->metadata(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->set_current_time(*reinterpret_cast<double*>(_v)); break;
        case 2: _t->set_record_to_keyframe(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->set_metadata(*reinterpret_cast<QVariantMap*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::Document::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::Document::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model8DocumentE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int glaxnimate::model::Document::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::model::Document::filename_changed(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void glaxnimate::model::Document::current_time_changing(FrameTime _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void glaxnimate::model::Document::current_time_changed(FrameTime _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void glaxnimate::model::Document::record_to_keyframe_changed(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void glaxnimate::model::Document::graphics_invalidated()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
