/********************************************************************************
** Form generated from reading UI file 'font_size_widget.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FONT_SIZE_WIDGET_H
#define UI_FONT_SIZE_WIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QListView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace glaxnimate {
namespace gui {
namespace font {

class Ui_FontSizeWidget
{
public:
    QVBoxLayout *verticalLayout;
    QDoubleSpinBox *spin_size;
    QListView *view_size;

    void setupUi(QWidget *glaxnimate__gui__font__FontSizeWidget)
    {
        if (glaxnimate__gui__font__FontSizeWidget->objectName().isEmpty())
            glaxnimate__gui__font__FontSizeWidget->setObjectName("glaxnimate__gui__font__FontSizeWidget");
        glaxnimate__gui__font__FontSizeWidget->resize(185, 300);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(glaxnimate__gui__font__FontSizeWidget->sizePolicy().hasHeightForWidth());
        glaxnimate__gui__font__FontSizeWidget->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(glaxnimate__gui__font__FontSizeWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        spin_size = new QDoubleSpinBox(glaxnimate__gui__font__FontSizeWidget);
        spin_size->setObjectName("spin_size");
        spin_size->setDecimals(0);
        spin_size->setMinimum(1.000000000000000);
        spin_size->setMaximum(1000.000000000000000);

        verticalLayout->addWidget(spin_size);

        view_size = new QListView(glaxnimate__gui__font__FontSizeWidget);
        view_size->setObjectName("view_size");

        verticalLayout->addWidget(view_size);


        retranslateUi(glaxnimate__gui__font__FontSizeWidget);

        QMetaObject::connectSlotsByName(glaxnimate__gui__font__FontSizeWidget);
    } // setupUi

    void retranslateUi(QWidget *glaxnimate__gui__font__FontSizeWidget)
    {
        (void)glaxnimate__gui__font__FontSizeWidget;
    } // retranslateUi

};

} // namespace font
} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace font {
namespace Ui {
    class FontSizeWidget: public Ui_FontSizeWidget {};
} // namespace Ui
} // namespace font
} // namespace gui
} // namespace glaxnimate

#endif // FONT_SIZE_WIDGET_H

