/****************************************************************************
** Meta object code from reading C++ file 'precomp_layer.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/core/model/shapes/precomp_layer.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'precomp_layer.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate5model12PreCompLayerE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::PreCompLayer::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model12PreCompLayerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::PreCompLayer",
        "composition",
        "property ref Composition",
        "size",
        "property QSizeF",
        "opacity",
        "property animated float",
        "opacity_changed",
        "",
        "op",
        "composition_changed",
        "on_transform_matrix_changed",
        "timing",
        "StretchableTime*",
        "Composition*",
        "QSizeF",
        "transform",
        "Transform*",
        "glaxnimate::model::AnimatableBase*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'opacity_changed'
        QtMocHelpers::SignalData<void(float)>(7, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 9 },
        }}),
        // Signal 'composition_changed'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_transform_matrix_changed'
        QtMocHelpers::SlotData<void()>(11, 8, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'timing'
        QtMocHelpers::PropertyData<StretchableTime*>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'composition'
        QtMocHelpers::PropertyData<Composition*>(1, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'size'
        QtMocHelpers::PropertyData<QSizeF>(3, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'transform'
        QtMocHelpers::PropertyData<Transform*>(16, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'opacity'
        QtMocHelpers::PropertyData<glaxnimate::model::AnimatableBase*>(5, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<PreCompLayer, qt_meta_tag_ZN10glaxnimate5model12PreCompLayerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::PreCompLayer::staticMetaObject = { {
    QMetaObject::SuperData::link<ShapeElement::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model12PreCompLayerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model12PreCompLayerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model12PreCompLayerE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::PreCompLayer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PreCompLayer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->opacity_changed((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 1: _t->composition_changed(); break;
        case 2: _t->on_transform_matrix_changed(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PreCompLayer::*)(float )>(_a, &PreCompLayer::opacity_changed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreCompLayer::*)()>(_a, &PreCompLayer::composition_changed, 1))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Composition* >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< StretchableTime* >(); break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Transform* >(); break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< glaxnimate::model::AnimatableBase* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<StretchableTime**>(_v) = _t->get_timing(); break;
        case 1: *reinterpret_cast<Composition**>(_v) = _t->get_composition(); break;
        case 2: *reinterpret_cast<QSizeF*>(_v) = _t->get_size(); break;
        case 3: *reinterpret_cast<Transform**>(_v) = _t->get_transform(); break;
        case 4: *reinterpret_cast<glaxnimate::model::AnimatableBase**>(_v) = _t->get_opacity(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->set_composition(*reinterpret_cast<Composition**>(_v)); break;
        case 2: _t->set_size(*reinterpret_cast<QSizeF*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::PreCompLayer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::PreCompLayer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model12PreCompLayerE_t>.strings))
        return static_cast<void*>(this);
    return ShapeElement::qt_metacast(_clname);
}

int glaxnimate::model::PreCompLayer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ShapeElement::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::model::PreCompLayer::opacity_changed(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void glaxnimate::model::PreCompLayer::composition_changed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
