/********************************************************************************
** Form generated from reading UI file 'script_console.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCRIPT_CONSOLE_H
#define UI_SCRIPT_CONSOLE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDockWidget>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "khistorycombobox.h"

QT_BEGIN_NAMESPACE

class Ui_ScriptConsole
{
public:
    QWidget *dockWidgetContents;
    QVBoxLayout *verticalLayout;
    QTextEdit *console_output;
    QHBoxLayout *horizontalLayout_2;
    KHistoryComboBox *console_input;
    QToolButton *btn_clear_console;
    QToolButton *check_persist;
    QToolButton *btn_reload;
    QComboBox *console_language;

    void setupUi(QDockWidget *ScriptConsole)
    {
        if (ScriptConsole->objectName().isEmpty())
            ScriptConsole->setObjectName("ScriptConsole");
        ScriptConsole->resize(400, 300);
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("utilities-terminal");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        ScriptConsole->setWindowIcon(icon);
        dockWidgetContents = new QWidget();
        dockWidgetContents->setObjectName("dockWidgetContents");
        verticalLayout = new QVBoxLayout(dockWidgetContents);
        verticalLayout->setObjectName("verticalLayout");
        console_output = new QTextEdit(dockWidgetContents);
        console_output->setObjectName("console_output");
        QFont font;
        font.setFamilies({QString::fromUtf8("Monospace")});
        console_output->setFont(font);
        console_output->setReadOnly(true);

        verticalLayout->addWidget(console_output);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        console_input = new KHistoryComboBox(dockWidgetContents);
        console_input->setObjectName("console_input");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(console_input->sizePolicy().hasHeightForWidth());
        console_input->setSizePolicy(sizePolicy);
        console_input->setFont(font);

        horizontalLayout_2->addWidget(console_input);

        btn_clear_console = new QToolButton(dockWidgetContents);
        btn_clear_console->setObjectName("btn_clear_console");
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("edit-clear-all")));
        btn_clear_console->setIcon(icon1);

        horizontalLayout_2->addWidget(btn_clear_console);

        check_persist = new QToolButton(dockWidgetContents);
        check_persist->setObjectName("check_persist");
        QIcon icon2;
        iconThemeName = QString::fromUtf8("pin");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        check_persist->setIcon(icon2);
        check_persist->setCheckable(true);

        horizontalLayout_2->addWidget(check_persist);

        btn_reload = new QToolButton(dockWidgetContents);
        btn_reload->setObjectName("btn_reload");
        QIcon icon3;
        iconThemeName = QString::fromUtf8("view-refresh");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_reload->setIcon(icon3);

        horizontalLayout_2->addWidget(btn_reload);

        console_language = new QComboBox(dockWidgetContents);
        console_language->setObjectName("console_language");

        horizontalLayout_2->addWidget(console_language);


        verticalLayout->addLayout(horizontalLayout_2);

        ScriptConsole->setWidget(dockWidgetContents);

        retranslateUi(ScriptConsole);
        QObject::connect(console_input, SIGNAL(returnPressed(QString)), ScriptConsole, SLOT(console_commit(QString)));
        QObject::connect(btn_clear_console, SIGNAL(clicked()), ScriptConsole, SLOT(console_clear()));

        QMetaObject::connectSlotsByName(ScriptConsole);
    } // setupUi

    void retranslateUi(QDockWidget *ScriptConsole)
    {
        ScriptConsole->setWindowTitle(tr2i18n("Script Console", nullptr));
#if QT_CONFIG(tooltip)
        btn_clear_console->setToolTip(tr2i18n("Clear Output", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_clear_console->setText(tr2i18n("Clear", nullptr));
#if QT_CONFIG(tooltip)
        check_persist->setToolTip(tr2i18n("Persist Logs", nullptr));
#endif // QT_CONFIG(tooltip)
        check_persist->setText(tr2i18n("Persist", nullptr));
#if QT_CONFIG(tooltip)
        btn_reload->setToolTip(tr2i18n("Reload Script Modules", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_reload->setText(tr2i18n("Reload", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ScriptConsole: public Ui_ScriptConsole {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SCRIPT_CONSOLE_H

