/****************************************************************************
** Meta object code from reading C++ file 'bitmap.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/core/model/assets/bitmap.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bitmap.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate5model6BitmapE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::Bitmap::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model6BitmapE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::Bitmap",
        "data",
        "property QByteArray",
        "filename",
        "property QString",
        "url",
        "format",
        "width",
        "property int",
        "height",
        "loaded",
        "",
        "refresh",
        "rebuild_embedded",
        "embed",
        "embedded",
        "on_refresh",
        "image",
        "QImage"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'loaded'
        QtMocHelpers::SignalData<void()>(10, 11, QMC::AccessPublic, QMetaType::Void),
        // Slot 'refresh'
        QtMocHelpers::SlotData<void(bool)>(12, 11, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Slot 'embed'
        QtMocHelpers::SlotData<void(bool)>(14, 11, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'on_refresh'
        QtMocHelpers::SlotData<void()>(16, 11, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'data'
        QtMocHelpers::PropertyData<QByteArray>(1, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'filename'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'url'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'format'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'width'
        QtMocHelpers::PropertyData<int>(7, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'height'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'embedded'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'image'
        QtMocHelpers::PropertyData<QImage>(17, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    4 },
            {    6,    4 },
            {    7,    8 },
            {    9,    8 },
    });
    return QtMocHelpers::metaObjectData<Bitmap, qt_meta_tag_ZN10glaxnimate5model6BitmapE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::Bitmap::staticMetaObject = { {
    QMetaObject::SuperData::link<Asset::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model6BitmapE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model6BitmapE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model6BitmapE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::Bitmap::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Bitmap *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->loaded(); break;
        case 1: _t->refresh((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->embed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->on_refresh(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Bitmap::*)()>(_a, &Bitmap::loaded, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QByteArray*>(_v) = _t->get_data(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->get_filename(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->get_url(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->get_format(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->get_width(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->get_height(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->embedded(); break;
        case 7: *reinterpret_cast<QImage*>(_v) = _t->get_image(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->set_data(*reinterpret_cast<QByteArray*>(_v)); break;
        case 1: _t->set_filename(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->set_url(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->embed(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::Bitmap::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::Bitmap::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model6BitmapE_t>.strings))
        return static_cast<void*>(this);
    return Asset::qt_metacast(_clname);
}

int glaxnimate::model::Bitmap::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Asset::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::model::Bitmap::loaded()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
