/****************************************************************************
** Meta object code from reading C++ file 'color_preview.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/external/Qt-Color-Widgets/include/QtColorWidgets/color_preview.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'color_preview.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13color_widgets12ColorPreviewE_t {};
} // unnamed namespace

template <> constexpr inline auto color_widgets::ColorPreview::qt_create_metaobjectdata<qt_meta_tag_ZN13color_widgets12ColorPreviewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "color_widgets::ColorPreview",
        "clicked",
        "",
        "colorChanged",
        "QColor",
        "comparisonColorChanged",
        "displayModeChanged",
        "DisplayMode",
        "backgroundChanged",
        "QBrush",
        "drawFrameChanged",
        "setColor",
        "c",
        "setComparisonColor",
        "color",
        "comparisonColor",
        "display_mode",
        "background",
        "drawFrame",
        "NoAlpha",
        "AllAlpha",
        "SplitAlpha",
        "SplitColor",
        "SplitColorReverse"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'clicked'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(QColor)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 2 },
        }}),
        // Signal 'comparisonColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 2 },
        }}),
        // Signal 'displayModeChanged'
        QtMocHelpers::SignalData<void(enum DisplayMode)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Signal 'backgroundChanged'
        QtMocHelpers::SignalData<void(const QBrush &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 2 },
        }}),
        // Signal 'drawFrameChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(const QColor &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 12 },
        }}),
        // Slot 'setComparisonColor'
        QtMocHelpers::SlotData<void(const QColor &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(14, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'comparisonColor'
        QtMocHelpers::PropertyData<QColor>(15, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'display_mode'
        QtMocHelpers::PropertyData<enum DisplayMode>(16, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 3),
        // property 'background'
        QtMocHelpers::PropertyData<QBrush>(17, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'drawFrame'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DisplayMode'
        QtMocHelpers::EnumData<enum DisplayMode>(7, 7, QMC::EnumFlags{}).add({
            {   19, DisplayMode::NoAlpha },
            {   20, DisplayMode::AllAlpha },
            {   21, DisplayMode::SplitAlpha },
            {   22, DisplayMode::SplitColor },
            {   23, DisplayMode::SplitColorReverse },
        }),
    };
    return QtMocHelpers::metaObjectData<ColorPreview, qt_meta_tag_ZN13color_widgets12ColorPreviewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject color_widgets::ColorPreview::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets12ColorPreviewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets12ColorPreviewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13color_widgets12ColorPreviewE_t>.metaTypes,
    nullptr
} };

void color_widgets::ColorPreview::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ColorPreview *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clicked(); break;
        case 1: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 2: _t->comparisonColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 3: _t->displayModeChanged((*reinterpret_cast<std::add_pointer_t<enum DisplayMode>>(_a[1]))); break;
        case 4: _t->backgroundChanged((*reinterpret_cast<std::add_pointer_t<QBrush>>(_a[1]))); break;
        case 5: _t->drawFrameChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 7: _t->setComparisonColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ColorPreview::*)()>(_a, &ColorPreview::clicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPreview::*)(QColor )>(_a, &ColorPreview::colorChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPreview::*)(QColor )>(_a, &ColorPreview::comparisonColorChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPreview::*)(DisplayMode )>(_a, &ColorPreview::displayModeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPreview::*)(const QBrush & )>(_a, &ColorPreview::backgroundChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPreview::*)(bool )>(_a, &ColorPreview::drawFrameChanged, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QColor*>(_v) = _t->color(); break;
        case 1: *reinterpret_cast<QColor*>(_v) = _t->comparisonColor(); break;
        case 2: *reinterpret_cast<enum DisplayMode*>(_v) = _t->displayMode(); break;
        case 3: *reinterpret_cast<QBrush*>(_v) = _t->background(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->drawFrame(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setColor(*reinterpret_cast<QColor*>(_v)); break;
        case 1: _t->setComparisonColor(*reinterpret_cast<QColor*>(_v)); break;
        case 2: _t->setDisplayMode(*reinterpret_cast<enum DisplayMode*>(_v)); break;
        case 3: _t->setBackground(*reinterpret_cast<QBrush*>(_v)); break;
        case 4: _t->setDrawFrame(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *color_widgets::ColorPreview::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *color_widgets::ColorPreview::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets12ColorPreviewE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int color_widgets::ColorPreview::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void color_widgets::ColorPreview::clicked()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void color_widgets::ColorPreview::colorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void color_widgets::ColorPreview::comparisonColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void color_widgets::ColorPreview::displayModeChanged(DisplayMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void color_widgets::ColorPreview::backgroundChanged(const QBrush & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void color_widgets::ColorPreview::drawFrameChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
