/********************************************************************************
** Form generated from reading UI file 'startup_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STARTUP_DIALOG_H
#define UI_STARTUP_DIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "widgets/spin2d.hpp"

namespace glaxnimate {
namespace gui {

class Ui_StartupDialog
{
public:
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QTableWidget *view_presets;
    QGroupBox *groupBox_3;
    QGridLayout *gridLayout;
    QLabel *label_3;
    QLabel *label;
    QLabel *label_2;
    QDoubleSpinBox *spin_duration;
    glaxnimate::gui::Spin2D *spin_size;
    QDoubleSpinBox *spin_fps;
    QComboBox *combo_time_units;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout;
    QListWidget *view_recent;
    QPushButton *button_open_browse;
    QHBoxLayout *horizontalLayout_2;
    QCheckBox *check_show_at_startup;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *StartupDialog)
    {
        if (StartupDialog->objectName().isEmpty())
            StartupDialog->setObjectName("StartupDialog");
        StartupDialog->resize(774, 564);
        verticalLayout_3 = new QVBoxLayout(StartupDialog);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        groupBox = new QGroupBox(StartupDialog);
        groupBox->setObjectName("groupBox");
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        view_presets = new QTableWidget(groupBox);
        if (view_presets->columnCount() < 5)
            view_presets->setColumnCount(5);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        view_presets->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        view_presets->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        view_presets->setHorizontalHeaderItem(2, __qtablewidgetitem2);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        view_presets->setHorizontalHeaderItem(3, __qtablewidgetitem3);
        QTableWidgetItem *__qtablewidgetitem4 = new QTableWidgetItem();
        view_presets->setHorizontalHeaderItem(4, __qtablewidgetitem4);
        view_presets->setObjectName("view_presets");
        view_presets->setSelectionMode(QAbstractItemView::SingleSelection);
        view_presets->setSelectionBehavior(QAbstractItemView::SelectRows);
        view_presets->setShowGrid(false);
        view_presets->verticalHeader()->setVisible(false);

        verticalLayout_2->addWidget(view_presets);

        groupBox_3 = new QGroupBox(groupBox);
        groupBox_3->setObjectName("groupBox_3");
        gridLayout = new QGridLayout(groupBox_3);
        gridLayout->setObjectName("gridLayout");
        label_3 = new QLabel(groupBox_3);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 2, 0, 1, 1);

        label = new QLabel(groupBox_3);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        label_2 = new QLabel(groupBox_3);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        spin_duration = new QDoubleSpinBox(groupBox_3);
        spin_duration->setObjectName("spin_duration");
        spin_duration->setMaximum(9999.989999999999782);

        gridLayout->addWidget(spin_duration, 2, 1, 1, 1);

        spin_size = new glaxnimate::gui::Spin2D(groupBox_3);
        spin_size->setObjectName("spin_size");

        gridLayout->addWidget(spin_size, 0, 1, 1, 2);

        spin_fps = new QDoubleSpinBox(groupBox_3);
        spin_fps->setObjectName("spin_fps");
        spin_fps->setMinimum(1.000000000000000);
        spin_fps->setMaximum(999.990000000000009);

        gridLayout->addWidget(spin_fps, 1, 1, 1, 2);

        combo_time_units = new QComboBox(groupBox_3);
        combo_time_units->addItem(QString());
        combo_time_units->addItem(QString());
        combo_time_units->setObjectName("combo_time_units");

        gridLayout->addWidget(combo_time_units, 2, 2, 1, 1);


        verticalLayout_2->addWidget(groupBox_3);


        horizontalLayout->addWidget(groupBox);

        groupBox_2 = new QGroupBox(StartupDialog);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout = new QVBoxLayout(groupBox_2);
        verticalLayout->setObjectName("verticalLayout");
        view_recent = new QListWidget(groupBox_2);
        view_recent->setObjectName("view_recent");

        verticalLayout->addWidget(view_recent);

        button_open_browse = new QPushButton(groupBox_2);
        button_open_browse->setObjectName("button_open_browse");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("document-open");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_open_browse->setIcon(icon);

        verticalLayout->addWidget(button_open_browse);


        horizontalLayout->addWidget(groupBox_2);

        horizontalLayout->setStretch(0, 2);
        horizontalLayout->setStretch(1, 1);

        verticalLayout_3->addLayout(horizontalLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        check_show_at_startup = new QCheckBox(StartupDialog);
        check_show_at_startup->setObjectName("check_show_at_startup");
        check_show_at_startup->setChecked(true);

        horizontalLayout_2->addWidget(check_show_at_startup);

        buttonBox = new QDialogButtonBox(StartupDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Ok);

        horizontalLayout_2->addWidget(buttonBox);


        verticalLayout_3->addLayout(horizontalLayout_2);


        retranslateUi(StartupDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, StartupDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, StartupDialog, qOverload<>(&QDialog::reject));
        QObject::connect(view_recent, SIGNAL(clicked(QModelIndex)), StartupDialog, SLOT(click_recent(QModelIndex)));
        QObject::connect(button_open_browse, &QPushButton::clicked, StartupDialog, qOverload<>(&QDialog::reject));
        QObject::connect(combo_time_units, SIGNAL(activated(int)), StartupDialog, SLOT(update_time_units()));
        QObject::connect(check_show_at_startup, SIGNAL(clicked(bool)), StartupDialog, SLOT(update_startup_enabled(bool)));
        QObject::connect(view_presets, SIGNAL(clicked(QModelIndex)), StartupDialog, SLOT(select_preset(QModelIndex)));
        QObject::connect(view_presets, SIGNAL(activated(QModelIndex)), StartupDialog, SLOT(select_preset(QModelIndex)));

        combo_time_units->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(StartupDialog);
    } // setupUi

    void retranslateUi(QDialog *StartupDialog)
    {
        StartupDialog->setWindowTitle(tr2i18n("Startup", nullptr));
        groupBox->setTitle(tr2i18n("Create New", nullptr));
        QTableWidgetItem *___qtablewidgetitem = view_presets->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(tr2i18n("Name", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = view_presets->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(tr2i18n("Size", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = view_presets->horizontalHeaderItem(2);
        ___qtablewidgetitem2->setText(tr2i18n("Framerate", nullptr));
        QTableWidgetItem *___qtablewidgetitem3 = view_presets->horizontalHeaderItem(3);
        ___qtablewidgetitem3->setText(tr2i18n("Aspect", nullptr));
        QTableWidgetItem *___qtablewidgetitem4 = view_presets->horizontalHeaderItem(4);
        ___qtablewidgetitem4->setText(tr2i18n("Duration", nullptr));
        groupBox_3->setTitle(tr2i18n("Settings", nullptr));
        label_3->setText(tr2i18n("Duration", nullptr));
        label->setText(tr2i18n("Size", nullptr));
        label_2->setText(tr2i18n("Framerate", nullptr));
        spin_fps->setSuffix(tr2i18n(" fps", nullptr));
        combo_time_units->setItemText(0, tr2i18n("Frames", nullptr));
        combo_time_units->setItemText(1, tr2i18n("Seconds", nullptr));

        groupBox_2->setTitle(tr2i18n("Open", nullptr));
        button_open_browse->setText(tr2i18n("Browse...", nullptr));
        check_show_at_startup->setText(tr2i18n("Show at startup", nullptr));
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class StartupDialog: public Ui_StartupDialog {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // STARTUP_DIALOG_H

