/****************************************************************************
** Meta object code from reading C++ file 'base.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/core/io/base.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'base.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate2io12ImportExportE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::io::ImportExport::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate2io12ImportExportE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::io::ImportExport",
        "message",
        "",
        "app::log::Severity",
        "severity",
        "progress_max_changed",
        "max",
        "progress",
        "value",
        "completed",
        "success",
        "can_handle",
        "glaxnimate::io::ImportExport::Direction",
        "direction",
        "can_handle_extension",
        "extension",
        "can_handle_filename",
        "filename",
        "save",
        "glaxnimate::model::Composition*",
        "comp",
        "QVariantMap",
        "setting_values",
        "load",
        "glaxnimate::model::Document*",
        "document",
        "data",
        "name_filter",
        "warning",
        "information",
        "error",
        "name",
        "slug",
        "extensions",
        "can_open",
        "can_save",
        "Direction",
        "Import",
        "Export"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'message'
        QtMocHelpers::SignalData<void(const QString &, app::log::Severity)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 1 }, { 0x80000000 | 3, 4 },
        }}),
        // Signal 'progress_max_changed'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'progress'
        QtMocHelpers::SignalData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Signal 'completed'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Method 'can_handle'
        QtMocHelpers::MethodData<bool(glaxnimate::io::ImportExport::Direction) const>(11, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Method 'can_handle_extension'
        QtMocHelpers::MethodData<bool(const QString &, glaxnimate::io::ImportExport::Direction) const>(14, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 15 }, { 0x80000000 | 12, 13 },
        }}),
        // Method 'can_handle_filename'
        QtMocHelpers::MethodData<bool(const QString &, glaxnimate::io::ImportExport::Direction) const>(16, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 17 }, { 0x80000000 | 12, 13 },
        }}),
        // Method 'save'
        QtMocHelpers::MethodData<QByteArray(glaxnimate::model::Composition *, const QVariantMap &, const QString &)>(18, 2, QMC::AccessPublic, QMetaType::QByteArray, {{
            { 0x80000000 | 19, 20 }, { 0x80000000 | 21, 22 }, { QMetaType::QString, 17 },
        }}),
        // Method 'save'
        QtMocHelpers::MethodData<QByteArray(glaxnimate::model::Composition *, const QVariantMap &)>(18, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QByteArray, {{
            { 0x80000000 | 19, 20 }, { 0x80000000 | 21, 22 },
        }}),
        // Method 'save'
        QtMocHelpers::MethodData<QByteArray(glaxnimate::model::Composition *)>(18, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QByteArray, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Method 'load'
        QtMocHelpers::MethodData<bool(glaxnimate::model::Document *, const QByteArray &, const QVariantMap &, const QString &)>(23, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 24, 25 }, { QMetaType::QByteArray, 26 }, { 0x80000000 | 21, 22 }, { QMetaType::QString, 17 },
        }}),
        // Method 'load'
        QtMocHelpers::MethodData<bool(glaxnimate::model::Document *, const QByteArray &, const QVariantMap &)>(23, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { 0x80000000 | 24, 25 }, { QMetaType::QByteArray, 26 }, { 0x80000000 | 21, 22 },
        }}),
        // Method 'load'
        QtMocHelpers::MethodData<bool(glaxnimate::model::Document *, const QByteArray &)>(23, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { 0x80000000 | 24, 25 }, { QMetaType::QByteArray, 26 },
        }}),
        // Method 'name_filter'
        QtMocHelpers::MethodData<QString() const>(27, 2, QMC::AccessPublic, QMetaType::QString),
        // Method 'warning'
        QtMocHelpers::MethodData<void(const QString &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 1 },
        }}),
        // Method 'information'
        QtMocHelpers::MethodData<void(const QString &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 1 },
        }}),
        // Method 'error'
        QtMocHelpers::MethodData<void(const QString &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 1 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'name'
        QtMocHelpers::PropertyData<QString>(31, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'slug'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'extensions'
        QtMocHelpers::PropertyData<QStringList>(33, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'can_open'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'can_save'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Direction'
        QtMocHelpers::EnumData<enum Direction>(36, 36, QMC::EnumFlags{}).add({
            {   37, Direction::Import },
            {   38, Direction::Export },
        }),
    };
    return QtMocHelpers::metaObjectData<ImportExport, qt_meta_tag_ZN10glaxnimate2io12ImportExportE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::io::ImportExport::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate2io12ImportExportE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate2io12ImportExportE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate2io12ImportExportE_t>.metaTypes,
    nullptr
} };

void glaxnimate::io::ImportExport::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ImportExport *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->message((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<app::log::Severity>>(_a[2]))); break;
        case 1: _t->progress_max_changed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->progress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->completed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: { bool _r = _t->can_handle((*reinterpret_cast<std::add_pointer_t<glaxnimate::io::ImportExport::Direction>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->can_handle_extension((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<glaxnimate::io::ImportExport::Direction>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->can_handle_filename((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<glaxnimate::io::ImportExport::Direction>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { QByteArray _r = _t->save((*reinterpret_cast<std::add_pointer_t<glaxnimate::model::Composition*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 8: { QByteArray _r = _t->save((*reinterpret_cast<std::add_pointer_t<glaxnimate::model::Composition*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 9: { QByteArray _r = _t->save((*reinterpret_cast<std::add_pointer_t<glaxnimate::model::Composition*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->load((*reinterpret_cast<std::add_pointer_t<glaxnimate::model::Document*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->load((*reinterpret_cast<std::add_pointer_t<glaxnimate::model::Document*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: { bool _r = _t->load((*reinterpret_cast<std::add_pointer_t<glaxnimate::model::Document*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: { QString _r = _t->name_filter();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->warning((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->information((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->error((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< app::log::Severity >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< glaxnimate::model::Composition* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< glaxnimate::model::Composition* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< glaxnimate::model::Composition* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< glaxnimate::model::Document* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< glaxnimate::model::Document* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< glaxnimate::model::Document* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ImportExport::*)(const QString & , app::log::Severity )>(_a, &ImportExport::message, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportExport::*)(int )>(_a, &ImportExport::progress_max_changed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportExport::*)(int )>(_a, &ImportExport::progress, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportExport::*)(bool )>(_a, &ImportExport::completed, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->slug(); break;
        case 2: *reinterpret_cast<QStringList*>(_v) = _t->extensions(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->can_open(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->can_save(); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::io::ImportExport::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::io::ImportExport::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate2io12ImportExportE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int glaxnimate::io::ImportExport::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::io::ImportExport::message(const QString & _t1, app::log::Severity _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void glaxnimate::io::ImportExport::progress_max_changed(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void glaxnimate::io::ImportExport::progress(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void glaxnimate::io::ImportExport::completed(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
