/********************************************************************************
** Form generated from reading UI file 'gradient_list_widget.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GRADIENT_LIST_WIDGET_H
#define UI_GRADIENT_LIST_WIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableView>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_GradientListWidget
{
public:
    QVBoxLayout *verticalLayout;
    QTableView *list_view;
    QHBoxLayout *horizontalLayout;
    QToolButton *btn_new;
    QToolButton *btn_preset;
    QSpacerItem *horizontalSpacer;
    QToolButton *btn_remove;
    QGridLayout *gridLayout;
    QLabel *label_2;
    QToolButton *btn_fill_radial;
    QToolButton *btn_fill_linear;
    QToolButton *btn_stroke_linear;
    QToolButton *btn_stroke_radial;
    QLabel *label;
    QToolButton *btn_fill_conical;
    QToolButton *btn_stroke_conical;

    void setupUi(QWidget *GradientListWidget)
    {
        if (GradientListWidget->objectName().isEmpty())
            GradientListWidget->setObjectName("GradientListWidget");
        GradientListWidget->resize(177, 353);
        verticalLayout = new QVBoxLayout(GradientListWidget);
        verticalLayout->setObjectName("verticalLayout");
        list_view = new QTableView(GradientListWidget);
        list_view->setObjectName("list_view");
        list_view->setSelectionMode(QAbstractItemView::SingleSelection);
        list_view->setSelectionBehavior(QAbstractItemView::SelectRows);
        list_view->horizontalHeader()->setVisible(true);
        list_view->verticalHeader()->setVisible(false);

        verticalLayout->addWidget(list_view);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        btn_new = new QToolButton(GradientListWidget);
        btn_new->setObjectName("btn_new");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("list-add");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_new->setIcon(icon);

        horizontalLayout->addWidget(btn_new);

        btn_preset = new QToolButton(GradientListWidget);
        btn_preset->setObjectName("btn_preset");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("folder");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_preset->setIcon(icon1);

        horizontalLayout->addWidget(btn_preset);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        btn_remove = new QToolButton(GradientListWidget);
        btn_remove->setObjectName("btn_remove");
        QIcon icon2;
        iconThemeName = QString::fromUtf8("list-remove");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_remove->setIcon(icon2);

        horizontalLayout->addWidget(btn_remove);


        verticalLayout->addLayout(horizontalLayout);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        label_2 = new QLabel(GradientListWidget);
        label_2->setObjectName("label_2");
        label_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_2, 1, 1, 1, 1);

        btn_fill_radial = new QToolButton(GradientListWidget);
        btn_fill_radial->setObjectName("btn_fill_radial");
        btn_fill_radial->setEnabled(false);
        QIcon icon3;
        iconThemeName = QString::fromUtf8("paint-gradient-radial");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_fill_radial->setIcon(icon3);
        btn_fill_radial->setCheckable(true);

        gridLayout->addWidget(btn_fill_radial, 0, 3, 1, 1);

        btn_fill_linear = new QToolButton(GradientListWidget);
        btn_fill_linear->setObjectName("btn_fill_linear");
        btn_fill_linear->setEnabled(false);
        QIcon icon4;
        iconThemeName = QString::fromUtf8("paint-gradient-linear");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon4 = QIcon::fromTheme(iconThemeName);
        } else {
            icon4.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_fill_linear->setIcon(icon4);
        btn_fill_linear->setCheckable(true);

        gridLayout->addWidget(btn_fill_linear, 0, 2, 1, 1);

        btn_stroke_linear = new QToolButton(GradientListWidget);
        btn_stroke_linear->setObjectName("btn_stroke_linear");
        btn_stroke_linear->setEnabled(false);
        btn_stroke_linear->setIcon(icon4);
        btn_stroke_linear->setCheckable(true);

        gridLayout->addWidget(btn_stroke_linear, 1, 2, 1, 1);

        btn_stroke_radial = new QToolButton(GradientListWidget);
        btn_stroke_radial->setObjectName("btn_stroke_radial");
        btn_stroke_radial->setEnabled(false);
        btn_stroke_radial->setIcon(icon3);
        btn_stroke_radial->setCheckable(true);

        gridLayout->addWidget(btn_stroke_radial, 1, 3, 1, 1);

        label = new QLabel(GradientListWidget);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label, 0, 1, 1, 1);

        btn_fill_conical = new QToolButton(GradientListWidget);
        btn_fill_conical->setObjectName("btn_fill_conical");
        btn_fill_conical->setEnabled(false);
        QIcon icon5(QIcon::fromTheme(QString::fromUtf8("paint-gradient-conical")));
        btn_fill_conical->setIcon(icon5);
        btn_fill_conical->setCheckable(true);

        gridLayout->addWidget(btn_fill_conical, 0, 4, 1, 1);

        btn_stroke_conical = new QToolButton(GradientListWidget);
        btn_stroke_conical->setObjectName("btn_stroke_conical");
        btn_stroke_conical->setEnabled(false);
        btn_stroke_conical->setIcon(icon5);
        btn_stroke_conical->setCheckable(true);

        gridLayout->addWidget(btn_stroke_conical, 1, 4, 1, 1);


        verticalLayout->addLayout(gridLayout);


        retranslateUi(GradientListWidget);
        QObject::connect(list_view, SIGNAL(clicked(QModelIndex)), GradientListWidget, SLOT(change_current_gradient()));
        QObject::connect(list_view, SIGNAL(entered(QModelIndex)), GradientListWidget, SLOT(change_current_gradient()));

        QMetaObject::connectSlotsByName(GradientListWidget);
    } // setupUi

    void retranslateUi(QWidget *GradientListWidget)
    {
#if QT_CONFIG(tooltip)
        btn_new->setToolTip(tr2i18n("Add Gradient", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_new->setText(tr2i18n("New", nullptr));
#if QT_CONFIG(tooltip)
        btn_preset->setToolTip(tr2i18n("Add from Preset", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_preset->setText(tr2i18n("Preset", nullptr));
#if QT_CONFIG(tooltip)
        btn_remove->setToolTip(tr2i18n("Remove Gradient", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_remove->setText(tr2i18n("Remove", nullptr));
        label_2->setText(tr2i18n("Stroke", nullptr));
#if QT_CONFIG(tooltip)
        btn_fill_radial->setToolTip(tr2i18n("Radial", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_fill_radial->setText(tr2i18n("Radial", nullptr));
#if QT_CONFIG(tooltip)
        btn_fill_linear->setToolTip(tr2i18n("Linear", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_fill_linear->setText(tr2i18n("Linear", nullptr));
#if QT_CONFIG(tooltip)
        btn_stroke_linear->setToolTip(tr2i18n("Linear", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_stroke_linear->setText(tr2i18n("Linear", nullptr));
#if QT_CONFIG(tooltip)
        btn_stroke_radial->setToolTip(tr2i18n("Radial", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_stroke_radial->setText(tr2i18n("Radial", nullptr));
        label->setText(tr2i18n("Fill", nullptr));
#if QT_CONFIG(tooltip)
        btn_fill_conical->setToolTip(tr2i18n("Conical", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_fill_conical->setText(tr2i18n("Conical", nullptr));
#if QT_CONFIG(tooltip)
        btn_stroke_conical->setToolTip(tr2i18n("Conical", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_stroke_conical->setText(tr2i18n("Conical", nullptr));
        (void)GradientListWidget;
    } // retranslateUi

};

namespace Ui {
    class GradientListWidget: public Ui_GradientListWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // GRADIENT_LIST_WIDGET_H

