/****************************************************************************
** Meta object code from reading C++ file 'bezier_item.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/gui/graphics/bezier_item.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bezier_item.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate3gui8graphics9PointItemE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::gui::graphics::PointItem::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate3gui8graphics9PointItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::gui::graphics::PointItem",
        "modified",
        "",
        "index",
        "math::bezier::Point",
        "point",
        "commit",
        "name",
        "tan_in_dragged",
        "QPointF",
        "p",
        "Qt::KeyboardModifiers",
        "mods",
        "tan_out_dragged",
        "pos_dragged",
        "on_modified",
        "on_commit"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'modified'
        QtMocHelpers::SignalData<void(int, const math::bezier::Point &, bool, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 4, 5 }, { QMetaType::Bool, 6 }, { QMetaType::QString, 7 },
        }}),
        // Slot 'tan_in_dragged'
        QtMocHelpers::SlotData<void(const QPointF &, Qt::KeyboardModifiers)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 11, 12 },
        }}),
        // Slot 'tan_out_dragged'
        QtMocHelpers::SlotData<void(const QPointF &, Qt::KeyboardModifiers)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 11, 12 },
        }}),
        // Slot 'pos_dragged'
        QtMocHelpers::SlotData<void(const QPointF &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'on_modified'
        QtMocHelpers::SlotData<void(bool, const QString &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 6 }, { QMetaType::QString, 7 },
        }}),
        // Slot 'on_modified'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'on_commit'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PointItem, qt_meta_tag_ZN10glaxnimate3gui8graphics9PointItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::gui::graphics::PointItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8graphics9PointItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8graphics9PointItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate3gui8graphics9PointItemE_t>.metaTypes,
    nullptr
} };

void glaxnimate::gui::graphics::PointItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PointItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->modified((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<math::bezier::Point>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 1: _t->tan_in_dragged((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[2]))); break;
        case 2: _t->tan_out_dragged((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[2]))); break;
        case 3: _t->pos_dragged((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 4: _t->on_modified((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 5: _t->on_modified((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->on_commit(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PointItem::*)(int , const math::bezier::Point & , bool , const QString & )>(_a, &PointItem::modified, 0))
            return;
    }
}

const QMetaObject *glaxnimate::gui::graphics::PointItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::gui::graphics::PointItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8graphics9PointItemE_t>.strings))
        return static_cast<void*>(this);
    return QGraphicsObject::qt_metacast(_clname);
}

int glaxnimate::gui::graphics::PointItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::gui::graphics::PointItem::modified(int _t1, const math::bezier::Point & _t2, bool _t3, const QString & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4);
}
namespace {
struct qt_meta_tag_ZN10glaxnimate3gui8graphics10BezierItemE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::gui::graphics::BezierItem::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate3gui8graphics10BezierItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::gui::graphics::BezierItem",
        "update_bezier",
        "",
        "math::bezier::Bezier",
        "bez",
        "set_bezier",
        "commit",
        "remove_point",
        "index",
        "make_first",
        "on_dragged",
        "math::bezier::Point",
        "point",
        "name",
        "refresh_from_position_property"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'update_bezier'
        QtMocHelpers::SlotData<void(const math::bezier::Bezier &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'set_bezier'
        QtMocHelpers::SlotData<void(const math::bezier::Bezier &, bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Bool, 6 },
        }}),
        // Slot 'set_bezier'
        QtMocHelpers::SlotData<void(const math::bezier::Bezier &)>(5, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'remove_point'
        QtMocHelpers::SlotData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'make_first'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'on_dragged'
        QtMocHelpers::SlotData<void(int, const math::bezier::Point &, bool, const QString &)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { 0x80000000 | 11, 12 }, { QMetaType::Bool, 6 }, { QMetaType::QString, 13 },
        }}),
        // Slot 'refresh_from_position_property'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BezierItem, qt_meta_tag_ZN10glaxnimate3gui8graphics10BezierItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::gui::graphics::BezierItem::staticMetaObject = { {
    QMetaObject::SuperData::link<TypedItem<Types::BezierItem>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8graphics10BezierItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8graphics10BezierItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate3gui8graphics10BezierItemE_t>.metaTypes,
    nullptr
} };

void glaxnimate::gui::graphics::BezierItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BezierItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->update_bezier((*reinterpret_cast<std::add_pointer_t<math::bezier::Bezier>>(_a[1]))); break;
        case 1: _t->set_bezier((*reinterpret_cast<std::add_pointer_t<math::bezier::Bezier>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 2: _t->set_bezier((*reinterpret_cast<std::add_pointer_t<math::bezier::Bezier>>(_a[1]))); break;
        case 3: _t->remove_point((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->make_first((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->on_dragged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<math::bezier::Point>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 6: _t->refresh_from_position_property(); break;
        default: ;
        }
    }
}

const QMetaObject *glaxnimate::gui::graphics::BezierItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::gui::graphics::BezierItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8graphics10BezierItemE_t>.strings))
        return static_cast<void*>(this);
    return TypedItem<Types::BezierItem>::qt_metacast(_clname);
}

int glaxnimate::gui::graphics::BezierItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TypedItem<Types::BezierItem>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
