/********************************************************************************
** Form generated from reading UI file 'resize_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RESIZE_DIALOG_H
#define UI_RESIZE_DIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

namespace glaxnimate {
namespace gui {

class Ui_ResizeDialog
{
public:
    QGridLayout *gridLayout;
    QLabel *label_2;
    QDialogButtonBox *buttonBox;
    QLabel *label;
    QVBoxLayout *verticalLayout;
    QPushButton *check_aspect;
    QCheckBox *check_scale_layers;
    QCheckBox *check_layer_ratio;
    QSpinBox *spin_width;
    QSpinBox *spin_height;

    void setupUi(QDialog *ResizeDialog)
    {
        if (ResizeDialog->objectName().isEmpty())
            ResizeDialog->setObjectName("ResizeDialog");
        ResizeDialog->resize(209, 178);
        gridLayout = new QGridLayout(ResizeDialog);
        gridLayout->setObjectName("gridLayout");
        label_2 = new QLabel(ResizeDialog);
        label_2->setObjectName("label_2");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy);

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        buttonBox = new QDialogButtonBox(ResizeDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout->addWidget(buttonBox, 4, 0, 1, 3);

        label = new QLabel(ResizeDialog);
        label->setObjectName("label");
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);

        gridLayout->addWidget(label, 0, 0, 1, 1);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        check_aspect = new QPushButton(ResizeDialog);
        check_aspect->setObjectName("check_aspect");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(check_aspect->sizePolicy().hasHeightForWidth());
        check_aspect->setSizePolicy(sizePolicy1);
        check_aspect->setText(QString::fromUtf8(""));
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("lock");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        check_aspect->setIcon(icon);
        check_aspect->setCheckable(true);
        check_aspect->setChecked(false);

        verticalLayout->addWidget(check_aspect);


        gridLayout->addLayout(verticalLayout, 0, 2, 2, 1);

        check_scale_layers = new QCheckBox(ResizeDialog);
        check_scale_layers->setObjectName("check_scale_layers");
        check_scale_layers->setChecked(true);

        gridLayout->addWidget(check_scale_layers, 2, 0, 1, 3);

        check_layer_ratio = new QCheckBox(ResizeDialog);
        check_layer_ratio->setObjectName("check_layer_ratio");
        check_layer_ratio->setChecked(true);

        gridLayout->addWidget(check_layer_ratio, 3, 0, 1, 3);

        spin_width = new QSpinBox(ResizeDialog);
        spin_width->setObjectName("spin_width");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(spin_width->sizePolicy().hasHeightForWidth());
        spin_width->setSizePolicy(sizePolicy2);
        spin_width->setMaximum(99999);

        gridLayout->addWidget(spin_width, 0, 1, 1, 1);

        spin_height = new QSpinBox(ResizeDialog);
        spin_height->setObjectName("spin_height");
        sizePolicy2.setHeightForWidth(spin_height->sizePolicy().hasHeightForWidth());
        spin_height->setSizePolicy(sizePolicy2);
        spin_height->setMaximum(99999);

        gridLayout->addWidget(spin_height, 1, 1, 1, 1);


        retranslateUi(ResizeDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, ResizeDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, ResizeDialog, qOverload<>(&QDialog::reject));
        QObject::connect(check_scale_layers, &QCheckBox::toggled, check_layer_ratio, &QCheckBox::setEnabled);
        QObject::connect(spin_height, SIGNAL(valueChanged(int)), ResizeDialog, SLOT(height_changed(int)));
        QObject::connect(spin_width, SIGNAL(valueChanged(int)), ResizeDialog, SLOT(width_changed(int)));
        QObject::connect(check_aspect, SIGNAL(clicked(bool)), ResizeDialog, SLOT(lock_changed(bool)));

        QMetaObject::connectSlotsByName(ResizeDialog);
    } // setupUi

    void retranslateUi(QDialog *ResizeDialog)
    {
        ResizeDialog->setWindowTitle(tr2i18n("Resize Document", nullptr));
        label_2->setText(tr2i18n("Height", nullptr));
        label->setText(tr2i18n("Width", nullptr));
#if QT_CONFIG(tooltip)
        check_aspect->setToolTip(tr2i18n("Lock Aspect Ratio", nullptr));
#endif // QT_CONFIG(tooltip)
        check_scale_layers->setText(tr2i18n("Scale Layers", nullptr));
        check_layer_ratio->setText(tr2i18n("Preserve Layer Aspect Ratio", nullptr));
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class ResizeDialog: public Ui_ResizeDialog {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // RESIZE_DIALOG_H

