/********************************************************************************
** Form generated from reading UI file 'font_style_widget.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FONT_STYLE_WIDGET_H
#define UI_FONT_STYLE_WIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "widgets/font/font_size_widget.hpp"

namespace glaxnimate {
namespace gui {
namespace font {

class Ui_FontStyleWidget
{
public:
    QGridLayout *gridLayout;
    QLabel *label;
    QLabel *label_2;
    QLabel *label_3;
    QLineEdit *edit_family;
    QVBoxLayout *verticalLayout;
    QListView *view_style;
    QGroupBox *groupBox;
    QFormLayout *formLayout;
    QLabel *label_4;
    QComboBox *combo_system;
    QLabel *label_5;
    QCheckBox *check_monospace;
    QLabel *label_6;
    QCheckBox *check_proportional;
    glaxnimate::gui::font::FontSizeWidget *size_widget;
    QTableView *view_family;

    void setupUi(QWidget *glaxnimate__gui__font__FontStyleWidget)
    {
        if (glaxnimate__gui__font__FontStyleWidget->objectName().isEmpty())
            glaxnimate__gui__font__FontStyleWidget->setObjectName("glaxnimate__gui__font__FontStyleWidget");
        glaxnimate__gui__font__FontStyleWidget->resize(704, 480);
        gridLayout = new QGridLayout(glaxnimate__gui__font__FontStyleWidget);
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(glaxnimate__gui__font__FontStyleWidget);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        label_2 = new QLabel(glaxnimate__gui__font__FontStyleWidget);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 0, 1, 1, 1);

        label_3 = new QLabel(glaxnimate__gui__font__FontStyleWidget);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 0, 2, 1, 1);

        edit_family = new QLineEdit(glaxnimate__gui__font__FontStyleWidget);
        edit_family->setObjectName("edit_family");

        gridLayout->addWidget(edit_family, 1, 0, 1, 1);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        view_style = new QListView(glaxnimate__gui__font__FontStyleWidget);
        view_style->setObjectName("view_style");

        verticalLayout->addWidget(view_style);

        groupBox = new QGroupBox(glaxnimate__gui__font__FontStyleWidget);
        groupBox->setObjectName("groupBox");
        formLayout = new QFormLayout(groupBox);
        formLayout->setObjectName("formLayout");
        label_4 = new QLabel(groupBox);
        label_4->setObjectName("label_4");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label_4);

        combo_system = new QComboBox(groupBox);
        combo_system->setObjectName("combo_system");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(combo_system->sizePolicy().hasHeightForWidth());
        combo_system->setSizePolicy(sizePolicy);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, combo_system);

        label_5 = new QLabel(groupBox);
        label_5->setObjectName("label_5");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label_5);

        check_monospace = new QCheckBox(groupBox);
        check_monospace->setObjectName("check_monospace");
        check_monospace->setText(QString::fromUtf8(""));
        check_monospace->setChecked(true);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, check_monospace);

        label_6 = new QLabel(groupBox);
        label_6->setObjectName("label_6");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, label_6);

        check_proportional = new QCheckBox(groupBox);
        check_proportional->setObjectName("check_proportional");
        check_proportional->setText(QString::fromUtf8(""));
        check_proportional->setChecked(true);

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, check_proportional);


        verticalLayout->addWidget(groupBox);


        gridLayout->addLayout(verticalLayout, 1, 1, 2, 1);

        size_widget = new glaxnimate::gui::font::FontSizeWidget(glaxnimate__gui__font__FontStyleWidget);
        size_widget->setObjectName("size_widget");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(size_widget->sizePolicy().hasHeightForWidth());
        size_widget->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(size_widget, 1, 2, 2, 1);

        view_family = new QTableView(glaxnimate__gui__font__FontStyleWidget);
        view_family->setObjectName("view_family");
        view_family->setSelectionMode(QAbstractItemView::SingleSelection);
        view_family->setSelectionBehavior(QAbstractItemView::SelectRows);
        view_family->setShowGrid(false);
        view_family->horizontalHeader()->setVisible(false);
        view_family->verticalHeader()->setVisible(false);

        gridLayout->addWidget(view_family, 2, 0, 1, 1);

        gridLayout->setColumnStretch(0, 20);
        gridLayout->setColumnStretch(1, 5);
        gridLayout->setColumnStretch(2, 5);
        QWidget::setTabOrder(edit_family, view_family);
        QWidget::setTabOrder(view_family, view_style);
        QWidget::setTabOrder(view_style, combo_system);
        QWidget::setTabOrder(combo_system, check_monospace);
        QWidget::setTabOrder(check_monospace, check_proportional);

        retranslateUi(glaxnimate__gui__font__FontStyleWidget);
        QObject::connect(edit_family, SIGNAL(textEdited(QString)), glaxnimate__gui__font__FontStyleWidget, SLOT(family_edited(QString)));
        QObject::connect(combo_system, SIGNAL(activated(int)), glaxnimate__gui__font__FontStyleWidget, SLOT(system_changed(int)));
        QObject::connect(check_monospace, SIGNAL(toggled(bool)), glaxnimate__gui__font__FontStyleWidget, SLOT(filter_flags_changed()));
        QObject::connect(check_proportional, SIGNAL(toggled(bool)), glaxnimate__gui__font__FontStyleWidget, SLOT(filter_flags_changed()));
        QObject::connect(view_family, SIGNAL(clicked(QModelIndex)), glaxnimate__gui__font__FontStyleWidget, SLOT(family_clicked(QModelIndex)));

        QMetaObject::connectSlotsByName(glaxnimate__gui__font__FontStyleWidget);
    } // setupUi

    void retranslateUi(QWidget *glaxnimate__gui__font__FontStyleWidget)
    {
        label->setText(tr2i18n("Font", nullptr));
        label_2->setText(tr2i18n("Font Style", nullptr));
        label_3->setText(tr2i18n("Size", nullptr));
        groupBox->setTitle(tr2i18n("Filters", nullptr));
        label_4->setText(tr2i18n("Writing System", nullptr));
        label_5->setText(tr2i18n("Monospace", nullptr));
        label_6->setText(tr2i18n("Proportional", nullptr));
        (void)glaxnimate__gui__font__FontStyleWidget;
    } // retranslateUi

};

} // namespace font
} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace font {
namespace Ui {
    class FontStyleWidget: public Ui_FontStyleWidget {};
} // namespace Ui
} // namespace font
} // namespace gui
} // namespace glaxnimate

#endif // FONT_STYLE_WIDGET_H

