/****************************************************************************
** Meta object code from reading C++ file 'swatch.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/external/Qt-Color-Widgets/include/QtColorWidgets/swatch.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'swatch.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13color_widgets6SwatchE_t {};
} // unnamed namespace

template <> constexpr inline auto color_widgets::Swatch::qt_create_metaobjectdata<qt_meta_tag_ZN13color_widgets6SwatchE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "color_widgets::Swatch",
        "paletteChanged",
        "",
        "ColorPalette",
        "palette",
        "selectedChanged",
        "selected",
        "colorSelected",
        "QColor",
        "color",
        "colorSizeChanged",
        "QSize",
        "colorSize",
        "maxColorSizeChanged",
        "colorSizePolicyChanged",
        "ColorSizePolicy",
        "colorSizePolicy",
        "doubleClicked",
        "index",
        "Qt::KeyboardModifiers",
        "modifiers",
        "rightClicked",
        "clicked",
        "forcedRowsChanged",
        "forcedRows",
        "forcedColumnsChanged",
        "forcedColumns",
        "readOnlyChanged",
        "readOnly",
        "borderChanged",
        "QPen",
        "border",
        "showClearColorChanged",
        "show",
        "setPalette",
        "setSelected",
        "setSelectedColor",
        "clearSelection",
        "setColorSize",
        "setMaxColorSize",
        "setColorSizePolicy",
        "setBorder",
        "setSelectionPen",
        "setForcedRows",
        "setForcedColumns",
        "setReadOnly",
        "removeSelected",
        "setShowClearColor",
        "paletteModified",
        "selection",
        "maxColorSize",
        "showClearColor",
        "Hint",
        "Minimum",
        "Fixed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'paletteChanged'
        QtMocHelpers::SignalData<void(const ColorPalette &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'selectedChanged'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'colorSelected'
        QtMocHelpers::SignalData<void(const QColor &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'colorSizeChanged'
        QtMocHelpers::SignalData<void(const QSize &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'maxColorSizeChanged'
        QtMocHelpers::SignalData<void(const QSize &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'colorSizePolicyChanged'
        QtMocHelpers::SignalData<void(enum ColorSizePolicy)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'doubleClicked'
        QtMocHelpers::SignalData<void(int, Qt::KeyboardModifiers)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { 0x80000000 | 19, 20 },
        }}),
        // Signal 'rightClicked'
        QtMocHelpers::SignalData<void(int, Qt::KeyboardModifiers)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { 0x80000000 | 19, 20 },
        }}),
        // Signal 'clicked'
        QtMocHelpers::SignalData<void(int, Qt::KeyboardModifiers)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { 0x80000000 | 19, 20 },
        }}),
        // Signal 'forcedRowsChanged'
        QtMocHelpers::SignalData<void(int)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Signal 'forcedColumnsChanged'
        QtMocHelpers::SignalData<void(int)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 26 },
        }}),
        // Signal 'readOnlyChanged'
        QtMocHelpers::SignalData<void(bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 28 },
        }}),
        // Signal 'borderChanged'
        QtMocHelpers::SignalData<void(const QPen &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Signal 'showClearColorChanged'
        QtMocHelpers::SignalData<void(bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 33 },
        }}),
        // Slot 'setPalette'
        QtMocHelpers::SlotData<void(const ColorPalette &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setSelected'
        QtMocHelpers::SlotData<void(int)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'setSelectedColor'
        QtMocHelpers::SlotData<bool(const QColor &)>(36, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'clearSelection'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setColorSize'
        QtMocHelpers::SlotData<void(const QSize &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'setMaxColorSize'
        QtMocHelpers::SlotData<void(const QSize &)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'setColorSizePolicy'
        QtMocHelpers::SlotData<void(enum ColorSizePolicy)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'setBorder'
        QtMocHelpers::SlotData<void(const QPen &)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Slot 'setSelectionPen'
        QtMocHelpers::SlotData<void(const QPen &)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 6 },
        }}),
        // Slot 'setForcedRows'
        QtMocHelpers::SlotData<void(int)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'setForcedColumns'
        QtMocHelpers::SlotData<void(int)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 26 },
        }}),
        // Slot 'setReadOnly'
        QtMocHelpers::SlotData<void(bool)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 28 },
        }}),
        // Slot 'removeSelected'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setShowClearColor'
        QtMocHelpers::SlotData<void(bool)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 33 },
        }}),
        // Slot 'paletteModified'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'palette'
        QtMocHelpers::PropertyData<ColorPalette>(4, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'selected'
        QtMocHelpers::PropertyData<int>(6, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'colorSize'
        QtMocHelpers::PropertyData<QSize>(12, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'colorSizePolicy'
        QtMocHelpers::PropertyData<enum ColorSizePolicy>(16, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'border'
        QtMocHelpers::PropertyData<QPen>(31, 0x80000000 | 30, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 12),
        // property 'selection'
        QtMocHelpers::PropertyData<QPen>(49, 0x80000000 | 30, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'forcedRows'
        QtMocHelpers::PropertyData<int>(24, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'forcedColumns'
        QtMocHelpers::PropertyData<int>(26, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'readOnly'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'maxColorSize'
        QtMocHelpers::PropertyData<QSize>(50, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'showClearColor'
        QtMocHelpers::PropertyData<bool>(51, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ColorSizePolicy'
        QtMocHelpers::EnumData<enum ColorSizePolicy>(15, 15, QMC::EnumFlags{}).add({
            {   52, ColorSizePolicy::Hint },
            {   53, ColorSizePolicy::Minimum },
            {   54, ColorSizePolicy::Fixed },
        }),
    };
    return QtMocHelpers::metaObjectData<Swatch, qt_meta_tag_ZN13color_widgets6SwatchE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject color_widgets::Swatch::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets6SwatchE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets6SwatchE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13color_widgets6SwatchE_t>.metaTypes,
    nullptr
} };

void color_widgets::Swatch::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Swatch *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->paletteChanged((*reinterpret_cast<std::add_pointer_t<ColorPalette>>(_a[1]))); break;
        case 1: _t->selectedChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->colorSelected((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 3: _t->colorSizeChanged((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 4: _t->maxColorSizeChanged((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 5: _t->colorSizePolicyChanged((*reinterpret_cast<std::add_pointer_t<enum ColorSizePolicy>>(_a[1]))); break;
        case 6: _t->doubleClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[2]))); break;
        case 7: _t->rightClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[2]))); break;
        case 8: _t->clicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[2]))); break;
        case 9: _t->forcedRowsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->forcedColumnsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->readOnlyChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->borderChanged((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 13: _t->showClearColorChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->setPalette((*reinterpret_cast<std::add_pointer_t<ColorPalette>>(_a[1]))); break;
        case 15: _t->setSelected((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: { bool _r = _t->setSelectedColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->clearSelection(); break;
        case 18: _t->setColorSize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 19: _t->setMaxColorSize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 20: _t->setColorSizePolicy((*reinterpret_cast<std::add_pointer_t<enum ColorSizePolicy>>(_a[1]))); break;
        case 21: _t->setBorder((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 22: _t->setSelectionPen((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 23: _t->setForcedRows((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 24: _t->setForcedColumns((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 25: _t->setReadOnly((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 26: _t->removeSelected(); break;
        case 27: _t->setShowClearColor((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 28: _t->paletteModified(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Swatch::*)(const ColorPalette & )>(_a, &Swatch::paletteChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Swatch::*)(int )>(_a, &Swatch::selectedChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Swatch::*)(const QColor & )>(_a, &Swatch::colorSelected, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Swatch::*)(const QSize & )>(_a, &Swatch::colorSizeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Swatch::*)(const QSize & )>(_a, &Swatch::maxColorSizeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Swatch::*)(ColorSizePolicy )>(_a, &Swatch::colorSizePolicyChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Swatch::*)(int , Qt::KeyboardModifiers )>(_a, &Swatch::doubleClicked, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Swatch::*)(int , Qt::KeyboardModifiers )>(_a, &Swatch::rightClicked, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Swatch::*)(int , Qt::KeyboardModifiers )>(_a, &Swatch::clicked, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Swatch::*)(int )>(_a, &Swatch::forcedRowsChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Swatch::*)(int )>(_a, &Swatch::forcedColumnsChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Swatch::*)(bool )>(_a, &Swatch::readOnlyChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Swatch::*)(const QPen & )>(_a, &Swatch::borderChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Swatch::*)(bool )>(_a, &Swatch::showClearColorChanged, 13))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<ColorPalette*>(_v) = _t->palette(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->selected(); break;
        case 2: *reinterpret_cast<QSize*>(_v) = _t->colorSize(); break;
        case 3: *reinterpret_cast<enum ColorSizePolicy*>(_v) = _t->colorSizePolicy(); break;
        case 4: *reinterpret_cast<QPen*>(_v) = _t->border(); break;
        case 5: *reinterpret_cast<QPen*>(_v) = _t->selectionPen(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->forcedRows(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->forcedColumns(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->readOnly(); break;
        case 9: *reinterpret_cast<QSize*>(_v) = _t->maxColorSize(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->showClearColor(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPalette(*reinterpret_cast<ColorPalette*>(_v)); break;
        case 1: _t->setSelected(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setColorSize(*reinterpret_cast<QSize*>(_v)); break;
        case 3: _t->setColorSizePolicy(*reinterpret_cast<enum ColorSizePolicy*>(_v)); break;
        case 4: _t->setBorder(*reinterpret_cast<QPen*>(_v)); break;
        case 5: _t->setSelectionPen(*reinterpret_cast<QPen*>(_v)); break;
        case 6: _t->setForcedRows(*reinterpret_cast<int*>(_v)); break;
        case 7: _t->setForcedColumns(*reinterpret_cast<int*>(_v)); break;
        case 8: _t->setReadOnly(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->setMaxColorSize(*reinterpret_cast<QSize*>(_v)); break;
        case 10: _t->setShowClearColor(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *color_widgets::Swatch::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *color_widgets::Swatch::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets6SwatchE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int color_widgets::Swatch::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 29)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 29;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void color_widgets::Swatch::paletteChanged(const ColorPalette & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void color_widgets::Swatch::selectedChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void color_widgets::Swatch::colorSelected(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void color_widgets::Swatch::colorSizeChanged(const QSize & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void color_widgets::Swatch::maxColorSizeChanged(const QSize & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void color_widgets::Swatch::colorSizePolicyChanged(ColorSizePolicy _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void color_widgets::Swatch::doubleClicked(int _t1, Qt::KeyboardModifiers _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void color_widgets::Swatch::rightClicked(int _t1, Qt::KeyboardModifiers _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void color_widgets::Swatch::clicked(int _t1, Qt::KeyboardModifiers _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void color_widgets::Swatch::forcedRowsChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void color_widgets::Swatch::forcedColumnsChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void color_widgets::Swatch::readOnlyChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void color_widgets::Swatch::borderChanged(const QPen & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void color_widgets::Swatch::showClearColorChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}
QT_WARNING_POP
