/********************************************************************************
** Form generated from reading UI file 'time_slider.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TIME_SLIDER_H
#define UI_TIME_SLIDER_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDockWidget>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QSlider>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "widgets/timeline/frame_controls_widget.hpp"

QT_BEGIN_NAMESPACE

class Ui_dock_time_slider
{
public:
    QWidget *dockWidgetContents;
    QHBoxLayout *horizontalLayout;
    glaxnimate::gui::FrameControlsWidget *play_controls_2;
    QSlider *scroll_time;

    void setupUi(QDockWidget *dock_time_slider)
    {
        if (dock_time_slider->objectName().isEmpty())
            dock_time_slider->setObjectName("dock_time_slider");
        dock_time_slider->resize(400, 300);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("player-time")));
        dock_time_slider->setWindowIcon(icon);
        dockWidgetContents = new QWidget();
        dockWidgetContents->setObjectName("dockWidgetContents");
        horizontalLayout = new QHBoxLayout(dockWidgetContents);
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        play_controls_2 = new glaxnimate::gui::FrameControlsWidget(dockWidgetContents);
        play_controls_2->setObjectName("play_controls_2");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(play_controls_2->sizePolicy().hasHeightForWidth());
        play_controls_2->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(play_controls_2);

        scroll_time = new QSlider(dockWidgetContents);
        scroll_time->setObjectName("scroll_time");
        scroll_time->setOrientation(Qt::Orientation::Horizontal);

        horizontalLayout->addWidget(scroll_time);

        dock_time_slider->setWidget(dockWidgetContents);

        retranslateUi(dock_time_slider);

        QMetaObject::connectSlotsByName(dock_time_slider);
    } // setupUi

    void retranslateUi(QDockWidget *dock_time_slider)
    {
        dock_time_slider->setWindowTitle(tr2i18n("Compact Timeline", nullptr));
    } // retranslateUi

};

namespace Ui {
    class dock_time_slider: public Ui_dock_time_slider {};
} // namespace Ui

QT_END_NAMESPACE

#endif // TIME_SLIDER_H

