/********************************************************************************
** Form generated from reading UI file 'keyframe_editor_widget.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KEYFRAME_EDITOR_WIDGET_H
#define UI_KEYFRAME_EDITOR_WIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "widgets/timeline/keyframe_transition_widget.hpp"

namespace glaxnimate {
namespace gui {

class Ui_KeyframeEditorWidget
{
public:
    QVBoxLayout *verticalLayout;
    glaxnimate::gui::KeyframeTransitionWidget *bezier_editor;
    QGroupBox *groupBox;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QComboBox *combo_before;
    QSpacerItem *horizontalSpacer;
    QLabel *label_2;
    QComboBox *combo_after;

    void setupUi(QWidget *glaxnimate__gui__KeyframeEditorWidget)
    {
        if (glaxnimate__gui__KeyframeEditorWidget->objectName().isEmpty())
            glaxnimate__gui__KeyframeEditorWidget->setObjectName("glaxnimate__gui__KeyframeEditorWidget");
        glaxnimate__gui__KeyframeEditorWidget->resize(618, 447);
        verticalLayout = new QVBoxLayout(glaxnimate__gui__KeyframeEditorWidget);
        verticalLayout->setObjectName("verticalLayout");
        bezier_editor = new glaxnimate::gui::KeyframeTransitionWidget(glaxnimate__gui__KeyframeEditorWidget);
        bezier_editor->setObjectName("bezier_editor");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(bezier_editor->sizePolicy().hasHeightForWidth());
        bezier_editor->setSizePolicy(sizePolicy);
        bezier_editor->setFocusPolicy(Qt::StrongFocus);

        verticalLayout->addWidget(bezier_editor);

        groupBox = new QGroupBox(glaxnimate__gui__KeyframeEditorWidget);
        groupBox->setObjectName("groupBox");
        horizontalLayout = new QHBoxLayout(groupBox);
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(groupBox);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        combo_before = new QComboBox(groupBox);
        combo_before->setObjectName("combo_before");

        horizontalLayout->addWidget(combo_before);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        label_2 = new QLabel(groupBox);
        label_2->setObjectName("label_2");

        horizontalLayout->addWidget(label_2);

        combo_after = new QComboBox(groupBox);
        combo_after->setObjectName("combo_after");

        horizontalLayout->addWidget(combo_after);


        verticalLayout->addWidget(groupBox);


        retranslateUi(glaxnimate__gui__KeyframeEditorWidget);
        QObject::connect(combo_before, SIGNAL(activated(int)), glaxnimate__gui__KeyframeEditorWidget, SLOT(preset_before(int)));
        QObject::connect(combo_after, SIGNAL(activated(int)), glaxnimate__gui__KeyframeEditorWidget, SLOT(preset_after(int)));

        QMetaObject::connectSlotsByName(glaxnimate__gui__KeyframeEditorWidget);
    } // setupUi

    void retranslateUi(QWidget *glaxnimate__gui__KeyframeEditorWidget)
    {
        groupBox->setTitle(tr2i18n("Presets", nullptr));
        label->setText(tr2i18n("Start", nullptr));
        label_2->setText(tr2i18n("Finish", nullptr));
        (void)glaxnimate__gui__KeyframeEditorWidget;
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class KeyframeEditorWidget: public Ui_KeyframeEditorWidget {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // KEYFRAME_EDITOR_WIDGET_H

