/********************************************************************************
** Form generated from reading UI file 'snippet_list_widget.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SNIPPET_LIST_WIDGET_H
#define UI_SNIPPET_LIST_WIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QListView>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace glaxnimate {
namespace gui {

class Ui_SnippetListWidget
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QToolButton *toolButton;
    QToolButton *toolButton_2;
    QToolButton *toolButton_3;
    QToolButton *toolButton_4;
    QSpacerItem *horizontalSpacer;
    QToolButton *toolButton_5;
    QListView *list_view;

    void setupUi(QWidget *SnippetListWidget)
    {
        if (SnippetListWidget->objectName().isEmpty())
            SnippetListWidget->setObjectName("SnippetListWidget");
        SnippetListWidget->resize(400, 300);
        verticalLayout = new QVBoxLayout(SnippetListWidget);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        toolButton = new QToolButton(SnippetListWidget);
        toolButton->setObjectName("toolButton");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("document-new");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        toolButton->setIcon(icon);

        horizontalLayout->addWidget(toolButton);

        toolButton_2 = new QToolButton(SnippetListWidget);
        toolButton_2->setObjectName("toolButton_2");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("document-edit");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        toolButton_2->setIcon(icon1);

        horizontalLayout->addWidget(toolButton_2);

        toolButton_3 = new QToolButton(SnippetListWidget);
        toolButton_3->setObjectName("toolButton_3");
        QIcon icon2;
        iconThemeName = QString::fromUtf8("edit-delete");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        toolButton_3->setIcon(icon2);

        horizontalLayout->addWidget(toolButton_3);

        toolButton_4 = new QToolButton(SnippetListWidget);
        toolButton_4->setObjectName("toolButton_4");
        QIcon icon3;
        iconThemeName = QString::fromUtf8("run-build");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        toolButton_4->setIcon(icon3);

        horizontalLayout->addWidget(toolButton_4);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        toolButton_5 = new QToolButton(SnippetListWidget);
        toolButton_5->setObjectName("toolButton_5");
        QIcon icon4(QIcon::fromTheme(QString::fromUtf8("view-refresh")));
        toolButton_5->setIcon(icon4);

        horizontalLayout->addWidget(toolButton_5);


        verticalLayout->addLayout(horizontalLayout);

        list_view = new QListView(SnippetListWidget);
        list_view->setObjectName("list_view");

        verticalLayout->addWidget(list_view);


        retranslateUi(SnippetListWidget);
        QObject::connect(toolButton, SIGNAL(clicked()), SnippetListWidget, SLOT(snippet_new()));
        QObject::connect(toolButton_2, SIGNAL(clicked()), SnippetListWidget, SLOT(snippet_edit()));
        QObject::connect(toolButton_3, SIGNAL(clicked()), SnippetListWidget, SLOT(snippet_delete()));
        QObject::connect(toolButton_4, SIGNAL(clicked()), SnippetListWidget, SLOT(snippet_run()));
        QObject::connect(toolButton_5, SIGNAL(clicked()), SnippetListWidget, SLOT(snippet_reload()));

        QMetaObject::connectSlotsByName(SnippetListWidget);
    } // setupUi

    void retranslateUi(QWidget *SnippetListWidget)
    {
#if QT_CONFIG(tooltip)
        toolButton->setToolTip(tr2i18n("New Snippet", nullptr));
#endif // QT_CONFIG(tooltip)
        toolButton->setText(tr2i18n("New", nullptr));
#if QT_CONFIG(tooltip)
        toolButton_2->setToolTip(tr2i18n("Edit the selected snippet", nullptr));
#endif // QT_CONFIG(tooltip)
        toolButton_2->setText(tr2i18n("Edit...", nullptr));
#if QT_CONFIG(tooltip)
        toolButton_3->setToolTip(tr2i18n("Delete selected snippet", nullptr));
#endif // QT_CONFIG(tooltip)
        toolButton_3->setText(tr2i18n("Delete", nullptr));
#if QT_CONFIG(tooltip)
        toolButton_4->setToolTip(tr2i18n("Run selected snippet", nullptr));
#endif // QT_CONFIG(tooltip)
        toolButton_4->setText(tr2i18n("Run", nullptr));
#if QT_CONFIG(tooltip)
        toolButton_5->setToolTip(tr2i18n("Reload snippets", nullptr));
#endif // QT_CONFIG(tooltip)
        toolButton_5->setText(tr2i18n("Reload", nullptr));
        (void)SnippetListWidget;
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class SnippetListWidget: public Ui_SnippetListWidget {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // SNIPPET_LIST_WIDGET_H

