/********************************************************************************
** Form generated from reading UI file 'timing_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TIMING_DIALOG_H
#define UI_TIMING_DIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_TimingDialog
{
public:
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout;
    QLabel *label_5;
    QDoubleSpinBox *spin_fps;
    QLabel *label_3;
    QDoubleSpinBox *spin_seconds;
    QLabel *label_4;
    QSpinBox *spin_frames;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QRadioButton *check_layer_keep;
    QRadioButton *check_layer_trim;
    QRadioButton *check_layer_scale;
    QDialogButtonBox *button_box;

    void setupUi(QDialog *TimingDialog)
    {
        if (TimingDialog->objectName().isEmpty())
            TimingDialog->setObjectName("TimingDialog");
        TimingDialog->resize(276, 288);
        verticalLayout = new QVBoxLayout(TimingDialog);
        verticalLayout->setObjectName("verticalLayout");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        label_5 = new QLabel(TimingDialog);
        label_5->setObjectName("label_5");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label_5);

        spin_fps = new QDoubleSpinBox(TimingDialog);
        spin_fps->setObjectName("spin_fps");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(spin_fps->sizePolicy().hasHeightForWidth());
        spin_fps->setSizePolicy(sizePolicy);
        spin_fps->setMinimum(1.000000000000000);
        spin_fps->setMaximum(999.990000000000009);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, spin_fps);

        label_3 = new QLabel(TimingDialog);
        label_3->setObjectName("label_3");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label_3);

        spin_seconds = new QDoubleSpinBox(TimingDialog);
        spin_seconds->setObjectName("spin_seconds");
        sizePolicy.setHeightForWidth(spin_seconds->sizePolicy().hasHeightForWidth());
        spin_seconds->setSizePolicy(sizePolicy);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, spin_seconds);

        label_4 = new QLabel(TimingDialog);
        label_4->setObjectName("label_4");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, label_4);

        spin_frames = new QSpinBox(TimingDialog);
        spin_frames->setObjectName("spin_frames");
        sizePolicy.setHeightForWidth(spin_frames->sizePolicy().hasHeightForWidth());
        spin_frames->setSizePolicy(sizePolicy);

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, spin_frames);


        verticalLayout->addLayout(formLayout);

        groupBox = new QGroupBox(TimingDialog);
        groupBox->setObjectName("groupBox");
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        check_layer_keep = new QRadioButton(groupBox);
        check_layer_keep->setObjectName("check_layer_keep");

        verticalLayout_2->addWidget(check_layer_keep);

        check_layer_trim = new QRadioButton(groupBox);
        check_layer_trim->setObjectName("check_layer_trim");
        check_layer_trim->setChecked(true);

        verticalLayout_2->addWidget(check_layer_trim);

        check_layer_scale = new QRadioButton(groupBox);
        check_layer_scale->setObjectName("check_layer_scale");

        verticalLayout_2->addWidget(check_layer_scale);


        verticalLayout->addWidget(groupBox);

        button_box = new QDialogButtonBox(TimingDialog);
        button_box->setObjectName("button_box");
        button_box->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(button_box);


        retranslateUi(TimingDialog);
        QObject::connect(button_box, SIGNAL(clicked(QAbstractButton*)), TimingDialog, SLOT(btn_clicked(QAbstractButton*)));
        QObject::connect(spin_fps, SIGNAL(valueChanged(double)), TimingDialog, SLOT(changed_fps(double)));
        QObject::connect(spin_seconds, SIGNAL(valueChanged(double)), TimingDialog, SLOT(changed_seconds(double)));
        QObject::connect(spin_frames, SIGNAL(valueChanged(int)), TimingDialog, SLOT(changed_frames(int)));

        QMetaObject::connectSlotsByName(TimingDialog);
    } // setupUi

    void retranslateUi(QDialog *TimingDialog)
    {
        TimingDialog->setWindowTitle(tr2i18n("Timing", nullptr));
        label_5->setText(tr2i18n("FPS", nullptr));
        label_3->setText(tr2i18n("Duration", nullptr));
        spin_seconds->setSuffix(tr2i18n("\"", nullptr));
        label_4->setText(tr2i18n("Frames", nullptr));
        groupBox->setTitle(tr2i18n("Layers", nullptr));
        check_layer_keep->setText(tr2i18n("Keep Initial Timing", nullptr));
        check_layer_trim->setText(tr2i18n("Trim Extra Frames", nullptr));
        check_layer_scale->setText(tr2i18n("Scale Timing", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TimingDialog: public Ui_TimingDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // TIMING_DIALOG_H

