/********************************************************************************
** Form generated from reading UI file 'frame_controls_widget.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FRAME_CONTROLS_WIDGET_H
#define UI_FRAME_CONTROLS_WIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace glaxnimate {
namespace gui {

class Ui_FrameControlsWidget
{
public:
    QHBoxLayout *layout;
    QSpinBox *spin_frame;
    QToolButton *button_play;
    QToolButton *button_loop;
    QToolButton *button_first;
    QToolButton *button_prev_kf;
    QToolButton *button_prev;
    QToolButton *button_next;
    QToolButton *button_next_kf;
    QToolButton *button_last;
    QToolButton *button_record;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *FrameControlsWidget)
    {
        if (FrameControlsWidget->objectName().isEmpty())
            FrameControlsWidget->setObjectName("FrameControlsWidget");
        FrameControlsWidget->resize(383, 45);
        layout = new QHBoxLayout(FrameControlsWidget);
        layout->setObjectName("layout");
        spin_frame = new QSpinBox(FrameControlsWidget);
        spin_frame->setObjectName("spin_frame");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(spin_frame->sizePolicy().hasHeightForWidth());
        spin_frame->setSizePolicy(sizePolicy);
        spin_frame->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        layout->addWidget(spin_frame);

        button_play = new QToolButton(FrameControlsWidget);
        button_play->setObjectName("button_play");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("media-playback-start");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_play->setIcon(icon);
        button_play->setCheckable(true);

        layout->addWidget(button_play);

        button_loop = new QToolButton(FrameControlsWidget);
        button_loop->setObjectName("button_loop");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("media-playlist-repeat");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_loop->setIcon(icon1);
        button_loop->setCheckable(true);
        button_loop->setChecked(true);

        layout->addWidget(button_loop);

        button_first = new QToolButton(FrameControlsWidget);
        button_first->setObjectName("button_first");
        QIcon icon2;
        iconThemeName = QString::fromUtf8("go-first");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_first->setIcon(icon2);

        layout->addWidget(button_first);

        button_prev_kf = new QToolButton(FrameControlsWidget);
        button_prev_kf->setObjectName("button_prev_kf");
        button_prev_kf->setEnabled(false);
        QIcon icon3;
        iconThemeName = QString::fromUtf8("go-previous-skip");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_prev_kf->setIcon(icon3);

        layout->addWidget(button_prev_kf);

        button_prev = new QToolButton(FrameControlsWidget);
        button_prev->setObjectName("button_prev");
        QIcon icon4;
        iconThemeName = QString::fromUtf8("go-previous");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon4 = QIcon::fromTheme(iconThemeName);
        } else {
            icon4.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_prev->setIcon(icon4);

        layout->addWidget(button_prev);

        button_next = new QToolButton(FrameControlsWidget);
        button_next->setObjectName("button_next");
        QIcon icon5;
        iconThemeName = QString::fromUtf8("go-next");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon5 = QIcon::fromTheme(iconThemeName);
        } else {
            icon5.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_next->setIcon(icon5);

        layout->addWidget(button_next);

        button_next_kf = new QToolButton(FrameControlsWidget);
        button_next_kf->setObjectName("button_next_kf");
        button_next_kf->setEnabled(false);
        QIcon icon6;
        iconThemeName = QString::fromUtf8("go-next-skip");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon6 = QIcon::fromTheme(iconThemeName);
        } else {
            icon6.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_next_kf->setIcon(icon6);

        layout->addWidget(button_next_kf);

        button_last = new QToolButton(FrameControlsWidget);
        button_last->setObjectName("button_last");
        QIcon icon7;
        iconThemeName = QString::fromUtf8("go-last");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon7 = QIcon::fromTheme(iconThemeName);
        } else {
            icon7.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_last->setIcon(icon7);

        layout->addWidget(button_last);

        button_record = new QToolButton(FrameControlsWidget);
        button_record->setObjectName("button_record");
        QIcon icon8;
        iconThemeName = QString::fromUtf8("media-record");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon8 = QIcon::fromTheme(iconThemeName);
        } else {
            icon8.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_record->setIcon(icon8);
        button_record->setCheckable(true);

        layout->addWidget(button_record);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        layout->addItem(horizontalSpacer);


        retranslateUi(FrameControlsWidget);
        QObject::connect(button_play, SIGNAL(clicked(bool)), FrameControlsWidget, SLOT(play_toggled(bool)));
        QObject::connect(button_first, SIGNAL(clicked()), FrameControlsWidget, SLOT(go_first()));
        QObject::connect(button_prev, SIGNAL(clicked()), FrameControlsWidget, SLOT(go_prev()));
        QObject::connect(button_next, SIGNAL(clicked()), FrameControlsWidget, SLOT(go_next()));
        QObject::connect(button_last, SIGNAL(clicked()), FrameControlsWidget, SLOT(go_last()));
        QObject::connect(spin_frame, SIGNAL(valueChanged(int)), FrameControlsWidget, SLOT(commit_time()));

        QMetaObject::connectSlotsByName(FrameControlsWidget);
    } // setupUi

    void retranslateUi(QWidget *FrameControlsWidget)
    {
        FrameControlsWidget->setWindowTitle(QString());
#if QT_CONFIG(tooltip)
        spin_frame->setToolTip(tr2i18n("Frame number", nullptr));
#endif // QT_CONFIG(tooltip)
        spin_frame->setSuffix(tr2i18n(" f", nullptr));
        button_play->setText(tr2i18n("Play", nullptr));
#if QT_CONFIG(tooltip)
        button_loop->setToolTip(tr2i18n("Loop animation", nullptr));
#endif // QT_CONFIG(tooltip)
        button_loop->setText(tr2i18n("Loop", nullptr));
#if QT_CONFIG(tooltip)
        button_first->setToolTip(tr2i18n("Go To First Frame", nullptr));
#endif // QT_CONFIG(tooltip)
        button_first->setText(tr2i18n("Go To First Frame", nullptr));
#if QT_CONFIG(tooltip)
        button_prev_kf->setToolTip(tr2i18n("Go To Previous Keyframe", nullptr));
#endif // QT_CONFIG(tooltip)
        button_prev_kf->setText(tr2i18n("Go To Previous Keyframe", nullptr));
#if QT_CONFIG(tooltip)
        button_prev->setToolTip(tr2i18n("Go To Previous Frame", nullptr));
#endif // QT_CONFIG(tooltip)
        button_prev->setText(tr2i18n("Go To Previous Frame", nullptr));
#if QT_CONFIG(tooltip)
        button_next->setToolTip(tr2i18n("Go To Next Frame", nullptr));
#endif // QT_CONFIG(tooltip)
        button_next->setText(tr2i18n("Go To Next Frame", nullptr));
#if QT_CONFIG(tooltip)
        button_next_kf->setToolTip(tr2i18n("Go To Next Keyframe", nullptr));
#endif // QT_CONFIG(tooltip)
        button_next_kf->setText(tr2i18n("Go To Next Keyframe", nullptr));
#if QT_CONFIG(tooltip)
        button_last->setToolTip(tr2i18n("Go To Last Frame", nullptr));
#endif // QT_CONFIG(tooltip)
        button_last->setText(tr2i18n("Go To Last Frame", nullptr));
#if QT_CONFIG(tooltip)
        button_record->setToolTip(tr2i18n("Record", nullptr));
#endif // QT_CONFIG(tooltip)
        button_record->setText(tr2i18n("Record", nullptr));
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class FrameControlsWidget: public Ui_FrameControlsWidget {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // FRAME_CONTROLS_WIDGET_H

