/********************************************************************************
** Form generated from reading UI file 'undo.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UNDO_H
#define UI_UNDO_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDockWidget>
#include <QtWidgets/QUndoView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_dock_undo
{
public:
    QWidget *dockWidgetContents;
    QVBoxLayout *verticalLayout;
    QUndoView *view_undo;

    void setupUi(QDockWidget *dock_undo)
    {
        if (dock_undo->objectName().isEmpty())
            dock_undo->setObjectName("dock_undo");
        dock_undo->resize(400, 300);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("edit-undo-history")));
        dock_undo->setWindowIcon(icon);
        dockWidgetContents = new QWidget();
        dockWidgetContents->setObjectName("dockWidgetContents");
        verticalLayout = new QVBoxLayout(dockWidgetContents);
        verticalLayout->setObjectName("verticalLayout");
        view_undo = new QUndoView(dockWidgetContents);
        view_undo->setObjectName("view_undo");
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("document-save")));
        view_undo->setCleanIcon(icon1);

        verticalLayout->addWidget(view_undo);

        dock_undo->setWidget(dockWidgetContents);

        retranslateUi(dock_undo);

        QMetaObject::connectSlotsByName(dock_undo);
    } // setupUi

    void retranslateUi(QDockWidget *dock_undo)
    {
        dock_undo->setWindowTitle(tr2i18n("Undo History", nullptr));
    } // retranslateUi

};

namespace Ui {
    class dock_undo: public Ui_dock_undo {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UNDO_H

