/********************************************************************************
** Form generated from reading UI file 'color_picker_widget.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_COLOR_PICKER_WIDGET_H
#define UI_COLOR_PICKER_WIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "QtColorWidgets/color_preview.hpp"

namespace glaxnimate {
namespace gui {

class Ui_ColorPickerWidget
{
public:
    QVBoxLayout *verticalLayout;
    QRadioButton *check_fill;
    QRadioButton *check_stroke;
    color_widgets::ColorPreview *color_preview;

    void setupUi(QWidget *ColorPickerWidget)
    {
        if (ColorPickerWidget->objectName().isEmpty())
            ColorPickerWidget->setObjectName("ColorPickerWidget");
        ColorPickerWidget->resize(148, 173);
        verticalLayout = new QVBoxLayout(ColorPickerWidget);
        verticalLayout->setObjectName("verticalLayout");
        check_fill = new QRadioButton(ColorPickerWidget);
        check_fill->setObjectName("check_fill");
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("format-fill-color")));
        check_fill->setIcon(icon);
        check_fill->setChecked(true);

        verticalLayout->addWidget(check_fill);

        check_stroke = new QRadioButton(ColorPickerWidget);
        check_stroke->setObjectName("check_stroke");
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("format-stroke-color")));
        check_stroke->setIcon(icon1);

        verticalLayout->addWidget(check_stroke);

        color_preview = new color_widgets::ColorPreview(ColorPickerWidget);
        color_preview->setObjectName("color_preview");
        color_preview->setColor(QColor(0, 0, 0));

        verticalLayout->addWidget(color_preview);


        retranslateUi(ColorPickerWidget);

        QMetaObject::connectSlotsByName(ColorPickerWidget);
    } // setupUi

    void retranslateUi(QWidget *ColorPickerWidget)
    {
        check_fill->setText(tr2i18n("Fill", nullptr));
        check_stroke->setText(tr2i18n("Stroke", nullptr));
        (void)ColorPickerWidget;
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class ColorPickerWidget: public Ui_ColorPickerWidget {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // COLOR_PICKER_WIDGET_H

