/****************************************************************************
** Meta object code from reading C++ file 'color_wheel.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/external/Qt-Color-Widgets/include/QtColorWidgets/color_wheel.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'color_wheel.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13color_widgets10ColorWheelE_t {};
} // unnamed namespace

template <> constexpr inline auto color_widgets::ColorWheel::qt_create_metaobjectdata<qt_meta_tag_ZN13color_widgets10ColorWheelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "color_widgets::ColorWheel",
        "colorChanged",
        "",
        "QColor",
        "colorSelected",
        "wheelWidthChanged",
        "selectorShapeChanged",
        "ShapeEnum",
        "shape",
        "rotatingSelectorChanged",
        "rotating",
        "colorSpaceChanged",
        "ColorSpaceEnum",
        "space",
        "mirroredSelectorChanged",
        "mirrored",
        "editingFinished",
        "setColor",
        "c",
        "setHue",
        "h",
        "setSaturation",
        "s",
        "setValue",
        "v",
        "setSelectorShape",
        "setRotatingSelector",
        "setColorSpace",
        "setMirroredSelector",
        "color",
        "hue",
        "saturation",
        "value",
        "wheelWidth",
        "selectorShape",
        "rotatingSelector",
        "colorSpace",
        "mirroredSelector",
        "ShapeTriangle",
        "ShapeSquare",
        "AngleEnum",
        "AngleFixed",
        "AngleRotating",
        "ColorHSV",
        "ColorHSL",
        "ColorLCH"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(QColor)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'colorSelected'
        QtMocHelpers::SignalData<void(QColor)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'wheelWidthChanged'
        QtMocHelpers::SignalData<void(unsigned )>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 2 },
        }}),
        // Signal 'selectorShapeChanged'
        QtMocHelpers::SignalData<void(enum ShapeEnum)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'rotatingSelectorChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Signal 'colorSpaceChanged'
        QtMocHelpers::SignalData<void(enum ColorSpaceEnum)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'mirroredSelectorChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Signal 'editingFinished'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(QColor)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 18 },
        }}),
        // Slot 'setHue'
        QtMocHelpers::SlotData<void(qreal)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 20 },
        }}),
        // Slot 'setSaturation'
        QtMocHelpers::SlotData<void(qreal)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 22 },
        }}),
        // Slot 'setValue'
        QtMocHelpers::SlotData<void(qreal)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 24 },
        }}),
        // Slot 'setSelectorShape'
        QtMocHelpers::SlotData<void(enum ShapeEnum)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'setRotatingSelector'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'setColorSpace'
        QtMocHelpers::SlotData<void(enum ColorSpaceEnum)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'setMirroredSelector'
        QtMocHelpers::SlotData<void(bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(29, 0x80000000 | 3, QMC::Readable | QMC::Writable | QMC::Designable | QMC::Scriptable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'hue'
        QtMocHelpers::PropertyData<qreal>(30, QMetaType::QReal, QMC::Readable | QMC::Writable | QMC::Scriptable | QMC::Stored | QMC::StdCppSet),
        // property 'saturation'
        QtMocHelpers::PropertyData<qreal>(31, QMetaType::QReal, QMC::Readable | QMC::Writable | QMC::Scriptable | QMC::Stored | QMC::StdCppSet),
        // property 'value'
        QtMocHelpers::PropertyData<qreal>(32, QMetaType::QReal, QMC::Readable | QMC::Writable | QMC::Scriptable | QMC::Stored | QMC::StdCppSet),
        // property 'wheelWidth'
        QtMocHelpers::PropertyData<uint>(33, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'selectorShape'
        QtMocHelpers::PropertyData<enum ShapeEnum>(34, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'rotatingSelector'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'colorSpace'
        QtMocHelpers::PropertyData<enum ColorSpaceEnum>(36, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'mirroredSelector'
        QtMocHelpers::PropertyData<bool>(37, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ShapeEnum'
        QtMocHelpers::EnumData<enum ShapeEnum>(7, 7, QMC::EnumFlags{}).add({
            {   38, ShapeEnum::ShapeTriangle },
            {   39, ShapeEnum::ShapeSquare },
        }),
        // enum 'AngleEnum'
        QtMocHelpers::EnumData<enum AngleEnum>(40, 40, QMC::EnumFlags{}).add({
            {   41, AngleEnum::AngleFixed },
            {   42, AngleEnum::AngleRotating },
        }),
        // enum 'ColorSpaceEnum'
        QtMocHelpers::EnumData<enum ColorSpaceEnum>(12, 12, QMC::EnumFlags{}).add({
            {   43, ColorSpaceEnum::ColorHSV },
            {   44, ColorSpaceEnum::ColorHSL },
            {   45, ColorSpaceEnum::ColorLCH },
        }),
    };
    return QtMocHelpers::metaObjectData<ColorWheel, qt_meta_tag_ZN13color_widgets10ColorWheelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject color_widgets::ColorWheel::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets10ColorWheelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets10ColorWheelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13color_widgets10ColorWheelE_t>.metaTypes,
    nullptr
} };

void color_widgets::ColorWheel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ColorWheel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 1: _t->colorSelected((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 2: _t->wheelWidthChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 3: _t->selectorShapeChanged((*reinterpret_cast<std::add_pointer_t<enum ShapeEnum>>(_a[1]))); break;
        case 4: _t->rotatingSelectorChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->colorSpaceChanged((*reinterpret_cast<std::add_pointer_t<enum ColorSpaceEnum>>(_a[1]))); break;
        case 6: _t->mirroredSelectorChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->editingFinished(); break;
        case 8: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 9: _t->setHue((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 10: _t->setSaturation((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 11: _t->setValue((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 12: _t->setSelectorShape((*reinterpret_cast<std::add_pointer_t<enum ShapeEnum>>(_a[1]))); break;
        case 13: _t->setRotatingSelector((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->setColorSpace((*reinterpret_cast<std::add_pointer_t<enum ColorSpaceEnum>>(_a[1]))); break;
        case 15: _t->setMirroredSelector((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ColorWheel::*)(QColor )>(_a, &ColorWheel::colorChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorWheel::*)(QColor )>(_a, &ColorWheel::colorSelected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorWheel::*)(unsigned  )>(_a, &ColorWheel::wheelWidthChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorWheel::*)(ShapeEnum )>(_a, &ColorWheel::selectorShapeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorWheel::*)(bool )>(_a, &ColorWheel::rotatingSelectorChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorWheel::*)(ColorSpaceEnum )>(_a, &ColorWheel::colorSpaceChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorWheel::*)(bool )>(_a, &ColorWheel::mirroredSelectorChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorWheel::*)()>(_a, &ColorWheel::editingFinished, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QColor*>(_v) = _t->color(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->hue(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->saturation(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->value(); break;
        case 4: *reinterpret_cast<uint*>(_v) = _t->wheelWidth(); break;
        case 5: *reinterpret_cast<enum ShapeEnum*>(_v) = _t->selectorShape(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->rotatingSelector(); break;
        case 7: *reinterpret_cast<enum ColorSpaceEnum*>(_v) = _t->colorSpace(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->mirroredSelector(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setColor(*reinterpret_cast<QColor*>(_v)); break;
        case 1: _t->setHue(*reinterpret_cast<qreal*>(_v)); break;
        case 2: _t->setSaturation(*reinterpret_cast<qreal*>(_v)); break;
        case 3: _t->setValue(*reinterpret_cast<qreal*>(_v)); break;
        case 4: _t->setWheelWidth(*reinterpret_cast<uint*>(_v)); break;
        case 5: _t->setSelectorShape(*reinterpret_cast<enum ShapeEnum*>(_v)); break;
        case 6: _t->setRotatingSelector(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setColorSpace(*reinterpret_cast<enum ColorSpaceEnum*>(_v)); break;
        case 8: _t->setMirroredSelector(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *color_widgets::ColorWheel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *color_widgets::ColorWheel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets10ColorWheelE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int color_widgets::ColorWheel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void color_widgets::ColorWheel::colorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void color_widgets::ColorWheel::colorSelected(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void color_widgets::ColorWheel::wheelWidthChanged(unsigned  _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void color_widgets::ColorWheel::selectorShapeChanged(ShapeEnum _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void color_widgets::ColorWheel::rotatingSelectorChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void color_widgets::ColorWheel::colorSpaceChanged(ColorSpaceEnum _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void color_widgets::ColorWheel::mirroredSelectorChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void color_widgets::ColorWheel::editingFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
