/********************************************************************************
** Form generated from reading UI file 'view_transform_widget.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_VIEW_TRANSFORM_WIDGET_H
#define UI_VIEW_TRANSFORM_WIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace glaxnimate {
namespace gui {

class Ui_ViewTransformWidget
{
public:
    QHBoxLayout *horizontalLayout;
    QToolButton *btn_view_fit;
    QLabel *label;
    QDoubleSpinBox *spin_zoom;
    QToolButton *btn_zoom_in;
    QToolButton *btn_zoom_out;
    QToolButton *btn_zoom_reset;
    QLabel *label_2;
    QDoubleSpinBox *spin_angle;
    QToolButton *btn_angle_reset;
    QToolButton *btn_flip_view;

    void setupUi(QWidget *ViewTransformWidget)
    {
        if (ViewTransformWidget->objectName().isEmpty())
            ViewTransformWidget->setObjectName("ViewTransformWidget");
        ViewTransformWidget->resize(429, 32);
        horizontalLayout = new QHBoxLayout(ViewTransformWidget);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        btn_view_fit = new QToolButton(ViewTransformWidget);
        btn_view_fit->setObjectName("btn_view_fit");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("zoom-fit-best");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_view_fit->setIcon(icon);

        horizontalLayout->addWidget(btn_view_fit);

        label = new QLabel(ViewTransformWidget);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        spin_zoom = new QDoubleSpinBox(ViewTransformWidget);
        spin_zoom->setObjectName("spin_zoom");
        spin_zoom->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        spin_zoom->setDecimals(0);
        spin_zoom->setMinimum(1.000000000000000);
        spin_zoom->setMaximum(10000.000000000000000);
        spin_zoom->setValue(100.000000000000000);

        horizontalLayout->addWidget(spin_zoom);

        btn_zoom_in = new QToolButton(ViewTransformWidget);
        btn_zoom_in->setObjectName("btn_zoom_in");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("zoom-in");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_zoom_in->setIcon(icon1);

        horizontalLayout->addWidget(btn_zoom_in);

        btn_zoom_out = new QToolButton(ViewTransformWidget);
        btn_zoom_out->setObjectName("btn_zoom_out");
        QIcon icon2;
        iconThemeName = QString::fromUtf8("zoom-out");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_zoom_out->setIcon(icon2);

        horizontalLayout->addWidget(btn_zoom_out);

        btn_zoom_reset = new QToolButton(ViewTransformWidget);
        btn_zoom_reset->setObjectName("btn_zoom_reset");
        QIcon icon3;
        iconThemeName = QString::fromUtf8("zoom-original");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_zoom_reset->setIcon(icon3);

        horizontalLayout->addWidget(btn_zoom_reset);

        label_2 = new QLabel(ViewTransformWidget);
        label_2->setObjectName("label_2");

        horizontalLayout->addWidget(label_2);

        spin_angle = new QDoubleSpinBox(ViewTransformWidget);
        spin_angle->setObjectName("spin_angle");
        spin_angle->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        spin_angle->setDecimals(0);
        spin_angle->setMinimum(0.000000000000000);
        spin_angle->setMaximum(359.000000000000000);
        spin_angle->setValue(0.000000000000000);

        horizontalLayout->addWidget(spin_angle);

        btn_angle_reset = new QToolButton(ViewTransformWidget);
        btn_angle_reset->setObjectName("btn_angle_reset");
        QIcon icon4;
        iconThemeName = QString::fromUtf8("rotation-allowed");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon4 = QIcon::fromTheme(iconThemeName);
        } else {
            icon4.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_angle_reset->setIcon(icon4);

        horizontalLayout->addWidget(btn_angle_reset);

        btn_flip_view = new QToolButton(ViewTransformWidget);
        btn_flip_view->setObjectName("btn_flip_view");
        QIcon icon5(QIcon::fromTheme(QString::fromUtf8("object-flip-horizontal")));
        btn_flip_view->setIcon(icon5);

        horizontalLayout->addWidget(btn_flip_view);


        retranslateUi(ViewTransformWidget);

        QMetaObject::connectSlotsByName(ViewTransformWidget);
    } // setupUi

    void retranslateUi(QWidget *ViewTransformWidget)
    {
#if QT_CONFIG(tooltip)
        btn_view_fit->setToolTip(tr2i18n("Fit View", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_view_fit->setText(tr2i18n("Fit View", nullptr));
        label->setText(tr2i18n("Zoom", nullptr));
        spin_zoom->setSuffix(tr2i18n("%", nullptr));
#if QT_CONFIG(tooltip)
        btn_zoom_in->setToolTip(tr2i18n("Zoom In", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_zoom_in->setText(tr2i18n("Zoom In", nullptr));
#if QT_CONFIG(tooltip)
        btn_zoom_out->setToolTip(tr2i18n("Zoom Out", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_zoom_out->setText(tr2i18n("Zoom Out", nullptr));
#if QT_CONFIG(tooltip)
        btn_zoom_reset->setToolTip(tr2i18n("Reset Zoom", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_zoom_reset->setText(tr2i18n("Reset Zoom", nullptr));
        label_2->setText(tr2i18n("Angle", nullptr));
        spin_angle->setSuffix(tr2i18n("\302\260", nullptr));
#if QT_CONFIG(tooltip)
        btn_angle_reset->setToolTip(tr2i18n("Reset Rotation", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_angle_reset->setText(tr2i18n("Reset Rotation", nullptr));
#if QT_CONFIG(tooltip)
        btn_flip_view->setToolTip(tr2i18n("Flip View", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_flip_view->setText(tr2i18n("Flip View", nullptr));
        (void)ViewTransformWidget;
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class ViewTransformWidget: public Ui_ViewTransformWidget {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // VIEW_TRANSFORM_WIDGET_H

