/********************************************************************************
** Form generated from reading UI file 'io_status_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IO_STATUS_DIALOG_H
#define UI_IO_STATUS_DIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

namespace glaxnimate {
namespace gui {

class Ui_IoStatusDialog
{
public:
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout;
    QLabel *icon_label;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QProgressBar *progress_bar;
    QSpacerItem *verticalSpacer;
    QGroupBox *group_box;
    QVBoxLayout *verticalLayout_2;
    QListWidget *list_widget;
    QDialogButtonBox *button_box;

    void setupUi(QDialog *IoStatusDialog)
    {
        if (IoStatusDialog->objectName().isEmpty())
            IoStatusDialog->setObjectName("IoStatusDialog");
        IoStatusDialog->resize(455, 298);
        verticalLayout_3 = new QVBoxLayout(IoStatusDialog);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        icon_label = new QLabel(IoStatusDialog);
        icon_label->setObjectName("icon_label");
        icon_label->setMinimumSize(QSize(64, 64));
        icon_label->setMaximumSize(QSize(64, 64));
        icon_label->setText(QString::fromUtf8(""));

        horizontalLayout->addWidget(icon_label);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(IoStatusDialog);
        label->setObjectName("label");

        verticalLayout->addWidget(label);

        progress_bar = new QProgressBar(IoStatusDialog);
        progress_bar->setObjectName("progress_bar");
        progress_bar->setMaximum(0);

        verticalLayout->addWidget(progress_bar);


        horizontalLayout->addLayout(verticalLayout);


        verticalLayout_3->addLayout(horizontalLayout);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);

        verticalLayout_3->addItem(verticalSpacer);

        group_box = new QGroupBox(IoStatusDialog);
        group_box->setObjectName("group_box");
        verticalLayout_2 = new QVBoxLayout(group_box);
        verticalLayout_2->setObjectName("verticalLayout_2");
        list_widget = new QListWidget(group_box);
        list_widget->setObjectName("list_widget");

        verticalLayout_2->addWidget(list_widget);


        verticalLayout_3->addWidget(group_box);

        button_box = new QDialogButtonBox(IoStatusDialog);
        button_box->setObjectName("button_box");
        button_box->setEnabled(false);
        button_box->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout_3->addWidget(button_box);


        retranslateUi(IoStatusDialog);
        QObject::connect(button_box, &QDialogButtonBox::rejected, IoStatusDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(IoStatusDialog);
    } // setupUi

    void retranslateUi(QDialog *IoStatusDialog)
    {
        IoStatusDialog->setWindowTitle(tr2i18n("IoStatusDialog", nullptr));
        label->setText(tr2i18n("/path/to/your/file/in.here", nullptr));
        group_box->setTitle(tr2i18n("Issues", nullptr));
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class IoStatusDialog: public Ui_IoStatusDialog {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // IO_STATUS_DIALOG_H

