/****************************************************************************
** Meta object code from reading C++ file 'compound_timeline_widget.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/gui/widgets/timeline/compound_timeline_widget.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'compound_timeline_widget.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate3gui22CompoundTimelineWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::gui::CompoundTimelineWidget::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate3gui22CompoundTimelineWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::gui::CompoundTimelineWidget",
        "switch_composition",
        "",
        "model::Composition*",
        "comp",
        "index",
        "current_node_changed",
        "model::VisualNode*",
        "node",
        "selection_changed",
        "std::vector<model::VisualNode*>",
        "selected",
        "deselected",
        "select_index",
        "QModelIndex",
        "_on_selection_changed",
        "QItemSelection",
        "select_line",
        "quintptr",
        "id",
        "replace_selection",
        "custom_context_menu",
        "QPoint",
        "p",
        "add_keyframe",
        "remove_keyframe",
        "on_scroll",
        "amount",
        "keyframe_action_enter",
        "keyframe_action_exit",
        "copy_keyframe",
        "paste_keyframe",
        "collapse_index",
        "expand_index",
        "click_index",
        "rows_removed",
        "first",
        "last"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'switch_composition'
        QtMocHelpers::SignalData<void(model::Composition *, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'current_node_changed'
        QtMocHelpers::SignalData<void(model::VisualNode *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'selection_changed'
        QtMocHelpers::SignalData<void(const std::vector<model::VisualNode*> &, const std::vector<model::VisualNode*> &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 10, 12 },
        }}),
        // Slot 'select_index'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 5 },
        }}),
        // Slot '_on_selection_changed'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 11 }, { 0x80000000 | 16, 12 },
        }}),
        // Slot 'select_line'
        QtMocHelpers::SlotData<void(quintptr, bool, bool)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { QMetaType::Bool, 11 }, { QMetaType::Bool, 20 },
        }}),
        // Slot 'custom_context_menu'
        QtMocHelpers::SlotData<void(const QPoint &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'add_keyframe'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'remove_keyframe'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_scroll'
        QtMocHelpers::SlotData<void(int)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'keyframe_action_enter'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'keyframe_action_exit'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'copy_keyframe'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'paste_keyframe'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'collapse_index'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 5 },
        }}),
        // Slot 'expand_index'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 5 },
        }}),
        // Slot 'click_index'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 5 },
        }}),
        // Slot 'rows_removed'
        QtMocHelpers::SlotData<void(const QModelIndex &, int, int)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 5 }, { QMetaType::Int, 36 }, { QMetaType::Int, 37 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CompoundTimelineWidget, qt_meta_tag_ZN10glaxnimate3gui22CompoundTimelineWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::gui::CompoundTimelineWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui22CompoundTimelineWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui22CompoundTimelineWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate3gui22CompoundTimelineWidgetE_t>.metaTypes,
    nullptr
} };

void glaxnimate::gui::CompoundTimelineWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CompoundTimelineWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->switch_composition((*reinterpret_cast<std::add_pointer_t<model::Composition*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->current_node_changed((*reinterpret_cast<std::add_pointer_t<model::VisualNode*>>(_a[1]))); break;
        case 2: _t->selection_changed((*reinterpret_cast<std::add_pointer_t<std::vector<model::VisualNode*>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::vector<model::VisualNode*>>>(_a[2]))); break;
        case 3: _t->select_index((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 4: _t->_on_selection_changed((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 5: _t->select_line((*reinterpret_cast<std::add_pointer_t<quintptr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 6: _t->custom_context_menu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 7: _t->add_keyframe(); break;
        case 8: _t->remove_keyframe(); break;
        case 9: _t->on_scroll((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->keyframe_action_enter(); break;
        case 11: _t->keyframe_action_exit(); break;
        case 12: _t->copy_keyframe(); break;
        case 13: _t->paste_keyframe(); break;
        case 14: _t->collapse_index((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 15: _t->expand_index((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 16: _t->click_index((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 17: _t->rows_removed((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CompoundTimelineWidget::*)(model::Composition * , int )>(_a, &CompoundTimelineWidget::switch_composition, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CompoundTimelineWidget::*)(model::VisualNode * )>(_a, &CompoundTimelineWidget::current_node_changed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CompoundTimelineWidget::*)(const std::vector<model::VisualNode*> & , const std::vector<model::VisualNode*> & )>(_a, &CompoundTimelineWidget::selection_changed, 2))
            return;
    }
}

const QMetaObject *glaxnimate::gui::CompoundTimelineWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::gui::CompoundTimelineWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui22CompoundTimelineWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int glaxnimate::gui::CompoundTimelineWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::gui::CompoundTimelineWidget::switch_composition(model::Composition * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void glaxnimate::gui::CompoundTimelineWidget::current_node_changed(model::VisualNode * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void glaxnimate::gui::CompoundTimelineWidget::selection_changed(const std::vector<model::VisualNode*> & _t1, const std::vector<model::VisualNode*> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}
QT_WARNING_POP
