/********************************************************************************
** Form generated from reading UI file 'logs.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LOGS_H
#define UI_LOGS_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDockWidget>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_dock_logs
{
public:
    QWidget *dockWidgetContents;
    QVBoxLayout *verticalLayout;
    QTableView *view_logs;

    void setupUi(QDockWidget *dock_logs)
    {
        if (dock_logs->objectName().isEmpty())
            dock_logs->setObjectName("dock_logs");
        dock_logs->resize(400, 300);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("utilities-log-viewer")));
        dock_logs->setWindowIcon(icon);
        dockWidgetContents = new QWidget();
        dockWidgetContents->setObjectName("dockWidgetContents");
        verticalLayout = new QVBoxLayout(dockWidgetContents);
        verticalLayout->setObjectName("verticalLayout");
        view_logs = new QTableView(dockWidgetContents);
        view_logs->setObjectName("view_logs");
        view_logs->setSelectionMode(QAbstractItemView::SelectionMode::ExtendedSelection);
        view_logs->setSelectionBehavior(QAbstractItemView::SelectionBehavior::SelectRows);
        view_logs->setTextElideMode(Qt::TextElideMode::ElideNone);
        view_logs->setShowGrid(false);

        verticalLayout->addWidget(view_logs);

        dock_logs->setWidget(dockWidgetContents);

        retranslateUi(dock_logs);

        QMetaObject::connectSlotsByName(dock_logs);
    } // setupUi

    void retranslateUi(QDockWidget *dock_logs)
    {
        dock_logs->setWindowTitle(tr2i18n("Logs", nullptr));
    } // retranslateUi

};

namespace Ui {
    class dock_logs: public Ui_dock_logs {};
} // namespace Ui

QT_END_NAMESPACE

#endif // LOGS_H

