/****************************************************************************
** Meta object code from reading C++ file 'color_palette.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/external/Qt-Color-Widgets/include/QtColorWidgets/color_palette.hpp"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'color_palette.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13color_widgets12ColorPaletteE_t {};
} // unnamed namespace

template <> constexpr inline auto color_widgets::ColorPalette::qt_create_metaobjectdata<qt_meta_tag_ZN13color_widgets12ColorPaletteE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "color_widgets::ColorPalette",
        "colorsChanged",
        "",
        "QList<std::pair<QColor,QString>>",
        "columnsChanged",
        "nameChanged",
        "fileNameChanged",
        "dirtyChanged",
        "colorChanged",
        "index",
        "colorRemoved",
        "colorAdded",
        "colorsUpdated",
        "setColumns",
        "columns",
        "setColors",
        "QList<QColor>",
        "colors",
        "setColorAt",
        "QColor",
        "color",
        "name",
        "setNameAt",
        "appendColor",
        "insertColor",
        "eraseColor",
        "save",
        "filename",
        "setName",
        "setFileName",
        "setDirty",
        "dirty",
        "colorAt",
        "nameAt",
        "loadColorTable",
        "QList<QRgb>",
        "color_table",
        "colorTable",
        "loadImage",
        "QImage",
        "image",
        "load",
        "QList<value_type>",
        "count",
        "fileName"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'colorsChanged'
        QtMocHelpers::SignalData<void(const QVector<QPair<QColor,QString> > &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'columnsChanged'
        QtMocHelpers::SignalData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'nameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'fileNameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'dirtyChanged'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Signal 'colorRemoved'
        QtMocHelpers::SignalData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Signal 'colorAdded'
        QtMocHelpers::SignalData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Signal 'colorsUpdated'
        QtMocHelpers::SignalData<void(const QVector<QPair<QColor,QString>> &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'setColumns'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'setColors'
        QtMocHelpers::SlotData<void(const QVector<QColor> &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'setColors'
        QtMocHelpers::SlotData<void(const QVector<QPair<QColor,QString> > &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 17 },
        }}),
        // Slot 'setColorAt'
        QtMocHelpers::SlotData<void(int, const QColor &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 }, { 0x80000000 | 19, 20 },
        }}),
        // Slot 'setColorAt'
        QtMocHelpers::SlotData<void(int, const QColor &, const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 }, { 0x80000000 | 19, 20 }, { QMetaType::QString, 21 },
        }}),
        // Slot 'setNameAt'
        QtMocHelpers::SlotData<void(int, const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 }, { QMetaType::QString, 21 },
        }}),
        // Slot 'setNameAt'
        QtMocHelpers::SlotData<void(int)>(22, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'appendColor'
        QtMocHelpers::SlotData<void(const QColor &, const QString &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { QMetaType::QString, 21 },
        }}),
        // Slot 'appendColor'
        QtMocHelpers::SlotData<void(const QColor &)>(23, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'insertColor'
        QtMocHelpers::SlotData<void(int, const QColor &, const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 }, { 0x80000000 | 19, 20 }, { QMetaType::QString, 21 },
        }}),
        // Slot 'insertColor'
        QtMocHelpers::SlotData<void(int, const QColor &)>(24, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 9 }, { 0x80000000 | 19, 20 },
        }}),
        // Slot 'eraseColor'
        QtMocHelpers::SlotData<void(int)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'save'
        QtMocHelpers::SlotData<bool(const QString &)>(26, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 27 },
        }}),
        // Slot 'save'
        QtMocHelpers::SlotData<bool()>(26, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setName'
        QtMocHelpers::SlotData<void(const QString &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Slot 'setFileName'
        QtMocHelpers::SlotData<void(const QString &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Slot 'setDirty'
        QtMocHelpers::SlotData<void(bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 31 },
        }}),
        // Method 'colorAt'
        QtMocHelpers::MethodData<QColor(int) const>(32, 2, QMC::AccessPublic, 0x80000000 | 19, {{
            { QMetaType::Int, 9 },
        }}),
        // Method 'nameAt'
        QtMocHelpers::MethodData<QString(int) const>(33, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::Int, 9 },
        }}),
        // Method 'loadColorTable'
        QtMocHelpers::MethodData<void(const QVector<QRgb> &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 35, 36 },
        }}),
        // Method 'colorTable'
        QtMocHelpers::MethodData<QVector<QRgb>() const>(37, 2, QMC::AccessPublic, 0x80000000 | 35),
        // Method 'loadImage'
        QtMocHelpers::MethodData<bool(const QImage &)>(38, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 39, 40 },
        }}),
        // Method 'load'
        QtMocHelpers::MethodData<bool(const QString &)>(41, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'colors'
        QtMocHelpers::PropertyData<QList<value_type>>(17, 0x80000000 | 42, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'columns'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'count'
        QtMocHelpers::PropertyData<int>(43, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'fileName'
        QtMocHelpers::PropertyData<QString>(44, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'dirty'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ColorPalette, qt_meta_tag_ZN13color_widgets12ColorPaletteE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject color_widgets::ColorPalette::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets12ColorPaletteE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets12ColorPaletteE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13color_widgets12ColorPaletteE_t>.metaTypes,
    nullptr
} };

void color_widgets::ColorPalette::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ColorPalette *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->colorsChanged((*reinterpret_cast<std::add_pointer_t<QList<std::pair<QColor,QString>>>>(_a[1]))); break;
        case 1: _t->columnsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->nameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->fileNameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->dirtyChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->colorRemoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->colorAdded((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->colorsUpdated((*reinterpret_cast<std::add_pointer_t<QList<std::pair<QColor,QString>>>>(_a[1]))); break;
        case 9: _t->setColumns((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->setColors((*reinterpret_cast<std::add_pointer_t<QList<QColor>>>(_a[1]))); break;
        case 11: _t->setColors((*reinterpret_cast<std::add_pointer_t<QList<std::pair<QColor,QString>>>>(_a[1]))); break;
        case 12: _t->setColorAt((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QColor>>(_a[2]))); break;
        case 13: _t->setColorAt((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QColor>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 14: _t->setNameAt((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 15: _t->setNameAt((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->appendColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 17: _t->appendColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 18: _t->insertColor((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QColor>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 19: _t->insertColor((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QColor>>(_a[2]))); break;
        case 20: _t->eraseColor((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 21: { bool _r = _t->save((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->save();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 23: _t->setName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: _t->setFileName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 25: _t->setDirty((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 26: { QColor _r = _t->colorAt((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QColor*>(_a[0]) = std::move(_r); }  break;
        case 27: { QString _r = _t->nameAt((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 28: _t->loadColorTable((*reinterpret_cast<std::add_pointer_t<QList<QRgb>>>(_a[1]))); break;
        case 29: { QList<QRgb> _r = _t->colorTable();
            if (_a[0]) *reinterpret_cast<QList<QRgb>*>(_a[0]) = std::move(_r); }  break;
        case 30: { bool _r = _t->loadImage((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 31: { bool _r = _t->load((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ColorPalette::*)(const QVector<QPair<QColor,QString> > & )>(_a, &ColorPalette::colorsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPalette::*)(int )>(_a, &ColorPalette::columnsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPalette::*)(const QString & )>(_a, &ColorPalette::nameChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPalette::*)(const QString & )>(_a, &ColorPalette::fileNameChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPalette::*)(bool )>(_a, &ColorPalette::dirtyChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPalette::*)(int )>(_a, &ColorPalette::colorChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPalette::*)(int )>(_a, &ColorPalette::colorRemoved, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPalette::*)(int )>(_a, &ColorPalette::colorAdded, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPalette::*)(const QVector<QPair<QColor,QString>> & )>(_a, &ColorPalette::colorsUpdated, 8))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QList<value_type>*>(_v) = _t->colors(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->columns(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->count(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->fileName(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->dirty(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setColors(*reinterpret_cast<QList<value_type>*>(_v)); break;
        case 1: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setColumns(*reinterpret_cast<int*>(_v)); break;
        case 4: _t->setFileName(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setDirty(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *color_widgets::ColorPalette::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *color_widgets::ColorPalette::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets12ColorPaletteE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int color_widgets::ColorPalette::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 32)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 32;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void color_widgets::ColorPalette::colorsChanged(const QVector<QPair<QColor,QString> > & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void color_widgets::ColorPalette::columnsChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void color_widgets::ColorPalette::nameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void color_widgets::ColorPalette::fileNameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void color_widgets::ColorPalette::dirtyChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void color_widgets::ColorPalette::colorChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void color_widgets::ColorPalette::colorRemoved(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void color_widgets::ColorPalette::colorAdded(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void color_widgets::ColorPalette::colorsUpdated(const QVector<QPair<QColor,QString>> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
