/****************************************************************************
** Meta object code from reading C++ file 'gradient.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/core/model/assets/gradient.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'gradient.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate5model14GradientColorsE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::GradientColors::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model14GradientColorsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::GradientColors",
        "colors",
        "property animated QGradientStops",
        "colors_changed",
        "",
        "QGradientStops",
        "split_segment",
        "segment_index",
        "factor",
        "QColor",
        "new_color",
        "remove_stop",
        "index",
        "glaxnimate::model::AnimatableBase*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'colors_changed'
        QtMocHelpers::SignalData<void(const QGradientStops &)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 4 },
        }}),
        // Method 'split_segment'
        QtMocHelpers::MethodData<void(int, float, const QColor &)>(6, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { QMetaType::Float, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Method 'split_segment'
        QtMocHelpers::MethodData<void(int, float)>(6, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { QMetaType::Float, 8 },
        }}),
        // Method 'split_segment'
        QtMocHelpers::MethodData<void(int)>(6, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Method 'remove_stop'
        QtMocHelpers::MethodData<void(int)>(11, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'colors'
        QtMocHelpers::PropertyData<glaxnimate::model::AnimatableBase*>(1, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<GradientColors, qt_meta_tag_ZN10glaxnimate5model14GradientColorsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::GradientColors::staticMetaObject = { {
    QMetaObject::SuperData::link<Asset::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model14GradientColorsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model14GradientColorsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model14GradientColorsE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::GradientColors::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GradientColors *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->colors_changed((*reinterpret_cast<std::add_pointer_t<QGradientStops>>(_a[1]))); break;
        case 1: _t->split_segment((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QColor>>(_a[3]))); break;
        case 2: _t->split_segment((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 3: _t->split_segment((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->remove_stop((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (GradientColors::*)(const QGradientStops & )>(_a, &GradientColors::colors_changed, 0))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< glaxnimate::model::AnimatableBase* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<glaxnimate::model::AnimatableBase**>(_v) = _t->get_colors(); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::GradientColors::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::GradientColors::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model14GradientColorsE_t>.strings))
        return static_cast<void*>(this);
    return Asset::qt_metacast(_clname);
}

int glaxnimate::model::GradientColors::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Asset::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::model::GradientColors::colors_changed(const QGradientStops & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN10glaxnimate5model8GradientE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::Gradient::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model8GradientE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::Gradient",
        "colors",
        "property ref GradientColors",
        "type",
        "property GradientType",
        "start_point",
        "property animated QPointF",
        "end_point",
        "highlight",
        "colors_changed_from",
        "",
        "GradientColors*",
        "old_use",
        "new_use",
        "radius",
        "t",
        "GradientType",
        "glaxnimate::model::AnimatableBase*",
        "Linear",
        "Radial",
        "Conical"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'colors_changed_from'
        QtMocHelpers::SignalData<void(GradientColors *, GradientColors *)>(9, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 11, 13 },
        }}),
        // Method 'radius'
        QtMocHelpers::MethodData<qreal(double) const>(14, 10, QMC::AccessPublic, QMetaType::QReal, {{
            { QMetaType::Double, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'colors'
        QtMocHelpers::PropertyData<GradientColors*>(1, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'type'
        QtMocHelpers::PropertyData<enum GradientType>(3, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'start_point'
        QtMocHelpers::PropertyData<glaxnimate::model::AnimatableBase*>(5, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'end_point'
        QtMocHelpers::PropertyData<glaxnimate::model::AnimatableBase*>(7, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'highlight'
        QtMocHelpers::PropertyData<glaxnimate::model::AnimatableBase*>(8, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'GradientType'
        QtMocHelpers::EnumData<enum GradientType>(16, 16, QMC::EnumFlags{}).add({
            {   18, GradientType::Linear },
            {   19, GradientType::Radial },
            {   20, GradientType::Conical },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    6 },
            {    8,    6 },
    });
    return QtMocHelpers::metaObjectData<Gradient, qt_meta_tag_ZN10glaxnimate5model8GradientE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::Gradient::staticMetaObject = { {
    QMetaObject::SuperData::link<BrushStyle::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model8GradientE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model8GradientE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model8GradientE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::Gradient::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Gradient *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->colors_changed_from((*reinterpret_cast<std::add_pointer_t<GradientColors*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GradientColors*>>(_a[2]))); break;
        case 1: { qreal _r = _t->radius((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qreal*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GradientColors* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Gradient::*)(GradientColors * , GradientColors * )>(_a, &Gradient::colors_changed_from, 0))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< GradientColors* >(); break;
        case 4:
        case 3:
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< glaxnimate::model::AnimatableBase* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<GradientColors**>(_v) = _t->get_colors(); break;
        case 1: *reinterpret_cast<enum GradientType*>(_v) = _t->get_type(); break;
        case 2: *reinterpret_cast<glaxnimate::model::AnimatableBase**>(_v) = _t->get_start_point(); break;
        case 3: *reinterpret_cast<glaxnimate::model::AnimatableBase**>(_v) = _t->get_end_point(); break;
        case 4: *reinterpret_cast<glaxnimate::model::AnimatableBase**>(_v) = _t->get_highlight(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->set_colors(*reinterpret_cast<GradientColors**>(_v)); break;
        case 1: _t->set_type(*reinterpret_cast<enum GradientType*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::Gradient::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::Gradient::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model8GradientE_t>.strings))
        return static_cast<void*>(this);
    return BrushStyle::qt_metacast(_clname);
}

int glaxnimate::model::Gradient::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = BrushStyle::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::model::Gradient::colors_changed_from(GradientColors * _t1, GradientColors * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
QT_WARNING_POP
