/****************************************************************************
** Meta object code from reading C++ file 'color_dialog.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/external/Qt-Color-Widgets/include/QtColorWidgets/color_dialog.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'color_dialog.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13color_widgets11ColorDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto color_widgets::ColorDialog::qt_create_metaobjectdata<qt_meta_tag_ZN13color_widgets11ColorDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "color_widgets::ColorDialog",
        "colorChanged",
        "",
        "QColor",
        "colorSelected",
        "wheelShapeChanged",
        "ColorWheel::ShapeEnum",
        "shape",
        "colorSpaceChanged",
        "ColorWheel::ColorSpaceEnum",
        "space",
        "wheelRotatingChanged",
        "rotating",
        "wheelMirroredChanged",
        "mirrored",
        "alphaEnabledChanged",
        "alphaEnabled",
        "setColor",
        "c",
        "showColor",
        "oldcolor",
        "setWheelShape",
        "setColorSpace",
        "setWheelRotating",
        "setWheelMirrored",
        "setAlphaEnabled",
        "a",
        "setColorInternal",
        "color",
        "set_hsv",
        "set_rgb",
        "set_alpha",
        "on_edit_hex_colorChanged",
        "on_edit_hex_colorEditingFinished",
        "on_buttonBox_clicked",
        "QAbstractButton*",
        "wheelShape",
        "colorSpace",
        "wheelRotating",
        "wheelMirrored",
        "ButtonMode",
        "OkCancel",
        "OkApplyCancel",
        "Close"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(QColor)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'colorSelected'
        QtMocHelpers::SignalData<void(QColor)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'wheelShapeChanged'
        QtMocHelpers::SignalData<void(ColorWheel::ShapeEnum)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'colorSpaceChanged'
        QtMocHelpers::SignalData<void(ColorWheel::ColorSpaceEnum)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'wheelRotatingChanged'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Signal 'wheelMirroredChanged'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Signal 'alphaEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(const QColor &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 18 },
        }}),
        // Slot 'showColor'
        QtMocHelpers::SlotData<void(const QColor &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 20 },
        }}),
        // Slot 'setWheelShape'
        QtMocHelpers::SlotData<void(ColorWheel::ShapeEnum)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'setColorSpace'
        QtMocHelpers::SlotData<void(ColorWheel::ColorSpaceEnum)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'setWheelRotating'
        QtMocHelpers::SlotData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Slot 'setWheelMirrored'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'setAlphaEnabled'
        QtMocHelpers::SlotData<void(bool)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 26 },
        }}),
        // Slot 'setColorInternal'
        QtMocHelpers::SlotData<void(const QColor &)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 28 },
        }}),
        // Slot 'set_hsv'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'set_rgb'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'set_alpha'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_edit_hex_colorChanged'
        QtMocHelpers::SlotData<void(const QColor &)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 28 },
        }}),
        // Slot 'on_edit_hex_colorEditingFinished'
        QtMocHelpers::SlotData<void(const QColor &)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 28 },
        }}),
        // Slot 'on_buttonBox_clicked'
        QtMocHelpers::SlotData<void(QAbstractButton *)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 35, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(28, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'wheelShape'
        QtMocHelpers::PropertyData<ColorWheel::ShapeEnum>(36, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'colorSpace'
        QtMocHelpers::PropertyData<ColorWheel::ColorSpaceEnum>(37, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'wheelRotating'
        QtMocHelpers::PropertyData<bool>(38, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'wheelMirrored'
        QtMocHelpers::PropertyData<bool>(39, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'alphaEnabled'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ButtonMode'
        QtMocHelpers::EnumData<enum ButtonMode>(40, 40, QMC::EnumFlags{}).add({
            {   41, ButtonMode::OkCancel },
            {   42, ButtonMode::OkApplyCancel },
            {   43, ButtonMode::Close },
        }),
    };
    return QtMocHelpers::metaObjectData<ColorDialog, qt_meta_tag_ZN13color_widgets11ColorDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN13color_widgets11ColorDialogE[] = {
    QMetaObject::SuperData::link<color_widgets::ColorWheel::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject color_widgets::ColorDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets11ColorDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets11ColorDialogE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN13color_widgets11ColorDialogE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13color_widgets11ColorDialogE_t>.metaTypes,
    nullptr
} };

void color_widgets::ColorDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ColorDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 1: _t->colorSelected((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 2: _t->wheelShapeChanged((*reinterpret_cast<std::add_pointer_t<ColorWheel::ShapeEnum>>(_a[1]))); break;
        case 3: _t->colorSpaceChanged((*reinterpret_cast<std::add_pointer_t<ColorWheel::ColorSpaceEnum>>(_a[1]))); break;
        case 4: _t->wheelRotatingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->wheelMirroredChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->alphaEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 8: _t->showColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 9: _t->setWheelShape((*reinterpret_cast<std::add_pointer_t<ColorWheel::ShapeEnum>>(_a[1]))); break;
        case 10: _t->setColorSpace((*reinterpret_cast<std::add_pointer_t<ColorWheel::ColorSpaceEnum>>(_a[1]))); break;
        case 11: _t->setWheelRotating((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->setWheelMirrored((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->setAlphaEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->setColorInternal((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 15: _t->set_hsv(); break;
        case 16: _t->set_rgb(); break;
        case 17: _t->set_alpha(); break;
        case 18: _t->on_edit_hex_colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 19: _t->on_edit_hex_colorEditingFinished((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 20: _t->on_buttonBox_clicked((*reinterpret_cast<std::add_pointer_t<QAbstractButton*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ColorDialog::*)(QColor )>(_a, &ColorDialog::colorChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorDialog::*)(QColor )>(_a, &ColorDialog::colorSelected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorDialog::*)(ColorWheel::ShapeEnum )>(_a, &ColorDialog::wheelShapeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorDialog::*)(ColorWheel::ColorSpaceEnum )>(_a, &ColorDialog::colorSpaceChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorDialog::*)(bool )>(_a, &ColorDialog::wheelRotatingChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorDialog::*)(bool )>(_a, &ColorDialog::wheelMirroredChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorDialog::*)(bool )>(_a, &ColorDialog::alphaEnabledChanged, 6))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QColor*>(_v) = _t->color(); break;
        case 1: *reinterpret_cast<ColorWheel::ShapeEnum*>(_v) = _t->wheelShape(); break;
        case 2: *reinterpret_cast<ColorWheel::ColorSpaceEnum*>(_v) = _t->colorSpace(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->wheelRotating(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->wheelMirrored(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->alphaEnabled(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setColor(*reinterpret_cast<QColor*>(_v)); break;
        case 1: _t->setWheelShape(*reinterpret_cast<ColorWheel::ShapeEnum*>(_v)); break;
        case 2: _t->setColorSpace(*reinterpret_cast<ColorWheel::ColorSpaceEnum*>(_v)); break;
        case 3: _t->setWheelRotating(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setWheelMirrored(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setAlphaEnabled(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *color_widgets::ColorDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *color_widgets::ColorDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets11ColorDialogE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int color_widgets::ColorDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 21;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void color_widgets::ColorDialog::colorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void color_widgets::ColorDialog::colorSelected(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void color_widgets::ColorDialog::wheelShapeChanged(ColorWheel::ShapeEnum _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void color_widgets::ColorDialog::colorSpaceChanged(ColorWheel::ColorSpaceEnum _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void color_widgets::ColorDialog::wheelRotatingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void color_widgets::ColorDialog::wheelMirroredChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void color_widgets::ColorDialog::alphaEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
