/********************************************************************************
** Form generated from reading UI file 'select_shape_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SELECT_SHAPE_DIALOG_H
#define UI_SELECT_SHAPE_DIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

namespace glaxnimate {
namespace gui {

class Ui_SelectShapeDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTreeView *view_document_node;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *glaxnimate__gui__SelectShapeDialog)
    {
        if (glaxnimate__gui__SelectShapeDialog->objectName().isEmpty())
            glaxnimate__gui__SelectShapeDialog->setObjectName("glaxnimate__gui__SelectShapeDialog");
        glaxnimate__gui__SelectShapeDialog->resize(400, 560);
        verticalLayout = new QVBoxLayout(glaxnimate__gui__SelectShapeDialog);
        verticalLayout->setObjectName("verticalLayout");
        view_document_node = new QTreeView(glaxnimate__gui__SelectShapeDialog);
        view_document_node->setObjectName("view_document_node");

        verticalLayout->addWidget(view_document_node);

        buttonBox = new QDialogButtonBox(glaxnimate__gui__SelectShapeDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(glaxnimate__gui__SelectShapeDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, glaxnimate__gui__SelectShapeDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, glaxnimate__gui__SelectShapeDialog, qOverload<>(&QDialog::reject));
        QObject::connect(view_document_node, SIGNAL(clicked(QModelIndex)), glaxnimate__gui__SelectShapeDialog, SLOT(select(QModelIndex)));
        QObject::connect(view_document_node, SIGNAL(activated(QModelIndex)), glaxnimate__gui__SelectShapeDialog, SLOT(select(QModelIndex)));

        QMetaObject::connectSlotsByName(glaxnimate__gui__SelectShapeDialog);
    } // setupUi

    void retranslateUi(QDialog *glaxnimate__gui__SelectShapeDialog)
    {
        glaxnimate__gui__SelectShapeDialog->setWindowTitle(tr2i18n("Select Shape", nullptr));
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class SelectShapeDialog: public Ui_SelectShapeDialog {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // SELECT_SHAPE_DIALOG_H

