/********************************************************************************
** Form generated from reading UI file 'compound_timeline_widget.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_COMPOUND_TIMELINE_WIDGET_H
#define UI_COMPOUND_TIMELINE_WIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QScrollBar>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "widgets/tab_bar/composition_tab_bar.hpp"
#include "widgets/timeline/timeline_treeview.hpp"
#include "widgets/timeline/timeline_widget.hpp"

namespace glaxnimate {
namespace gui {

class Ui_CompoundTimelineWidget
{
public:
    QAction *action_add_keyframe;
    QAction *action_remove_all_keyframes;
    QVBoxLayout *verticalLayout;
    glaxnimate::gui::CompositionTabBar *tab_bar;
    QFrame *frame;
    QHBoxLayout *horizontalLayout_4;
    QSplitter *splitter;
    glaxnimate::gui::TimelineTreeview *properties;
    QWidget *horizontalWidget;
    QHBoxLayout *horizontalLayout_2;
    glaxnimate::gui::TimelineWidget *timeline;
    QScrollBar *scrollbar;

    void setupUi(QWidget *glaxnimate__gui__CompoundTimelineWidget)
    {
        if (glaxnimate__gui__CompoundTimelineWidget->objectName().isEmpty())
            glaxnimate__gui__CompoundTimelineWidget->setObjectName("glaxnimate__gui__CompoundTimelineWidget");
        glaxnimate__gui__CompoundTimelineWidget->resize(718, 229);
        glaxnimate__gui__CompoundTimelineWidget->setContextMenuPolicy(Qt::ContextMenuPolicy::CustomContextMenu);
        action_add_keyframe = new QAction(glaxnimate__gui__CompoundTimelineWidget);
        action_add_keyframe->setObjectName("action_add_keyframe");
        action_remove_all_keyframes = new QAction(glaxnimate__gui__CompoundTimelineWidget);
        action_remove_all_keyframes->setObjectName("action_remove_all_keyframes");
        verticalLayout = new QVBoxLayout(glaxnimate__gui__CompoundTimelineWidget);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        tab_bar = new glaxnimate::gui::CompositionTabBar(glaxnimate__gui__CompoundTimelineWidget);
        tab_bar->setObjectName("tab_bar");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(tab_bar->sizePolicy().hasHeightForWidth());
        tab_bar->setSizePolicy(sizePolicy);

        verticalLayout->addWidget(tab_bar);

        frame = new QFrame(glaxnimate__gui__CompoundTimelineWidget);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::Shape::StyledPanel);
        frame->setFrameShadow(QFrame::Shadow::Sunken);
        horizontalLayout_4 = new QHBoxLayout(frame);
        horizontalLayout_4->setSpacing(0);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalLayout_4->setContentsMargins(0, 0, 0, 0);
        splitter = new QSplitter(frame);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Orientation::Horizontal);
        splitter->setChildrenCollapsible(false);
        properties = new glaxnimate::gui::TimelineTreeview(splitter);
        properties->setObjectName("properties");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(properties->sizePolicy().hasHeightForWidth());
        properties->setSizePolicy(sizePolicy1);
        properties->setMinimumSize(QSize(400, 0));
        properties->setFrameShape(QFrame::Shape::NoFrame);
        properties->setVerticalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
        properties->setHorizontalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
        properties->setDragEnabled(true);
        properties->setDragDropMode(QAbstractItemView::DragDropMode::InternalMove);
        properties->setAlternatingRowColors(true);
        properties->setSelectionMode(QAbstractItemView::SelectionMode::ExtendedSelection);
        properties->setSelectionBehavior(QAbstractItemView::SelectionBehavior::SelectRows);
        properties->setRootIsDecorated(true);
        properties->setUniformRowHeights(true);
        properties->setAnimated(false);
        splitter->addWidget(properties);
        properties->header()->setMinimumSectionSize(32);
        properties->header()->setDefaultSectionSize(32);
        horizontalWidget = new QWidget(splitter);
        horizontalWidget->setObjectName("horizontalWidget");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(1);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(horizontalWidget->sizePolicy().hasHeightForWidth());
        horizontalWidget->setSizePolicy(sizePolicy2);
        horizontalLayout_2 = new QHBoxLayout(horizontalWidget);
        horizontalLayout_2->setSpacing(0);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(8, 0, 0, 0);
        timeline = new glaxnimate::gui::TimelineWidget(horizontalWidget);
        timeline->setObjectName("timeline");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy3.setHorizontalStretch(1);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(timeline->sizePolicy().hasHeightForWidth());
        timeline->setSizePolicy(sizePolicy3);
        timeline->setFrameShape(QFrame::Shape::NoFrame);
        timeline->setVerticalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
        timeline->setHorizontalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAsNeeded);

        horizontalLayout_2->addWidget(timeline);

        splitter->addWidget(horizontalWidget);

        horizontalLayout_4->addWidget(splitter);

        scrollbar = new QScrollBar(frame);
        scrollbar->setObjectName("scrollbar");
        scrollbar->setPageStep(1);
        scrollbar->setOrientation(Qt::Orientation::Vertical);

        horizontalLayout_4->addWidget(scrollbar);


        verticalLayout->addWidget(frame);


        retranslateUi(glaxnimate__gui__CompoundTimelineWidget);
        QObject::connect(properties, SIGNAL(expanded(QModelIndex)), glaxnimate__gui__CompoundTimelineWidget, SLOT(expand_index(QModelIndex)));
        QObject::connect(properties, SIGNAL(collapsed(QModelIndex)), glaxnimate__gui__CompoundTimelineWidget, SLOT(collapse_index(QModelIndex)));
        QObject::connect(properties, SIGNAL(clicked(QModelIndex)), glaxnimate__gui__CompoundTimelineWidget, SLOT(click_index(QModelIndex)));
        QObject::connect(timeline, SIGNAL(scrolled(int)), scrollbar, SLOT(setValue(int)));

        QMetaObject::connectSlotsByName(glaxnimate__gui__CompoundTimelineWidget);
    } // setupUi

    void retranslateUi(QWidget *glaxnimate__gui__CompoundTimelineWidget)
    {
        action_add_keyframe->setText(tr2i18n("Add Keyframe", nullptr));
        action_remove_all_keyframes->setText(tr2i18n("&Clear Animations", nullptr));
        (void)glaxnimate__gui__CompoundTimelineWidget;
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class CompoundTimelineWidget: public Ui_CompoundTimelineWidget {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // COMPOUND_TIMELINE_WIDGET_H

