/****************************************************************************
** Meta object code from reading C++ file 'document_node.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/core/model/document_node.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'document_node.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate5model12DocumentNodeE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::DocumentNode::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model12DocumentNodeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::DocumentNode",
        "uuid",
        "property QUuid",
        "name",
        "property QString",
        "docnode_child_add_begin",
        "",
        "row",
        "docnode_child_add_end",
        "DocumentNode*",
        "node",
        "docnode_child_remove_begin",
        "docnode_child_remove_end",
        "docnode_child_move_begin",
        "from",
        "to",
        "docnode_child_move_end",
        "name_changed",
        "users_changed",
        "find_by_name",
        "glaxnimate::model::DocumentNode*",
        "find_by_uuid",
        "QUuid",
        "find_by_type_name",
        "QVariantList",
        "type_name"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'docnode_child_add_begin'
        QtMocHelpers::SignalData<void(int)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'docnode_child_add_end'
        QtMocHelpers::SignalData<void(DocumentNode *, int)>(8, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Int, 7 },
        }}),
        // Signal 'docnode_child_remove_begin'
        QtMocHelpers::SignalData<void(int)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'docnode_child_remove_end'
        QtMocHelpers::SignalData<void(DocumentNode *, int)>(12, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Int, 7 },
        }}),
        // Signal 'docnode_child_move_begin'
        QtMocHelpers::SignalData<void(int, int)>(13, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 }, { QMetaType::Int, 15 },
        }}),
        // Signal 'docnode_child_move_end'
        QtMocHelpers::SignalData<void(DocumentNode *, int, int)>(16, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Int, 14 }, { QMetaType::Int, 15 },
        }}),
        // Signal 'name_changed'
        QtMocHelpers::SignalData<void(const QString &)>(17, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'users_changed'
        QtMocHelpers::SignalData<void()>(18, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'find_by_name'
        QtMocHelpers::MethodData<glaxnimate::model::DocumentNode *(const QString &)>(19, 6, QMC::AccessPublic, 0x80000000 | 20, {{
            { QMetaType::QString, 3 },
        }}),
        // Method 'find_by_uuid'
        QtMocHelpers::MethodData<glaxnimate::model::DocumentNode *(const QUuid &)>(21, 6, QMC::AccessPublic, 0x80000000 | 20, {{
            { 0x80000000 | 22, 1 },
        }}),
        // Method 'find_by_type_name'
        QtMocHelpers::MethodData<QVariantList(const QString &)>(23, 6, QMC::AccessPublic, 0x80000000 | 24, {{
            { QMetaType::QString, 25 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'uuid'
        QtMocHelpers::PropertyData<QUuid>(1, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<DocumentNode, qt_meta_tag_ZN10glaxnimate5model12DocumentNodeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::DocumentNode::staticMetaObject = { {
    QMetaObject::SuperData::link<Object::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model12DocumentNodeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model12DocumentNodeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model12DocumentNodeE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::DocumentNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DocumentNode *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->docnode_child_add_begin((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->docnode_child_add_end((*reinterpret_cast<std::add_pointer_t<DocumentNode*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->docnode_child_remove_begin((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->docnode_child_remove_end((*reinterpret_cast<std::add_pointer_t<DocumentNode*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->docnode_child_move_begin((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 5: _t->docnode_child_move_end((*reinterpret_cast<std::add_pointer_t<DocumentNode*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 6: _t->name_changed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->users_changed(); break;
        case 8: { glaxnimate::model::DocumentNode* _r = _t->find_by_name((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<glaxnimate::model::DocumentNode**>(_a[0]) = std::move(_r); }  break;
        case 9: { glaxnimate::model::DocumentNode* _r = _t->find_by_uuid((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])));
            if (_a[0]) *reinterpret_cast<glaxnimate::model::DocumentNode**>(_a[0]) = std::move(_r); }  break;
        case 10: { QVariantList _r = _t->find_by_type_name((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DocumentNode* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DocumentNode* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DocumentNode* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DocumentNode::*)(int )>(_a, &DocumentNode::docnode_child_add_begin, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DocumentNode::*)(DocumentNode * , int )>(_a, &DocumentNode::docnode_child_add_end, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DocumentNode::*)(int )>(_a, &DocumentNode::docnode_child_remove_begin, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DocumentNode::*)(DocumentNode * , int )>(_a, &DocumentNode::docnode_child_remove_end, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DocumentNode::*)(int , int )>(_a, &DocumentNode::docnode_child_move_begin, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DocumentNode::*)(DocumentNode * , int , int )>(_a, &DocumentNode::docnode_child_move_end, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DocumentNode::*)(const QString & )>(_a, &DocumentNode::name_changed, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DocumentNode::*)()>(_a, &DocumentNode::users_changed, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QUuid*>(_v) = _t->get_uuid(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->get_name(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->set_name(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::DocumentNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::DocumentNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model12DocumentNodeE_t>.strings))
        return static_cast<void*>(this);
    return Object::qt_metacast(_clname);
}

int glaxnimate::model::DocumentNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Object::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::model::DocumentNode::docnode_child_add_begin(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void glaxnimate::model::DocumentNode::docnode_child_add_end(DocumentNode * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void glaxnimate::model::DocumentNode::docnode_child_remove_begin(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void glaxnimate::model::DocumentNode::docnode_child_remove_end(DocumentNode * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void glaxnimate::model::DocumentNode::docnode_child_move_begin(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void glaxnimate::model::DocumentNode::docnode_child_move_end(DocumentNode * _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}

// SIGNAL 6
void glaxnimate::model::DocumentNode::name_changed(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void glaxnimate::model::DocumentNode::users_changed()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
namespace {
struct qt_meta_tag_ZN10glaxnimate5model10VisualNodeE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::VisualNode::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model10VisualNodeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::VisualNode",
        "group_color",
        "property QColor",
        "visible",
        "property bool",
        "locked",
        "docnode_visible_changed",
        "",
        "docnode_locked_changed",
        "docnode_visible_recursive_changed",
        "docnode_group_color_changed",
        "QColor",
        "bounding_rect_changed",
        "transform_matrix_changed",
        "QTransform",
        "t",
        "group_transform_matrix_changed",
        "local_transform_matrix_changed",
        "visible_recursive",
        "locked_recursive",
        "selectable"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'docnode_visible_changed'
        QtMocHelpers::SignalData<void(bool)>(6, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'docnode_locked_changed'
        QtMocHelpers::SignalData<void(bool)>(8, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'docnode_visible_recursive_changed'
        QtMocHelpers::SignalData<void(bool)>(9, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'docnode_group_color_changed'
        QtMocHelpers::SignalData<void(const QColor &)>(10, 7, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 7 },
        }}),
        // Signal 'bounding_rect_changed'
        QtMocHelpers::SignalData<void()>(12, 7, QMC::AccessPublic, QMetaType::Void),
        // Signal 'transform_matrix_changed'
        QtMocHelpers::SignalData<void(const QTransform &)>(13, 7, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'group_transform_matrix_changed'
        QtMocHelpers::SignalData<void(const QTransform &)>(16, 7, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'local_transform_matrix_changed'
        QtMocHelpers::SignalData<void(const QTransform &)>(17, 7, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'group_color'
        QtMocHelpers::PropertyData<QColor>(1, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'visible'
        QtMocHelpers::PropertyData<bool>(3, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'locked'
        QtMocHelpers::PropertyData<bool>(5, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'visible_recursive'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'locked_recursive'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'selectable'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    4 },
    });
    return QtMocHelpers::metaObjectData<VisualNode, qt_meta_tag_ZN10glaxnimate5model10VisualNodeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::VisualNode::staticMetaObject = { {
    QMetaObject::SuperData::link<DocumentNode::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model10VisualNodeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model10VisualNodeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model10VisualNodeE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::VisualNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<VisualNode *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->docnode_visible_changed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->docnode_locked_changed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->docnode_visible_recursive_changed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->docnode_group_color_changed((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 4: _t->bounding_rect_changed(); break;
        case 5: _t->transform_matrix_changed((*reinterpret_cast<std::add_pointer_t<QTransform>>(_a[1]))); break;
        case 6: _t->group_transform_matrix_changed((*reinterpret_cast<std::add_pointer_t<QTransform>>(_a[1]))); break;
        case 7: _t->local_transform_matrix_changed((*reinterpret_cast<std::add_pointer_t<QTransform>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (VisualNode::*)(bool )>(_a, &VisualNode::docnode_visible_changed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (VisualNode::*)(bool )>(_a, &VisualNode::docnode_locked_changed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (VisualNode::*)(bool )>(_a, &VisualNode::docnode_visible_recursive_changed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (VisualNode::*)(const QColor & )>(_a, &VisualNode::docnode_group_color_changed, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (VisualNode::*)()>(_a, &VisualNode::bounding_rect_changed, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (VisualNode::*)(const QTransform & )>(_a, &VisualNode::transform_matrix_changed, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (VisualNode::*)(const QTransform & )>(_a, &VisualNode::group_transform_matrix_changed, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (VisualNode::*)(const QTransform & )>(_a, &VisualNode::local_transform_matrix_changed, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QColor*>(_v) = _t->get_group_color(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->get_visible(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->get_locked(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->docnode_visible_recursive(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->docnode_locked_recursive(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->docnode_selectable(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->set_group_color(*reinterpret_cast<QColor*>(_v)); break;
        case 1: _t->set_visible(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->set_locked(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::VisualNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::VisualNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model10VisualNodeE_t>.strings))
        return static_cast<void*>(this);
    return DocumentNode::qt_metacast(_clname);
}

int glaxnimate::model::VisualNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DocumentNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::model::VisualNode::docnode_visible_changed(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void glaxnimate::model::VisualNode::docnode_locked_changed(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void glaxnimate::model::VisualNode::docnode_visible_recursive_changed(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void glaxnimate::model::VisualNode::docnode_group_color_changed(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void glaxnimate::model::VisualNode::bounding_rect_changed()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void glaxnimate::model::VisualNode::transform_matrix_changed(const QTransform & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void glaxnimate::model::VisualNode::group_transform_matrix_changed(const QTransform & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void glaxnimate::model::VisualNode::local_transform_matrix_changed(const QTransform & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
