/********************************************************************************
** Form generated from reading UI file 'properties.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTIES_H
#define UI_PROPERTIES_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDockWidget>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_dock_properties
{
public:
    QWidget *dockWidgetContents;
    QVBoxLayout *verticalLayout;
    QTreeView *view_properties;

    void setupUi(QDockWidget *dock_properties)
    {
        if (dock_properties->objectName().isEmpty())
            dock_properties->setObjectName("dock_properties");
        dock_properties->resize(400, 300);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(dock_properties->sizePolicy().hasHeightForWidth());
        dock_properties->setSizePolicy(sizePolicy);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("document-properties")));
        dock_properties->setWindowIcon(icon);
        dockWidgetContents = new QWidget();
        dockWidgetContents->setObjectName("dockWidgetContents");
        verticalLayout = new QVBoxLayout(dockWidgetContents);
        verticalLayout->setObjectName("verticalLayout");
        view_properties = new QTreeView(dockWidgetContents);
        view_properties->setObjectName("view_properties");
        view_properties->setMinimumSize(QSize(333, 0));
        view_properties->setEditTriggers(QAbstractItemView::EditTrigger::DoubleClicked|QAbstractItemView::EditTrigger::EditKeyPressed|QAbstractItemView::EditTrigger::SelectedClicked);
        view_properties->setProperty("showDropIndicator", QVariant(false));
        view_properties->setAlternatingRowColors(true);

        verticalLayout->addWidget(view_properties);

        dock_properties->setWidget(dockWidgetContents);

        retranslateUi(dock_properties);

        QMetaObject::connectSlotsByName(dock_properties);
    } // setupUi

    void retranslateUi(QDockWidget *dock_properties)
    {
        dock_properties->setWindowTitle(tr2i18n("Properties", nullptr));
    } // retranslateUi

};

namespace Ui {
    class dock_properties: public Ui_dock_properties {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PROPERTIES_H

