/*
 * SPDX-FileCopyrightText: 2019-2025 Mattia Basaglia <dev@dragon.best>
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#include "rectangle_tool.hpp"

using namespace glaxnimate::gui;

tools::Autoreg<tools::RectangleTool> tools::RectangleTool::autoreg{max_priority};
