/********************************************************************************
** Form generated from reading UI file 'external_font_widget.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXTERNAL_FONT_WIDGET_H
#define UI_EXTERNAL_FONT_WIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "widgets/font/font_size_widget.hpp"

namespace glaxnimate {
namespace gui {
namespace font {

class Ui_ExternalFontWidget
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QLineEdit *edit_url;
    QPushButton *button_load;
    QPushButton *pushButton;
    QProgressBar *progress_bar;
    QWidget *widget_data;
    QGridLayout *gridLayout;
    QLabel *label_4;
    QLabel *label_2;
    QLabel *label_3;
    QTableView *view_fonts;
    QListWidget *view_style;
    glaxnimate::gui::font::FontSizeWidget *size_widget;

    void setupUi(QWidget *glaxnimate__gui__font__ExternalFontWidget)
    {
        if (glaxnimate__gui__font__ExternalFontWidget->objectName().isEmpty())
            glaxnimate__gui__font__ExternalFontWidget->setObjectName("glaxnimate__gui__font__ExternalFontWidget");
        glaxnimate__gui__font__ExternalFontWidget->resize(621, 421);
        verticalLayout = new QVBoxLayout(glaxnimate__gui__font__ExternalFontWidget);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(glaxnimate__gui__font__ExternalFontWidget);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        edit_url = new QLineEdit(glaxnimate__gui__font__ExternalFontWidget);
        edit_url->setObjectName("edit_url");

        horizontalLayout->addWidget(edit_url);

        button_load = new QPushButton(glaxnimate__gui__font__ExternalFontWidget);
        button_load->setObjectName("button_load");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("download");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_load->setIcon(icon);

        horizontalLayout->addWidget(button_load);

        pushButton = new QPushButton(glaxnimate__gui__font__ExternalFontWidget);
        pushButton->setObjectName("pushButton");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("document-open");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        pushButton->setIcon(icon1);

        horizontalLayout->addWidget(pushButton);


        verticalLayout->addLayout(horizontalLayout);

        progress_bar = new QProgressBar(glaxnimate__gui__font__ExternalFontWidget);
        progress_bar->setObjectName("progress_bar");

        verticalLayout->addWidget(progress_bar);

        widget_data = new QWidget(glaxnimate__gui__font__ExternalFontWidget);
        widget_data->setObjectName("widget_data");
        widget_data->setEnabled(false);
        gridLayout = new QGridLayout(widget_data);
        gridLayout->setObjectName("gridLayout");
        label_4 = new QLabel(widget_data);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 0, 0, 1, 1);

        label_2 = new QLabel(widget_data);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 0, 1, 1, 1);

        label_3 = new QLabel(widget_data);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 0, 2, 1, 1);

        view_fonts = new QTableView(widget_data);
        view_fonts->setObjectName("view_fonts");
        view_fonts->setAlternatingRowColors(true);
        view_fonts->setSelectionMode(QAbstractItemView::SingleSelection);
        view_fonts->setSelectionBehavior(QAbstractItemView::SelectRows);
        view_fonts->setShowGrid(false);
        view_fonts->setSortingEnabled(true);
        view_fonts->horizontalHeader()->setVisible(false);
        view_fonts->horizontalHeader()->setStretchLastSection(true);
        view_fonts->verticalHeader()->setVisible(false);

        gridLayout->addWidget(view_fonts, 1, 0, 1, 1);

        view_style = new QListWidget(widget_data);
        view_style->setObjectName("view_style");

        gridLayout->addWidget(view_style, 1, 1, 1, 1);

        size_widget = new glaxnimate::gui::font::FontSizeWidget(widget_data);
        size_widget->setObjectName("size_widget");

        gridLayout->addWidget(size_widget, 1, 2, 1, 1);

        gridLayout->setColumnStretch(0, 20);
        gridLayout->setColumnStretch(1, 5);
        gridLayout->setColumnStretch(2, 5);

        verticalLayout->addWidget(widget_data);


        retranslateUi(glaxnimate__gui__font__ExternalFontWidget);
        QObject::connect(pushButton, SIGNAL(clicked()), glaxnimate__gui__font__ExternalFontWidget, SLOT(url_from_file()));
        QObject::connect(button_load, SIGNAL(clicked()), glaxnimate__gui__font__ExternalFontWidget, SLOT(load_url()));
        QObject::connect(edit_url, SIGNAL(textChanged(QString)), glaxnimate__gui__font__ExternalFontWidget, SLOT(url_changed(QString)));

        QMetaObject::connectSlotsByName(glaxnimate__gui__font__ExternalFontWidget);
    } // setupUi

    void retranslateUi(QWidget *glaxnimate__gui__font__ExternalFontWidget)
    {
        label->setText(tr2i18n("File/URL", nullptr));
        button_load->setText(tr2i18n("Load", nullptr));
        pushButton->setText(tr2i18n("Open...", nullptr));
        label_4->setText(tr2i18n("Font", nullptr));
        label_2->setText(tr2i18n("Font Style", nullptr));
        label_3->setText(tr2i18n("Size", nullptr));
        (void)glaxnimate__gui__font__ExternalFontWidget;
    } // retranslateUi

};

} // namespace font
} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace font {
namespace Ui {
    class ExternalFontWidget: public Ui_ExternalFontWidget {};
} // namespace Ui
} // namespace font
} // namespace gui
} // namespace glaxnimate

#endif // EXTERNAL_FONT_WIDGET_H

