/********************************************************************************
** Form generated from reading UI file 'trace_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TRACE_DIALOG_H
#define UI_TRACE_DIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGraphicsView>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "QtColorWidgets/color_preview.hpp"
#include "QtColorWidgets/color_selector.hpp"

QT_BEGIN_NAMESPACE

class Ui_TraceDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *layout_main;
    QStackedWidget *stacked_preset_advanced;
    QWidget *widget_presets;
    QHBoxLayout *horizontalLayout_2;
    QListWidget *list_presets;
    QStackedWidget *stacked_preset;
    QWidget *page_7;
    QGridLayout *gridLayout_2;
    QSpinBox *spin_posterize;
    QTextBrowser *textBrowser;
    QLabel *label_8;
    QWidget *page_8;
    QVBoxLayout *verticalLayout_4;
    QTextEdit *textEdit_2;
    QWidget *page_9;
    QVBoxLayout *verticalLayout_6;
    QTextEdit *textEdit_3;
    QWidget *widget_advanced;
    QVBoxLayout *verticalLayout;
    QWidget *mode_container;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QComboBox *combo_mode;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_3;
    QStackedWidget *stacked_widget;
    QWidget *page;
    QFormLayout *formLayout_2;
    QLabel *label_4;
    color_widgets::ColorSelector *color_mono;
    QLabel *label_5;
    QCheckBox *check_inverted;
    QLabel *label_6;
    QSpinBox *spin_alpha_threshold;
    QWidget *page_2;
    QVBoxLayout *verticalLayout_5;
    QLabel *label_7;
    QListWidget *list_colors;
    QHBoxLayout *horizontalLayout_4;
    QToolButton *toolButton;
    QSpacerItem *horizontalSpacer;
    QToolButton *toolButton_2;
    QGridLayout *gridLayout;
    QSpinBox *spin_color_count;
    QDoubleSpinBox *spin_outline;
    QLabel *label_tolerance;
    QLabel *label_9;
    QSpinBox *spin_tolerance;
    QPushButton *button_detect_colors;
    QToolButton *button_quantize_options;
    QWidget *page_3;
    QGroupBox *group_potrace;
    QFormLayout *formLayout;
    QLabel *label_2;
    QDoubleSpinBox *spin_smoothness;
    QSpinBox *spin_min_area;
    QLabel *label_3;
    QGroupBox *group_preview;
    QVBoxLayout *group_preview_layout;
    QGraphicsView *preview;
    QSlider *preview_slider;
    QProgressBar *progress_bar;
    QHBoxLayout *horizontalLayout_7;
    QPushButton *button_update_preview;
    QSpacerItem *horizontalSpacer_2;
    QLabel *label_preview_zoom;
    QDoubleSpinBox *spin_preview_zoom;
    QHBoxLayout *layout_buttons;
    QPushButton *button_help;
    QPushButton *button_defaults;
    QPushButton *button_advanced;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *TraceDialog)
    {
        if (TraceDialog->objectName().isEmpty())
            TraceDialog->setObjectName("TraceDialog");
        TraceDialog->resize(771, 660);
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("bitmap-trace");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        TraceDialog->setWindowIcon(icon);
        verticalLayout_2 = new QVBoxLayout(TraceDialog);
        verticalLayout_2->setObjectName("verticalLayout_2");
        layout_main = new QHBoxLayout();
        layout_main->setObjectName("layout_main");
        stacked_preset_advanced = new QStackedWidget(TraceDialog);
        stacked_preset_advanced->setObjectName("stacked_preset_advanced");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(stacked_preset_advanced->sizePolicy().hasHeightForWidth());
        stacked_preset_advanced->setSizePolicy(sizePolicy);
        widget_presets = new QWidget();
        widget_presets->setObjectName("widget_presets");
        horizontalLayout_2 = new QHBoxLayout(widget_presets);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(0, 0, 0, -1);
        list_presets = new QListWidget(widget_presets);
        new QListWidgetItem(list_presets);
        new QListWidgetItem(list_presets);
        new QListWidgetItem(list_presets);
        list_presets->setObjectName("list_presets");
        list_presets->setMinimumSize(QSize(140, 550));
        list_presets->setMaximumSize(QSize(140, 16777215));
        list_presets->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        list_presets->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        list_presets->setIconSize(QSize(128, 128));
        list_presets->setFlow(QListView::TopToBottom);
        list_presets->setViewMode(QListView::IconMode);

        horizontalLayout_2->addWidget(list_presets);

        stacked_preset = new QStackedWidget(widget_presets);
        stacked_preset->setObjectName("stacked_preset");
        page_7 = new QWidget();
        page_7->setObjectName("page_7");
        gridLayout_2 = new QGridLayout(page_7);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        spin_posterize = new QSpinBox(page_7);
        spin_posterize->setObjectName("spin_posterize");
        spin_posterize->setMinimum(1);
        spin_posterize->setMaximum(999);
        spin_posterize->setValue(16);

        gridLayout_2->addWidget(spin_posterize, 1, 1, 1, 1);

        textBrowser = new QTextBrowser(page_7);
        textBrowser->setObjectName("textBrowser");
        textBrowser->setReadOnly(true);
        textBrowser->setTextInteractionFlags(Qt::LinksAccessibleByKeyboard|Qt::LinksAccessibleByMouse|Qt::TextBrowserInteraction|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);
        textBrowser->setOpenExternalLinks(true);

        gridLayout_2->addWidget(textBrowser, 0, 0, 1, 2);

        label_8 = new QLabel(page_7);
        label_8->setObjectName("label_8");

        gridLayout_2->addWidget(label_8, 1, 0, 1, 1);

        stacked_preset->addWidget(page_7);
        page_8 = new QWidget();
        page_8->setObjectName("page_8");
        verticalLayout_4 = new QVBoxLayout(page_8);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        textEdit_2 = new QTextEdit(page_8);
        textEdit_2->setObjectName("textEdit_2");
        textEdit_2->setReadOnly(true);
        textEdit_2->setTextInteractionFlags(Qt::LinksAccessibleByKeyboard|Qt::LinksAccessibleByMouse|Qt::TextBrowserInteraction|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        verticalLayout_4->addWidget(textEdit_2);

        stacked_preset->addWidget(page_8);
        page_9 = new QWidget();
        page_9->setObjectName("page_9");
        verticalLayout_6 = new QVBoxLayout(page_9);
        verticalLayout_6->setObjectName("verticalLayout_6");
        verticalLayout_6->setContentsMargins(0, 0, 0, 0);
        textEdit_3 = new QTextEdit(page_9);
        textEdit_3->setObjectName("textEdit_3");
        textEdit_3->setReadOnly(true);
        textEdit_3->setTextInteractionFlags(Qt::LinksAccessibleByKeyboard|Qt::LinksAccessibleByMouse|Qt::TextBrowserInteraction|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        verticalLayout_6->addWidget(textEdit_3);

        stacked_preset->addWidget(page_9);

        horizontalLayout_2->addWidget(stacked_preset);

        stacked_preset_advanced->addWidget(widget_presets);
        widget_advanced = new QWidget();
        widget_advanced->setObjectName("widget_advanced");
        verticalLayout = new QVBoxLayout(widget_advanced);
        verticalLayout->setObjectName("verticalLayout");
        mode_container = new QWidget(widget_advanced);
        mode_container->setObjectName("mode_container");
        horizontalLayout = new QHBoxLayout(mode_container);
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(mode_container);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        combo_mode = new QComboBox(mode_container);
        combo_mode->addItem(QString());
        combo_mode->addItem(QString());
        combo_mode->addItem(QString());
        combo_mode->addItem(QString());
        combo_mode->setObjectName("combo_mode");

        horizontalLayout->addWidget(combo_mode);


        verticalLayout->addWidget(mode_container);

        groupBox_2 = new QGroupBox(widget_advanced);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_3 = new QVBoxLayout(groupBox_2);
        verticalLayout_3->setObjectName("verticalLayout_3");
        stacked_widget = new QStackedWidget(groupBox_2);
        stacked_widget->setObjectName("stacked_widget");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(stacked_widget->sizePolicy().hasHeightForWidth());
        stacked_widget->setSizePolicy(sizePolicy1);
        page = new QWidget();
        page->setObjectName("page");
        formLayout_2 = new QFormLayout(page);
        formLayout_2->setObjectName("formLayout_2");
        label_4 = new QLabel(page);
        label_4->setObjectName("label_4");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, label_4);

        color_mono = new color_widgets::ColorSelector(page);
        color_mono->setObjectName("color_mono");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(color_mono->sizePolicy().hasHeightForWidth());
        color_mono->setSizePolicy(sizePolicy2);
        color_mono->setColor(QColor(0, 0, 0));

        formLayout_2->setWidget(0, QFormLayout::ItemRole::FieldRole, color_mono);

        label_5 = new QLabel(page);
        label_5->setObjectName("label_5");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::LabelRole, label_5);

        check_inverted = new QCheckBox(page);
        check_inverted->setObjectName("check_inverted");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::FieldRole, check_inverted);

        label_6 = new QLabel(page);
        label_6->setObjectName("label_6");

        formLayout_2->setWidget(2, QFormLayout::ItemRole::LabelRole, label_6);

        spin_alpha_threshold = new QSpinBox(page);
        spin_alpha_threshold->setObjectName("spin_alpha_threshold");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(spin_alpha_threshold->sizePolicy().hasHeightForWidth());
        spin_alpha_threshold->setSizePolicy(sizePolicy3);
        spin_alpha_threshold->setMaximum(255);
        spin_alpha_threshold->setValue(128);

        formLayout_2->setWidget(2, QFormLayout::ItemRole::FieldRole, spin_alpha_threshold);

        stacked_widget->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        verticalLayout_5 = new QVBoxLayout(page_2);
        verticalLayout_5->setObjectName("verticalLayout_5");
        label_7 = new QLabel(page_2);
        label_7->setObjectName("label_7");

        verticalLayout_5->addWidget(label_7);

        list_colors = new QListWidget(page_2);
        list_colors->setObjectName("list_colors");
        list_colors->setDragEnabled(true);
        list_colors->setDragDropMode(QAbstractItemView::InternalMove);
        list_colors->setDefaultDropAction(Qt::MoveAction);

        verticalLayout_5->addWidget(list_colors);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        toolButton = new QToolButton(page_2);
        toolButton->setObjectName("toolButton");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("list-add");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        toolButton->setIcon(icon1);

        horizontalLayout_4->addWidget(toolButton);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer);

        toolButton_2 = new QToolButton(page_2);
        toolButton_2->setObjectName("toolButton_2");
        QIcon icon2;
        iconThemeName = QString::fromUtf8("list-remove");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        toolButton_2->setIcon(icon2);

        horizontalLayout_4->addWidget(toolButton_2);


        verticalLayout_5->addLayout(horizontalLayout_4);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        spin_color_count = new QSpinBox(page_2);
        spin_color_count->setObjectName("spin_color_count");
        spin_color_count->setMaximum(32);

        gridLayout->addWidget(spin_color_count, 3, 2, 1, 1);

        spin_outline = new QDoubleSpinBox(page_2);
        spin_outline->setObjectName("spin_outline");
        spin_outline->setSingleStep(0.500000000000000);

        gridLayout->addWidget(spin_outline, 1, 2, 1, 1);

        label_tolerance = new QLabel(page_2);
        label_tolerance->setObjectName("label_tolerance");

        gridLayout->addWidget(label_tolerance, 0, 0, 1, 2);

        label_9 = new QLabel(page_2);
        label_9->setObjectName("label_9");

        gridLayout->addWidget(label_9, 1, 0, 1, 2);

        spin_tolerance = new QSpinBox(page_2);
        spin_tolerance->setObjectName("spin_tolerance");
        spin_tolerance->setMinimum(0);
        spin_tolerance->setMaximum(512);
        spin_tolerance->setSingleStep(10);
        spin_tolerance->setValue(10);

        gridLayout->addWidget(spin_tolerance, 0, 2, 1, 1);

        button_detect_colors = new QPushButton(page_2);
        button_detect_colors->setObjectName("button_detect_colors");

        gridLayout->addWidget(button_detect_colors, 3, 1, 1, 1);

        button_quantize_options = new QToolButton(page_2);
        button_quantize_options->setObjectName("button_quantize_options");
        QIcon icon3;
        iconThemeName = QString::fromUtf8("settings-configure");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_quantize_options->setIcon(icon3);

        gridLayout->addWidget(button_quantize_options, 3, 0, 1, 1);


        verticalLayout_5->addLayout(gridLayout);

        stacked_widget->addWidget(page_2);
        page_3 = new QWidget();
        page_3->setObjectName("page_3");
        stacked_widget->addWidget(page_3);

        verticalLayout_3->addWidget(stacked_widget);


        verticalLayout->addWidget(groupBox_2);

        group_potrace = new QGroupBox(widget_advanced);
        group_potrace->setObjectName("group_potrace");
        formLayout = new QFormLayout(group_potrace);
        formLayout->setObjectName("formLayout");
        label_2 = new QLabel(group_potrace);
        label_2->setObjectName("label_2");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label_2);

        spin_smoothness = new QDoubleSpinBox(group_potrace);
        spin_smoothness->setObjectName("spin_smoothness");
        sizePolicy3.setHeightForWidth(spin_smoothness->sizePolicy().hasHeightForWidth());
        spin_smoothness->setSizePolicy(sizePolicy3);
        spin_smoothness->setDecimals(0);
        spin_smoothness->setMaximum(100.000000000000000);
        spin_smoothness->setSingleStep(5.000000000000000);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, spin_smoothness);

        spin_min_area = new QSpinBox(group_potrace);
        spin_min_area->setObjectName("spin_min_area");
        sizePolicy3.setHeightForWidth(spin_min_area->sizePolicy().hasHeightForWidth());
        spin_min_area->setSizePolicy(sizePolicy3);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, spin_min_area);

        label_3 = new QLabel(group_potrace);
        label_3->setObjectName("label_3");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label_3);


        verticalLayout->addWidget(group_potrace);

        stacked_preset_advanced->addWidget(widget_advanced);

        layout_main->addWidget(stacked_preset_advanced);

        group_preview = new QGroupBox(TraceDialog);
        group_preview->setObjectName("group_preview");
        group_preview_layout = new QVBoxLayout(group_preview);
        group_preview_layout->setObjectName("group_preview_layout");
        preview = new QGraphicsView(group_preview);
        preview->setObjectName("preview");
        preview->setRenderHints(QPainter::Antialiasing|QPainter::TextAntialiasing);

        group_preview_layout->addWidget(preview);

        preview_slider = new QSlider(group_preview);
        preview_slider->setObjectName("preview_slider");
        preview_slider->setMaximum(100);
        preview_slider->setValue(100);
        preview_slider->setOrientation(Qt::Horizontal);

        group_preview_layout->addWidget(preview_slider);

        progress_bar = new QProgressBar(group_preview);
        progress_bar->setObjectName("progress_bar");
        progress_bar->setValue(24);

        group_preview_layout->addWidget(progress_bar);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        button_update_preview = new QPushButton(group_preview);
        button_update_preview->setObjectName("button_update_preview");
        QIcon icon4;
        iconThemeName = QString::fromUtf8("document-preview");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon4 = QIcon::fromTheme(iconThemeName);
        } else {
            icon4.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_update_preview->setIcon(icon4);

        horizontalLayout_7->addWidget(button_update_preview);

        horizontalSpacer_2 = new QSpacerItem(40, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_2);

        label_preview_zoom = new QLabel(group_preview);
        label_preview_zoom->setObjectName("label_preview_zoom");

        horizontalLayout_7->addWidget(label_preview_zoom);

        spin_preview_zoom = new QDoubleSpinBox(group_preview);
        spin_preview_zoom->setObjectName("spin_preview_zoom");
        spin_preview_zoom->setDecimals(0);
        spin_preview_zoom->setMinimum(1.000000000000000);
        spin_preview_zoom->setMaximum(10000.000000000000000);
        spin_preview_zoom->setStepType(QAbstractSpinBox::AdaptiveDecimalStepType);
        spin_preview_zoom->setValue(100.000000000000000);

        horizontalLayout_7->addWidget(spin_preview_zoom);


        group_preview_layout->addLayout(horizontalLayout_7);


        layout_main->addWidget(group_preview);


        verticalLayout_2->addLayout(layout_main);

        layout_buttons = new QHBoxLayout();
        layout_buttons->setObjectName("layout_buttons");
        button_help = new QPushButton(TraceDialog);
        button_help->setObjectName("button_help");
        QIcon icon5;
        iconThemeName = QString::fromUtf8("help-contents");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon5 = QIcon::fromTheme(iconThemeName);
        } else {
            icon5.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_help->setIcon(icon5);

        layout_buttons->addWidget(button_help);

        button_defaults = new QPushButton(TraceDialog);
        button_defaults->setObjectName("button_defaults");
        QIcon icon6;
        iconThemeName = QString::fromUtf8("kt-restore-defaults");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon6 = QIcon::fromTheme(iconThemeName);
        } else {
            icon6.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_defaults->setIcon(icon6);

        layout_buttons->addWidget(button_defaults);

        button_advanced = new QPushButton(TraceDialog);
        button_advanced->setObjectName("button_advanced");
        QIcon icon7;
        iconThemeName = QString::fromUtf8("preferences-system");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon7 = QIcon::fromTheme(iconThemeName);
        } else {
            icon7.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_advanced->setIcon(icon7);
        button_advanced->setCheckable(true);

        layout_buttons->addWidget(button_advanced);

        buttonBox = new QDialogButtonBox(TraceDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        layout_buttons->addWidget(buttonBox);


        verticalLayout_2->addLayout(layout_buttons);


        retranslateUi(TraceDialog);
        QObject::connect(button_update_preview, SIGNAL(clicked()), TraceDialog, SLOT(update_preview()));
        QObject::connect(spin_preview_zoom, SIGNAL(valueChanged(double)), TraceDialog, SLOT(zoom_preview(double)));
        QObject::connect(button_help, SIGNAL(clicked()), TraceDialog, SLOT(show_help()));
        QObject::connect(preview_slider, SIGNAL(sliderMoved(int)), TraceDialog, SLOT(preview_slide(int)));
        QObject::connect(button_defaults, SIGNAL(clicked()), TraceDialog, SLOT(reset_settings()));
        QObject::connect(list_presets, &QListWidget::currentRowChanged, stacked_preset, &QStackedWidget::setCurrentIndex);
        QObject::connect(button_advanced, SIGNAL(toggled(bool)), TraceDialog, SLOT(toggle_advanced(bool)));
        QObject::connect(list_presets, SIGNAL(currentRowChanged(int)), TraceDialog, SLOT(update_preview()));
        QObject::connect(button_detect_colors, SIGNAL(clicked()), TraceDialog, SLOT(auto_colors()));
        QObject::connect(button_quantize_options, SIGNAL(clicked()), TraceDialog, SLOT(color_options()));
        QObject::connect(toolButton, SIGNAL(clicked()), TraceDialog, SLOT(add_color()));
        QObject::connect(toolButton_2, SIGNAL(clicked()), TraceDialog, SLOT(remove_color()));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, TraceDialog, qOverload<>(&QDialog::reject));
        QObject::connect(buttonBox, SIGNAL(accepted()), TraceDialog, SLOT(apply()));

        stacked_preset_advanced->setCurrentIndex(0);
        list_presets->setCurrentRow(-1);
        stacked_preset->setCurrentIndex(1);
        stacked_widget->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(TraceDialog);
    } // setupUi

    void retranslateUi(QDialog *TraceDialog)
    {
        TraceDialog->setWindowTitle(tr2i18n("Trace Bitmap", nullptr));

        const bool __sortingEnabled = list_presets->isSortingEnabled();
        list_presets->setSortingEnabled(false);
        QListWidgetItem *___qlistwidgetitem = list_presets->item(0);
        ___qlistwidgetitem->setText(tr2i18n("Detailed", nullptr));
        QListWidgetItem *___qlistwidgetitem1 = list_presets->item(1);
        ___qlistwidgetitem1->setText(tr2i18n("Flat", nullptr));
        QListWidgetItem *___qlistwidgetitem2 = list_presets->item(2);
        ___qlistwidgetitem2->setText(tr2i18n("Pixel Art", nullptr));
        list_presets->setSortingEnabled(__sortingEnabled);

        textBrowser->setHtml(tr2i18n("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Noto Sans'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">This option is best suited for photos, paintings and detailed illustrations.</p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">The number of colors is reduced based on <span style=\" font-weight:600;\">Posterization Level</span> below.</p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left"
                        ":0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Note that there is a significant reduction in image quality when converting this kind of images to vector graphics.</p>\n"
"<p style=\" margin-top:37px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">(Image by <a href=\"https://linktr.ee/FellFallow\"><span style=\" text-decoration: underline; color:#2980b9;\">FellFallow</span></a>)</p></body></html>", nullptr));
        label_8->setText(tr2i18n("Posterization Level", nullptr));
        textEdit_2->setHtml(tr2i18n("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Noto Sans'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">This option is best suited for flat-shaded digital images, stickers, and cartoons.</p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">The number of colors will be reduced automatically, picking the most common colors in the source image.</p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-lef"
                        "t:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Very small details might get lost in the process.</p></body></html>", nullptr));
        textEdit_3->setHtml(tr2i18n("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Noto Sans'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">This is suited for pixel art. The traced vector will retain all colors in the original image as well the pixelated edges.</p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">This is only suitable for fairly small images with a low number of colors.</p></body></html>", nullptr));
        label->setText(tr2i18n("Mode", nullptr));
        combo_mode->setItemText(0, tr2i18n("Transparency", nullptr));
        combo_mode->setItemText(1, tr2i18n("Closest Color", nullptr));
        combo_mode->setItemText(2, tr2i18n("Extract Colors", nullptr));
        combo_mode->setItemText(3, tr2i18n("Pixel", nullptr));

        groupBox_2->setTitle(tr2i18n("Image Options", nullptr));
        label_4->setText(tr2i18n("Output Color", nullptr));
        label_5->setText(tr2i18n("Invert", nullptr));
        check_inverted->setText(QString());
        label_6->setText(tr2i18n("Threshold", nullptr));
        label_7->setText(tr2i18n("Colors", nullptr));
#if QT_CONFIG(tooltip)
        toolButton->setToolTip(tr2i18n("Add Color", nullptr));
#endif // QT_CONFIG(tooltip)
        toolButton->setText(tr2i18n("Add", nullptr));
#if QT_CONFIG(tooltip)
        toolButton_2->setToolTip(tr2i18n("Remove Color", nullptr));
#endif // QT_CONFIG(tooltip)
        toolButton_2->setText(tr2i18n("Remove", nullptr));
        label_tolerance->setText(tr2i18n("Tolerance", nullptr));
        label_9->setText(tr2i18n("Outline", nullptr));
        button_detect_colors->setText(tr2i18n("Detect Colors", nullptr));
#if QT_CONFIG(tooltip)
        button_quantize_options->setToolTip(tr2i18n("Configure Color Detection", nullptr));
#endif // QT_CONFIG(tooltip)
        button_quantize_options->setText(tr2i18n("Configure...", nullptr));
        group_potrace->setTitle(tr2i18n("Curve Options", nullptr));
        label_2->setText(tr2i18n("Smoothness", nullptr));
        spin_smoothness->setSuffix(tr2i18n("%", nullptr));
        label_3->setText(tr2i18n("Minimum Area", nullptr));
        group_preview->setTitle(tr2i18n("Preview", nullptr));
        button_update_preview->setText(tr2i18n("Update", nullptr));
        label_preview_zoom->setText(tr2i18n("Zoom", nullptr));
        spin_preview_zoom->setSuffix(tr2i18n("%", nullptr));
        button_help->setText(tr2i18n("Help...", nullptr));
        button_defaults->setText(tr2i18n("Revert to Defaults", nullptr));
        button_advanced->setText(tr2i18n("Advanced", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TraceDialog: public Ui_TraceDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // TRACE_DIALOG_H

