/****************************************************************************
** Meta object code from reading C++ file 'document_scene.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/gui/graphics/document_scene.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'document_scene.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate3gui8graphics13DocumentSceneE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::gui::graphics::DocumentScene::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate3gui8graphics13DocumentSceneE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::gui::graphics::DocumentScene",
        "node_user_selected",
        "",
        "std::vector<model::VisualNode*>",
        "selected",
        "deselected",
        "document_changed",
        "model::Document*",
        "new_doc",
        "old_doc",
        "drawing_background",
        "QPainter*",
        "painter",
        "QRectF",
        "rect",
        "connect_node",
        "model::DocumentNode*",
        "node",
        "disconnect_node",
        "move_node",
        "from",
        "to",
        "node_locked",
        "locked"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'node_user_selected'
        QtMocHelpers::SignalData<void(const std::vector<model::VisualNode*> &, const std::vector<model::VisualNode*> &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 5 },
        }}),
        // Signal 'document_changed'
        QtMocHelpers::SignalData<void(model::Document *, model::Document *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 7, 9 },
        }}),
        // Signal 'drawing_background'
        QtMocHelpers::SignalData<void(QPainter *, const QRectF &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 13, 14 },
        }}),
        // Slot 'connect_node'
        QtMocHelpers::SlotData<void(model::DocumentNode *)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'disconnect_node'
        QtMocHelpers::SlotData<void(model::DocumentNode *)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'move_node'
        QtMocHelpers::SlotData<void(model::DocumentNode *, int, int)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::Int, 20 }, { QMetaType::Int, 21 },
        }}),
        // Slot 'node_locked'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DocumentScene, qt_meta_tag_ZN10glaxnimate3gui8graphics13DocumentSceneE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::gui::graphics::DocumentScene::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsScene::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8graphics13DocumentSceneE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8graphics13DocumentSceneE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate3gui8graphics13DocumentSceneE_t>.metaTypes,
    nullptr
} };

void glaxnimate::gui::graphics::DocumentScene::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DocumentScene *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->node_user_selected((*reinterpret_cast<std::add_pointer_t<std::vector<model::VisualNode*>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::vector<model::VisualNode*>>>(_a[2]))); break;
        case 1: _t->document_changed((*reinterpret_cast<std::add_pointer_t<model::Document*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<model::Document*>>(_a[2]))); break;
        case 2: _t->drawing_background((*reinterpret_cast<std::add_pointer_t<QPainter*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[2]))); break;
        case 3: _t->connect_node((*reinterpret_cast<std::add_pointer_t<model::DocumentNode*>>(_a[1]))); break;
        case 4: _t->disconnect_node((*reinterpret_cast<std::add_pointer_t<model::DocumentNode*>>(_a[1]))); break;
        case 5: _t->move_node((*reinterpret_cast<std::add_pointer_t<model::DocumentNode*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 6: _t->node_locked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DocumentScene::*)(const std::vector<model::VisualNode*> & , const std::vector<model::VisualNode*> & )>(_a, &DocumentScene::node_user_selected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DocumentScene::*)(model::Document * , model::Document * )>(_a, &DocumentScene::document_changed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DocumentScene::*)(QPainter * , const QRectF & )>(_a, &DocumentScene::drawing_background, 2))
            return;
    }
}

const QMetaObject *glaxnimate::gui::graphics::DocumentScene::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::gui::graphics::DocumentScene::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8graphics13DocumentSceneE_t>.strings))
        return static_cast<void*>(this);
    return QGraphicsScene::qt_metacast(_clname);
}

int glaxnimate::gui::graphics::DocumentScene::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsScene::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::gui::graphics::DocumentScene::node_user_selected(const std::vector<model::VisualNode*> & _t1, const std::vector<model::VisualNode*> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void glaxnimate::gui::graphics::DocumentScene::document_changed(model::Document * _t1, model::Document * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void glaxnimate::gui::graphics::DocumentScene::drawing_background(QPainter * _t1, const QRectF & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}
QT_WARNING_POP
