/****************************************************************************
** Meta object code from reading C++ file 'offset_path.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/core/model/shapes/offset_path.hpp"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'offset_path.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate5model10OffsetPathE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::OffsetPath::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model10OffsetPathE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::OffsetPath",
        "amount",
        "property animated float",
        "miter_limit",
        "join",
        "property glaxnimate::model::Stroke::Join",
        "glaxnimate::model::AnimatableBase*",
        "glaxnimate::model::Stroke::Join"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'amount'
        QtMocHelpers::PropertyData<glaxnimate::model::AnimatableBase*>(1, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'miter_limit'
        QtMocHelpers::PropertyData<glaxnimate::model::AnimatableBase*>(3, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'join'
        QtMocHelpers::PropertyData<glaxnimate::model::Stroke::Join>(4, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    2 },
            {    4,    5 },
    });
    return QtMocHelpers::metaObjectData<OffsetPath, qt_meta_tag_ZN10glaxnimate5model10OffsetPathE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10glaxnimate5model10OffsetPathE[] = {
    QMetaObject::SuperData::link<glaxnimate::model::Stroke::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject glaxnimate::model::OffsetPath::staticMetaObject = { {
    QMetaObject::SuperData::link<StaticOverrides<OffsetPath,PathModifier>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model10OffsetPathE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model10OffsetPathE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10glaxnimate5model10OffsetPathE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model10OffsetPathE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::OffsetPath::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OffsetPath *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< glaxnimate::model::AnimatableBase* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<glaxnimate::model::AnimatableBase**>(_v) = _t->get_amount(); break;
        case 1: *reinterpret_cast<glaxnimate::model::AnimatableBase**>(_v) = _t->get_miter_limit(); break;
        case 2: *reinterpret_cast<glaxnimate::model::Stroke::Join*>(_v) = _t->get_join(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->set_join(*reinterpret_cast<glaxnimate::model::Stroke::Join*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::OffsetPath::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::OffsetPath::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model10OffsetPathE_t>.strings))
        return static_cast<void*>(this);
    return StaticOverrides<OffsetPath,PathModifier>::qt_metacast(_clname);
}

int glaxnimate::model::OffsetPath::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StaticOverrides<OffsetPath,PathModifier>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}
QT_WARNING_POP
