/********************************************************************************
** Form generated from reading UI file 'about_env_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ABOUT_ENV_DIALOG_H
#define UI_ABOUT_ENV_DIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace glaxnimate {
namespace gui {

class Ui_AboutEnvironmentDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    QTabWidget *tabWidget;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_3;
    QScrollArea *scrollArea_2;
    QWidget *scrollAreaWidgetContents_2;
    QVBoxLayout *verticalLayout_7;
    QLabel *label;
    QHBoxLayout *horizontalLayout_2;
    QLineEdit *line_user_data;
    QToolButton *toolButton;
    QLabel *label_3;
    QHBoxLayout *horizontalLayout_3;
    QLineEdit *line_settings;
    QToolButton *toolButton_2;
    QLabel *label_6;
    QHBoxLayout *horizontalLayout_5;
    QLineEdit *line_backup;
    QToolButton *toolButton_3;
    QLabel *label_2;
    QListWidget *view_data;
    QLabel *label_5;
    QListWidget *view_icons;
    QWidget *tab_4;
    QVBoxLayout *verticalLayout_8;
    QScrollArea *scrollArea_3;
    QWidget *scrollAreaWidgetContents_3;
    QVBoxLayout *verticalLayout_9;
    QLabel *label_7;
    QTableWidget *table_formats_input;
    QLabel *label_8;
    QTableWidget *table_formats_output;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *glaxnimate__gui__AboutEnvironmentDialog)
    {
        if (glaxnimate__gui__AboutEnvironmentDialog->objectName().isEmpty())
            glaxnimate__gui__AboutEnvironmentDialog->setObjectName("glaxnimate__gui__AboutEnvironmentDialog");
        glaxnimate__gui__AboutEnvironmentDialog->resize(505, 540);
        verticalLayout_2 = new QVBoxLayout(glaxnimate__gui__AboutEnvironmentDialog);
        verticalLayout_2->setObjectName("verticalLayout_2");
        tabWidget = new QTabWidget(glaxnimate__gui__AboutEnvironmentDialog);
        tabWidget->setObjectName("tabWidget");
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        verticalLayout_3 = new QVBoxLayout(tab_2);
        verticalLayout_3->setObjectName("verticalLayout_3");
        scrollArea_2 = new QScrollArea(tab_2);
        scrollArea_2->setObjectName("scrollArea_2");
        scrollArea_2->setWidgetResizable(true);
        scrollAreaWidgetContents_2 = new QWidget();
        scrollAreaWidgetContents_2->setObjectName("scrollAreaWidgetContents_2");
        scrollAreaWidgetContents_2->setGeometry(QRect(0, 0, 463, 438));
        verticalLayout_7 = new QVBoxLayout(scrollAreaWidgetContents_2);
        verticalLayout_7->setObjectName("verticalLayout_7");
        label = new QLabel(scrollAreaWidgetContents_2);
        label->setObjectName("label");

        verticalLayout_7->addWidget(label);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        line_user_data = new QLineEdit(scrollAreaWidgetContents_2);
        line_user_data->setObjectName("line_user_data");
        line_user_data->setReadOnly(true);

        horizontalLayout_2->addWidget(line_user_data);

        toolButton = new QToolButton(scrollAreaWidgetContents_2);
        toolButton->setObjectName("toolButton");
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("folder-open")));
        toolButton->setIcon(icon);

        horizontalLayout_2->addWidget(toolButton);


        verticalLayout_7->addLayout(horizontalLayout_2);

        label_3 = new QLabel(scrollAreaWidgetContents_2);
        label_3->setObjectName("label_3");

        verticalLayout_7->addWidget(label_3);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        line_settings = new QLineEdit(scrollAreaWidgetContents_2);
        line_settings->setObjectName("line_settings");
        line_settings->setReadOnly(true);

        horizontalLayout_3->addWidget(line_settings);

        toolButton_2 = new QToolButton(scrollAreaWidgetContents_2);
        toolButton_2->setObjectName("toolButton_2");
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("document-open")));
        toolButton_2->setIcon(icon1);

        horizontalLayout_3->addWidget(toolButton_2);


        verticalLayout_7->addLayout(horizontalLayout_3);

        label_6 = new QLabel(scrollAreaWidgetContents_2);
        label_6->setObjectName("label_6");

        verticalLayout_7->addWidget(label_6);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        line_backup = new QLineEdit(scrollAreaWidgetContents_2);
        line_backup->setObjectName("line_backup");
        line_backup->setReadOnly(true);

        horizontalLayout_5->addWidget(line_backup);

        toolButton_3 = new QToolButton(scrollAreaWidgetContents_2);
        toolButton_3->setObjectName("toolButton_3");
        toolButton_3->setIcon(icon);

        horizontalLayout_5->addWidget(toolButton_3);


        verticalLayout_7->addLayout(horizontalLayout_5);

        label_2 = new QLabel(scrollAreaWidgetContents_2);
        label_2->setObjectName("label_2");

        verticalLayout_7->addWidget(label_2);

        view_data = new QListWidget(scrollAreaWidgetContents_2);
        view_data->setObjectName("view_data");
        view_data->setVerticalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
        view_data->setHorizontalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
        view_data->setTextElideMode(Qt::TextElideMode::ElideMiddle);

        verticalLayout_7->addWidget(view_data);

        label_5 = new QLabel(scrollAreaWidgetContents_2);
        label_5->setObjectName("label_5");

        verticalLayout_7->addWidget(label_5);

        view_icons = new QListWidget(scrollAreaWidgetContents_2);
        view_icons->setObjectName("view_icons");
        view_icons->setVerticalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
        view_icons->setHorizontalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
        view_icons->setTextElideMode(Qt::TextElideMode::ElideMiddle);

        verticalLayout_7->addWidget(view_icons);

        scrollArea_2->setWidget(scrollAreaWidgetContents_2);

        verticalLayout_3->addWidget(scrollArea_2);

        tabWidget->addTab(tab_2, QString());
        tab_4 = new QWidget();
        tab_4->setObjectName("tab_4");
        verticalLayout_8 = new QVBoxLayout(tab_4);
        verticalLayout_8->setObjectName("verticalLayout_8");
        verticalLayout_8->setContentsMargins(0, 0, 0, 0);
        scrollArea_3 = new QScrollArea(tab_4);
        scrollArea_3->setObjectName("scrollArea_3");
        scrollArea_3->setFrameShape(QFrame::Shape::NoFrame);
        scrollArea_3->setWidgetResizable(true);
        scrollAreaWidgetContents_3 = new QWidget();
        scrollAreaWidgetContents_3->setObjectName("scrollAreaWidgetContents_3");
        scrollAreaWidgetContents_3->setGeometry(QRect(0, 0, 483, 458));
        verticalLayout_9 = new QVBoxLayout(scrollAreaWidgetContents_3);
        verticalLayout_9->setObjectName("verticalLayout_9");
        label_7 = new QLabel(scrollAreaWidgetContents_3);
        label_7->setObjectName("label_7");

        verticalLayout_9->addWidget(label_7);

        table_formats_input = new QTableWidget(scrollAreaWidgetContents_3);
        if (table_formats_input->columnCount() < 3)
            table_formats_input->setColumnCount(3);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        table_formats_input->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        table_formats_input->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        table_formats_input->setHorizontalHeaderItem(2, __qtablewidgetitem2);
        table_formats_input->setObjectName("table_formats_input");
        table_formats_input->setVerticalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
        table_formats_input->setEditTriggers(QAbstractItemView::EditTrigger::NoEditTriggers);
        table_formats_input->setTextElideMode(Qt::TextElideMode::ElideNone);
        table_formats_input->setHorizontalScrollMode(QAbstractItemView::ScrollMode::ScrollPerPixel);
        table_formats_input->verticalHeader()->setVisible(false);

        verticalLayout_9->addWidget(table_formats_input);

        label_8 = new QLabel(scrollAreaWidgetContents_3);
        label_8->setObjectName("label_8");

        verticalLayout_9->addWidget(label_8);

        table_formats_output = new QTableWidget(scrollAreaWidgetContents_3);
        if (table_formats_output->columnCount() < 3)
            table_formats_output->setColumnCount(3);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        table_formats_output->setHorizontalHeaderItem(0, __qtablewidgetitem3);
        QTableWidgetItem *__qtablewidgetitem4 = new QTableWidgetItem();
        table_formats_output->setHorizontalHeaderItem(1, __qtablewidgetitem4);
        QTableWidgetItem *__qtablewidgetitem5 = new QTableWidgetItem();
        table_formats_output->setHorizontalHeaderItem(2, __qtablewidgetitem5);
        table_formats_output->setObjectName("table_formats_output");
        table_formats_output->setVerticalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
        table_formats_output->setEditTriggers(QAbstractItemView::EditTrigger::NoEditTriggers);
        table_formats_output->setTextElideMode(Qt::TextElideMode::ElideNone);
        table_formats_output->setHorizontalScrollMode(QAbstractItemView::ScrollMode::ScrollPerPixel);
        table_formats_output->verticalHeader()->setVisible(false);

        verticalLayout_9->addWidget(table_formats_output);

        scrollArea_3->setWidget(scrollAreaWidgetContents_3);

        verticalLayout_8->addWidget(scrollArea_3);

        tabWidget->addTab(tab_4, QString());

        verticalLayout_2->addWidget(tabWidget);

        buttonBox = new QDialogButtonBox(glaxnimate__gui__AboutEnvironmentDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Orientation::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::StandardButton::Close);

        verticalLayout_2->addWidget(buttonBox);


        retranslateUi(glaxnimate__gui__AboutEnvironmentDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, glaxnimate__gui__AboutEnvironmentDialog, qOverload<>(&QDialog::reject));
        QObject::connect(view_data, SIGNAL(doubleClicked(QModelIndex)), glaxnimate__gui__AboutEnvironmentDialog, SLOT(dir_open(QModelIndex)));
        QObject::connect(view_icons, SIGNAL(doubleClicked(QModelIndex)), glaxnimate__gui__AboutEnvironmentDialog, SLOT(dir_open(QModelIndex)));
        QObject::connect(toolButton_2, SIGNAL(clicked()), glaxnimate__gui__AboutEnvironmentDialog, SLOT(open_settings_file()));
        QObject::connect(toolButton_3, SIGNAL(clicked()), glaxnimate__gui__AboutEnvironmentDialog, SLOT(open_backup()));
        QObject::connect(toolButton, SIGNAL(clicked()), glaxnimate__gui__AboutEnvironmentDialog, SLOT(open_user_data()));

        tabWidget->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(glaxnimate__gui__AboutEnvironmentDialog);
    } // setupUi

    void retranslateUi(QDialog *glaxnimate__gui__AboutEnvironmentDialog)
    {
        glaxnimate__gui__AboutEnvironmentDialog->setWindowTitle(tr2i18n("About Environment", nullptr));
        label->setText(tr2i18n("User Data Directory:", nullptr));
#if QT_CONFIG(tooltip)
        toolButton->setToolTip(tr2i18n("Open", nullptr));
#endif // QT_CONFIG(tooltip)
        toolButton->setText(tr2i18n("Open", nullptr));
        label_3->setText(tr2i18n("Settings File:", nullptr));
#if QT_CONFIG(tooltip)
        toolButton_2->setToolTip(tr2i18n("Open", nullptr));
#endif // QT_CONFIG(tooltip)
        toolButton_2->setText(tr2i18n("Open", nullptr));
        label_6->setText(tr2i18n("Backup Directory:", nullptr));
#if QT_CONFIG(tooltip)
        toolButton_3->setToolTip(tr2i18n("Open", nullptr));
#endif // QT_CONFIG(tooltip)
        toolButton_3->setText(tr2i18n("Open", nullptr));
        label_2->setText(tr2i18n("Data Directories:", nullptr));
        label_5->setText(tr2i18n("Icon Theme Paths:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), tr2i18n("Paths", nullptr));
        label_7->setText(tr2i18n("Input Formats", nullptr));
        QTableWidgetItem *___qtablewidgetitem = table_formats_input->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(tr2i18n("Name", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = table_formats_input->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(tr2i18n("ID", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = table_formats_input->horizontalHeaderItem(2);
        ___qtablewidgetitem2->setText(tr2i18n("Extensions", nullptr));
        label_8->setText(tr2i18n("Output Formats ", nullptr));
        QTableWidgetItem *___qtablewidgetitem3 = table_formats_output->horizontalHeaderItem(0);
        ___qtablewidgetitem3->setText(tr2i18n("Name", nullptr));
        QTableWidgetItem *___qtablewidgetitem4 = table_formats_output->horizontalHeaderItem(1);
        ___qtablewidgetitem4->setText(tr2i18n("ID", nullptr));
        QTableWidgetItem *___qtablewidgetitem5 = table_formats_output->horizontalHeaderItem(2);
        ___qtablewidgetitem5->setText(tr2i18n("Extensions", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_4), tr2i18n("Formats", nullptr));
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class AboutEnvironmentDialog: public Ui_AboutEnvironmentDialog {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // ABOUT_ENV_DIALOG_H

