/********************************************************************************
** Form generated from reading UI file 'plugin_settings_widget.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PLUGIN_SETTINGS_WIDGET_H
#define UI_PLUGIN_SETTINGS_WIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace glaxnimate {
namespace gui {

class Ui_PluginSettingsWidget
{
public:
    QHBoxLayout *horizontalLayout_2;
    QVBoxLayout *verticalLayout;
    QListWidget *list_plugins;
    QHBoxLayout *horizontalLayout;
    QToolButton *toolButton;
    QSpacerItem *horizontalSpacer;
    QToolButton *toolButton_2;
    QStackedWidget *stacked_widget;
    QWidget *page_noplugin;
    QWidget *page_plugin;
    QVBoxLayout *verticalLayout_2;
    QGroupBox *widget_plugin;
    QGridLayout *gridLayout;
    QLineEdit *line_version;
    QLineEdit *line_plugin_path;
    QHBoxLayout *horizontalLayout_3;
    QPushButton *btn_enable;
    QPushButton *btn_disable;
    QPushButton *btn_uninstall;
    QTableWidget *list_services;
    QLabel *label;
    QLineEdit *line_author;
    QPlainTextEdit *description;
    QLabel *label_3;
    QLabel *label_2;
    QLabel *label_5;
    QLabel *label_4;

    void setupUi(QWidget *PluginSettingsWidget)
    {
        if (PluginSettingsWidget->objectName().isEmpty())
            PluginSettingsWidget->setObjectName("PluginSettingsWidget");
        PluginSettingsWidget->resize(870, 735);
        horizontalLayout_2 = new QHBoxLayout(PluginSettingsWidget);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        list_plugins = new QListWidget(PluginSettingsWidget);
        list_plugins->setObjectName("list_plugins");

        verticalLayout->addWidget(list_plugins);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        toolButton = new QToolButton(PluginSettingsWidget);
        toolButton->setObjectName("toolButton");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("view-refresh");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        toolButton->setIcon(icon);

        horizontalLayout->addWidget(toolButton);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        toolButton_2 = new QToolButton(PluginSettingsWidget);
        toolButton_2->setObjectName("toolButton_2");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("install");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        toolButton_2->setIcon(icon1);

        horizontalLayout->addWidget(toolButton_2);


        verticalLayout->addLayout(horizontalLayout);


        horizontalLayout_2->addLayout(verticalLayout);

        stacked_widget = new QStackedWidget(PluginSettingsWidget);
        stacked_widget->setObjectName("stacked_widget");
        page_noplugin = new QWidget();
        page_noplugin->setObjectName("page_noplugin");
        stacked_widget->addWidget(page_noplugin);
        page_plugin = new QWidget();
        page_plugin->setObjectName("page_plugin");
        verticalLayout_2 = new QVBoxLayout(page_plugin);
        verticalLayout_2->setObjectName("verticalLayout_2");
        widget_plugin = new QGroupBox(page_plugin);
        widget_plugin->setObjectName("widget_plugin");
        widget_plugin->setTitle(QString::fromUtf8(" Plugin Name"));
        gridLayout = new QGridLayout(widget_plugin);
        gridLayout->setObjectName("gridLayout");
        line_version = new QLineEdit(widget_plugin);
        line_version->setObjectName("line_version");
        line_version->setReadOnly(true);

        gridLayout->addWidget(line_version, 1, 1, 1, 1);

        line_plugin_path = new QLineEdit(widget_plugin);
        line_plugin_path->setObjectName("line_plugin_path");
        line_plugin_path->setText(QString::fromUtf8("/path/to/the/plugin"));
        line_plugin_path->setReadOnly(true);

        gridLayout->addWidget(line_plugin_path, 0, 1, 1, 1);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        btn_enable = new QPushButton(widget_plugin);
        btn_enable->setObjectName("btn_enable");
        QIcon icon2;
        iconThemeName = QString::fromUtf8("package-install");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_enable->setIcon(icon2);

        horizontalLayout_3->addWidget(btn_enable);

        btn_disable = new QPushButton(widget_plugin);
        btn_disable->setObjectName("btn_disable");
        QIcon icon3;
        iconThemeName = QString::fromUtf8("package-remove");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_disable->setIcon(icon3);

        horizontalLayout_3->addWidget(btn_disable);

        btn_uninstall = new QPushButton(widget_plugin);
        btn_uninstall->setObjectName("btn_uninstall");
        QIcon icon4;
        iconThemeName = QString::fromUtf8("uninstall");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon4 = QIcon::fromTheme(iconThemeName);
        } else {
            icon4.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_uninstall->setIcon(icon4);

        horizontalLayout_3->addWidget(btn_uninstall);


        gridLayout->addLayout(horizontalLayout_3, 7, 0, 1, 2);

        list_services = new QTableWidget(widget_plugin);
        if (list_services->columnCount() < 2)
            list_services->setColumnCount(2);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        list_services->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        list_services->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        list_services->setObjectName("list_services");
        list_services->setSelectionMode(QAbstractItemView::NoSelection);
        list_services->setSelectionBehavior(QAbstractItemView::SelectRows);
        list_services->setShowGrid(false);
        list_services->horizontalHeader()->setStretchLastSection(true);
        list_services->verticalHeader()->setVisible(false);

        gridLayout->addWidget(list_services, 6, 0, 1, 2);

        label = new QLabel(widget_plugin);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        line_author = new QLineEdit(widget_plugin);
        line_author->setObjectName("line_author");
        line_author->setText(QString::fromUtf8("John Doe"));
        line_author->setReadOnly(true);

        gridLayout->addWidget(line_author, 2, 1, 1, 1);

        description = new QPlainTextEdit(widget_plugin);
        description->setObjectName("description");
        description->setReadOnly(true);

        gridLayout->addWidget(description, 4, 0, 1, 2);

        label_3 = new QLabel(widget_plugin);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 2, 0, 1, 1);

        label_2 = new QLabel(widget_plugin);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        label_5 = new QLabel(widget_plugin);
        label_5->setObjectName("label_5");

        gridLayout->addWidget(label_5, 3, 0, 1, 2);

        label_4 = new QLabel(widget_plugin);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 5, 0, 1, 2);


        verticalLayout_2->addWidget(widget_plugin);

        stacked_widget->addWidget(page_plugin);

        horizontalLayout_2->addWidget(stacked_widget);

        horizontalLayout_2->setStretch(0, 1);
        horizontalLayout_2->setStretch(1, 2);

        retranslateUi(PluginSettingsWidget);
        QObject::connect(toolButton, SIGNAL(clicked()), PluginSettingsWidget, SLOT(refresh_plugins()));
        QObject::connect(toolButton_2, SIGNAL(clicked()), PluginSettingsWidget, SLOT(install_dialog()));
        QObject::connect(btn_enable, SIGNAL(clicked()), PluginSettingsWidget, SLOT(enable_current()));
        QObject::connect(btn_disable, SIGNAL(clicked()), PluginSettingsWidget, SLOT(disable_current()));
        QObject::connect(btn_uninstall, SIGNAL(clicked()), PluginSettingsWidget, SLOT(uninstall_current()));
        QObject::connect(list_plugins, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)), PluginSettingsWidget, SLOT(current_changed(QListWidgetItem*)));
        QObject::connect(list_plugins, SIGNAL(itemClicked(QListWidgetItem*)), PluginSettingsWidget, SLOT(current_changed(QListWidgetItem*)));

        stacked_widget->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(PluginSettingsWidget);
    } // setupUi

    void retranslateUi(QWidget *PluginSettingsWidget)
    {
#if QT_CONFIG(tooltip)
        toolButton->setToolTip(tr2i18n("Reload Plugins", nullptr));
#endif // QT_CONFIG(tooltip)
        toolButton->setText(tr2i18n("Refresh", nullptr));
#if QT_CONFIG(tooltip)
        toolButton_2->setToolTip(tr2i18n("Install from file", nullptr));
#endif // QT_CONFIG(tooltip)
        toolButton_2->setText(tr2i18n("Install...", nullptr));
        line_version->setText(tr2i18n("123", nullptr));
        btn_enable->setText(tr2i18n("Enable", nullptr));
        btn_disable->setText(tr2i18n("Disable", nullptr));
        btn_uninstall->setText(tr2i18n("Uninstall", nullptr));
        QTableWidgetItem *___qtablewidgetitem = list_services->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(tr2i18n("Type", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = list_services->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(tr2i18n("Name", nullptr));
        label->setText(tr2i18n("Path", nullptr));
        label_3->setText(tr2i18n("Author", nullptr));
        label_2->setText(tr2i18n("Version", nullptr));
        label_5->setText(tr2i18n("Description", nullptr));
        label_4->setText(tr2i18n("Services", nullptr));
        (void)PluginSettingsWidget;
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class PluginSettingsWidget: public Ui_PluginSettingsWidget {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // PLUGIN_SETTINGS_WIDGET_H

