// This file is generated by kconfig_compiler_kf6 from glaxnimate_settings.kcfg.
// All changes you do to this file will be lost.
#ifndef GLAXNIMATE_GUI_GLAXNIMATESETTINGS_H
#define GLAXNIMATE_GUI_GLAXNIMATESETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

#include <QFontInfo>
#include <api_keys.hpp>
#include <settings/base.hpp>

namespace glaxnimate {
namespace gui {

class GlaxnimateSettings : public glaxnimate::gui::settings::GlaxnimateSettingsBase
{
  public:

    static GlaxnimateSettings *self();
    ~GlaxnimateSettings() override;

    /**
      Set startup_dialog
    */
    static
    void setStartup_dialog( bool v )
    {
      if (!self()->isStartup_dialogImmutable())
        self()->mStartup_dialog = v;
    }

    /**
      Get startup_dialog
    */
    static
    bool startup_dialog()
    {
      return self()->mStartup_dialog;
    }

    /**
      Is startup_dialog Immutable
    */
    static
    bool isStartup_dialogImmutable()
    {
      return self()->isImmutable( QStringLiteral( "startup_dialog" ) );
    }

    /**
      Get startup_dialog default value
    */
    static
    bool defaultStartup_dialogValue()
    {
        return defaultStartup_dialogValue_helper();
    }

    /**
      Get Item object corresponding to startup_dialog()
    */
    ItemBool *startup_dialogItem()
    {
      return mStartup_dialogItem;
    }

    /**
      Set timeline_scroll_horizontal
    */
    static
    void setTimeline_scroll_horizontal( bool v )
    {
      if (!self()->isTimeline_scroll_horizontalImmutable())
        self()->mTimeline_scroll_horizontal = v;
    }

    /**
      Get timeline_scroll_horizontal
    */
    static
    bool timeline_scroll_horizontal()
    {
      return self()->mTimeline_scroll_horizontal;
    }

    /**
      Is timeline_scroll_horizontal Immutable
    */
    static
    bool isTimeline_scroll_horizontalImmutable()
    {
      return self()->isImmutable( QStringLiteral( "timeline_scroll_horizontal" ) );
    }

    /**
      Get timeline_scroll_horizontal default value
    */
    static
    bool defaultTimeline_scroll_horizontalValue()
    {
        return defaultTimeline_scroll_horizontalValue_helper();
    }

    /**
      Get Item object corresponding to timeline_scroll_horizontal()
    */
    ItemBool *timeline_scroll_horizontalItem()
    {
      return mTimeline_scroll_horizontalItem;
    }

    /**
      Set debug_timeline
    */
    static
    void setDebug_timeline( bool v )
    {
      if (!self()->isDebug_timelineImmutable())
        self()->mDebug_timeline = v;
    }

    /**
      Get debug_timeline
    */
    static
    bool debug_timeline()
    {
      return self()->mDebug_timeline;
    }

    /**
      Is debug_timeline Immutable
    */
    static
    bool isDebug_timelineImmutable()
    {
      return self()->isImmutable( QStringLiteral( "debug_timeline" ) );
    }

    /**
      Get debug_timeline default value
    */
    static
    bool defaultDebug_timelineValue()
    {
        return defaultDebug_timelineValue_helper();
    }

    /**
      Get Item object corresponding to debug_timeline()
    */
    ItemBool *debug_timelineItem()
    {
      return mDebug_timelineItem;
    }

    /**
      Set layout
    */
    static
    void setLayout( int v )
    {
      if (!self()->isLayoutImmutable())
        self()->mLayout = v;
    }

    /**
      Get layout
    */
    static
    int layout()
    {
      return self()->mLayout;
    }

    /**
      Is layout Immutable
    */
    static
    bool isLayoutImmutable()
    {
      return self()->isImmutable( QStringLiteral( "layout" ) );
    }

    /**
      Get layout default value
    */
    static
    int defaultLayoutValue()
    {
        return defaultLayoutValue_helper();
    }

    /**
      Get Item object corresponding to layout()
    */
    ItemInt *layoutItem()
    {
      return mLayoutItem;
    }

    /**
      Set window_state
    */
    static
    void setWindow_state( const QString & v )
    {
      if (!self()->isWindow_stateImmutable())
        self()->mWindow_state = v;
    }

    /**
      Get window_state
    */
    static
    QString window_state()
    {
      return self()->mWindow_state;
    }

    /**
      Is window_state Immutable
    */
    static
    bool isWindow_stateImmutable()
    {
      return self()->isImmutable( QStringLiteral( "window_state" ) );
    }

    /**
      Get Item object corresponding to window_state()
    */
    ItemString *window_stateItem()
    {
      return mWindow_stateItem;
    }

    /**
      Set timeline_splitter
    */
    static
    void setTimeline_splitter( const QString & v )
    {
      if (!self()->isTimeline_splitterImmutable())
        self()->mTimeline_splitter = v;
    }

    /**
      Get timeline_splitter
    */
    static
    QString timeline_splitter()
    {
      return self()->mTimeline_splitter;
    }

    /**
      Is timeline_splitter Immutable
    */
    static
    bool isTimeline_splitterImmutable()
    {
      return self()->isImmutable( QStringLiteral( "timeline_splitter" ) );
    }

    /**
      Get Item object corresponding to timeline_splitter()
    */
    ItemString *timeline_splitterItem()
    {
      return mTimeline_splitterItem;
    }

    /**
      Set icon_theme
    */
    static
    void setIcon_theme( const QString & v )
    {
      if (!self()->isIcon_themeImmutable())
        self()->mIcon_theme = v;
    }

    /**
      Get icon_theme
    */
    static
    QString icon_theme()
    {
      return self()->mIcon_theme;
    }

    /**
      Is icon_theme Immutable
    */
    static
    bool isIcon_themeImmutable()
    {
      return self()->isImmutable( QStringLiteral( "icon_theme" ) );
    }

    /**
      Get Item object corresponding to icon_theme()
    */
    ItemString *icon_themeItem()
    {
      return mIcon_themeItem;
    }

    /**
      Set width
    */
    static
    void setWidth( int v )
    {
      if (v < 0)
      {
        qDebug() << "setWidth: value " << v << " is less than the minimum value of 0";
        v = 0;
      }

      if (v > 999999)
      {
        qDebug() << "setWidth: value " << v << " is greater than the maximum value of 999999";
        v = 999999;
      }

      if (!self()->isWidthImmutable())
        self()->mWidth = v;
    }

    /**
      Get width
    */
    static
    int width()
    {
      return self()->mWidth;
    }

    /**
      Is width Immutable
    */
    static
    bool isWidthImmutable()
    {
      return self()->isImmutable( QStringLiteral( "width" ) );
    }

    /**
      Get width default value
    */
    static
    int defaultWidthValue()
    {
        return defaultWidthValue_helper();
    }

    /**
      Get Item object corresponding to width()
    */
    ItemInt *widthItem()
    {
      return mWidthItem;
    }

    /**
      Set height
    */
    static
    void setHeight( int v )
    {
      if (v < 0)
      {
        qDebug() << "setHeight: value " << v << " is less than the minimum value of 0";
        v = 0;
      }

      if (v > 999999)
      {
        qDebug() << "setHeight: value " << v << " is greater than the maximum value of 999999";
        v = 999999;
      }

      if (!self()->isHeightImmutable())
        self()->mHeight = v;
    }

    /**
      Get height
    */
    static
    int height()
    {
      return self()->mHeight;
    }

    /**
      Is height Immutable
    */
    static
    bool isHeightImmutable()
    {
      return self()->isImmutable( QStringLiteral( "height" ) );
    }

    /**
      Get height default value
    */
    static
    int defaultHeightValue()
    {
        return defaultHeightValue_helper();
    }

    /**
      Get Item object corresponding to height()
    */
    ItemInt *heightItem()
    {
      return mHeightItem;
    }

    /**
      Set fps
    */
    static
    void setFps( double v )
    {
      if (v < 1)
      {
        qDebug() << "setFps: value " << v << " is less than the minimum value of 1";
        v = 1;
      }

      if (v > 999)
      {
        qDebug() << "setFps: value " << v << " is greater than the maximum value of 999";
        v = 999;
      }

      if (!self()->isFpsImmutable())
        self()->mFps = v;
    }

    /**
      Get fps
    */
    static
    double fps()
    {
      return self()->mFps;
    }

    /**
      Is fps Immutable
    */
    static
    bool isFpsImmutable()
    {
      return self()->isImmutable( QStringLiteral( "fps" ) );
    }

    /**
      Get fps default value
    */
    static
    double defaultFpsValue()
    {
        return defaultFpsValue_helper();
    }

    /**
      Get Item object corresponding to fps()
    */
    ItemDouble *fpsItem()
    {
      return mFpsItem;
    }

    /**
      Set duration
    */
    static
    void setDuration( double v )
    {
      if (v < 0)
      {
        qDebug() << "setDuration: value " << v << " is less than the minimum value of 0";
        v = 0;
      }
      if (!self()->isDurationImmutable())
        self()->mDuration = v;
    }

    /**
      Get duration
    */
    static
    double duration()
    {
      return self()->mDuration;
    }

    /**
      Is duration Immutable
    */
    static
    bool isDurationImmutable()
    {
      return self()->isImmutable( QStringLiteral( "duration" ) );
    }

    /**
      Get duration default value
    */
    static
    double defaultDurationValue()
    {
        return defaultDurationValue_helper();
    }

    /**
      Get Item object corresponding to duration()
    */
    ItemDouble *durationItem()
    {
      return mDurationItem;
    }

    /**
      Set backup_frequency
    */
    static
    void setBackup_frequency( int v )
    {
      if (!self()->isBackup_frequencyImmutable())
        self()->mBackup_frequency = v;
    }

    /**
      Get backup_frequency
    */
    static
    int backup_frequency()
    {
      return self()->mBackup_frequency;
    }

    /**
      Is backup_frequency Immutable
    */
    static
    bool isBackup_frequencyImmutable()
    {
      return self()->isImmutable( QStringLiteral( "backup_frequency" ) );
    }

    /**
      Get backup_frequency default value
    */
    static
    int defaultBackup_frequencyValue()
    {
        return defaultBackup_frequencyValue_helper();
    }

    /**
      Get Item object corresponding to backup_frequency()
    */
    ItemInt *backup_frequencyItem()
    {
      return mBackup_frequencyItem;
    }

    /**
      Set use_native_io_dialog
    */
    static
    void setUse_native_io_dialog( bool v )
    {
      if (!self()->isUse_native_io_dialogImmutable())
        self()->mUse_native_io_dialog = v;
    }

    /**
      Get use_native_io_dialog
    */
    static
    bool use_native_io_dialog()
    {
      return self()->mUse_native_io_dialog;
    }

    /**
      Is use_native_io_dialog Immutable
    */
    static
    bool isUse_native_io_dialogImmutable()
    {
      return self()->isImmutable( QStringLiteral( "use_native_io_dialog" ) );
    }

    /**
      Get use_native_io_dialog default value
    */
    static
    bool defaultUse_native_io_dialogValue()
    {
        return defaultUse_native_io_dialogValue_helper();
    }

    /**
      Get Item object corresponding to use_native_io_dialog()
    */
    ItemBool *use_native_io_dialogItem()
    {
      return mUse_native_io_dialogItem;
    }

    /**
      Set path
    */
    static
    void setPath( const QString & v )
    {
      if (!self()->isPathImmutable())
        self()->mPath = v;
    }

    /**
      Get path
    */
    static
    QString path()
    {
      return self()->mPath;
    }

    /**
      Is path Immutable
    */
    static
    bool isPathImmutable()
    {
      return self()->isImmutable( QStringLiteral( "path" ) );
    }

    /**
      Get Item object corresponding to path()
    */
    ItemPath *pathItem()
    {
      return mPathItem;
    }

    /**
      Set render_path
    */
    static
    void setRender_path( const QString & v )
    {
      if (!self()->isRender_pathImmutable())
        self()->mRender_path = v;
    }

    /**
      Get render_path
    */
    static
    QString render_path()
    {
      return self()->mRender_path;
    }

    /**
      Is render_path Immutable
    */
    static
    bool isRender_pathImmutable()
    {
      return self()->isImmutable( QStringLiteral( "render_path" ) );
    }

    /**
      Get Item object corresponding to render_path()
    */
    ItemPath *render_pathItem()
    {
      return mRender_pathItem;
    }

    /**
      Set import_path
    */
    static
    void setImport_path( const QString & v )
    {
      if (!self()->isImport_pathImmutable())
        self()->mImport_path = v;
    }

    /**
      Get import_path
    */
    static
    QString import_path()
    {
      return self()->mImport_path;
    }

    /**
      Is import_path Immutable
    */
    static
    bool isImport_pathImmutable()
    {
      return self()->isImmutable( QStringLiteral( "import_path" ) );
    }

    /**
      Get Item object corresponding to import_path()
    */
    ItemPath *import_pathItem()
    {
      return mImport_pathItem;
    }

    /**
      Set script_max_history
    */
    static
    void setScript_max_history( int v )
    {
      if (!self()->isScript_max_historyImmutable())
        self()->mScript_max_history = v;
    }

    /**
      Get script_max_history
    */
    static
    int script_max_history()
    {
      return self()->mScript_max_history;
    }

    /**
      Is script_max_history Immutable
    */
    static
    bool isScript_max_historyImmutable()
    {
      return self()->isImmutable( QStringLiteral( "script_max_history" ) );
    }

    /**
      Get script_max_history default value
    */
    static
    int defaultScript_max_historyValue()
    {
        return defaultScript_max_historyValue_helper();
    }

    /**
      Get Item object corresponding to script_max_history()
    */
    ItemInt *script_max_historyItem()
    {
      return mScript_max_historyItem;
    }

    /**
      Set script_history
    */
    static
    void setScript_history( const QStringList & v )
    {
      if (!self()->isScript_historyImmutable())
        self()->mScript_history = v;
    }

    /**
      Get script_history
    */
    static
    QStringList script_history()
    {
      return self()->mScript_history;
    }

    /**
      Is script_history Immutable
    */
    static
    bool isScript_historyImmutable()
    {
      return self()->isImmutable( QStringLiteral( "script_history" ) );
    }

    /**
      Get Item object corresponding to script_history()
    */
    ItemStringList *script_historyItem()
    {
      return mScript_historyItem;
    }

    /**
      Set shape_group
    */
    static
    void setShape_group( bool v )
    {
      if (!self()->isShape_groupImmutable())
        self()->mShape_group = v;
    }

    /**
      Get shape_group
    */
    static
    bool shape_group()
    {
      return self()->mShape_group;
    }

    /**
      Is shape_group Immutable
    */
    static
    bool isShape_groupImmutable()
    {
      return self()->isImmutable( QStringLiteral( "shape_group" ) );
    }

    /**
      Get shape_group default value
    */
    static
    bool defaultShape_groupValue()
    {
        return defaultShape_groupValue_helper();
    }

    /**
      Get Item object corresponding to shape_group()
    */
    ItemBool *shape_groupItem()
    {
      return mShape_groupItem;
    }

    /**
      Set shape_fill
    */
    static
    void setShape_fill( bool v )
    {
      if (!self()->isShape_fillImmutable())
        self()->mShape_fill = v;
    }

    /**
      Get shape_fill
    */
    static
    bool shape_fill()
    {
      return self()->mShape_fill;
    }

    /**
      Is shape_fill Immutable
    */
    static
    bool isShape_fillImmutable()
    {
      return self()->isImmutable( QStringLiteral( "shape_fill" ) );
    }

    /**
      Get shape_fill default value
    */
    static
    bool defaultShape_fillValue()
    {
        return defaultShape_fillValue_helper();
    }

    /**
      Get Item object corresponding to shape_fill()
    */
    ItemBool *shape_fillItem()
    {
      return mShape_fillItem;
    }

    /**
      Set shape_stroke
    */
    static
    void setShape_stroke( bool v )
    {
      if (!self()->isShape_strokeImmutable())
        self()->mShape_stroke = v;
    }

    /**
      Get shape_stroke
    */
    static
    bool shape_stroke()
    {
      return self()->mShape_stroke;
    }

    /**
      Is shape_stroke Immutable
    */
    static
    bool isShape_strokeImmutable()
    {
      return self()->isImmutable( QStringLiteral( "shape_stroke" ) );
    }

    /**
      Get shape_stroke default value
    */
    static
    bool defaultShape_strokeValue()
    {
        return defaultShape_strokeValue_helper();
    }

    /**
      Get Item object corresponding to shape_stroke()
    */
    ItemBool *shape_strokeItem()
    {
      return mShape_strokeItem;
    }

    /**
      Set edit_mask
    */
    static
    void setEdit_mask( bool v )
    {
      if (!self()->isEdit_maskImmutable())
        self()->mEdit_mask = v;
    }

    /**
      Get edit_mask
    */
    static
    bool edit_mask()
    {
      return self()->mEdit_mask;
    }

    /**
      Is edit_mask Immutable
    */
    static
    bool isEdit_maskImmutable()
    {
      return self()->isImmutable( QStringLiteral( "edit_mask" ) );
    }

    /**
      Get edit_mask default value
    */
    static
    bool defaultEdit_maskValue()
    {
        return defaultEdit_maskValue_helper();
    }

    /**
      Get Item object corresponding to edit_mask()
    */
    ItemBool *edit_maskItem()
    {
      return mEdit_maskItem;
    }

    /**
      Set color_main
    */
    static
    void setColor_main( const QColor & v )
    {
      if (!self()->isColor_mainImmutable())
        self()->mColor_main = v;
    }

    /**
      Get color_main
    */
    static
    QColor color_main()
    {
      return self()->mColor_main;
    }

    /**
      Is color_main Immutable
    */
    static
    bool isColor_mainImmutable()
    {
      return self()->isImmutable( QStringLiteral( "color_main" ) );
    }

    /**
      Get color_main default value
    */
    static
    QColor defaultColor_mainValue()
    {
        return defaultColor_mainValue_helper();
    }

    /**
      Get Item object corresponding to color_main()
    */
    ItemColor *color_mainItem()
    {
      return mColor_mainItem;
    }

    /**
      Set color_secondary
    */
    static
    void setColor_secondary( const QColor & v )
    {
      if (!self()->isColor_secondaryImmutable())
        self()->mColor_secondary = v;
    }

    /**
      Get color_secondary
    */
    static
    QColor color_secondary()
    {
      return self()->mColor_secondary;
    }

    /**
      Is color_secondary Immutable
    */
    static
    bool isColor_secondaryImmutable()
    {
      return self()->isImmutable( QStringLiteral( "color_secondary" ) );
    }

    /**
      Get color_secondary default value
    */
    static
    QColor defaultColor_secondaryValue()
    {
        return defaultColor_secondaryValue_helper();
    }

    /**
      Get Item object corresponding to color_secondary()
    */
    ItemColor *color_secondaryItem()
    {
      return mColor_secondaryItem;
    }

    /**
      Set stroke_width
    */
    static
    void setStroke_width( double v )
    {
      if (!self()->isStroke_widthImmutable())
        self()->mStroke_width = v;
    }

    /**
      Get stroke_width
    */
    static
    double stroke_width()
    {
      return self()->mStroke_width;
    }

    /**
      Is stroke_width Immutable
    */
    static
    bool isStroke_widthImmutable()
    {
      return self()->isImmutable( QStringLiteral( "stroke_width" ) );
    }

    /**
      Get stroke_width default value
    */
    static
    double defaultStroke_widthValue()
    {
        return defaultStroke_widthValue_helper();
    }

    /**
      Get Item object corresponding to stroke_width()
    */
    ItemDouble *stroke_widthItem()
    {
      return mStroke_widthItem;
    }

    /**
      Set stroke_cap
    */
    static
    void setStroke_cap( int v )
    {
      if (!self()->isStroke_capImmutable())
        self()->mStroke_cap = v;
    }

    /**
      Get stroke_cap
    */
    static
    int stroke_cap()
    {
      return self()->mStroke_cap;
    }

    /**
      Is stroke_cap Immutable
    */
    static
    bool isStroke_capImmutable()
    {
      return self()->isImmutable( QStringLiteral( "stroke_cap" ) );
    }

    /**
      Get stroke_cap default value
    */
    static
    int defaultStroke_capValue()
    {
        return defaultStroke_capValue_helper();
    }

    /**
      Get Item object corresponding to stroke_cap()
    */
    ItemEnum *stroke_capItem()
    {
      return mStroke_capItem;
    }

    /**
      Set stroke_join
    */
    static
    void setStroke_join( int v )
    {
      if (!self()->isStroke_joinImmutable())
        self()->mStroke_join = v;
    }

    /**
      Get stroke_join
    */
    static
    int stroke_join()
    {
      return self()->mStroke_join;
    }

    /**
      Is stroke_join Immutable
    */
    static
    bool isStroke_joinImmutable()
    {
      return self()->isImmutable( QStringLiteral( "stroke_join" ) );
    }

    /**
      Get stroke_join default value
    */
    static
    int defaultStroke_joinValue()
    {
        return defaultStroke_joinValue_helper();
    }

    /**
      Get Item object corresponding to stroke_join()
    */
    ItemEnum *stroke_joinItem()
    {
      return mStroke_joinItem;
    }

    /**
      Set stroke_miter
    */
    static
    void setStroke_miter( double v )
    {
      if (!self()->isStroke_miterImmutable())
        self()->mStroke_miter = v;
    }

    /**
      Get stroke_miter
    */
    static
    double stroke_miter()
    {
      return self()->mStroke_miter;
    }

    /**
      Is stroke_miter Immutable
    */
    static
    bool isStroke_miterImmutable()
    {
      return self()->isImmutable( QStringLiteral( "stroke_miter" ) );
    }

    /**
      Get stroke_miter default value
    */
    static
    double defaultStroke_miterValue()
    {
        return defaultStroke_miterValue_helper();
    }

    /**
      Get Item object corresponding to stroke_miter()
    */
    ItemDouble *stroke_miterItem()
    {
      return mStroke_miterItem;
    }

    /**
      Set star_type
    */
    static
    void setStar_type( int v )
    {
      if (!self()->isStar_typeImmutable())
        self()->mStar_type = v;
    }

    /**
      Get star_type
    */
    static
    int star_type()
    {
      return self()->mStar_type;
    }

    /**
      Is star_type Immutable
    */
    static
    bool isStar_typeImmutable()
    {
      return self()->isImmutable( QStringLiteral( "star_type" ) );
    }

    /**
      Get star_type default value
    */
    static
    int defaultStar_typeValue()
    {
        return defaultStar_typeValue_helper();
    }

    /**
      Get Item object corresponding to star_type()
    */
    ItemInt *star_typeItem()
    {
      return mStar_typeItem;
    }

    /**
      Set star_ratio
    */
    static
    void setStar_ratio( double v )
    {
      if (!self()->isStar_ratioImmutable())
        self()->mStar_ratio = v;
    }

    /**
      Get star_ratio
    */
    static
    double star_ratio()
    {
      return self()->mStar_ratio;
    }

    /**
      Is star_ratio Immutable
    */
    static
    bool isStar_ratioImmutable()
    {
      return self()->isImmutable( QStringLiteral( "star_ratio" ) );
    }

    /**
      Get star_ratio default value
    */
    static
    double defaultStar_ratioValue()
    {
        return defaultStar_ratioValue_helper();
    }

    /**
      Get Item object corresponding to star_ratio()
    */
    ItemDouble *star_ratioItem()
    {
      return mStar_ratioItem;
    }

    /**
      Set star_points
    */
    static
    void setStar_points( int v )
    {
      if (!self()->isStar_pointsImmutable())
        self()->mStar_points = v;
    }

    /**
      Get star_points
    */
    static
    int star_points()
    {
      return self()->mStar_points;
    }

    /**
      Is star_points Immutable
    */
    static
    bool isStar_pointsImmutable()
    {
      return self()->isImmutable( QStringLiteral( "star_points" ) );
    }

    /**
      Get star_points default value
    */
    static
    int defaultStar_pointsValue()
    {
        return defaultStar_pointsValue_helper();
    }

    /**
      Get Item object corresponding to star_points()
    */
    ItemInt *star_pointsItem()
    {
      return mStar_pointsItem;
    }

    /**
      Set favourite_font_families
    */
    static
    void setFavourite_font_families( const QStringList & v )
    {
      if (!self()->isFavourite_font_familiesImmutable())
        self()->mFavourite_font_families = v;
    }

    /**
      Get favourite_font_families
    */
    static
    QStringList favourite_font_families()
    {
      return self()->mFavourite_font_families;
    }

    /**
      Is favourite_font_families Immutable
    */
    static
    bool isFavourite_font_familiesImmutable()
    {
      return self()->isImmutable( QStringLiteral( "favourite_font_families" ) );
    }

    /**
      Get Item object corresponding to favourite_font_families()
    */
    ItemStringList *favourite_font_familiesItem()
    {
      return mFavourite_font_familiesItem;
    }

    /**
      Set text_family
    */
    static
    void setText_family( const QString & v )
    {
      if (!self()->isText_familyImmutable())
        self()->mText_family = v;
    }

    /**
      Get text_family
    */
    static
    QString text_family()
    {
      return self()->mText_family;
    }

    /**
      Is text_family Immutable
    */
    static
    bool isText_familyImmutable()
    {
      return self()->isImmutable( QStringLiteral( "text_family" ) );
    }

    /**
      Get text_family default value
    */
    static
    QString defaultText_familyValue()
    {
        return defaultText_familyValue_helper();
    }

    /**
      Get Item object corresponding to text_family()
    */
    ItemString *text_familyItem()
    {
      return mText_familyItem;
    }

    /**
      Set text_style
    */
    static
    void setText_style( const QString & v )
    {
      if (!self()->isText_styleImmutable())
        self()->mText_style = v;
    }

    /**
      Get text_style
    */
    static
    QString text_style()
    {
      return self()->mText_style;
    }

    /**
      Is text_style Immutable
    */
    static
    bool isText_styleImmutable()
    {
      return self()->isImmutable( QStringLiteral( "text_style" ) );
    }

    /**
      Get text_style default value
    */
    static
    QString defaultText_styleValue()
    {
        return defaultText_styleValue_helper();
    }

    /**
      Get Item object corresponding to text_style()
    */
    ItemString *text_styleItem()
    {
      return mText_styleItem;
    }

    /**
      Set text_size
    */
    static
    void setText_size( double v )
    {
      if (!self()->isText_sizeImmutable())
        self()->mText_size = v;
    }

    /**
      Get text_size
    */
    static
    double text_size()
    {
      return self()->mText_size;
    }

    /**
      Is text_size Immutable
    */
    static
    bool isText_sizeImmutable()
    {
      return self()->isImmutable( QStringLiteral( "text_size" ) );
    }

    /**
      Get text_size default value
    */
    static
    double defaultText_sizeValue()
    {
        return defaultText_sizeValue_helper();
    }

    /**
      Get Item object corresponding to text_size()
    */
    ItemDouble *text_sizeItem()
    {
      return mText_sizeItem;
    }

  protected:
    GlaxnimateSettings();
    friend class GlaxnimateSettingsHelper;


    // ui
    bool mStartup_dialog;
    static bool defaultStartup_dialogValue_helper();
    bool mTimeline_scroll_horizontal;
    static bool defaultTimeline_scroll_horizontalValue_helper();
    bool mDebug_timeline;
    static bool defaultDebug_timelineValue_helper();
    int mLayout;
    static int defaultLayoutValue_helper();
    QString mWindow_state;
    static QString defaultWindow_stateValue_helper();
    QString mTimeline_splitter;
    static QString defaultTimeline_splitterValue_helper();
    QString mIcon_theme;
    static QString defaultIcon_themeValue_helper();

    // defaults
    int mWidth;
    static int defaultWidthValue_helper();
    int mHeight;
    static int defaultHeightValue_helper();
    double mFps;
    static double defaultFpsValue_helper();
    double mDuration;
    static double defaultDurationValue_helper();

    // open_save
    int mBackup_frequency;
    static int defaultBackup_frequencyValue_helper();
    bool mUse_native_io_dialog;
    static bool defaultUse_native_io_dialogValue_helper();
    QString mPath;
    static QString defaultPathValue_helper();
    QString mRender_path;
    static QString defaultRender_pathValue_helper();
    QString mImport_path;
    static QString defaultImport_pathValue_helper();

    // scripting
    int mScript_max_history;
    static int defaultScript_max_historyValue_helper();
    QStringList mScript_history;
    static QStringList defaultScript_historyValue_helper();

    // tools
    bool mShape_group;
    static bool defaultShape_groupValue_helper();
    bool mShape_fill;
    static bool defaultShape_fillValue_helper();
    bool mShape_stroke;
    static bool defaultShape_strokeValue_helper();
    bool mEdit_mask;
    static bool defaultEdit_maskValue_helper();
    QColor mColor_main;
    static QColor defaultColor_mainValue_helper();
    QColor mColor_secondary;
    static QColor defaultColor_secondaryValue_helper();
    double mStroke_width;
    static double defaultStroke_widthValue_helper();
    int mStroke_cap;
    static int defaultStroke_capValue_helper();
    int mStroke_join;
    static int defaultStroke_joinValue_helper();
    double mStroke_miter;
    static double defaultStroke_miterValue_helper();
    int mStar_type;
    static int defaultStar_typeValue_helper();
    double mStar_ratio;
    static double defaultStar_ratioValue_helper();
    int mStar_points;
    static int defaultStar_pointsValue_helper();
    QStringList mFavourite_font_families;
    static QStringList defaultFavourite_font_familiesValue_helper();
    QString mText_family;
    static QString defaultText_familyValue_helper();
    QString mText_style;
    static QString defaultText_styleValue_helper();
    double mText_size;
    static double defaultText_sizeValue_helper();

  private:
    ItemBool *mStartup_dialogItem;
    ItemBool *mTimeline_scroll_horizontalItem;
    ItemBool *mDebug_timelineItem;
    ItemInt *mLayoutItem;
    ItemString *mWindow_stateItem;
    ItemString *mTimeline_splitterItem;
    ItemString *mIcon_themeItem;
    ItemInt *mWidthItem;
    ItemInt *mHeightItem;
    ItemDouble *mFpsItem;
    ItemDouble *mDurationItem;
    ItemInt *mBackup_frequencyItem;
    ItemBool *mUse_native_io_dialogItem;
    ItemPath *mPathItem;
    ItemPath *mRender_pathItem;
    ItemPath *mImport_pathItem;
    ItemInt *mScript_max_historyItem;
    ItemStringList *mScript_historyItem;
    ItemBool *mShape_groupItem;
    ItemBool *mShape_fillItem;
    ItemBool *mShape_strokeItem;
    ItemBool *mEdit_maskItem;
    ItemColor *mColor_mainItem;
    ItemColor *mColor_secondaryItem;
    ItemDouble *mStroke_widthItem;
    ItemEnum *mStroke_capItem;
    ItemEnum *mStroke_joinItem;
    ItemDouble *mStroke_miterItem;
    ItemInt *mStar_typeItem;
    ItemDouble *mStar_ratioItem;
    ItemInt *mStar_pointsItem;
    ItemStringList *mFavourite_font_familiesItem;
    ItemString *mText_familyItem;
    ItemString *mText_styleItem;
    ItemDouble *mText_sizeItem;
};

}
}

#endif

