/********************************************************************************
** Form generated from reading UI file 'follow_path_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FOLLOW_PATH_DIALOG_H
#define UI_FOLLOW_PATH_DIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <klocalizedstring.h>

namespace glaxnimate {
namespace gui {

class Ui_FollowPathDialog
{
public:
    QGridLayout *gridLayout;
    QLabel *label_4;
    QLabel *label_5;
    QDialogButtonBox *buttonBox;
    QLabel *label;
    QDoubleSpinBox *spin_start;
    QLabel *label_3;
    QComboBox *combo_units;
    QHBoxLayout *horizontalLayout;
    QLineEdit *line_shape_name;
    QPushButton *pushButton;
    QLabel *label_2;
    QDoubleSpinBox *spin_duration;
    QDoubleSpinBox *spin_end;
    QSpacerItem *verticalSpacer;

    void setupUi(QDialog *glaxnimate__gui__FollowPathDialog)
    {
        if (glaxnimate__gui__FollowPathDialog->objectName().isEmpty())
            glaxnimate__gui__FollowPathDialog->setObjectName("glaxnimate__gui__FollowPathDialog");
        glaxnimate__gui__FollowPathDialog->resize(356, 246);
        gridLayout = new QGridLayout(glaxnimate__gui__FollowPathDialog);
        gridLayout->setObjectName("gridLayout");
        label_4 = new QLabel(glaxnimate__gui__FollowPathDialog);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 3, 0, 1, 1);

        label_5 = new QLabel(glaxnimate__gui__FollowPathDialog);
        label_5->setObjectName("label_5");

        gridLayout->addWidget(label_5, 4, 0, 1, 1);

        buttonBox = new QDialogButtonBox(glaxnimate__gui__FollowPathDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout->addWidget(buttonBox, 6, 0, 1, 2);

        label = new QLabel(glaxnimate__gui__FollowPathDialog);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        spin_start = new QDoubleSpinBox(glaxnimate__gui__FollowPathDialog);
        spin_start->setObjectName("spin_start");

        gridLayout->addWidget(spin_start, 1, 1, 1, 1);

        label_3 = new QLabel(glaxnimate__gui__FollowPathDialog);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 2, 0, 1, 1);

        combo_units = new QComboBox(glaxnimate__gui__FollowPathDialog);
        combo_units->addItem(QString());
        combo_units->addItem(QString());
        combo_units->setObjectName("combo_units");

        gridLayout->addWidget(combo_units, 4, 1, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        line_shape_name = new QLineEdit(glaxnimate__gui__FollowPathDialog);
        line_shape_name->setObjectName("line_shape_name");
        line_shape_name->setText(QString::fromUtf8(""));
        line_shape_name->setReadOnly(true);

        horizontalLayout->addWidget(line_shape_name);

        pushButton = new QPushButton(glaxnimate__gui__FollowPathDialog);
        pushButton->setObjectName("pushButton");

        horizontalLayout->addWidget(pushButton);


        gridLayout->addLayout(horizontalLayout, 0, 1, 1, 1);

        label_2 = new QLabel(glaxnimate__gui__FollowPathDialog);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        spin_duration = new QDoubleSpinBox(glaxnimate__gui__FollowPathDialog);
        spin_duration->setObjectName("spin_duration");

        gridLayout->addWidget(spin_duration, 3, 1, 1, 1);

        spin_end = new QDoubleSpinBox(glaxnimate__gui__FollowPathDialog);
        spin_end->setObjectName("spin_end");

        gridLayout->addWidget(spin_end, 2, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 5, 1, 1, 1);


        retranslateUi(glaxnimate__gui__FollowPathDialog);
        QObject::connect(buttonBox, SIGNAL(accepted()), glaxnimate__gui__FollowPathDialog, SLOT(apply()));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, glaxnimate__gui__FollowPathDialog, qOverload<>(&QDialog::reject));
        QObject::connect(combo_units, SIGNAL(currentIndexChanged(int)), glaxnimate__gui__FollowPathDialog, SLOT(change_units(int)));
        QObject::connect(spin_duration, SIGNAL(valueChanged(double)), glaxnimate__gui__FollowPathDialog, SLOT(change_duration(double)));
        QObject::connect(spin_end, SIGNAL(valueChanged(double)), glaxnimate__gui__FollowPathDialog, SLOT(change_end(double)));
        QObject::connect(pushButton, SIGNAL(clicked()), glaxnimate__gui__FollowPathDialog, SLOT(select_path()));

        QMetaObject::connectSlotsByName(glaxnimate__gui__FollowPathDialog);
    } // setupUi

    void retranslateUi(QDialog *glaxnimate__gui__FollowPathDialog)
    {
        glaxnimate__gui__FollowPathDialog->setWindowTitle(tr2i18n("Follow Path", nullptr));
        label_4->setText(tr2i18n("Duration", nullptr));
        label_5->setText(tr2i18n("Time Units", nullptr));
        label->setText(tr2i18n("Shape", nullptr));
        label_3->setText(tr2i18n("End Time", nullptr));
        combo_units->setItemText(0, tr2i18n("Frames", nullptr));
        combo_units->setItemText(1, tr2i18n("Seconds", nullptr));

        pushButton->setText(tr2i18n("Choose...", nullptr));
        label_2->setText(tr2i18n("Start Time", nullptr));
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class FollowPathDialog: public Ui_FollowPathDialog {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // FOLLOW_PATH_DIALOG_H

