/********************************************************************************
** Form generated from reading UI file 'assets.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ASSETS_H
#define UI_ASSETS_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDockWidget>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "widgets/docks/custom_treeview.hpp"

QT_BEGIN_NAMESPACE

class Ui_dock_assets
{
public:
    QWidget *dockWidgetContents;
    QVBoxLayout *verticalLayout;
    glaxnimate::gui::CustomTreeView *view_assets;

    void setupUi(QDockWidget *dock_assets)
    {
        if (dock_assets->objectName().isEmpty())
            dock_assets->setObjectName("dock_assets");
        dock_assets->resize(400, 300);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("folder-stash")));
        dock_assets->setWindowIcon(icon);
        dockWidgetContents = new QWidget();
        dockWidgetContents->setObjectName("dockWidgetContents");
        verticalLayout = new QVBoxLayout(dockWidgetContents);
        verticalLayout->setObjectName("verticalLayout");
        view_assets = new glaxnimate::gui::CustomTreeView(dockWidgetContents);
        view_assets->setObjectName("view_assets");
        view_assets->setDragEnabled(true);
        view_assets->setDragDropMode(QAbstractItemView::DragDropMode::DragOnly);
        view_assets->setSelectionMode(QAbstractItemView::SelectionMode::SingleSelection);
        view_assets->setAutoExpandDelay(500);
        view_assets->header()->setVisible(false);
        view_assets->header()->setMinimumSectionSize(32);
        view_assets->header()->setDefaultSectionSize(32);
        view_assets->header()->setStretchLastSection(false);

        verticalLayout->addWidget(view_assets);

        dock_assets->setWidget(dockWidgetContents);

        retranslateUi(dock_assets);

        QMetaObject::connectSlotsByName(dock_assets);
    } // setupUi

    void retranslateUi(QDockWidget *dock_assets)
    {
        dock_assets->setWindowTitle(tr2i18n("Assets", nullptr));
    } // retranslateUi

};

namespace Ui {
    class dock_assets: public Ui_dock_assets {};
} // namespace Ui

QT_END_NAMESPACE

#endif // ASSETS_H

