/********************************************************************************
** Form generated from reading UI file 'document_swatch_widget.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DOCUMENT_SWATCH_WIDGET_H
#define UI_DOCUMENT_SWATCH_WIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "QtColorWidgets/swatch.hpp"

namespace glaxnimate {
namespace gui {

class Ui_DocumentSwatchWidget
{
public:
    QAction *action_generate;
    QAction *action_save;
    QAction *action_open;
    QVBoxLayout *verticalLayout;
    color_widgets::Swatch *swatch;
    QHBoxLayout *horizontalLayout_2;
    QToolButton *button_extra;
    QSpacerItem *horizontalSpacer;
    QToolButton *toolButton;
    QToolButton *toolButton_2;

    void setupUi(QWidget *DocumentSwatchWidget)
    {
        if (DocumentSwatchWidget->objectName().isEmpty())
            DocumentSwatchWidget->setObjectName("DocumentSwatchWidget");
        DocumentSwatchWidget->resize(312, 276);
        action_generate = new QAction(DocumentSwatchWidget);
        action_generate->setObjectName("action_generate");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("document-export");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        action_generate->setIcon(icon);
        action_save = new QAction(DocumentSwatchWidget);
        action_save->setObjectName("action_save");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("document-save");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        action_save->setIcon(icon1);
        action_open = new QAction(DocumentSwatchWidget);
        action_open->setObjectName("action_open");
        QIcon icon2;
        iconThemeName = QString::fromUtf8("document-open");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        action_open->setIcon(icon2);
        verticalLayout = new QVBoxLayout(DocumentSwatchWidget);
        verticalLayout->setObjectName("verticalLayout");
        swatch = new color_widgets::Swatch(DocumentSwatchWidget);
        swatch->setObjectName("swatch");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(swatch->sizePolicy().hasHeightForWidth());
        swatch->setSizePolicy(sizePolicy);
        swatch->setColorSize(QSize(32, 32));
        swatch->setShowClearColor(true);

        verticalLayout->addWidget(swatch);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        button_extra = new QToolButton(DocumentSwatchWidget);
        button_extra->setObjectName("button_extra");
        QIcon icon3;
        iconThemeName = QString::fromUtf8("open-menu-symbolic");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_extra->setIcon(icon3);

        horizontalLayout_2->addWidget(button_extra);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        toolButton = new QToolButton(DocumentSwatchWidget);
        toolButton->setObjectName("toolButton");
        QIcon icon4;
        iconThemeName = QString::fromUtf8("list-add");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon4 = QIcon::fromTheme(iconThemeName);
        } else {
            icon4.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        toolButton->setIcon(icon4);

        horizontalLayout_2->addWidget(toolButton);

        toolButton_2 = new QToolButton(DocumentSwatchWidget);
        toolButton_2->setObjectName("toolButton_2");
        QIcon icon5;
        iconThemeName = QString::fromUtf8("list-remove");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon5 = QIcon::fromTheme(iconThemeName);
        } else {
            icon5.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        toolButton_2->setIcon(icon5);

        horizontalLayout_2->addWidget(toolButton_2);


        verticalLayout->addLayout(horizontalLayout_2);


        retranslateUi(DocumentSwatchWidget);
        QObject::connect(toolButton_2, &QToolButton::clicked, swatch, qOverload<>(&color_widgets::Swatch::removeSelected));
        QObject::connect(toolButton, SIGNAL(clicked()), DocumentSwatchWidget, SLOT(swatch_add()));
        QObject::connect(swatch, SIGNAL(clicked(int,Qt::KeyboardModifiers)), DocumentSwatchWidget, SLOT(swatch_link(int,Qt::KeyboardModifiers)));
        QObject::connect(swatch, SIGNAL(rightClicked(int,Qt::KeyboardModifiers)), DocumentSwatchWidget, SLOT(swatch_menu(int)));

        QMetaObject::connectSlotsByName(DocumentSwatchWidget);
    } // setupUi

    void retranslateUi(QWidget *DocumentSwatchWidget)
    {
        action_generate->setText(tr2i18n("Generate", nullptr));
#if QT_CONFIG(tooltip)
        action_generate->setToolTip(tr2i18n("Generate Swatch from the open Document", nullptr));
#endif // QT_CONFIG(tooltip)
        action_save->setText(tr2i18n("Save Palette", nullptr));
        action_open->setText(tr2i18n("From Palette...", nullptr));
#if QT_CONFIG(tooltip)
        button_extra->setToolTip(tr2i18n("Swatch", nullptr));
#endif // QT_CONFIG(tooltip)
        button_extra->setText(tr2i18n("Swatch", nullptr));
#if QT_CONFIG(tooltip)
        toolButton->setToolTip(tr2i18n("Add color", nullptr));
#endif // QT_CONFIG(tooltip)
        toolButton->setText(tr2i18n("Add", nullptr));
        toolButton_2->setText(tr2i18n("Remove", nullptr));
        (void)DocumentSwatchWidget;
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class DocumentSwatchWidget: public Ui_DocumentSwatchWidget {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // DOCUMENT_SWATCH_WIDGET_H

