/********************************************************************************
** Form generated from reading UI file 'timeline.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TIMELINE_H
#define UI_TIMELINE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDockWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "widgets/timeline/compound_timeline_widget.hpp"
#include "widgets/timeline/frame_controls_widget.hpp"

QT_BEGIN_NAMESPACE

class Ui_dock_timeline
{
public:
    QWidget *dockWidgetContents;
    QVBoxLayout *verticalLayout;
    glaxnimate::gui::FrameControlsWidget *play_controls;
    glaxnimate::gui::CompoundTimelineWidget *timeline_widget;

    void setupUi(QDockWidget *dock_timeline)
    {
        if (dock_timeline->objectName().isEmpty())
            dock_timeline->setObjectName("dock_timeline");
        dock_timeline->resize(400, 300);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("player-time")));
        dock_timeline->setWindowIcon(icon);
        dockWidgetContents = new QWidget();
        dockWidgetContents->setObjectName("dockWidgetContents");
        verticalLayout = new QVBoxLayout(dockWidgetContents);
        verticalLayout->setObjectName("verticalLayout");
        play_controls = new glaxnimate::gui::FrameControlsWidget(dockWidgetContents);
        play_controls->setObjectName("play_controls");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(play_controls->sizePolicy().hasHeightForWidth());
        play_controls->setSizePolicy(sizePolicy);

        verticalLayout->addWidget(play_controls);

        timeline_widget = new glaxnimate::gui::CompoundTimelineWidget(dockWidgetContents);
        timeline_widget->setObjectName("timeline_widget");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(timeline_widget->sizePolicy().hasHeightForWidth());
        timeline_widget->setSizePolicy(sizePolicy1);

        verticalLayout->addWidget(timeline_widget);

        dock_timeline->setWidget(dockWidgetContents);

        retranslateUi(dock_timeline);

        QMetaObject::connectSlotsByName(dock_timeline);
    } // setupUi

    void retranslateUi(QDockWidget *dock_timeline)
    {
        dock_timeline->setWindowTitle(tr2i18n("&Timeline", nullptr));
    } // retranslateUi

};

namespace Ui {
    class dock_timeline: public Ui_dock_timeline {};
} // namespace Ui

QT_END_NAMESPACE

#endif // TIMELINE_H

