/********************************************************************************
** Form generated from reading UI file 'color_selector.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_COLOR_SELECTOR_H
#define UI_COLOR_SELECTOR_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "QtColorWidgets/color_line_edit.hpp"
#include "QtColorWidgets/color_palette_widget.hpp"
#include "QtColorWidgets/color_preview.hpp"
#include "QtColorWidgets/color_selector.hpp"
#include "QtColorWidgets/color_wheel.hpp"
#include "QtColorWidgets/gradient_slider.hpp"
#include "QtColorWidgets/hue_slider.hpp"

namespace glaxnimate {
namespace gui {

class Ui_ColorSelector
{
public:
    QVBoxLayout *verticalLayout;
    QComboBox *combo_box;
    QStackedWidget *tab_6;
    QWidget *tab_6Page1;
    QGridLayout *gridLayout;
    QLabel *label_2;
    QSpinBox *spinBox_3;
    color_widgets::HueSlider *slider_hsv_hue;
    color_widgets::GradientSlider *slider_hsv_value;
    QSpinBox *spinBox_2;
    QLabel *label;
    QSpinBox *spinBox;
    QLabel *label_3;
    color_widgets::GradientSlider *slider_hsv_sat;
    color_widgets::ColorWheel *color_hsv;
    QWidget *tab_6Page2;
    QGridLayout *gridLayout_3;
    color_widgets::HueSlider *slider_hsl_hue;
    QLabel *label_5;
    QSpinBox *spinBox_6;
    QSpinBox *spinBox_5;
    color_widgets::ColorWheel *color_hsl;
    QSpinBox *spinBox_7;
    color_widgets::GradientSlider *slider_hsl_sat;
    color_widgets::GradientSlider *slider_hsl_light;
    QLabel *label_6;
    QLabel *label_7;
    QWidget *tab_6Page3;
    QGridLayout *gridLayout_4;
    color_widgets::GradientSlider *slider_rgb_b;
    QLabel *label_10;
    color_widgets::GradientSlider *slider_rgb_g;
    QLabel *label_8;
    QLabel *label_9;
    color_widgets::GradientSlider *slider_rgb_r;
    QSpinBox *spinBox_8;
    QSpinBox *spinBox_9;
    QSpinBox *spinBox_10;
    QWidget *tab_6Page4;
    QGridLayout *gridLayout_5;
    QLabel *label_11;
    color_widgets::GradientSlider *slider_cmyk_c;
    QSpinBox *spinBox_11;
    QLabel *label_13;
    color_widgets::GradientSlider *slider_cmyk_m;
    QSpinBox *spinBox_13;
    QLabel *label_12;
    color_widgets::GradientSlider *slider_cmyk_y;
    QSpinBox *spinBox_12;
    QLabel *label_14;
    color_widgets::GradientSlider *slider_cmyk_k;
    QSpinBox *spinBox_14;
    QWidget *tab_6Page5;
    QVBoxLayout *verticalLayout_3;
    color_widgets::ColorPaletteWidget *palette_widget;
    QWidget *page;
    QGridLayout *gridLayout_2;
    color_widgets::GradientSlider *slider_alpha;
    QLabel *label_4;
    QSpinBox *spinBox_4;
    QHBoxLayout *horizontalLayout;
    QFrame *frame;
    QHBoxLayout *horizontalLayout_4;
    color_widgets::ColorSelector *color_preview;
    color_widgets::ColorSelector *color_preview_secondary;
    QToolButton *color_swap;
    QToolButton *btn_clear;
    color_widgets::ColorLineEdit *color_line_edit;

    void setupUi(QWidget *ColorSelector)
    {
        if (ColorSelector->objectName().isEmpty())
            ColorSelector->setObjectName("ColorSelector");
        ColorSelector->resize(345, 447);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ColorSelector->sizePolicy().hasHeightForWidth());
        ColorSelector->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(ColorSelector);
        verticalLayout->setObjectName("verticalLayout");
        combo_box = new QComboBox(ColorSelector);
        combo_box->addItem(QString());
        combo_box->addItem(QString());
        combo_box->addItem(QString());
        combo_box->addItem(QString());
        combo_box->addItem(QString());
        combo_box->addItem(QString());
        combo_box->setObjectName("combo_box");

        verticalLayout->addWidget(combo_box);

        tab_6 = new QStackedWidget(ColorSelector);
        tab_6->setObjectName("tab_6");
        tab_6Page1 = new QWidget();
        tab_6Page1->setObjectName("tab_6Page1");
        gridLayout = new QGridLayout(tab_6Page1);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        label_2 = new QLabel(tab_6Page1);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 2, 0, 1, 1);

        spinBox_3 = new QSpinBox(tab_6Page1);
        spinBox_3->setObjectName("spinBox_3");
        spinBox_3->setMaximum(255);

        gridLayout->addWidget(spinBox_3, 3, 2, 1, 1);

        slider_hsv_hue = new color_widgets::HueSlider(tab_6Page1);
        slider_hsv_hue->setObjectName("slider_hsv_hue");

        gridLayout->addWidget(slider_hsv_hue, 1, 1, 1, 1);

        slider_hsv_value = new color_widgets::GradientSlider(tab_6Page1);
        slider_hsv_value->setObjectName("slider_hsv_value");
        slider_hsv_value->setMaximum(255);

        gridLayout->addWidget(slider_hsv_value, 3, 1, 1, 1);

        spinBox_2 = new QSpinBox(tab_6Page1);
        spinBox_2->setObjectName("spinBox_2");
        spinBox_2->setMaximum(255);

        gridLayout->addWidget(spinBox_2, 2, 2, 1, 1);

        label = new QLabel(tab_6Page1);
        label->setObjectName("label");

        gridLayout->addWidget(label, 1, 0, 1, 1);

        spinBox = new QSpinBox(tab_6Page1);
        spinBox->setObjectName("spinBox");
        spinBox->setMaximum(359);

        gridLayout->addWidget(spinBox, 1, 2, 1, 1);

        label_3 = new QLabel(tab_6Page1);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 3, 0, 1, 1);

        slider_hsv_sat = new color_widgets::GradientSlider(tab_6Page1);
        slider_hsv_sat->setObjectName("slider_hsv_sat");
        slider_hsv_sat->setMaximum(255);

        gridLayout->addWidget(slider_hsv_sat, 2, 1, 1, 1);

        color_hsv = new color_widgets::ColorWheel(tab_6Page1);
        color_hsv->setObjectName("color_hsv");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(color_hsv->sizePolicy().hasHeightForWidth());
        color_hsv->setSizePolicy(sizePolicy1);
        color_hsv->setMinimumSize(QSize(128, 128));
        color_hsv->setMaximumSize(QSize(16777215, 230));
        color_hsv->setSelectorShape(color_widgets::ColorWheel::ShapeSquare);
        color_hsv->setRotatingSelector(false);

        gridLayout->addWidget(color_hsv, 0, 0, 1, 3);

        tab_6->addWidget(tab_6Page1);
        tab_6Page2 = new QWidget();
        tab_6Page2->setObjectName("tab_6Page2");
        gridLayout_3 = new QGridLayout(tab_6Page2);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        slider_hsl_hue = new color_widgets::HueSlider(tab_6Page2);
        slider_hsl_hue->setObjectName("slider_hsl_hue");

        gridLayout_3->addWidget(slider_hsl_hue, 1, 1, 1, 1);

        label_5 = new QLabel(tab_6Page2);
        label_5->setObjectName("label_5");

        gridLayout_3->addWidget(label_5, 1, 0, 1, 1);

        spinBox_6 = new QSpinBox(tab_6Page2);
        spinBox_6->setObjectName("spinBox_6");
        spinBox_6->setMaximum(255);

        gridLayout_3->addWidget(spinBox_6, 2, 2, 1, 1);

        spinBox_5 = new QSpinBox(tab_6Page2);
        spinBox_5->setObjectName("spinBox_5");
        spinBox_5->setMaximum(359);

        gridLayout_3->addWidget(spinBox_5, 1, 2, 1, 1);

        color_hsl = new color_widgets::ColorWheel(tab_6Page2);
        color_hsl->setObjectName("color_hsl");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(color_hsl->sizePolicy().hasHeightForWidth());
        color_hsl->setSizePolicy(sizePolicy2);
        color_hsl->setMinimumSize(QSize(128, 128));
        color_hsl->setSelectorShape(color_widgets::ColorWheel::ShapeSquare);
        color_hsl->setRotatingSelector(false);
        color_hsl->setColorSpace(color_widgets::ColorWheel::ColorHSL);

        gridLayout_3->addWidget(color_hsl, 0, 0, 1, 3);

        spinBox_7 = new QSpinBox(tab_6Page2);
        spinBox_7->setObjectName("spinBox_7");
        spinBox_7->setMaximum(255);

        gridLayout_3->addWidget(spinBox_7, 3, 2, 1, 1);

        slider_hsl_sat = new color_widgets::GradientSlider(tab_6Page2);
        slider_hsl_sat->setObjectName("slider_hsl_sat");
        slider_hsl_sat->setMaximum(255);

        gridLayout_3->addWidget(slider_hsl_sat, 2, 1, 1, 1);

        slider_hsl_light = new color_widgets::GradientSlider(tab_6Page2);
        slider_hsl_light->setObjectName("slider_hsl_light");
        slider_hsl_light->setMaximum(255);

        gridLayout_3->addWidget(slider_hsl_light, 3, 1, 1, 1);

        label_6 = new QLabel(tab_6Page2);
        label_6->setObjectName("label_6");

        gridLayout_3->addWidget(label_6, 2, 0, 1, 1);

        label_7 = new QLabel(tab_6Page2);
        label_7->setObjectName("label_7");

        gridLayout_3->addWidget(label_7, 3, 0, 1, 1);

        tab_6->addWidget(tab_6Page2);
        tab_6Page3 = new QWidget();
        tab_6Page3->setObjectName("tab_6Page3");
        gridLayout_4 = new QGridLayout(tab_6Page3);
        gridLayout_4->setObjectName("gridLayout_4");
        gridLayout_4->setContentsMargins(0, 0, 0, 0);
        slider_rgb_b = new color_widgets::GradientSlider(tab_6Page3);
        slider_rgb_b->setObjectName("slider_rgb_b");
        slider_rgb_b->setMaximum(255);

        gridLayout_4->addWidget(slider_rgb_b, 2, 1, 1, 1);

        label_10 = new QLabel(tab_6Page3);
        label_10->setObjectName("label_10");

        gridLayout_4->addWidget(label_10, 2, 0, 1, 1);

        slider_rgb_g = new color_widgets::GradientSlider(tab_6Page3);
        slider_rgb_g->setObjectName("slider_rgb_g");
        slider_rgb_g->setMaximum(255);

        gridLayout_4->addWidget(slider_rgb_g, 1, 1, 1, 1);

        label_8 = new QLabel(tab_6Page3);
        label_8->setObjectName("label_8");

        gridLayout_4->addWidget(label_8, 0, 0, 1, 1);

        label_9 = new QLabel(tab_6Page3);
        label_9->setObjectName("label_9");

        gridLayout_4->addWidget(label_9, 1, 0, 1, 1);

        slider_rgb_r = new color_widgets::GradientSlider(tab_6Page3);
        slider_rgb_r->setObjectName("slider_rgb_r");
        slider_rgb_r->setMaximum(255);

        gridLayout_4->addWidget(slider_rgb_r, 0, 1, 1, 1);

        spinBox_8 = new QSpinBox(tab_6Page3);
        spinBox_8->setObjectName("spinBox_8");
        spinBox_8->setMaximum(255);

        gridLayout_4->addWidget(spinBox_8, 0, 2, 1, 1);

        spinBox_9 = new QSpinBox(tab_6Page3);
        spinBox_9->setObjectName("spinBox_9");
        spinBox_9->setMaximum(255);

        gridLayout_4->addWidget(spinBox_9, 1, 2, 1, 1);

        spinBox_10 = new QSpinBox(tab_6Page3);
        spinBox_10->setObjectName("spinBox_10");
        spinBox_10->setMaximum(255);

        gridLayout_4->addWidget(spinBox_10, 2, 2, 1, 1);

        tab_6->addWidget(tab_6Page3);
        tab_6Page4 = new QWidget();
        tab_6Page4->setObjectName("tab_6Page4");
        gridLayout_5 = new QGridLayout(tab_6Page4);
        gridLayout_5->setObjectName("gridLayout_5");
        gridLayout_5->setContentsMargins(0, 0, 0, 0);
        label_11 = new QLabel(tab_6Page4);
        label_11->setObjectName("label_11");

        gridLayout_5->addWidget(label_11, 0, 0, 1, 1);

        slider_cmyk_c = new color_widgets::GradientSlider(tab_6Page4);
        slider_cmyk_c->setObjectName("slider_cmyk_c");
        slider_cmyk_c->setMaximum(255);

        gridLayout_5->addWidget(slider_cmyk_c, 0, 1, 1, 1);

        spinBox_11 = new QSpinBox(tab_6Page4);
        spinBox_11->setObjectName("spinBox_11");
        spinBox_11->setMaximum(255);

        gridLayout_5->addWidget(spinBox_11, 0, 2, 1, 1);

        label_13 = new QLabel(tab_6Page4);
        label_13->setObjectName("label_13");

        gridLayout_5->addWidget(label_13, 1, 0, 1, 1);

        slider_cmyk_m = new color_widgets::GradientSlider(tab_6Page4);
        slider_cmyk_m->setObjectName("slider_cmyk_m");
        slider_cmyk_m->setMaximum(255);

        gridLayout_5->addWidget(slider_cmyk_m, 1, 1, 1, 1);

        spinBox_13 = new QSpinBox(tab_6Page4);
        spinBox_13->setObjectName("spinBox_13");
        spinBox_13->setMaximum(255);

        gridLayout_5->addWidget(spinBox_13, 1, 2, 1, 1);

        label_12 = new QLabel(tab_6Page4);
        label_12->setObjectName("label_12");

        gridLayout_5->addWidget(label_12, 2, 0, 1, 1);

        slider_cmyk_y = new color_widgets::GradientSlider(tab_6Page4);
        slider_cmyk_y->setObjectName("slider_cmyk_y");
        slider_cmyk_y->setMaximum(255);

        gridLayout_5->addWidget(slider_cmyk_y, 2, 1, 1, 1);

        spinBox_12 = new QSpinBox(tab_6Page4);
        spinBox_12->setObjectName("spinBox_12");
        spinBox_12->setMaximum(255);

        gridLayout_5->addWidget(spinBox_12, 2, 2, 1, 1);

        label_14 = new QLabel(tab_6Page4);
        label_14->setObjectName("label_14");

        gridLayout_5->addWidget(label_14, 3, 0, 1, 1);

        slider_cmyk_k = new color_widgets::GradientSlider(tab_6Page4);
        slider_cmyk_k->setObjectName("slider_cmyk_k");
        slider_cmyk_k->setMaximum(255);

        gridLayout_5->addWidget(slider_cmyk_k, 3, 1, 1, 1);

        spinBox_14 = new QSpinBox(tab_6Page4);
        spinBox_14->setObjectName("spinBox_14");
        spinBox_14->setMaximum(255);

        gridLayout_5->addWidget(spinBox_14, 3, 2, 1, 1);

        tab_6->addWidget(tab_6Page4);
        tab_6Page5 = new QWidget();
        tab_6Page5->setObjectName("tab_6Page5");
        verticalLayout_3 = new QVBoxLayout(tab_6Page5);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, -1, 0, -1);
        palette_widget = new color_widgets::ColorPaletteWidget(tab_6Page5);
        palette_widget->setObjectName("palette_widget");

        verticalLayout_3->addWidget(palette_widget);

        tab_6->addWidget(tab_6Page5);
        page = new QWidget();
        page->setObjectName("page");
        tab_6->addWidget(page);

        verticalLayout->addWidget(tab_6);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        slider_alpha = new color_widgets::GradientSlider(ColorSelector);
        slider_alpha->setObjectName("slider_alpha");
        slider_alpha->setMaximum(255);

        gridLayout_2->addWidget(slider_alpha, 0, 1, 1, 1);

        label_4 = new QLabel(ColorSelector);
        label_4->setObjectName("label_4");

        gridLayout_2->addWidget(label_4, 0, 0, 1, 1);

        spinBox_4 = new QSpinBox(ColorSelector);
        spinBox_4->setObjectName("spinBox_4");
        spinBox_4->setMaximum(255);

        gridLayout_2->addWidget(spinBox_4, 0, 2, 1, 1);


        verticalLayout->addLayout(gridLayout_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        frame = new QFrame(ColorSelector);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        horizontalLayout_4 = new QHBoxLayout(frame);
        horizontalLayout_4->setSpacing(0);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalLayout_4->setContentsMargins(0, 0, 0, 0);
        color_preview = new color_widgets::ColorSelector(frame);
        color_preview->setObjectName("color_preview");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(color_preview->sizePolicy().hasHeightForWidth());
        color_preview->setSizePolicy(sizePolicy3);
        color_preview->setProperty("display_mode", QVariant::fromValue(color_widgets::ColorPreview::AllAlpha));
        color_preview->setDrawFrame(false);

        horizontalLayout_4->addWidget(color_preview);

        color_preview_secondary = new color_widgets::ColorSelector(frame);
        color_preview_secondary->setObjectName("color_preview_secondary");
        sizePolicy3.setHeightForWidth(color_preview_secondary->sizePolicy().hasHeightForWidth());
        color_preview_secondary->setSizePolicy(sizePolicy3);
        color_preview_secondary->setColor(QColor(0, 0, 0));
        color_preview_secondary->setProperty("display_mode", QVariant::fromValue(color_widgets::ColorPreview::AllAlpha));
        color_preview_secondary->setDrawFrame(false);

        horizontalLayout_4->addWidget(color_preview_secondary);


        horizontalLayout->addWidget(frame);

        color_swap = new QToolButton(ColorSelector);
        color_swap->setObjectName("color_swap");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("swap-panels");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("../../../../../.designer/backup"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        color_swap->setIcon(icon);

        horizontalLayout->addWidget(color_swap);

        btn_clear = new QToolButton(ColorSelector);
        btn_clear->setObjectName("btn_clear");
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("paint-none")));
        btn_clear->setIcon(icon1);

        horizontalLayout->addWidget(btn_clear);

        color_line_edit = new color_widgets::ColorLineEdit(ColorSelector);
        color_line_edit->setObjectName("color_line_edit");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(color_line_edit->sizePolicy().hasHeightForWidth());
        color_line_edit->setSizePolicy(sizePolicy4);
        color_line_edit->setMaximumSize(QSize(80, 16777215));
        QFont font;
        font.setFamilies({QString::fromUtf8("Monospace")});
        color_line_edit->setFont(font);
        color_line_edit->setShowAlpha(true);

        horizontalLayout->addWidget(color_line_edit);


        verticalLayout->addLayout(horizontalLayout);

        QWidget::setTabOrder(slider_hsv_hue, spinBox);
        QWidget::setTabOrder(spinBox, slider_hsv_sat);
        QWidget::setTabOrder(slider_hsv_sat, spinBox_2);
        QWidget::setTabOrder(spinBox_2, slider_hsv_value);
        QWidget::setTabOrder(slider_hsv_value, spinBox_3);
        QWidget::setTabOrder(spinBox_3, slider_alpha);
        QWidget::setTabOrder(slider_alpha, spinBox_4);
        QWidget::setTabOrder(spinBox_4, color_swap);
        QWidget::setTabOrder(color_swap, color_line_edit);
        QWidget::setTabOrder(color_line_edit, combo_box);
        QWidget::setTabOrder(combo_box, slider_hsl_hue);
        QWidget::setTabOrder(slider_hsl_hue, spinBox_5);
        QWidget::setTabOrder(spinBox_5, slider_hsl_sat);
        QWidget::setTabOrder(slider_hsl_sat, spinBox_6);
        QWidget::setTabOrder(spinBox_6, slider_hsl_light);
        QWidget::setTabOrder(slider_hsl_light, spinBox_7);
        QWidget::setTabOrder(spinBox_7, slider_rgb_r);
        QWidget::setTabOrder(slider_rgb_r, spinBox_8);
        QWidget::setTabOrder(spinBox_8, slider_rgb_g);
        QWidget::setTabOrder(slider_rgb_g, spinBox_9);
        QWidget::setTabOrder(spinBox_9, slider_rgb_b);
        QWidget::setTabOrder(slider_rgb_b, spinBox_10);
        QWidget::setTabOrder(spinBox_10, slider_cmyk_c);
        QWidget::setTabOrder(slider_cmyk_c, spinBox_11);
        QWidget::setTabOrder(spinBox_11, slider_cmyk_m);
        QWidget::setTabOrder(slider_cmyk_m, spinBox_13);
        QWidget::setTabOrder(spinBox_13, slider_cmyk_y);
        QWidget::setTabOrder(slider_cmyk_y, spinBox_12);
        QWidget::setTabOrder(spinBox_12, slider_cmyk_k);
        QWidget::setTabOrder(slider_cmyk_k, spinBox_14);

        retranslateUi(ColorSelector);
        QObject::connect(slider_alpha, SIGNAL(valueChanged(int)), spinBox_4, SLOT(setValue(int)));
        QObject::connect(spinBox_4, SIGNAL(valueChanged(int)), slider_alpha, SLOT(setValue(int)));
        QObject::connect(slider_hsv_hue, SIGNAL(valueChanged(int)), spinBox, SLOT(setValue(int)));
        QObject::connect(spinBox, SIGNAL(valueChanged(int)), slider_hsv_hue, SLOT(setValue(int)));
        QObject::connect(slider_hsv_sat, SIGNAL(valueChanged(int)), spinBox_2, SLOT(setValue(int)));
        QObject::connect(spinBox_2, SIGNAL(valueChanged(int)), slider_hsv_sat, SLOT(setValue(int)));
        QObject::connect(slider_hsv_value, SIGNAL(valueChanged(int)), spinBox_3, SLOT(setValue(int)));
        QObject::connect(spinBox_3, SIGNAL(valueChanged(int)), slider_hsv_value, SLOT(setValue(int)));
        QObject::connect(slider_hsl_hue, SIGNAL(valueChanged(int)), spinBox_5, SLOT(setValue(int)));
        QObject::connect(spinBox_5, SIGNAL(valueChanged(int)), slider_hsl_hue, SLOT(setValue(int)));
        QObject::connect(slider_hsl_sat, SIGNAL(valueChanged(int)), spinBox_6, SLOT(setValue(int)));
        QObject::connect(spinBox_6, SIGNAL(valueChanged(int)), slider_hsl_sat, SLOT(setValue(int)));
        QObject::connect(slider_hsl_light, SIGNAL(valueChanged(int)), spinBox_7, SLOT(setValue(int)));
        QObject::connect(spinBox_7, SIGNAL(valueChanged(int)), slider_hsl_light, SLOT(setValue(int)));
        QObject::connect(color_line_edit, SIGNAL(colorEdited(QColor)), ColorSelector, SLOT(color_update_alpha(QColor)));
        QObject::connect(color_hsv, SIGNAL(colorSelected(QColor)), ColorSelector, SLOT(color_update_noalpha(QColor)));
        QObject::connect(slider_hsv_hue, SIGNAL(valueChanged(int)), ColorSelector, SLOT(color_update_component(int)));
        QObject::connect(slider_hsv_sat, SIGNAL(valueChanged(int)), ColorSelector, SLOT(color_update_component(int)));
        QObject::connect(slider_hsv_value, SIGNAL(valueChanged(int)), ColorSelector, SLOT(color_update_component(int)));
        QObject::connect(slider_alpha, SIGNAL(valueChanged(int)), ColorSelector, SLOT(color_update_component(int)));
        QObject::connect(color_hsl, SIGNAL(colorSelected(QColor)), ColorSelector, SLOT(color_update_noalpha(QColor)));
        QObject::connect(slider_hsl_hue, SIGNAL(valueChanged(int)), ColorSelector, SLOT(color_update_component(int)));
        QObject::connect(slider_hsl_sat, SIGNAL(valueChanged(int)), ColorSelector, SLOT(color_update_component(int)));
        QObject::connect(slider_hsl_light, SIGNAL(valueChanged(int)), ColorSelector, SLOT(color_update_component(int)));
        QObject::connect(slider_rgb_b, SIGNAL(valueChanged(int)), ColorSelector, SLOT(color_update_component(int)));
        QObject::connect(slider_rgb_g, SIGNAL(valueChanged(int)), ColorSelector, SLOT(color_update_component(int)));
        QObject::connect(slider_rgb_r, SIGNAL(valueChanged(int)), ColorSelector, SLOT(color_update_component(int)));
        QObject::connect(slider_cmyk_k, SIGNAL(valueChanged(int)), ColorSelector, SLOT(color_update_component(int)));
        QObject::connect(slider_cmyk_y, SIGNAL(valueChanged(int)), ColorSelector, SLOT(color_update_component(int)));
        QObject::connect(slider_cmyk_m, SIGNAL(valueChanged(int)), ColorSelector, SLOT(color_update_component(int)));
        QObject::connect(slider_cmyk_c, SIGNAL(valueChanged(int)), ColorSelector, SLOT(color_update_component(int)));
        QObject::connect(palette_widget, SIGNAL(currentColorChanged(QColor)), ColorSelector, SLOT(color_update_alpha(QColor)));
        QObject::connect(slider_rgb_r, SIGNAL(valueChanged(int)), spinBox_8, SLOT(setValue(int)));
        QObject::connect(spinBox_8, SIGNAL(valueChanged(int)), slider_rgb_r, SLOT(setValue(int)));
        QObject::connect(slider_rgb_g, SIGNAL(valueChanged(int)), spinBox_9, SLOT(setValue(int)));
        QObject::connect(spinBox_9, SIGNAL(valueChanged(int)), slider_rgb_g, SLOT(setValue(int)));
        QObject::connect(slider_rgb_b, SIGNAL(valueChanged(int)), spinBox_10, SLOT(setValue(int)));
        QObject::connect(spinBox_10, SIGNAL(valueChanged(int)), slider_rgb_b, SLOT(setValue(int)));
        QObject::connect(slider_cmyk_c, SIGNAL(valueChanged(int)), spinBox_11, SLOT(setValue(int)));
        QObject::connect(spinBox_11, SIGNAL(valueChanged(int)), slider_cmyk_c, SLOT(setValue(int)));
        QObject::connect(slider_cmyk_m, SIGNAL(valueChanged(int)), spinBox_13, SLOT(setValue(int)));
        QObject::connect(spinBox_13, SIGNAL(valueChanged(int)), slider_cmyk_m, SLOT(setValue(int)));
        QObject::connect(slider_cmyk_y, SIGNAL(valueChanged(int)), spinBox_12, SLOT(setValue(int)));
        QObject::connect(spinBox_12, SIGNAL(valueChanged(int)), slider_cmyk_y, SLOT(setValue(int)));
        QObject::connect(slider_cmyk_k, SIGNAL(valueChanged(int)), spinBox_14, SLOT(setValue(int)));
        QObject::connect(spinBox_14, SIGNAL(valueChanged(int)), slider_cmyk_k, SLOT(setValue(int)));
        QObject::connect(color_swap, SIGNAL(clicked()), ColorSelector, SLOT(color_swap()));
        QObject::connect(color_preview, SIGNAL(colorSelected(QColor)), ColorSelector, SLOT(color_update_alpha(QColor)));
        QObject::connect(combo_box, &QComboBox::activated, tab_6, &QStackedWidget::setCurrentIndex);

        tab_6->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(ColorSelector);
    } // setupUi

    void retranslateUi(QWidget *ColorSelector)
    {
        combo_box->setItemText(0, tr2i18n("HSV", nullptr));
        combo_box->setItemText(1, tr2i18n("HSL", nullptr));
        combo_box->setItemText(2, tr2i18n("RGB", nullptr));
        combo_box->setItemText(3, tr2i18n("CMYK", nullptr));
        combo_box->setItemText(4, tr2i18n("Palette", nullptr));
        combo_box->setItemText(5, tr2i18n("None", nullptr));

#if QT_CONFIG(tooltip)
        combo_box->setToolTip(tr2i18n("Color Mode", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        label_2->setToolTip(tr2i18n("Saturation", nullptr));
#endif // QT_CONFIG(tooltip)
        label_2->setText(tr2i18n("S", nullptr));
#if QT_CONFIG(tooltip)
        spinBox_3->setToolTip(tr2i18n("Value", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        slider_hsv_hue->setToolTip(tr2i18n("Hue", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        slider_hsv_value->setToolTip(tr2i18n("Value", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        spinBox_2->setToolTip(tr2i18n("Saturation", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        label->setToolTip(tr2i18n("Hue", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(tr2i18n("H", nullptr));
#if QT_CONFIG(tooltip)
        spinBox->setToolTip(tr2i18n("Hue", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        label_3->setToolTip(tr2i18n("Value", nullptr));
#endif // QT_CONFIG(tooltip)
        label_3->setText(tr2i18n("V", nullptr));
#if QT_CONFIG(tooltip)
        slider_hsv_sat->setToolTip(tr2i18n("Saturation", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        slider_hsl_hue->setToolTip(tr2i18n("Hue", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        label_5->setToolTip(tr2i18n("Hue", nullptr));
#endif // QT_CONFIG(tooltip)
        label_5->setText(tr2i18n("H", nullptr));
#if QT_CONFIG(tooltip)
        spinBox_5->setToolTip(tr2i18n("Hue", nullptr));
#endif // QT_CONFIG(tooltip)
        label_6->setText(tr2i18n("S", nullptr));
        label_7->setText(tr2i18n("L", nullptr));
#if QT_CONFIG(tooltip)
        slider_rgb_b->setToolTip(tr2i18n("Blue", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        label_10->setToolTip(tr2i18n("Blue", nullptr));
#endif // QT_CONFIG(tooltip)
        label_10->setText(tr2i18n("B", nullptr));
#if QT_CONFIG(tooltip)
        slider_rgb_g->setToolTip(tr2i18n("Green", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        label_8->setToolTip(tr2i18n("Red", nullptr));
#endif // QT_CONFIG(tooltip)
        label_8->setText(tr2i18n("R", nullptr));
#if QT_CONFIG(tooltip)
        label_9->setToolTip(tr2i18n("Green", nullptr));
#endif // QT_CONFIG(tooltip)
        label_9->setText(tr2i18n("G", nullptr));
#if QT_CONFIG(tooltip)
        slider_rgb_r->setToolTip(tr2i18n("Red", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        spinBox_8->setToolTip(tr2i18n("Red", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        spinBox_9->setToolTip(tr2i18n("Green", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        spinBox_10->setToolTip(tr2i18n("Blue", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        label_11->setToolTip(tr2i18n("Cyan", nullptr));
#endif // QT_CONFIG(tooltip)
        label_11->setText(tr2i18n("C", nullptr));
#if QT_CONFIG(tooltip)
        slider_cmyk_c->setToolTip(tr2i18n("Cyan", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        spinBox_11->setToolTip(tr2i18n("Cyan", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        label_13->setToolTip(tr2i18n("Magenta", nullptr));
#endif // QT_CONFIG(tooltip)
        label_13->setText(tr2i18n("M", nullptr));
#if QT_CONFIG(tooltip)
        slider_cmyk_m->setToolTip(tr2i18n("Magenta", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        spinBox_13->setToolTip(tr2i18n("Magenta", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        label_12->setToolTip(tr2i18n("Yellow", nullptr));
#endif // QT_CONFIG(tooltip)
        label_12->setText(tr2i18n("Y", nullptr));
#if QT_CONFIG(tooltip)
        slider_cmyk_y->setToolTip(tr2i18n("Yellow", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        spinBox_12->setToolTip(tr2i18n("Yellow", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        label_14->setToolTip(tr2i18n("Black", nullptr));
#endif // QT_CONFIG(tooltip)
        label_14->setText(tr2i18n("K", nullptr));
#if QT_CONFIG(tooltip)
        slider_cmyk_k->setToolTip(tr2i18n("Black", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        spinBox_14->setToolTip(tr2i18n("Black", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        slider_alpha->setToolTip(tr2i18n("Alpha (Transparency)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        label_4->setToolTip(tr2i18n("Alpha (Transparency)", nullptr));
#endif // QT_CONFIG(tooltip)
        label_4->setText(tr2i18n("A", nullptr));
#if QT_CONFIG(tooltip)
        spinBox_4->setToolTip(tr2i18n("Alpha (Transparency)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        color_preview->setToolTip(tr2i18n("Primary Color", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        color_preview_secondary->setToolTip(tr2i18n("Secondary Color", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        color_swap->setToolTip(tr2i18n("Swap", nullptr));
#endif // QT_CONFIG(tooltip)
        color_swap->setText(tr2i18n("Swap", nullptr));
        btn_clear->setText(tr2i18n("Clear", nullptr));
        (void)ColorSelector;
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class ColorSelector: public Ui_ColorSelector {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // COLOR_SELECTOR_H

