/****************************************************************************
** Meta object code from reading C++ file 'canvas.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/gui/widgets/canvas.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'canvas.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate3gui6CanvasE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::gui::Canvas::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate3gui6CanvasE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::gui::Canvas",
        "zoomed",
        "",
        "percent",
        "rotated",
        "angle",
        "dropped",
        "const QMimeData*",
        "data",
        "mouse_moved",
        "QPointF",
        "p",
        "translate_view",
        "delta",
        "zoom_view",
        "factor",
        "zoom_view_anchor",
        "anchor",
        "set_zoom",
        "set_zoom_anchor",
        "flip_horizontal",
        "zoom_in",
        "zoom_out",
        "reset_zoom",
        "set_rotation",
        "radians",
        "reset_rotation",
        "view_fit"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'zoomed'
        QtMocHelpers::SignalData<void(qreal)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 3 },
        }}),
        // Signal 'rotated'
        QtMocHelpers::SignalData<void(qreal)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 5 },
        }}),
        // Signal 'dropped'
        QtMocHelpers::SignalData<void(const QMimeData *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'mouse_moved'
        QtMocHelpers::SignalData<void(const QPointF &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'translate_view'
        QtMocHelpers::SlotData<void(const QPointF &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 13 },
        }}),
        // Slot 'zoom_view'
        QtMocHelpers::SlotData<void(qreal)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 15 },
        }}),
        // Slot 'zoom_view_anchor'
        QtMocHelpers::SlotData<void(qreal, const QPointF &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 15 }, { 0x80000000 | 10, 17 },
        }}),
        // Slot 'set_zoom'
        QtMocHelpers::SlotData<void(qreal)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 15 },
        }}),
        // Slot 'set_zoom_anchor'
        QtMocHelpers::SlotData<void(qreal, const QPointF &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 15 }, { 0x80000000 | 10, 17 },
        }}),
        // Slot 'flip_horizontal'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'zoom_in'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'zoom_out'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reset_zoom'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'set_rotation'
        QtMocHelpers::SlotData<void(qreal)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 25 },
        }}),
        // Slot 'reset_rotation'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'view_fit'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Canvas, qt_meta_tag_ZN10glaxnimate3gui6CanvasE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::gui::Canvas::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui6CanvasE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui6CanvasE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate3gui6CanvasE_t>.metaTypes,
    nullptr
} };

void glaxnimate::gui::Canvas::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Canvas *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->zoomed((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 1: _t->rotated((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 2: _t->dropped((*reinterpret_cast<std::add_pointer_t<const QMimeData*>>(_a[1]))); break;
        case 3: _t->mouse_moved((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 4: _t->translate_view((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 5: _t->zoom_view((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 6: _t->zoom_view_anchor((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2]))); break;
        case 7: _t->set_zoom((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 8: _t->set_zoom_anchor((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2]))); break;
        case 9: _t->flip_horizontal(); break;
        case 10: _t->zoom_in(); break;
        case 11: _t->zoom_out(); break;
        case 12: _t->reset_zoom(); break;
        case 13: _t->set_rotation((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 14: _t->reset_rotation(); break;
        case 15: _t->view_fit(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Canvas::*)(qreal )>(_a, &Canvas::zoomed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Canvas::*)(qreal )>(_a, &Canvas::rotated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Canvas::*)(const QMimeData * )>(_a, &Canvas::dropped, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Canvas::*)(const QPointF & )>(_a, &Canvas::mouse_moved, 3))
            return;
    }
}

const QMetaObject *glaxnimate::gui::Canvas::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::gui::Canvas::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui6CanvasE_t>.strings))
        return static_cast<void*>(this);
    return QGraphicsView::qt_metacast(_clname);
}

int glaxnimate::gui::Canvas::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::gui::Canvas::zoomed(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void glaxnimate::gui::Canvas::rotated(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void glaxnimate::gui::Canvas::dropped(const QMimeData * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void glaxnimate::gui::Canvas::mouse_moved(const QPointF & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
