/****************************************************************************
** Meta object code from reading C++ file 'text.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/core/model/shapes/text.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'text.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate5model4FontE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::Font::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model4FontE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::Font",
        "family",
        "property QString",
        "size",
        "property float",
        "style",
        "line_height",
        "font_changed",
        "",
        "family_options",
        "QVariantList",
        "size_options",
        "style_options"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'font_changed'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'family'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'family_options'
        QtMocHelpers::PropertyData<QVariantList>(9, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'size'
        QtMocHelpers::PropertyData<float>(3, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'size_options'
        QtMocHelpers::PropertyData<QVariantList>(11, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'style'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'style_options'
        QtMocHelpers::PropertyData<QVariantList>(12, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'line_height'
        QtMocHelpers::PropertyData<float>(6, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    2 },
            {    6,    4 },
    });
    return QtMocHelpers::metaObjectData<Font, qt_meta_tag_ZN10glaxnimate5model4FontE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::Font::staticMetaObject = { {
    QMetaObject::SuperData::link<Object::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model4FontE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model4FontE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model4FontE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::Font::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Font *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->font_changed(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Font::*)()>(_a, &Font::font_changed, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->get_family(); break;
        case 1: *reinterpret_cast<QVariantList*>(_v) = _t->family_options(); break;
        case 2: *reinterpret_cast<float*>(_v) = _t->get_size(); break;
        case 3: *reinterpret_cast<QVariantList*>(_v) = _t->size_options(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->get_style(); break;
        case 5: *reinterpret_cast<QVariantList*>(_v) = _t->style_options(); break;
        case 6: *reinterpret_cast<float*>(_v) = _t->get_line_height(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->set_family(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->set_size(*reinterpret_cast<float*>(_v)); break;
        case 4: _t->set_style(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->set_line_height(*reinterpret_cast<float*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::Font::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::Font::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model4FontE_t>.strings))
        return static_cast<void*>(this);
    return Object::qt_metacast(_clname);
}

int glaxnimate::model::Font::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Object::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::model::Font::font_changed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN10glaxnimate5model9TextShapeE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::TextShape::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model9TextShapeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::TextShape",
        "text",
        "property QString",
        "position",
        "property animated QPointF",
        "path",
        "property ref model::ShapeElement",
        "path_offset",
        "property animated float",
        "glaxnimate::model::AnimatableBase*",
        "font",
        "Font*",
        "model::ShapeElement*"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'text'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'position'
        QtMocHelpers::PropertyData<glaxnimate::model::AnimatableBase*>(3, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'font'
        QtMocHelpers::PropertyData<Font*>(10, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'path'
        QtMocHelpers::PropertyData<model::ShapeElement*>(5, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'path_offset'
        QtMocHelpers::PropertyData<glaxnimate::model::AnimatableBase*>(7, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<TextShape, qt_meta_tag_ZN10glaxnimate5model9TextShapeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::TextShape::staticMetaObject = { {
    QMetaObject::SuperData::link<ShapeElement::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model9TextShapeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model9TextShapeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model9TextShapeE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::TextShape::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextShape *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Font* >(); break;
        case 4:
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< glaxnimate::model::AnimatableBase* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->get_text(); break;
        case 1: *reinterpret_cast<glaxnimate::model::AnimatableBase**>(_v) = _t->get_position(); break;
        case 2: *reinterpret_cast<Font**>(_v) = _t->get_font(); break;
        case 3: *reinterpret_cast<model::ShapeElement**>(_v) = _t->get_path(); break;
        case 4: *reinterpret_cast<glaxnimate::model::AnimatableBase**>(_v) = _t->get_path_offset(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->set_text(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->set_path(*reinterpret_cast<model::ShapeElement**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::TextShape::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::TextShape::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model9TextShapeE_t>.strings))
        return static_cast<void*>(this);
    return ShapeElement::qt_metacast(_clname);
}

int glaxnimate::model::TextShape::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ShapeElement::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
