/****************************************************************************
** Meta object code from reading C++ file 'view_transform_widget.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/gui/widgets/view_transform_widget.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'view_transform_widget.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate3gui19ViewTransformWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::gui::ViewTransformWidget::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate3gui19ViewTransformWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::gui::ViewTransformWidget",
        "zoom_in",
        "",
        "zoom_out",
        "zoom_changed",
        "percent",
        "angle_changed",
        "radians",
        "view_fit",
        "flip_view",
        "set_zoom",
        "set_angle",
        "set_flip",
        "flipped",
        "fuckyoumoc_on_zoom_changed",
        "fuckyoumoc_on_angle_changed",
        "degrees"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'zoom_in'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'zoom_out'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'zoom_changed'
        QtMocHelpers::SignalData<void(qreal)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 5 },
        }}),
        // Signal 'angle_changed'
        QtMocHelpers::SignalData<void(qreal)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 7 },
        }}),
        // Signal 'view_fit'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'flip_view'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'set_zoom'
        QtMocHelpers::SlotData<void(qreal)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 5 },
        }}),
        // Slot 'set_angle'
        QtMocHelpers::SlotData<void(qreal)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 7 },
        }}),
        // Slot 'set_flip'
        QtMocHelpers::SlotData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Slot 'fuckyoumoc_on_zoom_changed'
        QtMocHelpers::SlotData<void(qreal)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QReal, 5 },
        }}),
        // Slot 'fuckyoumoc_on_angle_changed'
        QtMocHelpers::SlotData<void(qreal)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QReal, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ViewTransformWidget, qt_meta_tag_ZN10glaxnimate3gui19ViewTransformWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::gui::ViewTransformWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui19ViewTransformWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui19ViewTransformWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate3gui19ViewTransformWidgetE_t>.metaTypes,
    nullptr
} };

void glaxnimate::gui::ViewTransformWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ViewTransformWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->zoom_in(); break;
        case 1: _t->zoom_out(); break;
        case 2: _t->zoom_changed((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 3: _t->angle_changed((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 4: _t->view_fit(); break;
        case 5: _t->flip_view(); break;
        case 6: _t->set_zoom((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 7: _t->set_angle((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 8: _t->set_flip((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->fuckyoumoc_on_zoom_changed((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 10: _t->fuckyoumoc_on_angle_changed((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ViewTransformWidget::*)()>(_a, &ViewTransformWidget::zoom_in, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewTransformWidget::*)()>(_a, &ViewTransformWidget::zoom_out, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewTransformWidget::*)(qreal )>(_a, &ViewTransformWidget::zoom_changed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewTransformWidget::*)(qreal )>(_a, &ViewTransformWidget::angle_changed, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewTransformWidget::*)()>(_a, &ViewTransformWidget::view_fit, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewTransformWidget::*)()>(_a, &ViewTransformWidget::flip_view, 5))
            return;
    }
}

const QMetaObject *glaxnimate::gui::ViewTransformWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::gui::ViewTransformWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui19ViewTransformWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int glaxnimate::gui::ViewTransformWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::gui::ViewTransformWidget::zoom_in()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void glaxnimate::gui::ViewTransformWidget::zoom_out()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void glaxnimate::gui::ViewTransformWidget::zoom_changed(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void glaxnimate::gui::ViewTransformWidget::angle_changed(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void glaxnimate::gui::ViewTransformWidget::view_fit()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void glaxnimate::gui::ViewTransformWidget::flip_view()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
