/********************************************************************************
** Form generated from reading UI file 'shape_tool_widget.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SHAPE_TOOL_WIDGET_H
#define UI_SHAPE_TOOL_WIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace glaxnimate {
namespace gui {

class Ui_ShapeToolWidget
{
public:
    QVBoxLayout *layout;
    QGroupBox *group_box;
    QVBoxLayout *verticalLayout;
    QRadioButton *check_raw_shape;
    QRadioButton *check_group;
    QRadioButton *check_layer;
    QCheckBox *check_fill;
    QCheckBox *check_stroke;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *ShapeToolWidget)
    {
        if (ShapeToolWidget->objectName().isEmpty())
            ShapeToolWidget->setObjectName("ShapeToolWidget");
        ShapeToolWidget->resize(239, 194);
        layout = new QVBoxLayout(ShapeToolWidget);
        layout->setObjectName("layout");
        group_box = new QGroupBox(ShapeToolWidget);
        group_box->setObjectName("group_box");
        verticalLayout = new QVBoxLayout(group_box);
        verticalLayout->setObjectName("verticalLayout");
        check_raw_shape = new QRadioButton(group_box);
        check_raw_shape->setObjectName("check_raw_shape");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("shapes");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        check_raw_shape->setIcon(icon);

        verticalLayout->addWidget(check_raw_shape);

        check_group = new QRadioButton(group_box);
        check_group->setObjectName("check_group");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("object-group");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        check_group->setIcon(icon1);
        check_group->setChecked(true);

        verticalLayout->addWidget(check_group);

        check_layer = new QRadioButton(group_box);
        check_layer->setObjectName("check_layer");
        QIcon icon2;
        iconThemeName = QString::fromUtf8("dialog-layers");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        check_layer->setIcon(icon2);

        verticalLayout->addWidget(check_layer);

        check_fill = new QCheckBox(group_box);
        check_fill->setObjectName("check_fill");
        QIcon icon3;
        iconThemeName = QString::fromUtf8("format-fill-color");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        check_fill->setIcon(icon3);
        check_fill->setChecked(true);

        verticalLayout->addWidget(check_fill);

        check_stroke = new QCheckBox(group_box);
        check_stroke->setObjectName("check_stroke");
        QIcon icon4;
        iconThemeName = QString::fromUtf8("format-stroke-color");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon4 = QIcon::fromTheme(iconThemeName);
        } else {
            icon4.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        check_stroke->setIcon(icon4);
        check_stroke->setChecked(true);

        verticalLayout->addWidget(check_stroke);


        layout->addWidget(group_box);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        layout->addItem(verticalSpacer);


        retranslateUi(ShapeToolWidget);
        QObject::connect(check_fill, SIGNAL(clicked()), ShapeToolWidget, SLOT(check_checks()));
        QObject::connect(check_stroke, SIGNAL(clicked()), ShapeToolWidget, SLOT(check_checks()));
        QObject::connect(check_layer, SIGNAL(clicked()), ShapeToolWidget, SLOT(check_checks()));
        QObject::connect(check_group, SIGNAL(clicked()), ShapeToolWidget, SLOT(check_checks()));
        QObject::connect(check_raw_shape, SIGNAL(clicked()), ShapeToolWidget, SLOT(check_checks()));

        QMetaObject::connectSlotsByName(ShapeToolWidget);
    } // setupUi

    void retranslateUi(QWidget *ShapeToolWidget)
    {
        group_box->setTitle(tr2i18n("Create", nullptr));
        check_raw_shape->setText(tr2i18n("Raw Shape", nullptr));
        check_group->setText(tr2i18n("Group", nullptr));
        check_layer->setText(tr2i18n("Layer", nullptr));
        check_fill->setText(tr2i18n("Fill", nullptr));
        check_stroke->setText(tr2i18n("Stroke", nullptr));
        (void)ShapeToolWidget;
    } // retranslateUi

};

} // namespace gui
} // namespace glaxnimate

namespace glaxnimate {
namespace gui {
namespace Ui {
    class ShapeToolWidget: public Ui_ShapeToolWidget {};
} // namespace Ui
} // namespace gui
} // namespace glaxnimate

#endif // SHAPE_TOOL_WIDGET_H

