/****************************************************************************
** Meta object code from reading C++ file 'document_swatch_widget.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/gui/widgets/shape_style/document_swatch_widget.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'document_swatch_widget.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate3gui20DocumentSwatchWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::gui::DocumentSwatchWidget::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate3gui20DocumentSwatchWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::gui::DocumentSwatchWidget",
        "current_color_def",
        "",
        "model::BrushStyle*",
        "def",
        "secondary_color_def",
        "needs_new_color",
        "swatch_link",
        "index",
        "Qt::KeyboardModifiers",
        "mod",
        "swatch_add",
        "swatch_palette_color_added",
        "swatch_palette_color_removed",
        "swatch_palette_color_changed",
        "swatch_doc_color_added",
        "position",
        "model::NamedColor*",
        "color",
        "swatch_doc_color_removed",
        "pos",
        "swatch_doc_color_changed",
        "swatch_menu",
        "generate",
        "open",
        "save"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'current_color_def'
        QtMocHelpers::SignalData<void(model::BrushStyle *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'secondary_color_def'
        QtMocHelpers::SignalData<void(model::BrushStyle *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'needs_new_color'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'swatch_link'
        QtMocHelpers::SlotData<void(int, Qt::KeyboardModifiers)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'swatch_add'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'swatch_palette_color_added'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'swatch_palette_color_removed'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'swatch_palette_color_changed'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'swatch_doc_color_added'
        QtMocHelpers::SlotData<void(int, model::NamedColor *)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 16 }, { 0x80000000 | 17, 18 },
        }}),
        // Slot 'swatch_doc_color_removed'
        QtMocHelpers::SlotData<void(int)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 20 },
        }}),
        // Slot 'swatch_doc_color_changed'
        QtMocHelpers::SlotData<void(int, model::NamedColor *)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 16 }, { 0x80000000 | 17, 18 },
        }}),
        // Slot 'swatch_menu'
        QtMocHelpers::SlotData<void(int)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'generate'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'open'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'save'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DocumentSwatchWidget, qt_meta_tag_ZN10glaxnimate3gui20DocumentSwatchWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::gui::DocumentSwatchWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui20DocumentSwatchWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui20DocumentSwatchWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate3gui20DocumentSwatchWidgetE_t>.metaTypes,
    nullptr
} };

void glaxnimate::gui::DocumentSwatchWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DocumentSwatchWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->current_color_def((*reinterpret_cast<std::add_pointer_t<model::BrushStyle*>>(_a[1]))); break;
        case 1: _t->secondary_color_def((*reinterpret_cast<std::add_pointer_t<model::BrushStyle*>>(_a[1]))); break;
        case 2: _t->needs_new_color(); break;
        case 3: _t->swatch_link((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[2]))); break;
        case 4: _t->swatch_add(); break;
        case 5: _t->swatch_palette_color_added((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->swatch_palette_color_removed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->swatch_palette_color_changed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->swatch_doc_color_added((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<model::NamedColor*>>(_a[2]))); break;
        case 9: _t->swatch_doc_color_removed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->swatch_doc_color_changed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<model::NamedColor*>>(_a[2]))); break;
        case 11: _t->swatch_menu((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->generate(); break;
        case 13: _t->open(); break;
        case 14: _t->save(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DocumentSwatchWidget::*)(model::BrushStyle * )>(_a, &DocumentSwatchWidget::current_color_def, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DocumentSwatchWidget::*)(model::BrushStyle * )>(_a, &DocumentSwatchWidget::secondary_color_def, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DocumentSwatchWidget::*)()>(_a, &DocumentSwatchWidget::needs_new_color, 2))
            return;
    }
}

const QMetaObject *glaxnimate::gui::DocumentSwatchWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::gui::DocumentSwatchWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui20DocumentSwatchWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int glaxnimate::gui::DocumentSwatchWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::gui::DocumentSwatchWidget::current_color_def(model::BrushStyle * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void glaxnimate::gui::DocumentSwatchWidget::secondary_color_def(model::BrushStyle * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void glaxnimate::gui::DocumentSwatchWidget::needs_new_color()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
