/****************************************************************************
** Meta object code from reading C++ file 'handle.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/gui/graphics/handle.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'handle.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate3gui8graphics10MoveHandleE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::gui::graphics::MoveHandle::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate3gui8graphics10MoveHandleE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::gui::graphics::MoveHandle",
        "drag_starting",
        "",
        "QPointF",
        "p",
        "Qt::KeyboardModifiers",
        "modifiers",
        "dragged",
        "dragged_x",
        "x",
        "dragged_y",
        "y",
        "drag_finished",
        "clicked"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'drag_starting'
        QtMocHelpers::SignalData<void(const QPointF &, Qt::KeyboardModifiers)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'dragged'
        QtMocHelpers::SignalData<void(const QPointF &, Qt::KeyboardModifiers)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'dragged_x'
        QtMocHelpers::SignalData<void(qreal, Qt::KeyboardModifiers)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 9 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'dragged_y'
        QtMocHelpers::SignalData<void(qreal, Qt::KeyboardModifiers)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 11 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'drag_finished'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'clicked'
        QtMocHelpers::SignalData<void(Qt::KeyboardModifiers)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MoveHandle, qt_meta_tag_ZN10glaxnimate3gui8graphics10MoveHandleE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::gui::graphics::MoveHandle::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8graphics10MoveHandleE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8graphics10MoveHandleE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate3gui8graphics10MoveHandleE_t>.metaTypes,
    nullptr
} };

void glaxnimate::gui::graphics::MoveHandle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MoveHandle *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->drag_starting((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[2]))); break;
        case 1: _t->dragged((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[2]))); break;
        case 2: _t->dragged_x((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[2]))); break;
        case 3: _t->dragged_y((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[2]))); break;
        case 4: _t->drag_finished(); break;
        case 5: _t->clicked((*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MoveHandle::*)(const QPointF & , Qt::KeyboardModifiers )>(_a, &MoveHandle::drag_starting, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MoveHandle::*)(const QPointF & , Qt::KeyboardModifiers )>(_a, &MoveHandle::dragged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MoveHandle::*)(qreal , Qt::KeyboardModifiers )>(_a, &MoveHandle::dragged_x, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MoveHandle::*)(qreal , Qt::KeyboardModifiers )>(_a, &MoveHandle::dragged_y, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MoveHandle::*)()>(_a, &MoveHandle::drag_finished, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MoveHandle::*)(Qt::KeyboardModifiers )>(_a, &MoveHandle::clicked, 5))
            return;
    }
}

const QMetaObject *glaxnimate::gui::graphics::MoveHandle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::gui::graphics::MoveHandle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui8graphics10MoveHandleE_t>.strings))
        return static_cast<void*>(this);
    return QGraphicsObject::qt_metacast(_clname);
}

int glaxnimate::gui::graphics::MoveHandle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::gui::graphics::MoveHandle::drag_starting(const QPointF & _t1, Qt::KeyboardModifiers _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void glaxnimate::gui::graphics::MoveHandle::dragged(const QPointF & _t1, Qt::KeyboardModifiers _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void glaxnimate::gui::graphics::MoveHandle::dragged_x(qreal _t1, Qt::KeyboardModifiers _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void glaxnimate::gui::graphics::MoveHandle::dragged_y(qreal _t1, Qt::KeyboardModifiers _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void glaxnimate::gui::graphics::MoveHandle::drag_finished()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void glaxnimate::gui::graphics::MoveHandle::clicked(Qt::KeyboardModifiers _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
