/*
 * SPDX-FileCopyrightText: 2019-2025 Mattia Basaglia <dev@dragon.best>
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#pragma once

namespace glaxnimate::model {
    
using FrameTime = double;

} // namespace glaxnimate::model
