/****************************************************************************
** Meta object code from reading C++ file 'frame_controls_widget.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/gui/widgets/timeline/frame_controls_widget.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'frame_controls_widget.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate3gui19FrameControlsWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::gui::FrameControlsWidget::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate3gui19FrameControlsWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::gui::FrameControlsWidget",
        "frame_selected",
        "",
        "frame",
        "record_toggled",
        "enabled",
        "play_started",
        "play_stopped",
        "loop_changed",
        "b",
        "min_changed",
        "min",
        "max_changed",
        "max",
        "fps_changed",
        "fps",
        "play",
        "pause",
        "toggle_play",
        "set_record_enabled",
        "set_loop",
        "loop",
        "go_first",
        "go_prev",
        "go_next",
        "go_last",
        "play_toggled",
        "commit_time"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'frame_selected'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'record_toggled'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Signal 'play_started'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'play_stopped'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loop_changed'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'min_changed'
        QtMocHelpers::SignalData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Signal 'max_changed'
        QtMocHelpers::SignalData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Signal 'fps_changed'
        QtMocHelpers::SignalData<void(qreal)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 15 },
        }}),
        // Slot 'play'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'pause'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggle_play'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'set_record_enabled'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Slot 'set_loop'
        QtMocHelpers::SlotData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 21 },
        }}),
        // Slot 'go_first'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'go_prev'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'go_next'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'go_last'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'play_toggled'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'commit_time'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FrameControlsWidget, qt_meta_tag_ZN10glaxnimate3gui19FrameControlsWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::gui::FrameControlsWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui19FrameControlsWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui19FrameControlsWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate3gui19FrameControlsWidgetE_t>.metaTypes,
    nullptr
} };

void glaxnimate::gui::FrameControlsWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FrameControlsWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->frame_selected((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->record_toggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->play_started(); break;
        case 3: _t->play_stopped(); break;
        case 4: _t->loop_changed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->min_changed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->max_changed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->fps_changed((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 8: _t->play(); break;
        case 9: _t->pause(); break;
        case 10: _t->toggle_play(); break;
        case 11: _t->set_record_enabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->set_loop((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->go_first(); break;
        case 14: _t->go_prev(); break;
        case 15: _t->go_next(); break;
        case 16: _t->go_last(); break;
        case 17: _t->play_toggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->commit_time(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FrameControlsWidget::*)(int )>(_a, &FrameControlsWidget::frame_selected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameControlsWidget::*)(bool )>(_a, &FrameControlsWidget::record_toggled, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameControlsWidget::*)()>(_a, &FrameControlsWidget::play_started, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameControlsWidget::*)()>(_a, &FrameControlsWidget::play_stopped, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameControlsWidget::*)(bool )>(_a, &FrameControlsWidget::loop_changed, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameControlsWidget::*)(int )>(_a, &FrameControlsWidget::min_changed, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameControlsWidget::*)(int )>(_a, &FrameControlsWidget::max_changed, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameControlsWidget::*)(qreal )>(_a, &FrameControlsWidget::fps_changed, 7))
            return;
    }
}

const QMetaObject *glaxnimate::gui::FrameControlsWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::gui::FrameControlsWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate3gui19FrameControlsWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int glaxnimate::gui::FrameControlsWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::gui::FrameControlsWidget::frame_selected(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void glaxnimate::gui::FrameControlsWidget::record_toggled(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void glaxnimate::gui::FrameControlsWidget::play_started()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void glaxnimate::gui::FrameControlsWidget::play_stopped()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void glaxnimate::gui::FrameControlsWidget::loop_changed(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void glaxnimate::gui::FrameControlsWidget::min_changed(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void glaxnimate::gui::FrameControlsWidget::max_changed(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void glaxnimate::gui::FrameControlsWidget::fps_changed(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
