/****************************************************************************
** Meta object code from reading C++ file 'assets.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/core/model/assets/assets.hpp"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'assets.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate5model14NamedColorListE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::NamedColorList::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model14NamedColorListE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::NamedColorList",
        "values",
        "property list NamedColor",
        "color_changed",
        "",
        "position",
        "model::NamedColor*",
        "color",
        "color_added",
        "color_removed",
        "QVariantList"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'color_changed'
        QtMocHelpers::SignalData<void(int, model::NamedColor *)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'color_added'
        QtMocHelpers::SignalData<void(int, model::NamedColor *)>(8, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'color_removed'
        QtMocHelpers::SignalData<void(int, model::NamedColor *)>(9, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { 0x80000000 | 6, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'values'
        QtMocHelpers::PropertyData<QVariantList>(1, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<NamedColorList, qt_meta_tag_ZN10glaxnimate5model14NamedColorListE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::NamedColorList::staticMetaObject = { {
    QMetaObject::SuperData::link<AssetListBase<NamedColor,NamedColorList>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model14NamedColorListE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model14NamedColorListE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model14NamedColorListE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::NamedColorList::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NamedColorList *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->color_changed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<model::NamedColor*>>(_a[2]))); break;
        case 1: _t->color_added((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<model::NamedColor*>>(_a[2]))); break;
        case 2: _t->color_removed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<model::NamedColor*>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (NamedColorList::*)(int , model::NamedColor * )>(_a, &NamedColorList::color_changed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (NamedColorList::*)(int , model::NamedColor * )>(_a, &NamedColorList::color_added, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (NamedColorList::*)(int , model::NamedColor * )>(_a, &NamedColorList::color_removed, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QVariantList*>(_v) = _t->get_values(); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::NamedColorList::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::NamedColorList::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model14NamedColorListE_t>.strings))
        return static_cast<void*>(this);
    return AssetListBase<NamedColor,NamedColorList>::qt_metacast(_clname);
}

int glaxnimate::model::NamedColorList::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AssetListBase<NamedColor,NamedColorList>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::model::NamedColorList::color_changed(int _t1, model::NamedColor * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void glaxnimate::model::NamedColorList::color_added(int _t1, model::NamedColor * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void glaxnimate::model::NamedColorList::color_removed(int _t1, model::NamedColor * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN10glaxnimate5model10BitmapListE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::BitmapList::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model10BitmapListE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::BitmapList",
        "values",
        "property list Bitmap",
        "QVariantList"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'values'
        QtMocHelpers::PropertyData<QVariantList>(1, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<BitmapList, qt_meta_tag_ZN10glaxnimate5model10BitmapListE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::BitmapList::staticMetaObject = { {
    QMetaObject::SuperData::link<AssetListBase<Bitmap,BitmapList>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model10BitmapListE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model10BitmapListE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model10BitmapListE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::BitmapList::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BitmapList *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QVariantList*>(_v) = _t->get_values(); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::BitmapList::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::BitmapList::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model10BitmapListE_t>.strings))
        return static_cast<void*>(this);
    return AssetListBase<Bitmap,BitmapList>::qt_metacast(_clname);
}

int glaxnimate::model::BitmapList::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AssetListBase<Bitmap,BitmapList>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN10glaxnimate5model18GradientColorsListE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::GradientColorsList::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model18GradientColorsListE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::GradientColorsList",
        "values",
        "property list GradientColors",
        "QVariantList"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'values'
        QtMocHelpers::PropertyData<QVariantList>(1, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<GradientColorsList, qt_meta_tag_ZN10glaxnimate5model18GradientColorsListE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::GradientColorsList::staticMetaObject = { {
    QMetaObject::SuperData::link<AssetListBase<GradientColors,GradientColorsList>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model18GradientColorsListE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model18GradientColorsListE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model18GradientColorsListE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::GradientColorsList::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GradientColorsList *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QVariantList*>(_v) = _t->get_values(); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::GradientColorsList::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::GradientColorsList::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model18GradientColorsListE_t>.strings))
        return static_cast<void*>(this);
    return AssetListBase<GradientColors,GradientColorsList>::qt_metacast(_clname);
}

int glaxnimate::model::GradientColorsList::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AssetListBase<GradientColors,GradientColorsList>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN10glaxnimate5model12GradientListE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::GradientList::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model12GradientListE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::GradientList",
        "values",
        "property list Gradient",
        "QVariantList"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'values'
        QtMocHelpers::PropertyData<QVariantList>(1, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<GradientList, qt_meta_tag_ZN10glaxnimate5model12GradientListE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::GradientList::staticMetaObject = { {
    QMetaObject::SuperData::link<AssetListBase<Gradient,GradientList>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model12GradientListE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model12GradientListE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model12GradientListE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::GradientList::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GradientList *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QVariantList*>(_v) = _t->get_values(); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::GradientList::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::GradientList::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model12GradientListE_t>.strings))
        return static_cast<void*>(this);
    return AssetListBase<Gradient,GradientList>::qt_metacast(_clname);
}

int glaxnimate::model::GradientList::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AssetListBase<Gradient,GradientList>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN10glaxnimate5model15CompositionListE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::CompositionList::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model15CompositionListE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::CompositionList",
        "values",
        "property list Composition",
        "precomp_added",
        "",
        "model::Composition*",
        "obj",
        "position",
        "QVariantList"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'precomp_added'
        QtMocHelpers::SignalData<void(model::Composition *, int)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { QMetaType::Int, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'values'
        QtMocHelpers::PropertyData<QVariantList>(1, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<CompositionList, qt_meta_tag_ZN10glaxnimate5model15CompositionListE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::CompositionList::staticMetaObject = { {
    QMetaObject::SuperData::link<AssetListBase<Composition,CompositionList>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model15CompositionListE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model15CompositionListE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model15CompositionListE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::CompositionList::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CompositionList *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->precomp_added((*reinterpret_cast<std::add_pointer_t<model::Composition*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CompositionList::*)(model::Composition * , int )>(_a, &CompositionList::precomp_added, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QVariantList*>(_v) = _t->get_values(); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::CompositionList::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::CompositionList::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model15CompositionListE_t>.strings))
        return static_cast<void*>(this);
    return AssetListBase<Composition,CompositionList>::qt_metacast(_clname);
}

int glaxnimate::model::CompositionList::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AssetListBase<Composition,CompositionList>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::model::CompositionList::precomp_added(model::Composition * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN10glaxnimate5model8FontListE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::FontList::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model8FontListE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::FontList",
        "values",
        "property list EmbeddedFont",
        "font_added",
        "",
        "model::EmbeddedFont*",
        "font",
        "QVariantList"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'font_added'
        QtMocHelpers::SignalData<void(model::EmbeddedFont *)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'values'
        QtMocHelpers::PropertyData<QVariantList>(1, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<FontList, qt_meta_tag_ZN10glaxnimate5model8FontListE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::FontList::staticMetaObject = { {
    QMetaObject::SuperData::link<AssetListBase<EmbeddedFont,FontList>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model8FontListE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model8FontListE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model8FontListE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::FontList::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FontList *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->font_added((*reinterpret_cast<std::add_pointer_t<model::EmbeddedFont*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FontList::*)(model::EmbeddedFont * )>(_a, &FontList::font_added, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QVariantList*>(_v) = _t->get_values(); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::FontList::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::FontList::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model8FontListE_t>.strings))
        return static_cast<void*>(this);
    return AssetListBase<EmbeddedFont,FontList>::qt_metacast(_clname);
}

int glaxnimate::model::FontList::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AssetListBase<EmbeddedFont,FontList>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::model::FontList::font_added(model::EmbeddedFont * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN10glaxnimate5model6AssetsE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::Assets::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model6AssetsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::Assets",
        "add_color",
        "glaxnimate::model::NamedColor*",
        "",
        "QColor",
        "color",
        "name",
        "add_image_file",
        "glaxnimate::model::Bitmap*",
        "filename",
        "embed",
        "add_image",
        "QImage",
        "image",
        "store_as",
        "add_gradient_colors",
        "glaxnimate::model::GradientColors*",
        "index",
        "add_gradient",
        "glaxnimate::model::Gradient*",
        "add_font",
        "glaxnimate::model::EmbeddedFont*",
        "ttf_data",
        "font_by_index",
        "database_index",
        "add_composition",
        "glaxnimate::model::Composition*",
        "colors",
        "NamedColorList*",
        "images",
        "BitmapList*",
        "gradient_colors",
        "GradientColorsList*",
        "gradients",
        "GradientList*",
        "compositions",
        "CompositionList*",
        "fonts",
        "FontList*"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'add_color'
        QtMocHelpers::MethodData<glaxnimate::model::NamedColor *(const QColor &, const QString &)>(1, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { 0x80000000 | 4, 5 }, { QMetaType::QString, 6 },
        }}),
        // Method 'add_color'
        QtMocHelpers::MethodData<glaxnimate::model::NamedColor *(const QColor &)>(1, 3, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 2, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Method 'add_image_file'
        QtMocHelpers::MethodData<glaxnimate::model::Bitmap *(const QString &, bool)>(7, 3, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QString, 9 }, { QMetaType::Bool, 10 },
        }}),
        // Method 'add_image'
        QtMocHelpers::MethodData<glaxnimate::model::Bitmap *(const QImage &, const QString &)>(11, 3, QMC::AccessPublic, 0x80000000 | 8, {{
            { 0x80000000 | 12, 13 }, { QMetaType::QString, 14 },
        }}),
        // Method 'add_image'
        QtMocHelpers::MethodData<glaxnimate::model::Bitmap *(const QImage &)>(11, 3, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 8, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Method 'add_gradient_colors'
        QtMocHelpers::MethodData<glaxnimate::model::GradientColors *(int)>(15, 3, QMC::AccessPublic, 0x80000000 | 16, {{
            { QMetaType::Int, 17 },
        }}),
        // Method 'add_gradient_colors'
        QtMocHelpers::MethodData<glaxnimate::model::GradientColors *()>(15, 3, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 16),
        // Method 'add_gradient'
        QtMocHelpers::MethodData<glaxnimate::model::Gradient *(int)>(18, 3, QMC::AccessPublic, 0x80000000 | 19, {{
            { QMetaType::Int, 17 },
        }}),
        // Method 'add_gradient'
        QtMocHelpers::MethodData<glaxnimate::model::Gradient *()>(18, 3, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 19),
        // Method 'add_font'
        QtMocHelpers::MethodData<glaxnimate::model::EmbeddedFont *(const QByteArray &)>(20, 3, QMC::AccessPublic, 0x80000000 | 21, {{
            { QMetaType::QByteArray, 22 },
        }}),
        // Method 'font_by_index'
        QtMocHelpers::MethodData<glaxnimate::model::EmbeddedFont *(int) const>(23, 3, QMC::AccessPublic, 0x80000000 | 21, {{
            { QMetaType::Int, 24 },
        }}),
        // Method 'add_composition'
        QtMocHelpers::MethodData<glaxnimate::model::Composition *()>(25, 3, QMC::AccessPublic, 0x80000000 | 26),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'colors'
        QtMocHelpers::PropertyData<NamedColorList*>(27, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'images'
        QtMocHelpers::PropertyData<BitmapList*>(29, 0x80000000 | 30, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'gradient_colors'
        QtMocHelpers::PropertyData<GradientColorsList*>(31, 0x80000000 | 32, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'gradients'
        QtMocHelpers::PropertyData<GradientList*>(33, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'compositions'
        QtMocHelpers::PropertyData<CompositionList*>(35, 0x80000000 | 36, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'fonts'
        QtMocHelpers::PropertyData<FontList*>(37, 0x80000000 | 38, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Assets, qt_meta_tag_ZN10glaxnimate5model6AssetsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::Assets::staticMetaObject = { {
    QMetaObject::SuperData::link<DocumentNode::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model6AssetsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model6AssetsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model6AssetsE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::Assets::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Assets *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { glaxnimate::model::NamedColor* _r = _t->add_color((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<glaxnimate::model::NamedColor**>(_a[0]) = std::move(_r); }  break;
        case 1: { glaxnimate::model::NamedColor* _r = _t->add_color((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])));
            if (_a[0]) *reinterpret_cast<glaxnimate::model::NamedColor**>(_a[0]) = std::move(_r); }  break;
        case 2: { glaxnimate::model::Bitmap* _r = _t->add_image_file((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<glaxnimate::model::Bitmap**>(_a[0]) = std::move(_r); }  break;
        case 3: { glaxnimate::model::Bitmap* _r = _t->add_image((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<glaxnimate::model::Bitmap**>(_a[0]) = std::move(_r); }  break;
        case 4: { glaxnimate::model::Bitmap* _r = _t->add_image((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1])));
            if (_a[0]) *reinterpret_cast<glaxnimate::model::Bitmap**>(_a[0]) = std::move(_r); }  break;
        case 5: { glaxnimate::model::GradientColors* _r = _t->add_gradient_colors((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<glaxnimate::model::GradientColors**>(_a[0]) = std::move(_r); }  break;
        case 6: { glaxnimate::model::GradientColors* _r = _t->add_gradient_colors();
            if (_a[0]) *reinterpret_cast<glaxnimate::model::GradientColors**>(_a[0]) = std::move(_r); }  break;
        case 7: { glaxnimate::model::Gradient* _r = _t->add_gradient((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<glaxnimate::model::Gradient**>(_a[0]) = std::move(_r); }  break;
        case 8: { glaxnimate::model::Gradient* _r = _t->add_gradient();
            if (_a[0]) *reinterpret_cast<glaxnimate::model::Gradient**>(_a[0]) = std::move(_r); }  break;
        case 9: { glaxnimate::model::EmbeddedFont* _r = _t->add_font((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<glaxnimate::model::EmbeddedFont**>(_a[0]) = std::move(_r); }  break;
        case 10: { glaxnimate::model::EmbeddedFont* _r = _t->font_by_index((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<glaxnimate::model::EmbeddedFont**>(_a[0]) = std::move(_r); }  break;
        case 11: { glaxnimate::model::Composition* _r = _t->add_composition();
            if (_a[0]) *reinterpret_cast<glaxnimate::model::Composition**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< BitmapList* >(); break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< CompositionList* >(); break;
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< FontList* >(); break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< GradientColorsList* >(); break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< GradientList* >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< NamedColorList* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<NamedColorList**>(_v) = _t->get_colors(); break;
        case 1: *reinterpret_cast<BitmapList**>(_v) = _t->get_images(); break;
        case 2: *reinterpret_cast<GradientColorsList**>(_v) = _t->get_gradient_colors(); break;
        case 3: *reinterpret_cast<GradientList**>(_v) = _t->get_gradients(); break;
        case 4: *reinterpret_cast<CompositionList**>(_v) = _t->get_compositions(); break;
        case 5: *reinterpret_cast<FontList**>(_v) = _t->get_fonts(); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::Assets::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::Assets::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model6AssetsE_t>.strings))
        return static_cast<void*>(this);
    return DocumentNode::qt_metacast(_clname);
}

int glaxnimate::model::Assets::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DocumentNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}
QT_WARNING_POP
