/****************************************************************************
** Meta object code from reading C++ file 'composition.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/core/model/assets/composition.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'composition.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate5model11CompositionE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::Composition::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model11CompositionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::Composition",
        "shapes",
        "property list model::ShapeElement",
        "fps",
        "property float",
        "width",
        "height",
        "fps_changed",
        "",
        "width_changed",
        "height_changed",
        "render_image",
        "QImage",
        "time",
        "QSize",
        "size",
        "QColor",
        "background",
        "QVariantList",
        "animation",
        "AnimationContainer*",
        "QSizeF",
        "rect",
        "QRectF"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fps_changed'
        QtMocHelpers::SignalData<void(float)>(7, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 3 },
        }}),
        // Signal 'width_changed'
        QtMocHelpers::SignalData<void(float)>(9, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 8 },
        }}),
        // Signal 'height_changed'
        QtMocHelpers::SignalData<void(float)>(10, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 8 },
        }}),
        // Method 'render_image'
        QtMocHelpers::MethodData<QImage(float, QSize, const QColor &) const>(11, 8, QMC::AccessPublic, 0x80000000 | 12, {{
            { QMetaType::Float, 13 }, { 0x80000000 | 14, 15 }, { 0x80000000 | 16, 17 },
        }}),
        // Method 'render_image'
        QtMocHelpers::MethodData<QImage(float, QSize) const>(11, 8, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 12, {{
            { QMetaType::Float, 13 }, { 0x80000000 | 14, 15 },
        }}),
        // Method 'render_image'
        QtMocHelpers::MethodData<QImage(float) const>(11, 8, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 12, {{
            { QMetaType::Float, 13 },
        }}),
        // Method 'render_image'
        QtMocHelpers::MethodData<QImage() const>(11, 8, QMC::AccessPublic, 0x80000000 | 12),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'shapes'
        QtMocHelpers::PropertyData<QVariantList>(1, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'animation'
        QtMocHelpers::PropertyData<AnimationContainer*>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'fps'
        QtMocHelpers::PropertyData<float>(3, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'width'
        QtMocHelpers::PropertyData<float>(5, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'height'
        QtMocHelpers::PropertyData<float>(6, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'size'
        QtMocHelpers::PropertyData<QSizeF>(15, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'rect'
        QtMocHelpers::PropertyData<QRectF>(22, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    4 },
            {    6,    4 },
    });
    return QtMocHelpers::metaObjectData<Composition, qt_meta_tag_ZN10glaxnimate5model11CompositionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::Composition::staticMetaObject = { {
    QMetaObject::SuperData::link<VisualNode::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model11CompositionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model11CompositionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model11CompositionE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::Composition::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Composition *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fps_changed((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 1: _t->width_changed((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 2: _t->height_changed((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 3: { QImage _r = _t->render_image((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSize>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QColor>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QImage*>(_a[0]) = std::move(_r); }  break;
        case 4: { QImage _r = _t->render_image((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSize>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QImage*>(_a[0]) = std::move(_r); }  break;
        case 5: { QImage _r = _t->render_image((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QImage*>(_a[0]) = std::move(_r); }  break;
        case 6: { QImage _r = _t->render_image();
            if (_a[0]) *reinterpret_cast<QImage*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Composition::*)(float )>(_a, &Composition::fps_changed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Composition::*)(float )>(_a, &Composition::width_changed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Composition::*)(float )>(_a, &Composition::height_changed, 2))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< AnimationContainer* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QVariantList*>(_v) = _t->get_shapes(); break;
        case 1: *reinterpret_cast<AnimationContainer**>(_v) = _t->get_animation(); break;
        case 2: *reinterpret_cast<float*>(_v) = _t->get_fps(); break;
        case 3: *reinterpret_cast<float*>(_v) = _t->get_width(); break;
        case 4: *reinterpret_cast<float*>(_v) = _t->get_height(); break;
        case 5: *reinterpret_cast<QSizeF*>(_v) = _t->size(); break;
        case 6: *reinterpret_cast<QRectF*>(_v) = _t->rect(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->set_fps(*reinterpret_cast<float*>(_v)); break;
        case 3: _t->set_width(*reinterpret_cast<float*>(_v)); break;
        case 4: _t->set_height(*reinterpret_cast<float*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::Composition::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::Composition::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model11CompositionE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "AssetBase"))
        return static_cast< AssetBase*>(this);
    return VisualNode::qt_metacast(_clname);
}

int glaxnimate::model::Composition::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = VisualNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void glaxnimate::model::Composition::fps_changed(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void glaxnimate::model::Composition::width_changed(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void glaxnimate::model::Composition::height_changed(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
