/********************************************************************************
** Form generated from reading UI file 'lottiefiles_search_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LOTTIEFILES_SEARCH_DIALOG_H
#define UI_LOTTIEFILES_SEARCH_DIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_LottieFilesSearchDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QLineEdit *input_query;
    QPushButton *pushButton_4;
    QScrollArea *result_area;
    QWidget *result_area_content;
    QGridLayout *result_area_layout;
    QProgressBar *progress_bar;
    QLabel *label_error;
    QHBoxLayout *horizontalLayout_3;
    QPushButton *button_previous;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *button_next;
    QHBoxLayout *horizontalLayout;
    QPushButton *button_open;
    QPushButton *button_import;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushButton_3;

    void setupUi(QDialog *LottieFilesSearchDialog)
    {
        if (LottieFilesSearchDialog->objectName().isEmpty())
            LottieFilesSearchDialog->setObjectName("LottieFilesSearchDialog");
        LottieFilesSearchDialog->resize(822, 646);
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("lottiefiles");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        LottieFilesSearchDialog->setWindowIcon(icon);
        verticalLayout = new QVBoxLayout(LottieFilesSearchDialog);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        input_query = new QLineEdit(LottieFilesSearchDialog);
        input_query->setObjectName("input_query");

        horizontalLayout_2->addWidget(input_query);

        pushButton_4 = new QPushButton(LottieFilesSearchDialog);
        pushButton_4->setObjectName("pushButton_4");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("search");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        pushButton_4->setIcon(icon1);

        horizontalLayout_2->addWidget(pushButton_4);


        verticalLayout->addLayout(horizontalLayout_2);

        result_area = new QScrollArea(LottieFilesSearchDialog);
        result_area->setObjectName("result_area");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(result_area->sizePolicy().hasHeightForWidth());
        result_area->setSizePolicy(sizePolicy);
        result_area->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
        result_area->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        result_area->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
        result_area->setWidgetResizable(true);
        result_area_content = new QWidget();
        result_area_content->setObjectName("result_area_content");
        result_area_content->setGeometry(QRect(0, 0, 785, 448));
        result_area_layout = new QGridLayout(result_area_content);
        result_area_layout->setSpacing(0);
        result_area_layout->setObjectName("result_area_layout");
        result_area->setWidget(result_area_content);

        verticalLayout->addWidget(result_area);

        progress_bar = new QProgressBar(LottieFilesSearchDialog);
        progress_bar->setObjectName("progress_bar");

        verticalLayout->addWidget(progress_bar);

        label_error = new QLabel(LottieFilesSearchDialog);
        label_error->setObjectName("label_error");
        label_error->setText(QString::fromUtf8("Network Error"));

        verticalLayout->addWidget(label_error);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        button_previous = new QPushButton(LottieFilesSearchDialog);
        button_previous->setObjectName("button_previous");
        QIcon icon2(QIcon::fromTheme(QString::fromUtf8("go-previous")));
        button_previous->setIcon(icon2);

        horizontalLayout_3->addWidget(button_previous);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);

        button_next = new QPushButton(LottieFilesSearchDialog);
        button_next->setObjectName("button_next");
        QIcon icon3;
        iconThemeName = QString::fromUtf8("go-next");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_next->setIcon(icon3);

        horizontalLayout_3->addWidget(button_next);


        verticalLayout->addLayout(horizontalLayout_3);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        button_open = new QPushButton(LottieFilesSearchDialog);
        button_open->setObjectName("button_open");
        QIcon icon4;
        iconThemeName = QString::fromUtf8("document-open");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon4 = QIcon::fromTheme(iconThemeName);
        } else {
            icon4.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_open->setIcon(icon4);

        horizontalLayout->addWidget(button_open);

        button_import = new QPushButton(LottieFilesSearchDialog);
        button_import->setObjectName("button_import");
        QIcon icon5;
        iconThemeName = QString::fromUtf8("document-import");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon5 = QIcon::fromTheme(iconThemeName);
        } else {
            icon5.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_import->setIcon(icon5);

        horizontalLayout->addWidget(button_import);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushButton_3 = new QPushButton(LottieFilesSearchDialog);
        pushButton_3->setObjectName("pushButton_3");
        QIcon icon6;
        iconThemeName = QString::fromUtf8("dialog-cancel");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon6 = QIcon::fromTheme(iconThemeName);
        } else {
            icon6.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        pushButton_3->setIcon(icon6);

        horizontalLayout->addWidget(pushButton_3);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(LottieFilesSearchDialog);
        QObject::connect(button_open, SIGNAL(clicked()), LottieFilesSearchDialog, SLOT(clicked_open()));
        QObject::connect(button_import, SIGNAL(clicked()), LottieFilesSearchDialog, SLOT(clicked_import()));
        QObject::connect(pushButton_3, &QPushButton::clicked, LottieFilesSearchDialog, qOverload<>(&QDialog::reject));
        QObject::connect(pushButton_4, SIGNAL(clicked()), LottieFilesSearchDialog, SLOT(clicked_search()));
        QObject::connect(button_next, SIGNAL(clicked()), LottieFilesSearchDialog, SLOT(clicked_next()));
        QObject::connect(button_previous, SIGNAL(clicked()), LottieFilesSearchDialog, SLOT(clicked_previous()));

        QMetaObject::connectSlotsByName(LottieFilesSearchDialog);
    } // setupUi

    void retranslateUi(QDialog *LottieFilesSearchDialog)
    {
        LottieFilesSearchDialog->setWindowTitle(tr2i18n("Import From LottieFiles", nullptr));
        pushButton_4->setText(tr2i18n("Search", nullptr));
        button_previous->setText(tr2i18n("Previous Page", nullptr));
        button_next->setText(tr2i18n("Next Page", nullptr));
        button_open->setText(tr2i18n("Open", nullptr));
        button_import->setText(tr2i18n("Import", nullptr));
        pushButton_3->setText(tr2i18n("Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LottieFilesSearchDialog: public Ui_LottieFilesSearchDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // LOTTIEFILES_SEARCH_DIALOG_H

