/****************************************************************************
** Meta object code from reading C++ file 'hue_slider.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/external/Qt-Color-Widgets/include/QtColorWidgets/hue_slider.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'hue_slider.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13color_widgets9HueSliderE_t {};
} // unnamed namespace

template <> constexpr inline auto color_widgets::HueSlider::qt_create_metaobjectdata<qt_meta_tag_ZN13color_widgets9HueSliderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "color_widgets::HueSlider",
        "colorHueChanged",
        "",
        "colorHue",
        "colorChanged",
        "QColor",
        "colorAlphaChanged",
        "v",
        "colorSaturationChanged",
        "colorValueChanged",
        "setColorValue",
        "value",
        "setColorSaturation",
        "setColorAlpha",
        "alpha",
        "setColorHue",
        "setColor",
        "color",
        "setFullColor",
        "colorSaturation",
        "colorValue",
        "colorAlpha"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'colorHueChanged'
        QtMocHelpers::SignalData<void(qreal)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 3 },
        }}),
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(QColor)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'colorAlphaChanged'
        QtMocHelpers::SignalData<void(qreal)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 7 },
        }}),
        // Signal 'colorSaturationChanged'
        QtMocHelpers::SignalData<void(qreal)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 7 },
        }}),
        // Signal 'colorValueChanged'
        QtMocHelpers::SignalData<void(qreal)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 7 },
        }}),
        // Slot 'setColorValue'
        QtMocHelpers::SlotData<void(qreal)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 11 },
        }}),
        // Slot 'setColorSaturation'
        QtMocHelpers::SlotData<void(qreal)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 11 },
        }}),
        // Slot 'setColorAlpha'
        QtMocHelpers::SlotData<void(qreal)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 14 },
        }}),
        // Slot 'setColorHue'
        QtMocHelpers::SlotData<void(qreal)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 3 },
        }}),
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(const QColor &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 17 },
        }}),
        // Slot 'setFullColor'
        QtMocHelpers::SlotData<void(const QColor &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'colorSaturation'
        QtMocHelpers::PropertyData<qreal>(19, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'colorValue'
        QtMocHelpers::PropertyData<qreal>(20, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'colorAlpha'
        QtMocHelpers::PropertyData<qreal>(21, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(17, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'colorHue'
        QtMocHelpers::PropertyData<qreal>(3, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<HueSlider, qt_meta_tag_ZN13color_widgets9HueSliderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject color_widgets::HueSlider::staticMetaObject = { {
    QMetaObject::SuperData::link<GradientSlider::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets9HueSliderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets9HueSliderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13color_widgets9HueSliderE_t>.metaTypes,
    nullptr
} };

void color_widgets::HueSlider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<HueSlider *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->colorHueChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 1: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 2: _t->colorAlphaChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 3: _t->colorSaturationChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 4: _t->colorValueChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 5: _t->setColorValue((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 6: _t->setColorSaturation((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 7: _t->setColorAlpha((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 8: _t->setColorHue((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 9: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 10: _t->setFullColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (HueSlider::*)(qreal )>(_a, &HueSlider::colorHueChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (HueSlider::*)(QColor )>(_a, &HueSlider::colorChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (HueSlider::*)(qreal )>(_a, &HueSlider::colorAlphaChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (HueSlider::*)(qreal )>(_a, &HueSlider::colorSaturationChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (HueSlider::*)(qreal )>(_a, &HueSlider::colorValueChanged, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qreal*>(_v) = _t->colorSaturation(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->colorValue(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->colorAlpha(); break;
        case 3: *reinterpret_cast<QColor*>(_v) = _t->color(); break;
        case 4: *reinterpret_cast<qreal*>(_v) = _t->colorHue(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setColorSaturation(*reinterpret_cast<qreal*>(_v)); break;
        case 1: _t->setColorValue(*reinterpret_cast<qreal*>(_v)); break;
        case 2: _t->setColorAlpha(*reinterpret_cast<qreal*>(_v)); break;
        case 3: _t->setColor(*reinterpret_cast<QColor*>(_v)); break;
        case 4: _t->setColorHue(*reinterpret_cast<qreal*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *color_widgets::HueSlider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *color_widgets::HueSlider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets9HueSliderE_t>.strings))
        return static_cast<void*>(this);
    return GradientSlider::qt_metacast(_clname);
}

int color_widgets::HueSlider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GradientSlider::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void color_widgets::HueSlider::colorHueChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void color_widgets::HueSlider::colorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void color_widgets::HueSlider::colorAlphaChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void color_widgets::HueSlider::colorSaturationChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void color_widgets::HueSlider::colorValueChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
