/****************************************************************************
** Meta object code from reading C++ file 'polystar.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/src/core/model/shapes/polystar.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'polystar.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10glaxnimate5model8PolyStarE_t {};
} // unnamed namespace

template <> constexpr inline auto glaxnimate::model::PolyStar::qt_create_metaobjectdata<qt_meta_tag_ZN10glaxnimate5model8PolyStarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "glaxnimate::model::PolyStar",
        "type",
        "property StarType",
        "position",
        "property animated QPointF",
        "outer_radius",
        "property animated float",
        "inner_radius",
        "angle",
        "points",
        "property animated int",
        "outer_roundness",
        "inner_roundness",
        "StarType",
        "glaxnimate::model::AnimatableBase*",
        "Star",
        "Polygon"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'type'
        QtMocHelpers::PropertyData<enum StarType>(1, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'position'
        QtMocHelpers::PropertyData<glaxnimate::model::AnimatableBase*>(3, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'outer_radius'
        QtMocHelpers::PropertyData<glaxnimate::model::AnimatableBase*>(5, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'inner_radius'
        QtMocHelpers::PropertyData<glaxnimate::model::AnimatableBase*>(7, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'angle'
        QtMocHelpers::PropertyData<glaxnimate::model::AnimatableBase*>(8, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'points'
        QtMocHelpers::PropertyData<glaxnimate::model::AnimatableBase*>(9, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'outer_roundness'
        QtMocHelpers::PropertyData<glaxnimate::model::AnimatableBase*>(11, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'inner_roundness'
        QtMocHelpers::PropertyData<glaxnimate::model::AnimatableBase*>(12, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'StarType'
        QtMocHelpers::EnumData<enum StarType>(13, 13, QMC::EnumFlags{}).add({
            {   15, StarType::Star },
            {   16, StarType::Polygon },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    6 },
            {    8,    6 },
            {    9,   10 },
            {   11,    6 },
            {   12,    6 },
    });
    return QtMocHelpers::metaObjectData<PolyStar, qt_meta_tag_ZN10glaxnimate5model8PolyStarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject glaxnimate::model::PolyStar::staticMetaObject = { {
    QMetaObject::SuperData::link<Shape::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model8PolyStarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model8PolyStarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10glaxnimate5model8PolyStarE_t>.metaTypes,
    nullptr
} };

void glaxnimate::model::PolyStar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PolyStar *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
        case 6:
        case 5:
        case 4:
        case 3:
        case 2:
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< glaxnimate::model::AnimatableBase* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum StarType*>(_v) = _t->get_type(); break;
        case 1: *reinterpret_cast<glaxnimate::model::AnimatableBase**>(_v) = _t->get_position(); break;
        case 2: *reinterpret_cast<glaxnimate::model::AnimatableBase**>(_v) = _t->get_outer_radius(); break;
        case 3: *reinterpret_cast<glaxnimate::model::AnimatableBase**>(_v) = _t->get_inner_radius(); break;
        case 4: *reinterpret_cast<glaxnimate::model::AnimatableBase**>(_v) = _t->get_angle(); break;
        case 5: *reinterpret_cast<glaxnimate::model::AnimatableBase**>(_v) = _t->get_points(); break;
        case 6: *reinterpret_cast<glaxnimate::model::AnimatableBase**>(_v) = _t->get_outer_roundness(); break;
        case 7: *reinterpret_cast<glaxnimate::model::AnimatableBase**>(_v) = _t->get_inner_roundness(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->set_type(*reinterpret_cast<enum StarType*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *glaxnimate::model::PolyStar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *glaxnimate::model::PolyStar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10glaxnimate5model8PolyStarE_t>.strings))
        return static_cast<void*>(this);
    return Shape::qt_metacast(_clname);
}

int glaxnimate::model::PolyStar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Shape::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
