/********************************************************************************
** Form generated from reading UI file 'layers.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LAYERS_H
#define UI_LAYERS_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDockWidget>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "widgets/docks/layer_view.hpp"

QT_BEGIN_NAMESPACE

class Ui_dock_layers
{
public:
    QWidget *dockWidgetContents;
    QVBoxLayout *verticalLayout;
    glaxnimate::gui::LayerView *view_document_node;
    QHBoxLayout *horizontalLayout;
    QToolButton *btn_layer_add;
    QToolButton *btn_layer_duplicate;
    QToolButton *btn_layer_remove;
    QSpacerItem *horizontalSpacer;
    QToolButton *btn_layer_top;
    QToolButton *btn_layer_up;
    QToolButton *btn_layer_down;
    QToolButton *btn_layer_bottom;

    void setupUi(QDockWidget *dock_layers)
    {
        if (dock_layers->objectName().isEmpty())
            dock_layers->setObjectName("dock_layers");
        dock_layers->resize(400, 300);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(dock_layers->sizePolicy().hasHeightForWidth());
        dock_layers->setSizePolicy(sizePolicy);
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("dialog-layers");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        dock_layers->setWindowIcon(icon);
        dockWidgetContents = new QWidget();
        dockWidgetContents->setObjectName("dockWidgetContents");
        verticalLayout = new QVBoxLayout(dockWidgetContents);
        verticalLayout->setObjectName("verticalLayout");
        view_document_node = new glaxnimate::gui::LayerView(dockWidgetContents);
        view_document_node->setObjectName("view_document_node");
        view_document_node->setDragEnabled(true);
        view_document_node->setDragDropMode(QAbstractItemView::InternalMove);
        view_document_node->setSelectionMode(QAbstractItemView::ExtendedSelection);
        view_document_node->setAutoExpandDelay(500);
        view_document_node->header()->setVisible(false);
        view_document_node->header()->setMinimumSectionSize(32);
        view_document_node->header()->setDefaultSectionSize(32);
        view_document_node->header()->setStretchLastSection(false);

        verticalLayout->addWidget(view_document_node);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        btn_layer_add = new QToolButton(dockWidgetContents);
        btn_layer_add->setObjectName("btn_layer_add");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("layer-new");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("../../../../../../.designer/backup"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_layer_add->setIcon(icon1);

        horizontalLayout->addWidget(btn_layer_add);

        btn_layer_duplicate = new QToolButton(dockWidgetContents);
        btn_layer_duplicate->setObjectName("btn_layer_duplicate");
        QIcon icon2;
        iconThemeName = QString::fromUtf8("layer-duplicate");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("../../../../../../.designer/backup"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_layer_duplicate->setIcon(icon2);

        horizontalLayout->addWidget(btn_layer_duplicate);

        btn_layer_remove = new QToolButton(dockWidgetContents);
        btn_layer_remove->setObjectName("btn_layer_remove");
        QIcon icon3;
        iconThemeName = QString::fromUtf8("layer-delete");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("../../../../../../.designer/backup"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_layer_remove->setIcon(icon3);

        horizontalLayout->addWidget(btn_layer_remove);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        btn_layer_top = new QToolButton(dockWidgetContents);
        btn_layer_top->setObjectName("btn_layer_top");
        QIcon icon4;
        iconThemeName = QString::fromUtf8("layer-top");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon4 = QIcon::fromTheme(iconThemeName);
        } else {
            icon4.addFile(QString::fromUtf8("../../../../../../.designer/backup"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_layer_top->setIcon(icon4);

        horizontalLayout->addWidget(btn_layer_top);

        btn_layer_up = new QToolButton(dockWidgetContents);
        btn_layer_up->setObjectName("btn_layer_up");
        QIcon icon5;
        iconThemeName = QString::fromUtf8("layer-raise");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon5 = QIcon::fromTheme(iconThemeName);
        } else {
            icon5.addFile(QString::fromUtf8("../../../../../../.designer/backup"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_layer_up->setIcon(icon5);

        horizontalLayout->addWidget(btn_layer_up);

        btn_layer_down = new QToolButton(dockWidgetContents);
        btn_layer_down->setObjectName("btn_layer_down");
        QIcon icon6;
        iconThemeName = QString::fromUtf8("layer-lower");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon6 = QIcon::fromTheme(iconThemeName);
        } else {
            icon6.addFile(QString::fromUtf8("../../../../../../.designer/backup"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_layer_down->setIcon(icon6);

        horizontalLayout->addWidget(btn_layer_down);

        btn_layer_bottom = new QToolButton(dockWidgetContents);
        btn_layer_bottom->setObjectName("btn_layer_bottom");
        QIcon icon7;
        iconThemeName = QString::fromUtf8("layer-bottom");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon7 = QIcon::fromTheme(iconThemeName);
        } else {
            icon7.addFile(QString::fromUtf8("../../../../../../.designer/backup"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        btn_layer_bottom->setIcon(icon7);

        horizontalLayout->addWidget(btn_layer_bottom);


        verticalLayout->addLayout(horizontalLayout);

        dock_layers->setWidget(dockWidgetContents);

        retranslateUi(dock_layers);

        QMetaObject::connectSlotsByName(dock_layers);
    } // setupUi

    void retranslateUi(QDockWidget *dock_layers)
    {
        dock_layers->setWindowTitle(tr2i18n("Layers", nullptr));
#if QT_CONFIG(tooltip)
        btn_layer_add->setToolTip(tr2i18n("Add", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_layer_add->setText(tr2i18n("Add", nullptr));
#if QT_CONFIG(tooltip)
        btn_layer_duplicate->setToolTip(tr2i18n("Duplicate", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_layer_duplicate->setText(tr2i18n("Duplicate", nullptr));
#if QT_CONFIG(tooltip)
        btn_layer_remove->setToolTip(tr2i18n("Remove", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_layer_remove->setText(tr2i18n("Remove", nullptr));
#if QT_CONFIG(tooltip)
        btn_layer_top->setToolTip(tr2i18n("Move To Top", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_layer_top->setText(tr2i18n("Move To Top", nullptr));
#if QT_CONFIG(tooltip)
        btn_layer_up->setToolTip(tr2i18n("Move Up", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_layer_up->setText(tr2i18n("Move Up", nullptr));
#if QT_CONFIG(tooltip)
        btn_layer_down->setToolTip(tr2i18n("Move Down", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_layer_down->setText(tr2i18n("Move Down", nullptr));
#if QT_CONFIG(tooltip)
        btn_layer_bottom->setToolTip(tr2i18n("Move To Bottom", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_layer_bottom->setText(tr2i18n("Move To Bottom", nullptr));
    } // retranslateUi

};

namespace Ui {
    class dock_layers: public Ui_dock_layers {};
} // namespace Ui

QT_END_NAMESPACE

#endif // LAYERS_H

