/****************************************************************************
** Meta object code from reading C++ file 'color_palette_widget.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../glaxnimate/external/Qt-Color-Widgets/include/QtColorWidgets/color_palette_widget.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'color_palette_widget.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13color_widgets18ColorPaletteWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto color_widgets::ColorPaletteWidget::qt_create_metaobjectdata<qt_meta_tag_ZN13color_widgets18ColorPaletteWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "color_widgets::ColorPaletteWidget",
        "modelChanged",
        "",
        "ColorPaletteModel*",
        "model",
        "colorSizeChanged",
        "QSize",
        "colorSize",
        "colorSizePolicyChanged",
        "Swatch::ColorSizePolicy",
        "colorSizePolicy",
        "forcedRowsChanged",
        "forcedRows",
        "forcedColumnsChanged",
        "forcedColumns",
        "readOnlyChanged",
        "readOnly",
        "currentColorChanged",
        "QColor",
        "currentColor",
        "index",
        "borderChanged",
        "QPen",
        "border",
        "currentRowChanged",
        "currentRow",
        "currentPaletteChanged",
        "ColorPalette",
        "palette",
        "defaultColorChanged",
        "color",
        "setModel",
        "setColorSize",
        "setColorSizePolicy",
        "setBorder",
        "setForcedRows",
        "setForcedColumns",
        "setReadOnly",
        "clearCurrentColor",
        "setCurrentColor",
        "name",
        "setCurrentRow",
        "setDefaultColor",
        "on_palette_list_currentIndexChanged",
        "on_swatch_doubleClicked",
        "color_widgets::Swatch::ColorSizePolicy",
        "currentPalette",
        "defaultColor"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'modelChanged'
        QtMocHelpers::SignalData<void(ColorPaletteModel *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'colorSizeChanged'
        QtMocHelpers::SignalData<void(const QSize &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'colorSizePolicyChanged'
        QtMocHelpers::SignalData<void(Swatch::ColorSizePolicy)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'forcedRowsChanged'
        QtMocHelpers::SignalData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Signal 'forcedColumnsChanged'
        QtMocHelpers::SignalData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Signal 'readOnlyChanged'
        QtMocHelpers::SignalData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Signal 'currentColorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Signal 'currentColorChanged'
        QtMocHelpers::SignalData<void(int)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 20 },
        }}),
        // Signal 'borderChanged'
        QtMocHelpers::SignalData<void(const QPen &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Signal 'currentRowChanged'
        QtMocHelpers::SignalData<void(int)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Signal 'currentPaletteChanged'
        QtMocHelpers::SignalData<void(const ColorPalette &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Signal 'defaultColorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 30 },
        }}),
        // Slot 'setModel'
        QtMocHelpers::SlotData<void(ColorPaletteModel *)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setColorSize'
        QtMocHelpers::SlotData<void(const QSize &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'setColorSizePolicy'
        QtMocHelpers::SlotData<void(Swatch::ColorSizePolicy)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'setBorder'
        QtMocHelpers::SlotData<void(const QPen &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'setForcedRows'
        QtMocHelpers::SlotData<void(int)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'setForcedColumns'
        QtMocHelpers::SlotData<void(int)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'setReadOnly'
        QtMocHelpers::SlotData<void(bool)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'clearCurrentColor'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setCurrentColor'
        QtMocHelpers::SlotData<bool(const QColor &)>(39, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 18, 30 },
        }}),
        // Slot 'setCurrentColor'
        QtMocHelpers::SlotData<bool(const QString &)>(39, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 40 },
        }}),
        // Slot 'setCurrentColor'
        QtMocHelpers::SlotData<bool(int)>(39, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 20 },
        }}),
        // Slot 'setCurrentRow'
        QtMocHelpers::SlotData<void(int)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'setDefaultColor'
        QtMocHelpers::SlotData<void(const QColor &)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 30 },
        }}),
        // Slot 'on_palette_list_currentIndexChanged'
        QtMocHelpers::SlotData<void(int)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 20 },
        }}),
        // Slot 'on_swatch_doubleClicked'
        QtMocHelpers::SlotData<void(int)>(44, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 20 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'model'
        QtMocHelpers::PropertyData<ColorPaletteModel*>(4, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'colorSize'
        QtMocHelpers::PropertyData<QSize>(7, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'colorSizePolicy'
        QtMocHelpers::PropertyData<color_widgets::Swatch::ColorSizePolicy>(10, 0x80000000 | 45, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'border'
        QtMocHelpers::PropertyData<QPen>(23, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 8),
        // property 'forcedRows'
        QtMocHelpers::PropertyData<int>(12, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'forcedColumns'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'readOnly'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'currentColor'
        QtMocHelpers::PropertyData<QColor>(19, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'currentRow'
        QtMocHelpers::PropertyData<int>(25, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'currentPalette'
        QtMocHelpers::PropertyData<ColorPalette>(46, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 10),
        // property 'defaultColor'
        QtMocHelpers::PropertyData<QColor>(47, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 11),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ColorPaletteWidget, qt_meta_tag_ZN13color_widgets18ColorPaletteWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN13color_widgets18ColorPaletteWidgetE[] = {
    QMetaObject::SuperData::link<color_widgets::Swatch::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject color_widgets::ColorPaletteWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets18ColorPaletteWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets18ColorPaletteWidgetE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN13color_widgets18ColorPaletteWidgetE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13color_widgets18ColorPaletteWidgetE_t>.metaTypes,
    nullptr
} };

void color_widgets::ColorPaletteWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ColorPaletteWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->modelChanged((*reinterpret_cast<std::add_pointer_t<ColorPaletteModel*>>(_a[1]))); break;
        case 1: _t->colorSizeChanged((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 2: _t->colorSizePolicyChanged((*reinterpret_cast<std::add_pointer_t<Swatch::ColorSizePolicy>>(_a[1]))); break;
        case 3: _t->forcedRowsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->forcedColumnsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->readOnlyChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->currentColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 7: _t->currentColorChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->borderChanged((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 9: _t->currentRowChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->currentPaletteChanged((*reinterpret_cast<std::add_pointer_t<ColorPalette>>(_a[1]))); break;
        case 11: _t->defaultColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 12: _t->setModel((*reinterpret_cast<std::add_pointer_t<ColorPaletteModel*>>(_a[1]))); break;
        case 13: _t->setColorSize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 14: _t->setColorSizePolicy((*reinterpret_cast<std::add_pointer_t<Swatch::ColorSizePolicy>>(_a[1]))); break;
        case 15: _t->setBorder((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 16: _t->setForcedRows((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->setForcedColumns((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->setReadOnly((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->clearCurrentColor(); break;
        case 20: { bool _r = _t->setCurrentColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->setCurrentColor((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->setCurrentColor((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 23: _t->setCurrentRow((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 24: _t->setDefaultColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 25: _t->on_palette_list_currentIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 26: _t->on_swatch_doubleClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ColorPaletteModel* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ColorPaletteModel* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ColorPaletteWidget::*)(ColorPaletteModel * )>(_a, &ColorPaletteWidget::modelChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPaletteWidget::*)(const QSize & )>(_a, &ColorPaletteWidget::colorSizeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPaletteWidget::*)(Swatch::ColorSizePolicy )>(_a, &ColorPaletteWidget::colorSizePolicyChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPaletteWidget::*)(int )>(_a, &ColorPaletteWidget::forcedRowsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPaletteWidget::*)(int )>(_a, &ColorPaletteWidget::forcedColumnsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPaletteWidget::*)(bool )>(_a, &ColorPaletteWidget::readOnlyChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPaletteWidget::*)(const QColor & )>(_a, &ColorPaletteWidget::currentColorChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPaletteWidget::*)(int )>(_a, &ColorPaletteWidget::currentColorChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPaletteWidget::*)(const QPen & )>(_a, &ColorPaletteWidget::borderChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPaletteWidget::*)(int )>(_a, &ColorPaletteWidget::currentRowChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPaletteWidget::*)(const ColorPalette & )>(_a, &ColorPaletteWidget::currentPaletteChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (ColorPaletteWidget::*)(const QColor & )>(_a, &ColorPaletteWidget::defaultColorChanged, 11))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< ColorPaletteModel* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<ColorPaletteModel**>(_v) = _t->model(); break;
        case 1: *reinterpret_cast<QSize*>(_v) = _t->colorSize(); break;
        case 2: *reinterpret_cast<color_widgets::Swatch::ColorSizePolicy*>(_v) = _t->colorSizePolicy(); break;
        case 3: *reinterpret_cast<QPen*>(_v) = _t->border(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->forcedRows(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->forcedColumns(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->readOnly(); break;
        case 7: *reinterpret_cast<QColor*>(_v) = _t->currentColor(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->currentRow(); break;
        case 9: *reinterpret_cast<ColorPalette*>(_v) = _t->currentPalette(); break;
        case 10: *reinterpret_cast<QColor*>(_v) = _t->defaultColor(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setModel(*reinterpret_cast<ColorPaletteModel**>(_v)); break;
        case 1: _t->setColorSize(*reinterpret_cast<QSize*>(_v)); break;
        case 2: _t->setColorSizePolicy(*reinterpret_cast<color_widgets::Swatch::ColorSizePolicy*>(_v)); break;
        case 3: _t->setBorder(*reinterpret_cast<QPen*>(_v)); break;
        case 4: _t->setForcedRows(*reinterpret_cast<int*>(_v)); break;
        case 5: _t->setForcedColumns(*reinterpret_cast<int*>(_v)); break;
        case 6: _t->setReadOnly(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setCurrentColor(*reinterpret_cast<QColor*>(_v)); break;
        case 8: _t->setCurrentRow(*reinterpret_cast<int*>(_v)); break;
        case 10: _t->setDefaultColor(*reinterpret_cast<QColor*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *color_widgets::ColorPaletteWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *color_widgets::ColorPaletteWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13color_widgets18ColorPaletteWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int color_widgets::ColorPaletteWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void color_widgets::ColorPaletteWidget::modelChanged(ColorPaletteModel * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void color_widgets::ColorPaletteWidget::colorSizeChanged(const QSize & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void color_widgets::ColorPaletteWidget::colorSizePolicyChanged(Swatch::ColorSizePolicy _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void color_widgets::ColorPaletteWidget::forcedRowsChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void color_widgets::ColorPaletteWidget::forcedColumnsChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void color_widgets::ColorPaletteWidget::readOnlyChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void color_widgets::ColorPaletteWidget::currentColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void color_widgets::ColorPaletteWidget::currentColorChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void color_widgets::ColorPaletteWidget::borderChanged(const QPen & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void color_widgets::ColorPaletteWidget::currentRowChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void color_widgets::ColorPaletteWidget::currentPaletteChanged(const ColorPalette & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void color_widgets::ColorPaletteWidget::defaultColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}
QT_WARNING_POP
