/********************************************************************************
** Form generated from reading UI file 'document_metadata_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DOCUMENT_METADATA_DIALOG_H
#define UI_DOCUMENT_METADATA_DIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_DocumentMetadataDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    QWidget *tab;
    QFormLayout *formLayout;
    QLabel *label_2;
    QLineEdit *edit_author;
    QLabel *label_3;
    QLineEdit *edit_description;
    QLabel *label_4;
    QPlainTextEdit *edit_keywords;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_2;
    QTableWidget *table_widget;
    QDialogButtonBox *button_box;

    void setupUi(QDialog *DocumentMetadataDialog)
    {
        if (DocumentMetadataDialog->objectName().isEmpty())
            DocumentMetadataDialog->setObjectName("DocumentMetadataDialog");
        DocumentMetadataDialog->resize(400, 303);
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("documentinfo");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        DocumentMetadataDialog->setWindowIcon(icon);
        verticalLayout = new QVBoxLayout(DocumentMetadataDialog);
        verticalLayout->setObjectName("verticalLayout");
        tabWidget = new QTabWidget(DocumentMetadataDialog);
        tabWidget->setObjectName("tabWidget");
        tabWidget->setDocumentMode(true);
        tab = new QWidget();
        tab->setObjectName("tab");
        formLayout = new QFormLayout(tab);
        formLayout->setObjectName("formLayout");
        label_2 = new QLabel(tab);
        label_2->setObjectName("label_2");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label_2);

        edit_author = new QLineEdit(tab);
        edit_author->setObjectName("edit_author");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, edit_author);

        label_3 = new QLabel(tab);
        label_3->setObjectName("label_3");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label_3);

        edit_description = new QLineEdit(tab);
        edit_description->setObjectName("edit_description");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, edit_description);

        label_4 = new QLabel(tab);
        label_4->setObjectName("label_4");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, label_4);

        edit_keywords = new QPlainTextEdit(tab);
        edit_keywords->setObjectName("edit_keywords");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, edit_keywords);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        verticalLayout_2 = new QVBoxLayout(tab_2);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        table_widget = new QTableWidget(tab_2);
        if (table_widget->columnCount() < 2)
            table_widget->setColumnCount(2);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        table_widget->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        table_widget->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        table_widget->setObjectName("table_widget");
        table_widget->horizontalHeader()->setStretchLastSection(true);
        table_widget->verticalHeader()->setVisible(false);

        verticalLayout_2->addWidget(table_widget);

        tabWidget->addTab(tab_2, QString());

        verticalLayout->addWidget(tabWidget);

        button_box = new QDialogButtonBox(DocumentMetadataDialog);
        button_box->setObjectName("button_box");
        button_box->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(button_box);

#if QT_CONFIG(shortcut)
        label_2->setBuddy(edit_author);
        label_3->setBuddy(edit_description);
        label_4->setBuddy(edit_keywords);
#endif // QT_CONFIG(shortcut)

        retranslateUi(DocumentMetadataDialog);
        QObject::connect(table_widget, SIGNAL(itemChanged(QTableWidgetItem*)), DocumentMetadataDialog, SLOT(update_item(QTableWidgetItem*)));
        QObject::connect(button_box, SIGNAL(clicked(QAbstractButton*)), DocumentMetadataDialog, SLOT(button_clicked(QAbstractButton*)));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(DocumentMetadataDialog);
    } // setupUi

    void retranslateUi(QDialog *DocumentMetadataDialog)
    {
        DocumentMetadataDialog->setWindowTitle(tr2i18n("Document Metadata", nullptr));
        label_2->setText(tr2i18n("Author", nullptr));
        label_3->setText(tr2i18n("Description", nullptr));
        label_4->setText(tr2i18n("Keywords", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), tr2i18n("Common", nullptr));
        QTableWidgetItem *___qtablewidgetitem = table_widget->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(tr2i18n("Name", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = table_widget->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(tr2i18n("Value", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), tr2i18n("Custom", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DocumentMetadataDialog: public Ui_DocumentMetadataDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // DOCUMENT_METADATA_DIALOG_H

