/********************************************************************************
** Form generated from reading UI file 'stroke_style_widget.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STROKE_STYLE_WIDGET_H
#define UI_STROKE_STYLE_WIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "widgets/shape_style/color_selector.hpp"

QT_BEGIN_NAMESPACE

class Ui_StrokeStyleWidget
{
public:
    QVBoxLayout *main_layout;
    QTabWidget *tab_widget;
    QWidget *tab;
    QFormLayout *formLayout;
    QLabel *label;
    QDoubleSpinBox *spin_stroke_width;
    QLabel *label_2;
    QHBoxLayout *layout_cap;
    QToolButton *button_cap_butt;
    QSpacerItem *horizontalSpacer;
    QToolButton *button_cap_round;
    QSpacerItem *horizontalSpacer_2;
    QToolButton *button_cap_square;
    QLabel *label_3;
    QHBoxLayout *layout_join;
    QToolButton *button_join_bevel;
    QSpacerItem *horizontalSpacer_3;
    QToolButton *button_join_round;
    QSpacerItem *horizontalSpacer_4;
    QToolButton *button_join_miter;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer_5;
    QDoubleSpinBox *spin_miter;
    QWidget *tab_3;
    QVBoxLayout *verticalLayout_3;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_2;
    glaxnimate::gui::ColorSelector *color_selector;
    QWidget *tab_2;
    QWidget *frame;

    void setupUi(QWidget *StrokeStyleWidget)
    {
        if (StrokeStyleWidget->objectName().isEmpty())
            StrokeStyleWidget->setObjectName("StrokeStyleWidget");
        StrokeStyleWidget->resize(262, 284);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(StrokeStyleWidget->sizePolicy().hasHeightForWidth());
        StrokeStyleWidget->setSizePolicy(sizePolicy);
        main_layout = new QVBoxLayout(StrokeStyleWidget);
        main_layout->setObjectName("main_layout");
        tab_widget = new QTabWidget(StrokeStyleWidget);
        tab_widget->setObjectName("tab_widget");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(tab_widget->sizePolicy().hasHeightForWidth());
        tab_widget->setSizePolicy(sizePolicy1);
        tab = new QWidget();
        tab->setObjectName("tab");
        formLayout = new QFormLayout(tab);
        formLayout->setObjectName("formLayout");
        label = new QLabel(tab);
        label->setObjectName("label");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label);

        spin_stroke_width = new QDoubleSpinBox(tab);
        spin_stroke_width->setObjectName("spin_stroke_width");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(spin_stroke_width->sizePolicy().hasHeightForWidth());
        spin_stroke_width->setSizePolicy(sizePolicy2);
        spin_stroke_width->setMaximum(999.990000000000009);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, spin_stroke_width);

        label_2 = new QLabel(tab);
        label_2->setObjectName("label_2");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label_2);

        layout_cap = new QHBoxLayout();
        layout_cap->setObjectName("layout_cap");
        button_cap_butt = new QToolButton(tab);
        button_cap_butt->setObjectName("button_cap_butt");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("stroke-cap-butt");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_cap_butt->setIcon(icon);
        button_cap_butt->setCheckable(true);

        layout_cap->addWidget(button_cap_butt);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        layout_cap->addItem(horizontalSpacer);

        button_cap_round = new QToolButton(tab);
        button_cap_round->setObjectName("button_cap_round");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("stroke-cap-round");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_cap_round->setIcon(icon1);
        button_cap_round->setCheckable(true);

        layout_cap->addWidget(button_cap_round);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        layout_cap->addItem(horizontalSpacer_2);

        button_cap_square = new QToolButton(tab);
        button_cap_square->setObjectName("button_cap_square");
        QIcon icon2;
        iconThemeName = QString::fromUtf8("stroke-cap-square");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_cap_square->setIcon(icon2);
        button_cap_square->setCheckable(true);

        layout_cap->addWidget(button_cap_square);


        formLayout->setLayout(1, QFormLayout::ItemRole::FieldRole, layout_cap);

        label_3 = new QLabel(tab);
        label_3->setObjectName("label_3");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, label_3);

        layout_join = new QHBoxLayout();
        layout_join->setObjectName("layout_join");
        button_join_bevel = new QToolButton(tab);
        button_join_bevel->setObjectName("button_join_bevel");
        QIcon icon3;
        iconThemeName = QString::fromUtf8("stroke-join-bevel");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_join_bevel->setIcon(icon3);
        button_join_bevel->setCheckable(true);

        layout_join->addWidget(button_join_bevel);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        layout_join->addItem(horizontalSpacer_3);

        button_join_round = new QToolButton(tab);
        button_join_round->setObjectName("button_join_round");
        QIcon icon4;
        iconThemeName = QString::fromUtf8("stroke-join-round");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon4 = QIcon::fromTheme(iconThemeName);
        } else {
            icon4.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_join_round->setIcon(icon4);
        button_join_round->setCheckable(true);

        layout_join->addWidget(button_join_round);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        layout_join->addItem(horizontalSpacer_4);

        button_join_miter = new QToolButton(tab);
        button_join_miter->setObjectName("button_join_miter");
        QIcon icon5;
        iconThemeName = QString::fromUtf8("stroke-join-miter");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon5 = QIcon::fromTheme(iconThemeName);
        } else {
            icon5.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        button_join_miter->setIcon(icon5);
        button_join_miter->setCheckable(true);

        layout_join->addWidget(button_join_miter);


        formLayout->setLayout(2, QFormLayout::ItemRole::FieldRole, layout_join);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_5);

        spin_miter = new QDoubleSpinBox(tab);
        spin_miter->setObjectName("spin_miter");
        spin_miter->setEnabled(false);
        spin_miter->setDecimals(2);
        spin_miter->setMaximum(999.990000000000009);

        horizontalLayout_2->addWidget(spin_miter);


        formLayout->setLayout(3, QFormLayout::ItemRole::FieldRole, horizontalLayout_2);

        tab_widget->addTab(tab, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName("tab_3");
        verticalLayout_3 = new QVBoxLayout(tab_3);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        scrollArea = new QScrollArea(tab_3);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 242, 165));
        verticalLayout_2 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        color_selector = new glaxnimate::gui::ColorSelector(scrollAreaWidgetContents);
        color_selector->setObjectName("color_selector");

        verticalLayout_2->addWidget(color_selector);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout_3->addWidget(scrollArea);

        tab_widget->addTab(tab_3, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        tab_2->setEnabled(false);
        tab_widget->addTab(tab_2, QString());

        main_layout->addWidget(tab_widget);

        frame = new QWidget(StrokeStyleWidget);
        frame->setObjectName("frame");
        sizePolicy.setHeightForWidth(frame->sizePolicy().hasHeightForWidth());
        frame->setSizePolicy(sizePolicy);
        frame->setMinimumSize(QSize(0, 64));
        frame->setMaximumSize(QSize(16777215, 128));

        main_layout->addWidget(frame);


        retranslateUi(StrokeStyleWidget);
        QObject::connect(button_join_miter, &QToolButton::toggled, spin_miter, &QDoubleSpinBox::setEnabled);
        QObject::connect(button_cap_butt, SIGNAL(clicked()), StrokeStyleWidget, SLOT(check_cap()));
        QObject::connect(button_cap_round, SIGNAL(clicked()), StrokeStyleWidget, SLOT(check_cap()));
        QObject::connect(button_cap_square, SIGNAL(clicked()), StrokeStyleWidget, SLOT(check_cap()));
        QObject::connect(button_join_miter, SIGNAL(clicked()), StrokeStyleWidget, SLOT(check_join()));
        QObject::connect(button_join_bevel, SIGNAL(clicked()), StrokeStyleWidget, SLOT(check_join()));
        QObject::connect(button_join_round, SIGNAL(clicked()), StrokeStyleWidget, SLOT(check_join()));
        QObject::connect(spin_stroke_width, SIGNAL(valueChanged(double)), StrokeStyleWidget, SLOT(check_width(double)));
        QObject::connect(spin_miter, SIGNAL(valueChanged(double)), StrokeStyleWidget, SLOT(check_miter(double)));
        QObject::connect(color_selector, SIGNAL(current_color_changed(QColor)), StrokeStyleWidget, SLOT(check_color(QColor)));
        QObject::connect(color_selector, SIGNAL(current_color_committed(QColor)), StrokeStyleWidget, SLOT(color_committed(QColor)));
        QObject::connect(spin_stroke_width, SIGNAL(editingFinished()), StrokeStyleWidget, SLOT(commit_width()));
        QObject::connect(color_selector, SIGNAL(current_color_cleared()), StrokeStyleWidget, SLOT(clear_color()));

        tab_widget->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(StrokeStyleWidget);
    } // setupUi

    void retranslateUi(QWidget *StrokeStyleWidget)
    {
        label->setText(tr2i18n("Width", nullptr));
#if QT_CONFIG(tooltip)
        spin_stroke_width->setToolTip(tr2i18n("Stroke Width", nullptr));
#endif // QT_CONFIG(tooltip)
        label_2->setText(tr2i18n("Cap", nullptr));
#if QT_CONFIG(tooltip)
        button_cap_butt->setToolTip(tr2i18n("Butt", nullptr));
#endif // QT_CONFIG(tooltip)
        button_cap_butt->setText(tr2i18n("Butt", nullptr));
#if QT_CONFIG(tooltip)
        button_cap_round->setToolTip(tr2i18n("Round", nullptr));
#endif // QT_CONFIG(tooltip)
        button_cap_round->setText(tr2i18n("Round", nullptr));
#if QT_CONFIG(tooltip)
        button_cap_square->setToolTip(tr2i18n("Square", nullptr));
#endif // QT_CONFIG(tooltip)
        button_cap_square->setText(tr2i18n("Square", nullptr));
        label_3->setText(tr2i18n("Join", nullptr));
#if QT_CONFIG(tooltip)
        button_join_bevel->setToolTip(tr2i18n("Bevel", nullptr));
#endif // QT_CONFIG(tooltip)
        button_join_bevel->setText(tr2i18n("Bevel", nullptr));
#if QT_CONFIG(tooltip)
        button_join_round->setToolTip(tr2i18n("Round", nullptr));
#endif // QT_CONFIG(tooltip)
        button_join_round->setText(tr2i18n("Round", nullptr));
#if QT_CONFIG(tooltip)
        button_join_miter->setToolTip(tr2i18n("Miter", nullptr));
#endif // QT_CONFIG(tooltip)
        button_join_miter->setText(tr2i18n("Miter", nullptr));
#if QT_CONFIG(tooltip)
        spin_miter->setToolTip(tr2i18n("Miter Limit", nullptr));
#endif // QT_CONFIG(tooltip)
        tab_widget->setTabText(tab_widget->indexOf(tab), tr2i18n("Style", nullptr));
        tab_widget->setTabText(tab_widget->indexOf(tab_3), tr2i18n("Color", nullptr));
        tab_widget->setTabText(tab_widget->indexOf(tab_2), tr2i18n("Dashes", nullptr));
        (void)StrokeStyleWidget;
    } // retranslateUi

};

namespace Ui {
    class StrokeStyleWidget: public Ui_StrokeStyleWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // STROKE_STYLE_WIDGET_H

